////////////////////////////////////////////////////////
//
// GEM - Graphics Environment for Multimedia
//
// Implementation file
//
// Copyright (c) 2002 IOhannes m zmoelnig. forum::für::umläute. IEM
//	zmoelnig@iem.kug.ac.at
//  For information on usage and redistribution, and for a DISCLAIMER
//  *  OF ALL WARRANTIES, see the file, "GEM.LICENSE.TERMS"
//
//  this file has been generated...
////////////////////////////////////////////////////////

#include "GEMglRasterPos2dv.h"

CPPEXTERN_NEW_WITH_TWO_ARGS ( GEMglRasterPos2dv , t_floatarg, A_DEFFLOAT, t_floatarg, A_DEFFLOAT)

/////////////////////////////////////////////////////////
//
// GEMglRasterPos2dv
//
/////////////////////////////////////////////////////////
// Constructor
//
GEMglRasterPos2dv :: GEMglRasterPos2dv	(t_floatarg arg0=0, t_floatarg arg1=0) {
vMess(arg0, arg1);
	m_inlet = inlet_new(this->x_obj, &this->x_obj->ob_pd, &s_float, gensym("v"));
}
/////////////////////////////////////////////////////////
// Destructor
//
GEMglRasterPos2dv :: ~GEMglRasterPos2dv () {
	inlet_free(m_inlet);
}

/////////////////////////////////////////////////////////
// Render
//
void GEMglRasterPos2dv :: render(GemState *state) {
	glRasterPos2dv (v);
}

/////////////////////////////////////////////////////////
// variable
//
void GEMglRasterPos2dv :: vMess (t_float arg0, t_float arg1) {	// FUN
	v[0]=(GLdouble)arg0;
	v[1]=(GLdouble)arg1;
	setModified();
}

/////////////////////////////////////////////////////////
// static member functions
//

void GEMglRasterPos2dv :: obj_setupCallback(t_class *classPtr) {
	 class_addmethod(classPtr, (t_method)&GEMglRasterPos2dv::vMessCallback,  	gensym("v"), A_DEFFLOAT, A_DEFFLOAT, A_NULL);
}

void GEMglRasterPos2dv :: vMessCallback (void* data, t_floatarg arg0, t_floatarg arg1) {
	GetMyClass(data)->vMess ( arg0, arg1);
}
