        return _GL_UNDEFINED;
    case 4:
        if (!strcmp(name, "GL_Q")) return GL_Q;
        if (!strcmp(name, "GL_R")) return GL_R;
        if (!strcmp(name, "GL_S")) return GL_S;
        if (!strcmp(name, "GL_T")) return GL_T;
        return _GL_UNDEFINED;
    case 5:
        if (!strcmp(name, "GL_2D")) return GL_2D;
        if (!strcmp(name, "GL_3D")) return GL_3D;
        if (!strcmp(name, "GL_CW")) return GL_CW;
        if (!strcmp(name, "GL_OR")) return GL_OR;
        if (!strcmp(name, "GL_R8")) return GL_R8;
        if (!strcmp(name, "GL_RG")) return GL_RG;
        return _GL_UNDEFINED;
    case 6:
        if (!strcmp(name, "GL_ADD")) return GL_ADD;
        if (!strcmp(name, "GL_AND")) return GL_AND;
        if (!strcmp(name, "GL_BGR")) return GL_BGR;
        if (!strcmp(name, "GL_CCW")) return GL_CCW;
        if (!strcmp(name, "GL_EXP")) return GL_EXP;
        if (!strcmp(name, "GL_FOG")) return GL_FOG;
        if (!strcmp(name, "GL_INT")) return GL_INT;
        if (!strcmp(name, "GL_MAX")) return GL_MAX;
        if (!strcmp(name, "GL_MIN")) return GL_MIN;
        if (!strcmp(name, "GL_NOR")) return GL_NOR;
        if (!strcmp(name, "GL_ONE")) return GL_ONE;
        if (!strcmp(name, "GL_R16")) return GL_R16;
        if (!strcmp(name, "GL_R8I")) return GL_R8I;
        if (!strcmp(name, "GL_RED")) return GL_RED;
        if (!strcmp(name, "GL_RED")) return GL_RED;
        if (!strcmp(name, "GL_RG8")) return GL_RG8;
        if (!strcmp(name, "GL_RGB")) return GL_RGB;
        if (!strcmp(name, "GL_SET")) return GL_SET;
        if (!strcmp(name, "GL_V2F")) return GL_V2F;
        if (!strcmp(name, "GL_V3F")) return GL_V3F;
        if (!strcmp(name, "GL_XOR")) return GL_XOR;
        return _GL_UNDEFINED;
    case 7:
        if (!strcmp(name, "GL_AUX0")) return GL_AUX0;
        if (!strcmp(name, "GL_AUX1")) return GL_AUX1;
        if (!strcmp(name, "GL_AUX2")) return GL_AUX2;
        if (!strcmp(name, "GL_AUX3")) return GL_AUX3;
        if (!strcmp(name, "GL_BACK")) return GL_BACK;
        if (!strcmp(name, "GL_BGRA")) return GL_BGRA;
        if (!strcmp(name, "GL_BGRA")) return GL_BGRA;
        if (!strcmp(name, "GL_BLUE")) return GL_BLUE;
        if (!strcmp(name, "GL_BOOL")) return GL_BOOL;
        if (!strcmp(name, "GL_BYTE")) return GL_BYTE;
        if (!strcmp(name, "GL_BYTE")) return GL_BYTE;
        if (!strcmp(name, "GL_COPY")) return GL_COPY;
        if (!strcmp(name, "GL_DECR")) return GL_DECR;
        if (!strcmp(name, "GL_EXP2")) return GL_EXP2;
        if (!strcmp(name, "GL_FILL")) return GL_FILL;
        if (!strcmp(name, "GL_FLAT")) return GL_FLAT;
        if (!strcmp(name, "GL_INCR")) return GL_INCR;
        if (!strcmp(name, "GL_KEEP")) return GL_KEEP;
        if (!strcmp(name, "GL_LEFT")) return GL_LEFT;
        if (!strcmp(name, "GL_LESS")) return GL_LESS;
        if (!strcmp(name, "GL_LINE")) return GL_LINE;
        if (!strcmp(name, "GL_LOAD")) return GL_LOAD;
        if (!strcmp(name, "GL_MULT")) return GL_MULT;
        if (!strcmp(name, "GL_NAND")) return GL_NAND;
        if (!strcmp(name, "GL_NONE")) return GL_NONE;
        if (!strcmp(name, "GL_NOOP")) return GL_NOOP;
        if (!strcmp(name, "GL_R16F")) return GL_R16F;
        if (!strcmp(name, "GL_R16I")) return GL_R16I;
        if (!strcmp(name, "GL_R32F")) return GL_R32F;
        if (!strcmp(name, "GL_R32I")) return GL_R32I;
        if (!strcmp(name, "GL_R8UI")) return GL_R8UI;
        if (!strcmp(name, "GL_RG16")) return GL_RG16;
        if (!strcmp(name, "GL_RG8I")) return GL_RG8I;
        if (!strcmp(name, "GL_RGB4")) return GL_RGB4;
        if (!strcmp(name, "GL_RGB5")) return GL_RGB5;
        if (!strcmp(name, "GL_RGB8")) return GL_RGB8;
        if (!strcmp(name, "GL_RGBA")) return GL_RGBA;
        if (!strcmp(name, "GL_SRGB")) return GL_SRGB;
        if (!strcmp(name, "GL_SRGB")) return GL_SRGB;
        if (!strcmp(name, "GL_TRUE")) return GL_TRUE;
        if (!strcmp(name, "GL_ZERO")) return GL_ZERO;
        return _GL_UNDEFINED;
    case 8:
        if (!strcmp(name, "GL_ACCUM")) return GL_ACCUM;
        if (!strcmp(name, "GL_ALPHA")) return GL_ALPHA;
        if (!strcmp(name, "GL_BLEND")) return GL_BLEND;
        if (!strcmp(name, "GL_CLAMP")) return GL_CLAMP;
        if (!strcmp(name, "GL_CLEAR")) return GL_CLEAR;
        if (!strcmp(name, "GL_COEFF")) return GL_COEFF;
        if (!strcmp(name, "GL_COLOR")) return GL_COLOR;
        if (!strcmp(name, "GL_DECAL")) return GL_DECAL;
        if (!strcmp(name, "GL_DEPTH")) return GL_DEPTH;
        if (!strcmp(name, "GL_EQUAL")) return GL_EQUAL;
        if (!strcmp(name, "GL_EQUIV")) return GL_EQUIV;
        if (!strcmp(name, "GL_FALSE")) return GL_FALSE;
        if (!strcmp(name, "GL_FLOAT")) return GL_FLOAT;
        if (!strcmp(name, "GL_FLOAT")) return GL_FLOAT;
        if (!strcmp(name, "GL_FRONT")) return GL_FRONT;
        if (!strcmp(name, "GL_GREEN")) return GL_GREEN;
        if (!strcmp(name, "GL_INDEX")) return GL_INDEX;
        if (!strcmp(name, "GL_LINES")) return GL_LINES;
        if (!strcmp(name, "GL_NEVER")) return GL_NEVER;
        if (!strcmp(name, "GL_ORDER")) return GL_ORDER;
        if (!strcmp(name, "GL_POINT")) return GL_POINT;
        if (!strcmp(name, "GL_QUADS")) return GL_QUADS;
        if (!strcmp(name, "GL_R16UI")) return GL_R16UI;
        if (!strcmp(name, "GL_R32UI")) return GL_R32UI;
        if (!strcmp(name, "GL_RG16F")) return GL_RG16F;
        if (!strcmp(name, "GL_RG16I")) return GL_RG16I;
        if (!strcmp(name, "GL_RG32F")) return GL_RG32F;
        if (!strcmp(name, "GL_RG32I")) return GL_RG32I;
        if (!strcmp(name, "GL_RG8UI")) return GL_RG8UI;
        if (!strcmp(name, "GL_RGB10")) return GL_RGB10;
        if (!strcmp(name, "GL_RGB12")) return GL_RGB12;
        if (!strcmp(name, "GL_RGB16")) return GL_RGB16;
        if (!strcmp(name, "GL_RGB8I")) return GL_RGB8I;
        if (!strcmp(name, "GL_RGBA2")) return GL_RGBA2;
        if (!strcmp(name, "GL_RGBA4")) return GL_RGBA4;
        if (!strcmp(name, "GL_RGBA8")) return GL_RGBA8;
        if (!strcmp(name, "GL_RIGHT")) return GL_RIGHT;
        if (!strcmp(name, "GL_SHORT")) return GL_SHORT;
        if (!strcmp(name, "GL_SRGB8")) return GL_SRGB8;
        if (!strcmp(name, "GL_W_EXT")) return GL_W_EXT;
        if (!strcmp(name, "GL_X_EXT")) return GL_X_EXT;
        if (!strcmp(name, "GL_Y_EXT")) return GL_Y_EXT;
        if (!strcmp(name, "GL_Z_EXT")) return GL_Z_EXT;
        return _GL_UNDEFINED;
    case 9:
        if (!strcmp(name, "GL_ALPHA4")) return GL_ALPHA4;
        if (!strcmp(name, "GL_ALPHA8")) return GL_ALPHA8;
        if (!strcmp(name, "GL_ALWAYS")) return GL_ALWAYS;
        if (!strcmp(name, "GL_BITMAP")) return GL_BITMAP;
        if (!strcmp(name, "GL_DITHER")) return GL_DITHER;
        if (!strcmp(name, "GL_DOMAIN")) return GL_DOMAIN;
        if (!strcmp(name, "GL_DOUBLE")) return GL_DOUBLE;
        if (!strcmp(name, "GL_GEQUAL")) return GL_GEQUAL;
        if (!strcmp(name, "GL_INVERT")) return GL_INVERT;
        if (!strcmp(name, "GL_LEQUAL")) return GL_LEQUAL;
        if (!strcmp(name, "GL_LIGHT0")) return GL_LIGHT0;
        if (!strcmp(name, "GL_LIGHT1")) return GL_LIGHT1;
        if (!strcmp(name, "GL_LIGHT2")) return GL_LIGHT2;
        if (!strcmp(name, "GL_LIGHT3")) return GL_LIGHT3;
        if (!strcmp(name, "GL_LIGHT4")) return GL_LIGHT4;
        if (!strcmp(name, "GL_LIGHT5")) return GL_LIGHT5;
        if (!strcmp(name, "GL_LIGHT6")) return GL_LIGHT6;
        if (!strcmp(name, "GL_LIGHT7")) return GL_LIGHT7;
        if (!strcmp(name, "GL_LINEAR")) return GL_LINEAR;
        if (!strcmp(name, "GL_MINMAX")) return GL_MINMAX;
        if (!strcmp(name, "GL_NICEST")) return GL_NICEST;
        if (!strcmp(name, "GL_POINTS")) return GL_POINTS;
        if (!strcmp(name, "GL_REDUCE")) return GL_REDUCE;
        if (!strcmp(name, "GL_RENDER")) return GL_RENDER;
        if (!strcmp(name, "GL_REPEAT")) return GL_REPEAT;
        if (!strcmp(name, "GL_RETURN")) return GL_RETURN;
        if (!strcmp(name, "GL_RG16UI")) return GL_RG16UI;
        if (!strcmp(name, "GL_RG32UI")) return GL_RG32UI;
        if (!strcmp(name, "GL_RGB16F")) return GL_RGB16F;
        if (!strcmp(name, "GL_RGB16I")) return GL_RGB16I;
        if (!strcmp(name, "GL_RGB32F")) return GL_RGB32F;
        if (!strcmp(name, "GL_RGB32I")) return GL_RGB32I;
        if (!strcmp(name, "GL_RGB8UI")) return GL_RGB8UI;
        if (!strcmp(name, "GL_RGBA12")) return GL_RGBA12;
        if (!strcmp(name, "GL_RGBA16")) return GL_RGBA16;
        if (!strcmp(name, "GL_RGBA8I")) return GL_RGBA8I;
        if (!strcmp(name, "GL_SELECT")) return GL_SELECT;
        if (!strcmp(name, "GL_SMOOTH")) return GL_SMOOTH;
        if (!strcmp(name, "GL_STEREO")) return GL_STEREO;
        if (!strcmp(name, "GL_VENDOR")) return GL_VENDOR;
        if (!strcmp(name, "GL_ZOOM_X")) return GL_ZOOM_X;
        if (!strcmp(name, "GL_ZOOM_Y")) return GL_ZOOM_Y;
        return _GL_UNDEFINED;
    case 10:
        if (!strcmp(name, "GL_2_BYTES")) return GL_2_BYTES;
        if (!strcmp(name, "GL_3_BYTES")) return GL_3_BYTES;
        if (!strcmp(name, "GL_422_EXT")) return GL_422_EXT;
        if (!strcmp(name, "GL_4_BYTES")) return GL_4_BYTES;
        if (!strcmp(name, "GL_ADD_ATI")) return GL_ADD_ATI;
        if (!strcmp(name, "GL_ALPHA12")) return GL_ALPHA12;
        if (!strcmp(name, "GL_ALPHA16")) return GL_ALPHA16;
        if (!strcmp(name, "GL_AMBIENT")) return GL_AMBIENT;
        if (!strcmp(name, "GL_BGR_EXT")) return GL_BGR_EXT;
        if (!strcmp(name, "GL_C3F_V3F")) return GL_C3F_V3F;
        if (!strcmp(name, "GL_CND_ATI")) return GL_CND_ATI;
        if (!strcmp(name, "GL_COMBINE")) return GL_COMBINE;
        if (!strcmp(name, "GL_COMPILE")) return GL_COMPILE;
        if (!strcmp(name, "GL_DIFFUSE")) return GL_DIFFUSE;
        if (!strcmp(name, "GL_DSDT_NV")) return GL_DSDT_NV;
        if (!strcmp(name, "GL_DSDT_NV")) return GL_DSDT_NV;
        if (!strcmp(name, "GL_FASTEST")) return GL_FASTEST;
        if (!strcmp(name, "GL_FOG_BIT")) return GL_FOG_BIT;
        if (!strcmp(name, "GL_FOG_END")) return GL_FOG_END;
        if (!strcmp(name, "GL_GREATER")) return GL_GREATER;
        if (!strcmp(name, "GL_HILO_NV")) return GL_HILO_NV;
        if (!strcmp(name, "GL_HILO_NV")) return GL_HILO_NV;
        if (!strcmp(name, "GL_MAD_ATI")) return GL_MAD_ATI;
        if (!strcmp(name, "GL_MAX_EXT")) return GL_MAX_EXT;
        if (!strcmp(name, "GL_MIN_EXT")) return GL_MIN_EXT;
        if (!strcmp(name, "GL_MOV_ATI")) return GL_MOV_ATI;
        if (!strcmp(name, "GL_MUL_ATI")) return GL_MUL_ATI;
        if (!strcmp(name, "GL_N3F_V3F")) return GL_N3F_V3F;
        if (!strcmp(name, "GL_NEAREST")) return GL_NEAREST;
        if (!strcmp(name, "GL_ONE_EXT")) return GL_ONE_EXT;
        if (!strcmp(name, "GL_POLYGON")) return GL_POLYGON;
        if (!strcmp(name, "GL_REPLACE")) return GL_REPLACE;
        if (!strcmp(name, "GL_RGB16UI")) return GL_RGB16UI;
        if (!strcmp(name, "GL_RGB32UI")) return GL_RGB32UI;
        if (!strcmp(name, "GL_RGB5_A1")) return GL_RGB5_A1;
        if (!strcmp(name, "GL_RGB9_E5")) return GL_RGB9_E5;
        if (!strcmp(name, "GL_RGBA16F")) return GL_RGBA16F;
        if (!strcmp(name, "GL_RGBA16I")) return GL_RGBA16I;
        if (!strcmp(name, "GL_RGBA32F")) return GL_RGBA32F;
        if (!strcmp(name, "GL_RGBA32I")) return GL_RGBA32I;
        if (!strcmp(name, "GL_RGBA8UI")) return GL_RGBA8UI;
        if (!strcmp(name, "GL_S3_s3tc")) return GL_S3_s3tc;
        if (!strcmp(name, "GL_SAMPLES")) return GL_SAMPLES;
        if (!strcmp(name, "GL_STENCIL")) return GL_STENCIL;
        if (!strcmp(name, "GL_SUB_ATI")) return GL_SUB_ATI;
        if (!strcmp(name, "GL_T2F_V3F")) return GL_T2F_V3F;
        if (!strcmp(name, "GL_T4F_V4F")) return GL_T4F_V4F;
        if (!strcmp(name, "GL_TEXTURE")) return GL_TEXTURE;
        if (!strcmp(name, "GL_VERSION")) return GL_VERSION;
        return _GL_UNDEFINED;
    case 11:
        if (!strcmp(name, "GL_3D_COLOR")) return GL_3D_COLOR;
        if (!strcmp(name, "GL_ABGR_EXT")) return GL_ABGR_EXT;
        if (!strcmp(name, "GL_BGRA_EXT")) return GL_BGRA_EXT;
        if (!strcmp(name, "GL_BOOL_ARB")) return GL_BOOL_ARB;
        if (!strcmp(name, "GL_C4UB_V2F")) return GL_C4UB_V2F;
        if (!strcmp(name, "GL_C4UB_V3F")) return GL_C4UB_V3F;
        if (!strcmp(name, "GL_CMYK_EXT")) return GL_CMYK_EXT;
        if (!strcmp(name, "GL_CND0_ATI")) return GL_CND0_ATI;
        if (!strcmp(name, "GL_CONSTANT")) return GL_CONSTANT;
        if (!strcmp(name, "GL_DOT3_ATI")) return GL_DOT3_ATI;
        if (!strcmp(name, "GL_DOT3_RGB")) return GL_DOT3_RGB;
        if (!strcmp(name, "GL_DOT4_ATI")) return GL_DOT4_ATI;
        if (!strcmp(name, "GL_DSDT8_NV")) return GL_DSDT8_NV;
        if (!strcmp(name, "GL_DSDT8_NV")) return GL_DSDT8_NV;
        if (!strcmp(name, "GL_DUDV_ATI")) return GL_DUDV_ATI;
        if (!strcmp(name, "GL_EMISSION")) return GL_EMISSION;
        if (!strcmp(name, "GL_EVAL_BIT")) return GL_EVAL_BIT;
        if (!strcmp(name, "GL_EXT_abgr")) return GL_EXT_abgr;
        if (!strcmp(name, "GL_EXT_bgra")) return GL_EXT_bgra;
        if (!strcmp(name, "GL_FEEDBACK")) return GL_FEEDBACK;
        if (!strcmp(name, "GL_FOG_HINT")) return GL_FOG_HINT;
        if (!strcmp(name, "GL_FOG_MODE")) return GL_FOG_MODE;
        if (!strcmp(name, "GL_FRAME_NV")) return GL_FRAME_NV;
        if (!strcmp(name, "GL_FUNC_ADD")) return GL_FUNC_ADD;
        if (!strcmp(name, "GL_HILO8_NV")) return GL_HILO8_NV;
        if (!strcmp(name, "GL_HINT_BIT")) return GL_HINT_BIT;
        if (!strcmp(name, "GL_INT_VEC2")) return GL_INT_VEC2;
        if (!strcmp(name, "GL_INT_VEC3")) return GL_INT_VEC3;
        if (!strcmp(name, "GL_INT_VEC4")) return GL_INT_VEC4;
        if (!strcmp(name, "GL_LERP_ATI")) return GL_LERP_ATI;
        if (!strcmp(name, "GL_LIGHTING")) return GL_LIGHTING;
        if (!strcmp(name, "GL_LINE_BIT")) return GL_LINE_BIT;
        if (!strcmp(name, "GL_LIST_BIT")) return GL_LIST_BIT;
        if (!strcmp(name, "GL_LOGIC_OP")) return GL_LOGIC_OP;
        if (!strcmp(name, "GL_MODULATE")) return GL_MODULATE;
        if (!strcmp(name, "GL_NO_ERROR")) return GL_NO_ERROR;
        if (!strcmp(name, "GL_NOTEQUAL")) return GL_NOTEQUAL;
        if (!strcmp(name, "GL_NV_fence")) return GL_NV_fence;
        if (!strcmp(name, "GL_POSITION")) return GL_POSITION;
        if (!strcmp(name, "GL_PREVIOUS")) return GL_PREVIOUS;
        if (!strcmp(name, "GL_R3_G3_B2")) return GL_R3_G3_B2;
        if (!strcmp(name, "GL_RED_BIAS")) return GL_RED_BIAS;
        if (!strcmp(name, "GL_RED_BITS")) return GL_RED_BITS;
        if (!strcmp(name, "GL_RENDERER")) return GL_RENDERER;
        if (!strcmp(name, "GL_RGB10_A2")) return GL_RGB10_A2;
        if (!strcmp(name, "GL_RGB2_EXT")) return GL_RGB2_EXT;
        if (!strcmp(name, "GL_RGB4_EXT")) return GL_RGB4_EXT;
        if (!strcmp(name, "GL_RGB5_EXT")) return GL_RGB5_EXT;
        if (!strcmp(name, "GL_RGB8_EXT")) return GL_RGB8_EXT;
        if (!strcmp(name, "GL_RGBA16UI")) return GL_RGBA16UI;
        if (!strcmp(name, "GL_RGBA32UI")) return GL_RGBA32UI;
        if (!strcmp(name, "GL_RGB_S3TC")) return GL_RGB_S3TC;
        if (!strcmp(name, "GL_SPECULAR")) return GL_SPECULAR;
        if (!strcmp(name, "GL_SRC0_RGB")) return GL_SRC0_RGB;
        if (!strcmp(name, "GL_SRC1_RGB")) return GL_SRC1_RGB;
        if (!strcmp(name, "GL_SRC2_RGB")) return GL_SRC2_RGB;
        if (!strcmp(name, "GL_SRGB_EXT")) return GL_SRGB_EXT;
        if (!strcmp(name, "GL_SUBTRACT")) return GL_SUBTRACT;
        if (!strcmp(name, "GL_TEXTURE0")) return GL_TEXTURE0;
        if (!strcmp(name, "GL_TEXTURE1")) return GL_TEXTURE1;
        if (!strcmp(name, "GL_TEXTURE2")) return GL_TEXTURE2;
        if (!strcmp(name, "GL_TEXTURE3")) return GL_TEXTURE3;
        if (!strcmp(name, "GL_TEXTURE4")) return GL_TEXTURE4;
        if (!strcmp(name, "GL_TEXTURE5")) return GL_TEXTURE5;
        if (!strcmp(name, "GL_TEXTURE6")) return GL_TEXTURE6;
        if (!strcmp(name, "GL_TEXTURE7")) return GL_TEXTURE7;
        if (!strcmp(name, "GL_TEXTURE8")) return GL_TEXTURE8;
        if (!strcmp(name, "GL_TEXTURE9")) return GL_TEXTURE9;
        if (!strcmp(name, "GL_VIEWPORT")) return GL_VIEWPORT;
        if (!strcmp(name, "GL_ZERO_EXT")) return GL_ZERO_EXT;
        return _GL_UNDEFINED;
    case 12:
        if (!strcmp(name, "GL_1PASS_EXT")) return GL_1PASS_EXT;
        if (!strcmp(name, "GL_BACK_LEFT")) return GL_BACK_LEFT;
        if (!strcmp(name, "GL_BLEND_DST")) return GL_BLEND_DST;
        if (!strcmp(name, "GL_BLEND_SRC")) return GL_BLEND_SRC;
        if (!strcmp(name, "GL_BLUE_BIAS")) return GL_BLUE_BIAS;
        if (!strcmp(name, "GL_BLUE_BITS")) return GL_BLUE_BITS;
        if (!strcmp(name, "GL_BOOL_VEC2")) return GL_BOOL_VEC2;
        if (!strcmp(name, "GL_BOOL_VEC3")) return GL_BOOL_VEC3;
        if (!strcmp(name, "GL_BOOL_VEC4")) return GL_BOOL_VEC4;
        if (!strcmp(name, "GL_CMYKA_EXT")) return GL_CMYKA_EXT;
        if (!strcmp(name, "GL_COLOR_SUM")) return GL_COLOR_SUM;
        if (!strcmp(name, "GL_CON_0_ATI")) return GL_CON_0_ATI;
        if (!strcmp(name, "GL_CON_1_ATI")) return GL_CON_1_ATI;
        if (!strcmp(name, "GL_CON_2_ATI")) return GL_CON_2_ATI;
        if (!strcmp(name, "GL_CON_3_ATI")) return GL_CON_3_ATI;
        if (!strcmp(name, "GL_CON_4_ATI")) return GL_CON_4_ATI;
        if (!strcmp(name, "GL_CON_5_ATI")) return GL_CON_5_ATI;
        if (!strcmp(name, "GL_CON_6_ATI")) return GL_CON_6_ATI;
        if (!strcmp(name, "GL_CON_7_ATI")) return GL_CON_7_ATI;
        if (!strcmp(name, "GL_CUBIC_EXT")) return GL_CUBIC_EXT;
        if (!strcmp(name, "GL_CULL_FACE")) return GL_CULL_FACE;
        if (!strcmp(name, "GL_DECR_WRAP")) return GL_DECR_WRAP;
        if (!strcmp(name, "GL_DONT_CARE")) return GL_DONT_CARE;
        if (!strcmp(name, "GL_DOT3_RGBA")) return GL_DOT3_RGBA;
        if (!strcmp(name, "GL_DST_ALPHA")) return GL_DST_ALPHA;
        if (!strcmp(name, "GL_DST_COLOR")) return GL_DST_COLOR;
        if (!strcmp(name, "GL_EDGE_FLAG")) return GL_EDGE_FLAG;
        if (!strcmp(name, "GL_EXT_cmyka")) return GL_EXT_cmyka;
        if (!strcmp(name, "GL_EYE_PLANE")) return GL_EYE_PLANE;
        if (!strcmp(name, "GL_FIELDS_NV")) return GL_FIELDS_NV;
        if (!strcmp(name, "GL_FOG_COLOR")) return GL_FOG_COLOR;
        if (!strcmp(name, "GL_FOG_COORD")) return GL_FOG_COORD;
        if (!strcmp(name, "GL_FOG_INDEX")) return GL_FOG_INDEX;
        if (!strcmp(name, "GL_FOG_START")) return GL_FOG_START;
        if (!strcmp(name, "GL_HILO16_NV")) return GL_HILO16_NV;
        if (!strcmp(name, "GL_HILO16_NV")) return GL_HILO16_NV;
        if (!strcmp(name, "GL_HISTOGRAM")) return GL_HISTOGRAM;
        if (!strcmp(name, "GL_INCR_WRAP")) return GL_INCR_WRAP;
        if (!strcmp(name, "GL_INTENSITY")) return GL_INTENSITY;
        if (!strcmp(name, "GL_LINE_LOOP")) return GL_LINE_LOOP;
        if (!strcmp(name, "GL_LIST_BASE")) return GL_LIST_BASE;
        if (!strcmp(name, "GL_LIST_MODE")) return GL_LIST_MODE;
        if (!strcmp(name, "GL_LOCAL_EXT")) return GL_LOCAL_EXT;
        if (!strcmp(name, "GL_LUMINANCE")) return GL_LUMINANCE;
        if (!strcmp(name, "GL_MAP_COLOR")) return GL_MAP_COLOR;
        if (!strcmp(name, "GL_MODELVIEW")) return GL_MODELVIEW;
        if (!strcmp(name, "GL_NORMALIZE")) return GL_NORMALIZE;
        if (!strcmp(name, "GL_PHONG_WIN")) return GL_PHONG_WIN;
        if (!strcmp(name, "GL_POINT_BIT")) return GL_POINT_BIT;
        if (!strcmp(name, "GL_READ_ONLY")) return GL_READ_ONLY;
        if (!strcmp(name, "GL_RED_SCALE")) return GL_RED_SCALE;
        if (!strcmp(name, "GL_REG_0_ATI")) return GL_REG_0_ATI;
        if (!strcmp(name, "GL_REG_1_ATI")) return GL_REG_1_ATI;
        if (!strcmp(name, "GL_REG_2_ATI")) return GL_REG_2_ATI;
        if (!strcmp(name, "GL_REG_3_ATI")) return GL_REG_3_ATI;
        if (!strcmp(name, "GL_REG_4_ATI")) return GL_REG_4_ATI;
        if (!strcmp(name, "GL_REG_5_ATI")) return GL_REG_5_ATI;
        if (!strcmp(name, "GL_RGB10_EXT")) return GL_RGB10_EXT;
        if (!strcmp(name, "GL_RGB12_EXT")) return GL_RGB12_EXT;
        if (!strcmp(name, "GL_RGB16_EXT")) return GL_RGB16_EXT;
        if (!strcmp(name, "GL_RGB4_S3TC")) return GL_RGB4_S3TC;
        if (!strcmp(name, "GL_RGB8I_EXT")) return GL_RGB8I_EXT;
        if (!strcmp(name, "GL_RGBA2_EXT")) return GL_RGBA2_EXT;
        if (!strcmp(name, "GL_RGBA4_EXT")) return GL_RGBA4_EXT;
        if (!strcmp(name, "GL_RGBA8_EXT")) return GL_RGBA8_EXT;
        if (!strcmp(name, "GL_RGBA_MODE")) return GL_RGBA_MODE;
        if (!strcmp(name, "GL_RGBA_S3TC")) return GL_RGBA_S3TC;
        if (!strcmp(name, "GL_RGB_SCALE")) return GL_RGB_SCALE;
        if (!strcmp(name, "GL_SHININESS")) return GL_SHININESS;
        if (!strcmp(name, "GL_SPARE0_NV")) return GL_SPARE0_NV;
        if (!strcmp(name, "GL_SPARE1_NV")) return GL_SPARE1_NV;
        if (!strcmp(name, "GL_SRC_ALPHA")) return GL_SRC_ALPHA;
        if (!strcmp(name, "GL_SRC_COLOR")) return GL_SRC_COLOR;
        if (!strcmp(name, "GL_SRGB8_EXT")) return GL_SRGB8_EXT;
        if (!strcmp(name, "GL_TEXTURE10")) return GL_TEXTURE10;
        if (!strcmp(name, "GL_TEXTURE11")) return GL_TEXTURE11;
        if (!strcmp(name, "GL_TEXTURE12")) return GL_TEXTURE12;
        if (!strcmp(name, "GL_TEXTURE13")) return GL_TEXTURE13;
        if (!strcmp(name, "GL_TEXTURE14")) return GL_TEXTURE14;
        if (!strcmp(name, "GL_TEXTURE15")) return GL_TEXTURE15;
        if (!strcmp(name, "GL_TEXTURE16")) return GL_TEXTURE16;
        if (!strcmp(name, "GL_TEXTURE17")) return GL_TEXTURE17;
        if (!strcmp(name, "GL_TEXTURE18")) return GL_TEXTURE18;
        if (!strcmp(name, "GL_TEXTURE19")) return GL_TEXTURE19;
        if (!strcmp(name, "GL_TEXTURE20")) return GL_TEXTURE20;
        if (!strcmp(name, "GL_TEXTURE21")) return GL_TEXTURE21;
        if (!strcmp(name, "GL_TEXTURE22")) return GL_TEXTURE22;
        if (!strcmp(name, "GL_TEXTURE23")) return GL_TEXTURE23;
        if (!strcmp(name, "GL_TEXTURE24")) return GL_TEXTURE24;
        if (!strcmp(name, "GL_TEXTURE25")) return GL_TEXTURE25;
        if (!strcmp(name, "GL_TEXTURE26")) return GL_TEXTURE26;
        if (!strcmp(name, "GL_TEXTURE27")) return GL_TEXTURE27;
        if (!strcmp(name, "GL_TEXTURE28")) return GL_TEXTURE28;
        if (!strcmp(name, "GL_TEXTURE29")) return GL_TEXTURE29;
        if (!strcmp(name, "GL_TEXTURE30")) return GL_TEXTURE30;
        if (!strcmp(name, "GL_TEXTURE31")) return GL_TEXTURE31;
        if (!strcmp(name, "GL_TRIANGLES")) return GL_TRIANGLES;
        return _GL_UNDEFINED;
    case 13:
        if (!strcmp(name, "GL_1PASS_SGIS")) return GL_1PASS_SGIS;
        if (!strcmp(name, "GL_2X_BIT_ATI")) return GL_2X_BIT_ATI;
        if (!strcmp(name, "GL_4X_BIT_ATI")) return GL_4X_BIT_ATI;
        if (!strcmp(name, "GL_8X_BIT_ATI")) return GL_8X_BIT_ATI;
        if (!strcmp(name, "GL_ADD_SIGNED")) return GL_ADD_SIGNED;
        if (!strcmp(name, "GL_ALPHA4_EXT")) return GL_ALPHA4_EXT;
        if (!strcmp(name, "GL_ALPHA8_EXT")) return GL_ALPHA8_EXT;
        if (!strcmp(name, "GL_ALPHA_BIAS")) return GL_ALPHA_BIAS;
        if (!strcmp(name, "GL_ALPHA_BITS")) return GL_ALPHA_BITS;
        if (!strcmp(name, "GL_ALPHA_TEST")) return GL_ALPHA_TEST;
        if (!strcmp(name, "GL_ARB_shadow")) return GL_ARB_shadow;
        if (!strcmp(name, "GL_BACK_RIGHT")) return GL_BACK_RIGHT;
        if (!strcmp(name, "GL_BLUE_SCALE")) return GL_BLUE_SCALE;
        if (!strcmp(name, "GL_DEPTH_BIAS")) return GL_DEPTH_BIAS;
        if (!strcmp(name, "GL_DEPTH_BITS")) return GL_DEPTH_BITS;
        if (!strcmp(name, "GL_DEPTH_FUNC")) return GL_DEPTH_FUNC;
        if (!strcmp(name, "GL_DEPTH_TEST")) return GL_DEPTH_TEST;
        if (!strcmp(name, "GL_DISCARD_NV")) return GL_DISCARD_NV;
        if (!strcmp(name, "GL_DOUBLE_EXT")) return GL_DOUBLE_EXT;
        if (!strcmp(name, "GL_DS_BIAS_NV")) return GL_DS_BIAS_NV;
        if (!strcmp(name, "GL_DT_BIAS_NV")) return GL_DT_BIAS_NV;
        if (!strcmp(name, "GL_DU8DV8_ATI")) return GL_DU8DV8_ATI;
        if (!strcmp(name, "GL_ENABLE_BIT")) return GL_ENABLE_BIT;
        if (!strcmp(name, "GL_EVAL_2D_NV")) return GL_EVAL_2D_NV;
        if (!strcmp(name, "GL_EXTENSIONS")) return GL_EXTENSIONS;
        if (!strcmp(name, "GL_EYE_LINEAR")) return GL_EYE_LINEAR;
        if (!strcmp(name, "GL_FIXED_ONLY")) return GL_FIXED_ONLY;
        if (!strcmp(name, "GL_FLOAT_MAT2")) return GL_FLOAT_MAT2;
        if (!strcmp(name, "GL_FLOAT_MAT3")) return GL_FLOAT_MAT3;
        if (!strcmp(name, "GL_FLOAT_MAT4")) return GL_FLOAT_MAT4;
        if (!strcmp(name, "GL_FLOAT_R_NV")) return GL_FLOAT_R_NV;
        if (!strcmp(name, "GL_FLOAT_VEC2")) return GL_FLOAT_VEC2;
        if (!strcmp(name, "GL_FLOAT_VEC3")) return GL_FLOAT_VEC3;
        if (!strcmp(name, "GL_FLOAT_VEC4")) return GL_FLOAT_VEC4;
        if (!strcmp(name, "GL_FRONT_FACE")) return GL_FRONT_FACE;
        if (!strcmp(name, "GL_FRONT_LEFT")) return GL_FRONT_LEFT;
        if (!strcmp(name, "GL_GREEN_BIAS")) return GL_GREEN_BIAS;
        if (!strcmp(name, "GL_GREEN_BITS")) return GL_GREEN_BITS;
        if (!strcmp(name, "GL_HALF_APPLE")) return GL_HALF_APPLE;
        if (!strcmp(name, "GL_HALF_FLOAT")) return GL_HALF_FLOAT;
        if (!strcmp(name, "GL_HI_BIAS_NV")) return GL_HI_BIAS_NV;
        if (!strcmp(name, "GL_INDEX_BITS")) return GL_INDEX_BITS;
        if (!strcmp(name, "GL_INDEX_MODE")) return GL_INDEX_MODE;
        if (!strcmp(name, "GL_INTENSITY4")) return GL_INTENSITY4;
        if (!strcmp(name, "GL_INTENSITY8")) return GL_INTENSITY8;
        if (!strcmp(name, "GL_INVERSE_NV")) return GL_INVERSE_NV;
        if (!strcmp(name, "GL_LINE_STRIP")) return GL_LINE_STRIP;
        if (!strcmp(name, "GL_LINE_TOKEN")) return GL_LINE_TOKEN;
        if (!strcmp(name, "GL_LINE_WIDTH")) return GL_LINE_WIDTH;
        if (!strcmp(name, "GL_LIST_INDEX")) return GL_LIST_INDEX;
        if (!strcmp(name, "GL_LO_BIAS_NV")) return GL_LO_BIAS_NV;
        if (!strcmp(name, "GL_LOWER_LEFT")) return GL_LOWER_LEFT;
        if (!strcmp(name, "GL_LUMINANCE4")) return GL_LUMINANCE4;
        if (!strcmp(name, "GL_LUMINANCE8")) return GL_LUMINANCE8;
        if (!strcmp(name, "GL_MAP1_INDEX")) return GL_MAP1_INDEX;
        if (!strcmp(name, "GL_MAP2_INDEX")) return GL_MAP2_INDEX;
        if (!strcmp(name, "GL_MATRIX0_NV")) return GL_MATRIX0_NV;
        if (!strcmp(name, "GL_MATRIX1_NV")) return GL_MATRIX1_NV;
        if (!strcmp(name, "GL_MATRIX2_NV")) return GL_MATRIX2_NV;
        if (!strcmp(name, "GL_MATRIX3_NV")) return GL_MATRIX3_NV;
        if (!strcmp(name, "GL_MATRIX4_NV")) return GL_MATRIX4_NV;
        if (!strcmp(name, "GL_MATRIX5_NV")) return GL_MATRIX5_NV;
        if (!strcmp(name, "GL_MATRIX6_NV")) return GL_MATRIX6_NV;
        if (!strcmp(name, "GL_MATRIX7_NV")) return GL_MATRIX7_NV;
        if (!strcmp(name, "GL_MATRIX_EXT")) return GL_MATRIX_EXT;
        if (!strcmp(name, "GL_MAX_LIGHTS")) return GL_MAX_LIGHTS;
        if (!strcmp(name, "GL_MINMAX_EXT")) return GL_MINMAX_EXT;
        if (!strcmp(name, "GL_NORMAL_MAP")) return GL_NORMAL_MAP;
        if (!strcmp(name, "GL_OP_ADD_EXT")) return GL_OP_ADD_EXT;
        if (!strcmp(name, "GL_OP_MAX_EXT")) return GL_OP_MAX_EXT;
        if (!strcmp(name, "GL_OP_MIN_EXT")) return GL_OP_MIN_EXT;
        if (!strcmp(name, "GL_OP_MOV_EXT")) return GL_OP_MOV_EXT;
        if (!strcmp(name, "GL_OP_MUL_EXT")) return GL_OP_MUL_EXT;
        if (!strcmp(name, "GL_OP_SUB_EXT")) return GL_OP_SUB_EXT;
        if (!strcmp(name, "GL_OR_REVERSE")) return GL_OR_REVERSE;
        if (!strcmp(name, "GL_POINT_SIZE")) return GL_POINT_SIZE;
        if (!strcmp(name, "GL_PROJECTION")) return GL_PROJECTION;
        if (!strcmp(name, "GL_QUAD_STRIP")) return GL_QUAD_STRIP;
        if (!strcmp(name, "GL_QUERY_WAIT")) return GL_QUERY_WAIT;
        if (!strcmp(name, "GL_READ_WRITE")) return GL_READ_WRITE;
        if (!strcmp(name, "GL_REDUCE_EXT")) return GL_REDUCE_EXT;
        if (!strcmp(name, "GL_RGB16F_ARB")) return GL_RGB16F_ARB;
        if (!strcmp(name, "GL_RGB16I_EXT")) return GL_RGB16I_EXT;
        if (!strcmp(name, "GL_RGB32F_ARB")) return GL_RGB32F_ARB;
        if (!strcmp(name, "GL_RGB32I_EXT")) return GL_RGB32I_EXT;
        if (!strcmp(name, "GL_RGB8UI_EXT")) return GL_RGB8UI_EXT;
        if (!strcmp(name, "GL_RGBA12_EXT")) return GL_RGBA12_EXT;
        if (!strcmp(name, "GL_RGBA16_EXT")) return GL_RGBA16_EXT;
        if (!strcmp(name, "GL_RGBA4_S3TC")) return GL_RGBA4_S3TC;
        if (!strcmp(name, "GL_RGBA8I_EXT")) return GL_RGBA8I_EXT;
        if (!strcmp(name, "GL_RG_INTEGER")) return GL_RG_INTEGER;
        if (!strcmp(name, "GL_SAMPLER_1D")) return GL_SAMPLER_1D;
        if (!strcmp(name, "GL_SAMPLER_2D")) return GL_SAMPLER_2D;
        if (!strcmp(name, "GL_SAMPLER_3D")) return GL_SAMPLER_3D;
        if (!strcmp(name, "GL_SCALAR_EXT")) return GL_SCALAR_EXT;
        if (!strcmp(name, "GL_SGIX_async")) return GL_SGIX_async;
        if (!strcmp(name, "GL_SGIX_ycrcb")) return GL_SGIX_ycrcb;
        if (!strcmp(name, "GL_SLUMINANCE")) return GL_SLUMINANCE;
        if (!strcmp(name, "GL_SPHERE_MAP")) return GL_SPHERE_MAP;
        if (!strcmp(name, "GL_SRC0_ALPHA")) return GL_SRC0_ALPHA;
        if (!strcmp(name, "GL_SRC1_ALPHA")) return GL_SRC1_ALPHA;
        if (!strcmp(name, "GL_SRC2_ALPHA")) return GL_SRC2_ALPHA;
        if (!strcmp(name, "GL_SRGB_ALPHA")) return GL_SRGB_ALPHA;
        if (!strcmp(name, "GL_STATIC_ATI")) return GL_STATIC_ATI;
        if (!strcmp(name, "GL_SUN_vertex")) return GL_SUN_vertex;
        if (!strcmp(name, "GL_TEXTURE_1D")) return GL_TEXTURE_1D;
        if (!strcmp(name, "GL_TEXTURE_1D")) return GL_TEXTURE_1D;
        if (!strcmp(name, "GL_TEXTURE_2D")) return GL_TEXTURE_2D;
        if (!strcmp(name, "GL_TEXTURE_2D")) return GL_TEXTURE_2D;
        if (!strcmp(name, "GL_TEXTURE_3D")) return GL_TEXTURE_3D;
        if (!strcmp(name, "GL_UPPER_LEFT")) return GL_UPPER_LEFT;
        if (!strcmp(name, "GL_VECTOR_EXT")) return GL_VECTOR_EXT;
        if (!strcmp(name, "GL_WRITE_ONLY")) return GL_WRITE_ONLY;
        if (!strcmp(name, "GL_YCBCR_MESA")) return GL_YCBCR_MESA;
        return _GL_UNDEFINED;
    case 14:
        if (!strcmp(name, "GL_2PASS_0_EXT")) return GL_2PASS_0_EXT;
        if (!strcmp(name, "GL_2PASS_1_EXT")) return GL_2PASS_1_EXT;
        if (!strcmp(name, "GL_422_REV_EXT")) return GL_422_REV_EXT;
        if (!strcmp(name, "GL_4PASS_0_EXT")) return GL_4PASS_0_EXT;
        if (!strcmp(name, "GL_4PASS_1_EXT")) return GL_4PASS_1_EXT;
        if (!strcmp(name, "GL_4PASS_2_EXT")) return GL_4PASS_2_EXT;
        if (!strcmp(name, "GL_4PASS_3_EXT")) return GL_4PASS_3_EXT;
        if (!strcmp(name, "GL_ALPHA12_EXT")) return GL_ALPHA12_EXT;
        if (!strcmp(name, "GL_ALPHA16_EXT")) return GL_ALPHA16_EXT;
        if (!strcmp(name, "GL_ALPHA8I_EXT")) return GL_ALPHA8I_EXT;
        if (!strcmp(name, "GL_ALPHA_SCALE")) return GL_ALPHA_SCALE;
        if (!strcmp(name, "GL_AND_REVERSE")) return GL_AND_REVERSE;
        if (!strcmp(name, "GL_APPLE_fence")) return GL_APPLE_fence;
        if (!strcmp(name, "GL_ARB_imaging")) return GL_ARB_imaging;
        if (!strcmp(name, "GL_ATI_meminfo")) return GL_ATI_meminfo;
        if (!strcmp(name, "GL_AUTO_NORMAL")) return GL_AUTO_NORMAL;
        if (!strcmp(name, "GL_AUX_BUFFERS")) return GL_AUX_BUFFERS;
        if (!strcmp(name, "GL_AVERAGE_EXT")) return GL_AVERAGE_EXT;
        if (!strcmp(name, "GL_BGR_INTEGER")) return GL_BGR_INTEGER;
        if (!strcmp(name, "GL_BLEND_COLOR")) return GL_BLEND_COLOR;
        if (!strcmp(name, "GL_BUFFER_SIZE")) return GL_BUFFER_SIZE;
        if (!strcmp(name, "GL_C4F_N3F_V3F")) return GL_C4F_N3F_V3F;
        if (!strcmp(name, "GL_CLIP_PLANE0")) return GL_CLIP_PLANE0;
        if (!strcmp(name, "GL_CLIP_PLANE1")) return GL_CLIP_PLANE1;
        if (!strcmp(name, "GL_CLIP_PLANE2")) return GL_CLIP_PLANE2;
        if (!strcmp(name, "GL_CLIP_PLANE3")) return GL_CLIP_PLANE3;
        if (!strcmp(name, "GL_CLIP_PLANE4")) return GL_CLIP_PLANE4;
        if (!strcmp(name, "GL_CLIP_PLANE5")) return GL_CLIP_PLANE5;
        if (!strcmp(name, "GL_COLOR_ARRAY")) return GL_COLOR_ARRAY;
        if (!strcmp(name, "GL_COLOR_INDEX")) return GL_COLOR_INDEX;
        if (!strcmp(name, "GL_COLOR_TABLE")) return GL_COLOR_TABLE;
        if (!strcmp(name, "GL_COMBINE4_NV")) return GL_COMBINE4_NV;
        if (!strcmp(name, "GL_COMBINE_ARB")) return GL_COMBINE_ARB;
        if (!strcmp(name, "GL_COMBINE_EXT")) return GL_COMBINE_EXT;
        if (!strcmp(name, "GL_COMBINE_RGB")) return GL_COMBINE_RGB;
        if (!strcmp(name, "GL_CURRENT_BIT")) return GL_CURRENT_BIT;
        if (!strcmp(name, "GL_DEPTH_RANGE")) return GL_DEPTH_RANGE;
        if (!strcmp(name, "GL_DEPTH_SCALE")) return GL_DEPTH_SCALE;
        if (!strcmp(name, "GL_DISCARD_ATI")) return GL_DISCARD_ATI;
        if (!strcmp(name, "GL_DRAW_BUFFER")) return GL_DRAW_BUFFER;
        if (!strcmp(name, "GL_DSDT_MAG_NV")) return GL_DSDT_MAG_NV;
        if (!strcmp(name, "GL_DSDT_MAG_NV")) return GL_DSDT_MAG_NV;
        if (!strcmp(name, "GL_DS_SCALE_NV")) return GL_DS_SCALE_NV;
        if (!strcmp(name, "GL_DT_SCALE_NV")) return GL_DT_SCALE_NV;
        if (!strcmp(name, "GL_DYNAMIC_ATI")) return GL_DYNAMIC_ATI;
        if (!strcmp(name, "GL_ENV_ADD_EXT")) return GL_ENV_ADD_EXT;
        if (!strcmp(name, "GL_EXT_texture")) return GL_EXT_texture;
        if (!strcmp(name, "GL_FENCE_APPLE")) return GL_FENCE_APPLE;
        if (!strcmp(name, "GL_FLOAT_RG_NV")) return GL_FLOAT_RG_NV;
        if (!strcmp(name, "GL_FOG_DENSITY")) return GL_FOG_DENSITY;
        if (!strcmp(name, "GL_FRAMEBUFFER")) return GL_FRAMEBUFFER;
        if (!strcmp(name, "GL_FRONT_RIGHT")) return GL_FRONT_RIGHT;
        if (!strcmp(name, "GL_GREEN_SCALE")) return GL_GREEN_SCALE;
        if (!strcmp(name, "GL_HI_SCALE_NV")) return GL_HI_SCALE_NV;
        if (!strcmp(name, "GL_IDENTITY_NV")) return GL_IDENTITY_NV;
        if (!strcmp(name, "GL_INDEX_ARRAY")) return GL_INDEX_ARRAY;
        if (!strcmp(name, "GL_INDEX_SHIFT")) return GL_INDEX_SHIFT;
        if (!strcmp(name, "GL_INTENSITY12")) return GL_INTENSITY12;
        if (!strcmp(name, "GL_INTENSITY16")) return GL_INTENSITY16;
        if (!strcmp(name, "GL_INTERPOLATE")) return GL_INTERPOLATE;
        if (!strcmp(name, "GL_LINE_SMOOTH")) return GL_LINE_SMOOTH;
        if (!strcmp(name, "GL_LINK_STATUS")) return GL_LINK_STATUS;
        if (!strcmp(name, "GL_LO_SCALE_NV")) return GL_LO_SCALE_NV;
        if (!strcmp(name, "GL_LUMINANCE12")) return GL_LUMINANCE12;
        if (!strcmp(name, "GL_LUMINANCE16")) return GL_LUMINANCE16;
        if (!strcmp(name, "GL_MAP1_NORMAL")) return GL_MAP1_NORMAL;
        if (!strcmp(name, "GL_MAP2_NORMAL")) return GL_MAP2_NORMAL;
        if (!strcmp(name, "GL_MAP_STENCIL")) return GL_MAP_STENCIL;
        if (!strcmp(name, "GL_MATRIX0_ARB")) return GL_MATRIX0_ARB;
        if (!strcmp(name, "GL_MATRIX1_ARB")) return GL_MATRIX1_ARB;
        if (!strcmp(name, "GL_MATRIX2_ARB")) return GL_MATRIX2_ARB;
        if (!strcmp(name, "GL_MATRIX3_ARB")) return GL_MATRIX3_ARB;
        if (!strcmp(name, "GL_MATRIX4_ARB")) return GL_MATRIX4_ARB;
        if (!strcmp(name, "GL_MATRIX5_ARB")) return GL_MATRIX5_ARB;
        if (!strcmp(name, "GL_MATRIX6_ARB")) return GL_MATRIX6_ARB;
        if (!strcmp(name, "GL_MATRIX7_ARB")) return GL_MATRIX7_ARB;
        if (!strcmp(name, "GL_MATRIX8_ARB")) return GL_MATRIX8_ARB;
        if (!strcmp(name, "GL_MATRIX9_ARB")) return GL_MATRIX9_ARB;
        if (!strcmp(name, "GL_MATRIX_MODE")) return GL_MATRIX_MODE;
        if (!strcmp(name, "GL_MAX_SAMPLES")) return GL_MAX_SAMPLES;
        if (!strcmp(name, "GL_MINMAX_SINK")) return GL_MINMAX_SINK;
        if (!strcmp(name, "GL_MULTISAMPLE")) return GL_MULTISAMPLE;
        if (!strcmp(name, "GL_OP_DOT3_EXT")) return GL_OP_DOT3_EXT;
        if (!strcmp(name, "GL_OP_DOT4_EXT")) return GL_OP_DOT4_EXT;
        if (!strcmp(name, "GL_OP_FRAC_EXT")) return GL_OP_FRAC_EXT;
        if (!strcmp(name, "GL_OP_MADD_EXT")) return GL_OP_MADD_EXT;
        if (!strcmp(name, "GL_OR_INVERTED")) return GL_OR_INVERTED;
        if (!strcmp(name, "GL_PERTURB_EXT")) return GL_PERTURB_EXT;
        if (!strcmp(name, "GL_POINT_TOKEN")) return GL_POINT_TOKEN;
        if (!strcmp(name, "GL_POLYGON_BIT")) return GL_POLYGON_BIT;
        if (!strcmp(name, "GL_READ_BUFFER")) return GL_READ_BUFFER;
        if (!strcmp(name, "GL_RED_BIT_ATI")) return GL_RED_BIT_ATI;
        if (!strcmp(name, "GL_RED_INTEGER")) return GL_RED_INTEGER;
        if (!strcmp(name, "GL_RENDER_MODE")) return GL_RENDER_MODE;
        if (!strcmp(name, "GL_REPLACE_EXT")) return GL_REPLACE_EXT;
        if (!strcmp(name, "GL_RESTART_SUN")) return GL_RESTART_SUN;
        if (!strcmp(name, "GL_RGB16UI_EXT")) return GL_RGB16UI_EXT;
        if (!strcmp(name, "GL_RGB32UI_EXT")) return GL_RGB32UI_EXT;
        if (!strcmp(name, "GL_RGB5_A1_EXT")) return GL_RGB5_A1_EXT;
        if (!strcmp(name, "GL_RGB9_E5_EXT")) return GL_RGB9_E5_EXT;
        if (!strcmp(name, "GL_RGBA16F_ARB")) return GL_RGBA16F_ARB;
        if (!strcmp(name, "GL_RGBA16I_EXT")) return GL_RGBA16I_EXT;
        if (!strcmp(name, "GL_RGBA32F_ARB")) return GL_RGBA32F_ARB;
        if (!strcmp(name, "GL_RGBA32I_EXT")) return GL_RGBA32I_EXT;
        if (!strcmp(name, "GL_RGBA8UI_EXT")) return GL_RGBA8UI_EXT;
        if (!strcmp(name, "GL_RGB_INTEGER")) return GL_RGB_INTEGER;
        if (!strcmp(name, "GL_SAMPLES_ARB")) return GL_SAMPLES_ARB;
        if (!strcmp(name, "GL_SAMPLES_EXT")) return GL_SAMPLES_EXT;
        if (!strcmp(name, "GL_SCISSOR_BIT")) return GL_SCISSOR_BIT;
        if (!strcmp(name, "GL_SCISSOR_BOX")) return GL_SCISSOR_BOX;
        if (!strcmp(name, "GL_SGIX_shadow")) return GL_SGIX_shadow;
        if (!strcmp(name, "GL_SGIX_sprite")) return GL_SGIX_sprite;
        if (!strcmp(name, "GL_SHADE_MODEL")) return GL_SHADE_MODEL;
        if (!strcmp(name, "GL_SHADER_TYPE")) return GL_SHADER_TYPE;
        if (!strcmp(name, "GL_SLUMINANCE8")) return GL_SLUMINANCE8;
        if (!strcmp(name, "GL_SOURCE0_RGB")) return GL_SOURCE0_RGB;
        if (!strcmp(name, "GL_SOURCE1_RGB")) return GL_SOURCE1_RGB;
        if (!strcmp(name, "GL_SOURCE2_RGB")) return GL_SOURCE2_RGB;
        if (!strcmp(name, "GL_SPOT_CUTOFF")) return GL_SPOT_CUTOFF;
        if (!strcmp(name, "GL_STATIC_COPY")) return GL_STATIC_COPY;
        if (!strcmp(name, "GL_STATIC_DRAW")) return GL_STATIC_DRAW;
        if (!strcmp(name, "GL_STATIC_READ")) return GL_STATIC_READ;
        if (!strcmp(name, "GL_STENCIL_REF")) return GL_STENCIL_REF;
        if (!strcmp(name, "GL_STREAM_COPY")) return GL_STREAM_COPY;
        if (!strcmp(name, "GL_STREAM_DRAW")) return GL_STREAM_DRAW;
        if (!strcmp(name, "GL_STREAM_READ")) return GL_STREAM_READ;
        if (!strcmp(name, "GL_T2F_C3F_V3F")) return GL_T2F_C3F_V3F;
        if (!strcmp(name, "GL_T2F_N3F_V3F")) return GL_T2F_N3F_V3F;
        if (!strcmp(name, "GL_TEXTURE_BIT")) return GL_TEXTURE_BIT;
        if (!strcmp(name, "GL_TEXTURE_ENV")) return GL_TEXTURE_ENV;
        if (!strcmp(name, "GL_VARIANT_EXT")) return GL_VARIANT_EXT;
        if (!strcmp(name, "GL_VERSION_1_1")) return GL_VERSION_1_1;
        if (!strcmp(name, "GL_VERSION_1_2")) return GL_VERSION_1_2;
        if (!strcmp(name, "GL_VERSION_1_3")) return GL_VERSION_1_3;
        if (!strcmp(name, "GL_VERSION_1_4")) return GL_VERSION_1_4;
        if (!strcmp(name, "GL_VERSION_1_5")) return GL_VERSION_1_5;
        if (!strcmp(name, "GL_VERSION_2_0")) return GL_VERSION_2_0;
        if (!strcmp(name, "GL_VERSION_2_1")) return GL_VERSION_2_1;
        if (!strcmp(name, "GL_VERSION_3_0")) return GL_VERSION_3_0;
        if (!strcmp(name, "GL_WRAP_BORDER")) return GL_WRAP_BORDER;
        return _GL_UNDEFINED;
    case 15:
        if (!strcmp(name, "GL_2PASS_0_SGIS")) return GL_2PASS_0_SGIS;
        if (!strcmp(name, "GL_2PASS_1_SGIS")) return GL_2PASS_1_SGIS;
        if (!strcmp(name, "GL_3DFX_tbuffer")) return GL_3DFX_tbuffer;
        if (!strcmp(name, "GL_4PASS_0_SGIS")) return GL_4PASS_0_SGIS;
        if (!strcmp(name, "GL_4PASS_1_SGIS")) return GL_4PASS_1_SGIS;
        if (!strcmp(name, "GL_4PASS_2_SGIS")) return GL_4PASS_2_SGIS;
        if (!strcmp(name, "GL_4PASS_3_SGIS")) return GL_4PASS_3_SGIS;
        if (!strcmp(name, "GL_ALPHA16F_ARB")) return GL_ALPHA16F_ARB;
        if (!strcmp(name, "GL_ALPHA16I_EXT")) return GL_ALPHA16I_EXT;
        if (!strcmp(name, "GL_ALPHA32F_ARB")) return GL_ALPHA32F_ARB;
        if (!strcmp(name, "GL_ALPHA32I_EXT")) return GL_ALPHA32I_EXT;
        if (!strcmp(name, "GL_ALPHA8UI_EXT")) return GL_ALPHA8UI_EXT;
        if (!strcmp(name, "GL_AND_INVERTED")) return GL_AND_INVERTED;
        if (!strcmp(name, "GL_ARRAY_BUFFER")) return GL_ARRAY_BUFFER;
        if (!strcmp(name, "GL_BGRA_INTEGER")) return GL_BGRA_INTEGER;
        if (!strcmp(name, "GL_BIAS_BIT_ATI")) return GL_BIAS_BIT_ATI;
        if (!strcmp(name, "GL_BITMAP_TOKEN")) return GL_BITMAP_TOKEN;
        if (!strcmp(name, "GL_BLUE_BIT_ATI")) return GL_BLUE_BIT_ATI;
        if (!strcmp(name, "GL_BLUE_INTEGER")) return GL_BLUE_INTEGER;
        if (!strcmp(name, "GL_BUFFER_USAGE")) return GL_BUFFER_USAGE;
        if (!strcmp(name, "GL_COLOR_MATRIX")) return GL_COLOR_MATRIX;
        if (!strcmp(name, "GL_COMBINER0_NV")) return GL_COMBINER0_NV;
        if (!strcmp(name, "GL_COMBINER1_NV")) return GL_COMBINER1_NV;
        if (!strcmp(name, "GL_COMBINER2_NV")) return GL_COMBINER2_NV;
        if (!strcmp(name, "GL_COMBINER3_NV")) return GL_COMBINER3_NV;
        if (!strcmp(name, "GL_COMBINER4_NV")) return GL_COMBINER4_NV;
        if (!strcmp(name, "GL_COMBINER5_NV")) return GL_COMBINER5_NV;
        if (!strcmp(name, "GL_COMBINER6_NV")) return GL_COMBINER6_NV;
        if (!strcmp(name, "GL_COMBINER7_NV")) return GL_COMBINER7_NV;
        if (!strcmp(name, "GL_COMP_BIT_ATI")) return GL_COMP_BIT_ATI;
        if (!strcmp(name, "GL_CONSTANT_ARB")) return GL_CONSTANT_ARB;
        if (!strcmp(name, "GL_CONSTANT_EXT")) return GL_CONSTANT_EXT;
        if (!strcmp(name, "GL_CONST_EYE_NV")) return GL_CONST_EYE_NV;
        if (!strcmp(name, "GL_DEPTH_BUFFER")) return GL_DEPTH_BUFFER;
        if (!strcmp(name, "GL_DOT2_ADD_ATI")) return GL_DOT2_ADD_ATI;
        if (!strcmp(name, "GL_DOT3_RGB_ARB")) return GL_DOT3_RGB_ARB;
        if (!strcmp(name, "GL_DOT3_RGB_EXT")) return GL_DOT3_RGB_EXT;
        if (!strcmp(name, "GL_DOUBLEBUFFER")) return GL_DOUBLEBUFFER;
        if (!strcmp(name, "GL_DRAW_BUFFER0")) return GL_DRAW_BUFFER0;
        if (!strcmp(name, "GL_DRAW_BUFFER1")) return GL_DRAW_BUFFER1;
        if (!strcmp(name, "GL_DRAW_BUFFER2")) return GL_DRAW_BUFFER2;
        if (!strcmp(name, "GL_DRAW_BUFFER3")) return GL_DRAW_BUFFER3;
        if (!strcmp(name, "GL_DRAW_BUFFER4")) return GL_DRAW_BUFFER4;
        if (!strcmp(name, "GL_DRAW_BUFFER5")) return GL_DRAW_BUFFER5;
        if (!strcmp(name, "GL_DRAW_BUFFER6")) return GL_DRAW_BUFFER6;
        if (!strcmp(name, "GL_DRAW_BUFFER7")) return GL_DRAW_BUFFER7;
        if (!strcmp(name, "GL_DRAW_BUFFER8")) return GL_DRAW_BUFFER8;
        if (!strcmp(name, "GL_DRAW_BUFFER9")) return GL_DRAW_BUFFER9;
        if (!strcmp(name, "GL_DYNAMIC_COPY")) return GL_DYNAMIC_COPY;
        if (!strcmp(name, "GL_DYNAMIC_DRAW")) return GL_DYNAMIC_DRAW;
        if (!strcmp(name, "GL_DYNAMIC_READ")) return GL_DYNAMIC_READ;
        if (!strcmp(name, "GL_ENV_COPY_EXT")) return GL_ENV_COPY_EXT;
        if (!strcmp(name, "GL_E_TIMES_F_NV")) return GL_E_TIMES_F_NV;
        if (!strcmp(name, "GL_FLOAT_MAT2x3")) return GL_FLOAT_MAT2x3;
        if (!strcmp(name, "GL_FLOAT_MAT2x4")) return GL_FLOAT_MAT2x4;
        if (!strcmp(name, "GL_FLOAT_MAT3x2")) return GL_FLOAT_MAT3x2;
        if (!strcmp(name, "GL_FLOAT_MAT3x4")) return GL_FLOAT_MAT3x4;
        if (!strcmp(name, "GL_FLOAT_MAT4x2")) return GL_FLOAT_MAT4x2;
        if (!strcmp(name, "GL_FLOAT_MAT4x3")) return GL_FLOAT_MAT4x3;
        if (!strcmp(name, "GL_FLOAT_R16_NV")) return GL_FLOAT_R16_NV;
        if (!strcmp(name, "GL_FLOAT_R32_NV")) return GL_FLOAT_R32_NV;
        if (!strcmp(name, "GL_FLOAT_RGB_NV")) return GL_FLOAT_RGB_NV;
        if (!strcmp(name, "GL_FUNC_ADD_EXT")) return GL_FUNC_ADD_EXT;
        if (!strcmp(name, "GL_HALF_BIT_ATI")) return GL_HALF_BIT_ATI;
        if (!strcmp(name, "GL_INDEX_OFFSET")) return GL_INDEX_OFFSET;
        if (!strcmp(name, "GL_INT_VEC2_ARB")) return GL_INT_VEC2_ARB;
        if (!strcmp(name, "GL_INT_VEC3_ARB")) return GL_INT_VEC3_ARB;
        if (!strcmp(name, "GL_INT_VEC4_ARB")) return GL_INT_VEC4_ARB;
        if (!strcmp(name, "GL_INVALID_ENUM")) return GL_INVALID_ENUM;
        if (!strcmp(name, "GL_KTX_Z_REGION")) return GL_KTX_Z_REGION;
        if (!strcmp(name, "GL_LIGHTING_BIT")) return GL_LIGHTING_BIT;
        if (!strcmp(name, "GL_LINE_STIPPLE")) return GL_LINE_STIPPLE;
        if (!strcmp(name, "GL_MAP1_COLOR_4")) return GL_MAP1_COLOR_4;
        if (!strcmp(name, "GL_MAP2_COLOR_4")) return GL_MAP2_COLOR_4;
        if (!strcmp(name, "GL_MAP_READ_BIT")) return GL_MAP_READ_BIT;
        if (!strcmp(name, "GL_MATRIX10_ARB")) return GL_MATRIX10_ARB;
        if (!strcmp(name, "GL_MATRIX11_ARB")) return GL_MATRIX11_ARB;
        if (!strcmp(name, "GL_MATRIX12_ARB")) return GL_MATRIX12_ARB;
        if (!strcmp(name, "GL_MATRIX13_ARB")) return GL_MATRIX13_ARB;
        if (!strcmp(name, "GL_MATRIX14_ARB")) return GL_MATRIX14_ARB;
        if (!strcmp(name, "GL_MATRIX15_ARB")) return GL_MATRIX15_ARB;
        if (!strcmp(name, "GL_MATRIX16_ARB")) return GL_MATRIX16_ARB;
        if (!strcmp(name, "GL_MATRIX17_ARB")) return GL_MATRIX17_ARB;
        if (!strcmp(name, "GL_MATRIX18_ARB")) return GL_MATRIX18_ARB;
        if (!strcmp(name, "GL_MATRIX19_ARB")) return GL_MATRIX19_ARB;
        if (!strcmp(name, "GL_MATRIX20_ARB")) return GL_MATRIX20_ARB;
        if (!strcmp(name, "GL_MATRIX21_ARB")) return GL_MATRIX21_ARB;
        if (!strcmp(name, "GL_MATRIX22_ARB")) return GL_MATRIX22_ARB;
        if (!strcmp(name, "GL_MATRIX23_ARB")) return GL_MATRIX23_ARB;
        if (!strcmp(name, "GL_MATRIX24_ARB")) return GL_MATRIX24_ARB;
        if (!strcmp(name, "GL_MATRIX25_ARB")) return GL_MATRIX25_ARB;
        if (!strcmp(name, "GL_MATRIX26_ARB")) return GL_MATRIX26_ARB;
        if (!strcmp(name, "GL_MATRIX27_ARB")) return GL_MATRIX27_ARB;
        if (!strcmp(name, "GL_MATRIX28_ARB")) return GL_MATRIX28_ARB;
        if (!strcmp(name, "GL_MATRIX29_ARB")) return GL_MATRIX29_ARB;
        if (!strcmp(name, "GL_MATRIX30_ARB")) return GL_MATRIX30_ARB;
        if (!strcmp(name, "GL_MATRIX31_ARB")) return GL_MATRIX31_ARB;
        if (!strcmp(name, "GL_MAX_RED_SGIS")) return GL_MAX_RED_SGIS;
        if (!strcmp(name, "GL_MIN_RED_SGIS")) return GL_MIN_RED_SGIS;
        if (!strcmp(name, "GL_NORMAL_ARRAY")) return GL_NORMAL_ARRAY;
        if (!strcmp(name, "GL_OBJECT_PLANE")) return GL_OBJECT_PLANE;
        if (!strcmp(name, "GL_OML_resample")) return GL_OML_resample;
        if (!strcmp(name, "GL_OP_CLAMP_EXT")) return GL_OP_CLAMP_EXT;
        if (!strcmp(name, "GL_OPERAND0_RGB")) return GL_OPERAND0_RGB;
        if (!strcmp(name, "GL_OPERAND1_RGB")) return GL_OPERAND1_RGB;
        if (!strcmp(name, "GL_OPERAND2_RGB")) return GL_OPERAND2_RGB;
        if (!strcmp(name, "GL_OP_FLOOR_EXT")) return GL_OP_FLOOR_EXT;
        if (!strcmp(name, "GL_OP_INDEX_EXT")) return GL_OP_INDEX_EXT;
        if (!strcmp(name, "GL_OP_POWER_EXT")) return GL_OP_POWER_EXT;
        if (!strcmp(name, "GL_OP_RECIP_EXT")) return GL_OP_RECIP_EXT;
        if (!strcmp(name, "GL_OP_ROUND_EXT")) return GL_OP_ROUND_EXT;
        if (!strcmp(name, "GL_POINT_SMOOTH")) return GL_POINT_SMOOTH;
        if (!strcmp(name, "GL_POINT_SPRITE")) return GL_POINT_SPRITE;
        if (!strcmp(name, "GL_POLYGON_MODE")) return GL_POLYGON_MODE;
        if (!strcmp(name, "GL_PRESERVE_ATI")) return GL_PRESERVE_ATI;
        if (!strcmp(name, "GL_PREVIOUS_ARB")) return GL_PREVIOUS_ARB;
        if (!strcmp(name, "GL_PREVIOUS_EXT")) return GL_PREVIOUS_EXT;
        if (!strcmp(name, "GL_QUERY_RESULT")) return GL_QUERY_RESULT;
        if (!strcmp(name, "GL_R1UI_V3F_SUN")) return GL_R1UI_V3F_SUN;
        if (!strcmp(name, "GL_RENDERBUFFER")) return GL_RENDERBUFFER;
        if (!strcmp(name, "GL_RGB10_A2_EXT")) return GL_RGB10_A2_EXT;
        if (!strcmp(name, "GL_RGBA16UI_EXT")) return GL_RGBA16UI_EXT;
        if (!strcmp(name, "GL_RGBA32UI_EXT")) return GL_RGBA32UI_EXT;
        if (!strcmp(name, "GL_RGBA_INTEGER")) return GL_RGBA_INTEGER;
        if (!strcmp(name, "GL_SAMPLER_CUBE")) return GL_SAMPLER_CUBE;
        if (!strcmp(name, "GL_SAMPLES_3DFX")) return GL_SAMPLES_3DFX;
        if (!strcmp(name, "GL_SAMPLES_SGIS")) return GL_SAMPLES_SGIS;
        if (!strcmp(name, "GL_SCISSOR_TEST")) return GL_SCISSOR_TEST;
        if (!strcmp(name, "GL_SEPARABLE_2D")) return GL_SEPARABLE_2D;
        if (!strcmp(name, "GL_SGIX_clipmap")) return GL_SGIX_clipmap;
        if (!strcmp(name, "GL_SINGLE_COLOR")) return GL_SINGLE_COLOR;
        if (!strcmp(name, "GL_SRGB8_ALPHA8")) return GL_SRGB8_ALPHA8;
        if (!strcmp(name, "GL_STENCIL_BITS")) return GL_STENCIL_BITS;
        if (!strcmp(name, "GL_STENCIL_FAIL")) return GL_STENCIL_FAIL;
        if (!strcmp(name, "GL_STENCIL_FUNC")) return GL_STENCIL_FUNC;
        if (!strcmp(name, "GL_STENCIL_TEST")) return GL_STENCIL_TEST;
        if (!strcmp(name, "GL_SUBTRACT_ARB")) return GL_SUBTRACT_ARB;
        if (!strcmp(name, "GL_T2F_C4UB_V3F")) return GL_T2F_C4UB_V3F;
        if (!strcmp(name, "GL_TEXTURE0_ARB")) return GL_TEXTURE0_ARB;
        if (!strcmp(name, "GL_TEXTURE1_ARB")) return GL_TEXTURE1_ARB;
        if (!strcmp(name, "GL_TEXTURE2_ARB")) return GL_TEXTURE2_ARB;
        if (!strcmp(name, "GL_TEXTURE3_ARB")) return GL_TEXTURE3_ARB;
        if (!strcmp(name, "GL_TEXTURE4_ARB")) return GL_TEXTURE4_ARB;
        if (!strcmp(name, "GL_TEXTURE5_ARB")) return GL_TEXTURE5_ARB;
        if (!strcmp(name, "GL_TEXTURE6_ARB")) return GL_TEXTURE6_ARB;
        if (!strcmp(name, "GL_TEXTURE7_ARB")) return GL_TEXTURE7_ARB;
        if (!strcmp(name, "GL_TEXTURE8_ARB")) return GL_TEXTURE8_ARB;
        if (!strcmp(name, "GL_TEXTURE9_ARB")) return GL_TEXTURE9_ARB;
        if (!strcmp(name, "GL_TRANSPOSE_NV")) return GL_TRANSPOSE_NV;
        if (!strcmp(name, "GL_TRIANGLE_FAN")) return GL_TRIANGLE_FAN;
        if (!strcmp(name, "GL_UNSIGNED_INT")) return GL_UNSIGNED_INT;
        if (!strcmp(name, "GL_UNSIGNED_INT")) return GL_UNSIGNED_INT;
        if (!strcmp(name, "GL_VERTEX_ARRAY")) return GL_VERTEX_ARRAY;
        if (!strcmp(name, "GL_VERTEX_ID_NV")) return GL_VERTEX_ID_NV;
        if (!strcmp(name, "GL_VIEWPORT_BIT")) return GL_VIEWPORT_BIT;
        return _GL_UNDEFINED;
    case 16:
        if (!strcmp(name, "GL_ALPHA16UI_EXT")) return GL_ALPHA16UI_EXT;
        if (!strcmp(name, "GL_ALPHA32UI_EXT")) return GL_ALPHA32UI_EXT;
        if (!strcmp(name, "GL_ALPHA_INTEGER")) return GL_ALPHA_INTEGER;
        if (!strcmp(name, "GL_BLEND_DST_RGB")) return GL_BLEND_DST_RGB;
        if (!strcmp(name, "GL_BLEND_SRC_RGB")) return GL_BLEND_SRC_RGB;
        if (!strcmp(name, "GL_BOOL_VEC2_ARB")) return GL_BOOL_VEC2_ARB;
        if (!strcmp(name, "GL_BOOL_VEC3_ARB")) return GL_BOOL_VEC3_ARB;
        if (!strcmp(name, "GL_BOOL_VEC4_ARB")) return GL_BOOL_VEC4_ARB;
        if (!strcmp(name, "GL_BUFFER_ACCESS")) return GL_BUFFER_ACCESS;
        if (!strcmp(name, "GL_BUFFER_MAPPED")) return GL_BUFFER_MAPPED;
        if (!strcmp(name, "GL_CLAMP_TO_EDGE")) return GL_CLAMP_TO_EDGE;
        if (!strcmp(name, "GL_COLOR_INDEXES")) return GL_COLOR_INDEXES;
        if (!strcmp(name, "GL_COLOR_SUM_ARB")) return GL_COLOR_SUM_ARB;
        if (!strcmp(name, "GL_COLOR_SUM_EXT")) return GL_COLOR_SUM_EXT;
        if (!strcmp(name, "GL_COMBINE_ALPHA")) return GL_COMBINE_ALPHA;
        if (!strcmp(name, "GL_COMPRESSED_RG")) return GL_COMPRESSED_RG;
        if (!strcmp(name, "GL_CONTEXT_FLAGS")) return GL_CONTEXT_FLAGS;
        if (!strcmp(name, "GL_COORD_REPLACE")) return GL_COORD_REPLACE;
        if (!strcmp(name, "GL_COPY_INVERTED")) return GL_COPY_INVERTED;
        if (!strcmp(name, "GL_CULL_MODES_NV")) return GL_CULL_MODES_NV;
        if (!strcmp(name, "GL_CURRENT_COLOR")) return GL_CURRENT_COLOR;
        if (!strcmp(name, "GL_CURRENT_INDEX")) return GL_CURRENT_INDEX;
        if (!strcmp(name, "GL_CURRENT_QUERY")) return GL_CURRENT_QUERY;
        if (!strcmp(name, "GL_DECR_WRAP_EXT")) return GL_DECR_WRAP_EXT;
        if (!strcmp(name, "GL_DELETE_STATUS")) return GL_DELETE_STATUS;
        if (!strcmp(name, "GL_DEPTH_STENCIL")) return GL_DEPTH_STENCIL;
        if (!strcmp(name, "GL_DOT3_RGBA_ARB")) return GL_DOT3_RGBA_ARB;
        if (!strcmp(name, "GL_DOT3_RGBA_EXT")) return GL_DOT3_RGBA_EXT;
        if (!strcmp(name, "GL_DRAW_BUFFER10")) return GL_DRAW_BUFFER10;
        if (!strcmp(name, "GL_DRAW_BUFFER11")) return GL_DRAW_BUFFER11;
        if (!strcmp(name, "GL_DRAW_BUFFER12")) return GL_DRAW_BUFFER12;
        if (!strcmp(name, "GL_DRAW_BUFFER13")) return GL_DRAW_BUFFER13;
        if (!strcmp(name, "GL_DRAW_BUFFER14")) return GL_DRAW_BUFFER14;
        if (!strcmp(name, "GL_DRAW_BUFFER15")) return GL_DRAW_BUFFER15;
        if (!strcmp(name, "GL_DSDT8_MAG8_NV")) return GL_DSDT8_MAG8_NV;
        if (!strcmp(name, "GL_DSDT8_MAG8_NV")) return GL_DSDT8_MAG8_NV;
        if (!strcmp(name, "GL_EMBOSS_MAP_NV")) return GL_EMBOSS_MAP_NV;
        if (!strcmp(name, "GL_ENV_BLEND_EXT")) return GL_ENV_BLEND_EXT;
        if (!strcmp(name, "GL_EXT_Cg_shader")) return GL_EXT_Cg_shader;
        if (!strcmp(name, "GL_EXT_fog_coord")) return GL_EXT_fog_coord;
        if (!strcmp(name, "GL_EXT_histogram")) return GL_EXT_histogram;
        if (!strcmp(name, "GL_EXT_texture3D")) return GL_EXT_texture3D;
        if (!strcmp(name, "GL_EYE_LINE_SGIS")) return GL_EYE_LINE_SGIS;
        if (!strcmp(name, "GL_EYE_RADIAL_NV")) return GL_EYE_RADIAL_NV;
        if (!strcmp(name, "GL_FLOAT_RG16_NV")) return GL_FLOAT_RG16_NV;
        if (!strcmp(name, "GL_FLOAT_RG32_NV")) return GL_FLOAT_RG32_NV;
        if (!strcmp(name, "GL_FLOAT_RGBA_NV")) return GL_FLOAT_RGBA_NV;
        if (!strcmp(name, "GL_FOG_COORD_SRC")) return GL_FOG_COORD_SRC;
        if (!strcmp(name, "GL_FUNC_SUBTRACT")) return GL_FUNC_SUBTRACT;
        if (!strcmp(name, "GL_GREEN_BIT_ATI")) return GL_GREEN_BIT_ATI;
        if (!strcmp(name, "GL_GREEN_INTEGER")) return GL_GREEN_INTEGER;
        if (!strcmp(name, "GL_HALF_FLOAT_NV")) return GL_HALF_FLOAT_NV;
        if (!strcmp(name, "GL_HISTOGRAM_EXT")) return GL_HISTOGRAM_EXT;
        if (!strcmp(name, "GL_IGNORE_BORDER")) return GL_IGNORE_BORDER;
        if (!strcmp(name, "GL_INCR_WRAP_EXT")) return GL_INCR_WRAP_EXT;
        if (!strcmp(name, "GL_INDEX_BIT_PGI")) return GL_INDEX_BIT_PGI;
        if (!strcmp(name, "GL_INTENSITY_EXT")) return GL_INTENSITY_EXT;
        if (!strcmp(name, "GL_INTERLACE_OML")) return GL_INTERLACE_OML;
        if (!strcmp(name, "GL_INVALID_VALUE")) return GL_INVALID_VALUE;
        if (!strcmp(name, "GL_INVARIANT_EXT")) return GL_INVARIANT_EXT;
        if (!strcmp(name, "GL_LOGIC_OP_MODE")) return GL_LOGIC_OP_MODE;
        if (!strcmp(name, "GL_MAJOR_VERSION")) return GL_MAJOR_VERSION;
        if (!strcmp(name, "GL_MAP1_VERTEX_3")) return GL_MAP1_VERTEX_3;
        if (!strcmp(name, "GL_MAP1_VERTEX_4")) return GL_MAP1_VERTEX_4;
        if (!strcmp(name, "GL_MAP2_VERTEX_3")) return GL_MAP2_VERTEX_3;
        if (!strcmp(name, "GL_MAP2_VERTEX_4")) return GL_MAP2_VERTEX_4;
        if (!strcmp(name, "GL_MAP_WRITE_BIT")) return GL_MAP_WRITE_BIT;
        if (!strcmp(name, "GL_MAX_BLUE_SGIS")) return GL_MAX_BLUE_SGIS;
        if (!strcmp(name, "GL_MIN_BLUE_SGIS")) return GL_MIN_BLUE_SGIS;
        if (!strcmp(name, "GL_MINMAX_FORMAT")) return GL_MINMAX_FORMAT;
        if (!strcmp(name, "GL_MINOR_VERSION")) return GL_MINOR_VERSION;
        if (!strcmp(name, "GL_NORMAL_MAP_NV")) return GL_NORMAL_MAP_NV;
        if (!strcmp(name, "GL_NV_evaluators")) return GL_NV_evaluators;
        if (!strcmp(name, "GL_NV_half_float")) return GL_NV_half_float;
        if (!strcmp(name, "GL_OBJECT_LINEAR")) return GL_OBJECT_LINEAR;
        if (!strcmp(name, "GL_OML_interlace")) return GL_OML_interlace;
        if (!strcmp(name, "GL_OML_subsample")) return GL_OML_subsample;
        if (!strcmp(name, "GL_OP_NEGATE_EXT")) return GL_OP_NEGATE_EXT;
        if (!strcmp(name, "GL_OP_SET_GE_EXT")) return GL_OP_SET_GE_EXT;
        if (!strcmp(name, "GL_OP_SET_LT_EXT")) return GL_OP_SET_LT_EXT;
        if (!strcmp(name, "GL_OUT_OF_MEMORY")) return GL_OUT_OF_MEMORY;
        if (!strcmp(name, "GL_POLYGON_TOKEN")) return GL_POLYGON_TOKEN;
        if (!strcmp(name, "GL_PRIMARY_COLOR")) return GL_PRIMARY_COLOR;
        if (!strcmp(name, "GL_QUAD_MESH_SUN")) return GL_QUAD_MESH_SUN;
        if (!strcmp(name, "GL_QUERY_NO_WAIT")) return GL_QUERY_NO_WAIT;
        if (!strcmp(name, "GL_QUERY_WAIT_NV")) return GL_QUERY_WAIT_NV;
        if (!strcmp(name, "GL_READ_ONLY_ARB")) return GL_READ_ONLY_ARB;
        if (!strcmp(name, "GL_RGB_SCALE_ARB")) return GL_RGB_SCALE_ARB;
        if (!strcmp(name, "GL_RGB_SCALE_EXT")) return GL_RGB_SCALE_EXT;
        if (!strcmp(name, "GL_SGIX_resample")) return GL_SGIX_resample;
        if (!strcmp(name, "GL_SIGNED_RGB_NV")) return GL_SIGNED_RGB_NV;
        if (!strcmp(name, "GL_SIGNED_RGB_NV")) return GL_SIGNED_RGB_NV;
        if (!strcmp(name, "GL_SOURCE0_ALPHA")) return GL_SOURCE0_ALPHA;
        if (!strcmp(name, "GL_SOURCE1_ALPHA")) return GL_SOURCE1_ALPHA;
        if (!strcmp(name, "GL_SOURCE2_ALPHA")) return GL_SOURCE2_ALPHA;
        if (!strcmp(name, "GL_SPOT_EXPONENT")) return GL_SPOT_EXPONENT;
        if (!strcmp(name, "GL_STENCIL_INDEX")) return GL_STENCIL_INDEX;
        if (!strcmp(name, "GL_SUBPIXEL_BITS")) return GL_SUBPIXEL_BITS;
        if (!strcmp(name, "GL_TEXTURE10_ARB")) return GL_TEXTURE10_ARB;
        if (!strcmp(name, "GL_TEXTURE11_ARB")) return GL_TEXTURE11_ARB;
        if (!strcmp(name, "GL_TEXTURE12_ARB")) return GL_TEXTURE12_ARB;
        if (!strcmp(name, "GL_TEXTURE13_ARB")) return GL_TEXTURE13_ARB;
        if (!strcmp(name, "GL_TEXTURE14_ARB")) return GL_TEXTURE14_ARB;
        if (!strcmp(name, "GL_TEXTURE15_ARB")) return GL_TEXTURE15_ARB;
        if (!strcmp(name, "GL_TEXTURE16_ARB")) return GL_TEXTURE16_ARB;
        if (!strcmp(name, "GL_TEXTURE17_ARB")) return GL_TEXTURE17_ARB;
        if (!strcmp(name, "GL_TEXTURE18_ARB")) return GL_TEXTURE18_ARB;
        if (!strcmp(name, "GL_TEXTURE19_ARB")) return GL_TEXTURE19_ARB;
        if (!strcmp(name, "GL_TEXTURE20_ARB")) return GL_TEXTURE20_ARB;
        if (!strcmp(name, "GL_TEXTURE21_ARB")) return GL_TEXTURE21_ARB;
        if (!strcmp(name, "GL_TEXTURE22_ARB")) return GL_TEXTURE22_ARB;
        if (!strcmp(name, "GL_TEXTURE23_ARB")) return GL_TEXTURE23_ARB;
        if (!strcmp(name, "GL_TEXTURE24_ARB")) return GL_TEXTURE24_ARB;
        if (!strcmp(name, "GL_TEXTURE25_ARB")) return GL_TEXTURE25_ARB;
        if (!strcmp(name, "GL_TEXTURE26_ARB")) return GL_TEXTURE26_ARB;
        if (!strcmp(name, "GL_TEXTURE27_ARB")) return GL_TEXTURE27_ARB;
        if (!strcmp(name, "GL_TEXTURE28_ARB")) return GL_TEXTURE28_ARB;
        if (!strcmp(name, "GL_TEXTURE29_ARB")) return GL_TEXTURE29_ARB;
        if (!strcmp(name, "GL_TEXTURE30_ARB")) return GL_TEXTURE30_ARB;
        if (!strcmp(name, "GL_TEXTURE31_ARB")) return GL_TEXTURE31_ARB;
        if (!strcmp(name, "GL_TEXTURE_DEPTH")) return GL_TEXTURE_DEPTH;
        if (!strcmp(name, "GL_TEXTURE_GEN_Q")) return GL_TEXTURE_GEN_Q;
        if (!strcmp(name, "GL_TEXTURE_GEN_R")) return GL_TEXTURE_GEN_R;
        if (!strcmp(name, "GL_TEXTURE_GEN_S")) return GL_TEXTURE_GEN_S;
        if (!strcmp(name, "GL_TEXTURE_GEN_T")) return GL_TEXTURE_GEN_T;
        if (!strcmp(name, "GL_TEXTURE_WIDTH")) return GL_TEXTURE_WIDTH;
        if (!strcmp(name, "GL_TRANSFORM_BIT")) return GL_TRANSFORM_BIT;
        if (!strcmp(name, "GL_UNSIGNED_BYTE")) return GL_UNSIGNED_BYTE;
        if (!strcmp(name, "GL_VARIABLE_A_NV")) return GL_VARIABLE_A_NV;
        if (!strcmp(name, "GL_VARIABLE_B_NV")) return GL_VARIABLE_B_NV;
        if (!strcmp(name, "GL_VARIABLE_C_NV")) return GL_VARIABLE_C_NV;
        if (!strcmp(name, "GL_VARIABLE_D_NV")) return GL_VARIABLE_D_NV;
        if (!strcmp(name, "GL_VARIABLE_E_NV")) return GL_VARIABLE_E_NV;
        if (!strcmp(name, "GL_VARIABLE_F_NV")) return GL_VARIABLE_F_NV;
        if (!strcmp(name, "GL_VARIABLE_G_NV")) return GL_VARIABLE_G_NV;
        if (!strcmp(name, "GL_VERTEX_SHADER")) return GL_VERTEX_SHADER;
        if (!strcmp(name, "GL_WIN_swap_hint")) return GL_WIN_swap_hint;
        return _GL_UNDEFINED;
    case 17:
        if (!strcmp(name, "GL_ACCUM_RED_BITS")) return GL_ACCUM_RED_BITS;
        if (!strcmp(name, "GL_ACTIVE_TEXTURE")) return GL_ACTIVE_TEXTURE;
        if (!strcmp(name, "GL_ADD_SIGNED_ARB")) return GL_ADD_SIGNED_ARB;
        if (!strcmp(name, "GL_ADD_SIGNED_EXT")) return GL_ADD_SIGNED_EXT;
        if (!strcmp(name, "GL_ALPHA_MAX_SGIX")) return GL_ALPHA_MAX_SGIX;
        if (!strcmp(name, "GL_ALPHA_MIN_SGIX")) return GL_ALPHA_MIN_SGIX;
        if (!strcmp(name, "GL_ALPHA_TEST_REF")) return GL_ALPHA_TEST_REF;
        if (!strcmp(name, "GL_ARB_texture_rg")) return GL_ARB_texture_rg;
        if (!strcmp(name, "GL_ARB_window_pos")) return GL_ARB_window_pos;
        if (!strcmp(name, "GL_BLEND_EQUATION")) return GL_BLEND_EQUATION;
        if (!strcmp(name, "GL_CLIP_DISTANCE0")) return GL_CLIP_DISTANCE0;
        if (!strcmp(name, "GL_CLIP_DISTANCE1")) return GL_CLIP_DISTANCE1;
        if (!strcmp(name, "GL_CLIP_DISTANCE2")) return GL_CLIP_DISTANCE2;
        if (!strcmp(name, "GL_CLIP_DISTANCE3")) return GL_CLIP_DISTANCE3;
        if (!strcmp(name, "GL_CLIP_DISTANCE4")) return GL_CLIP_DISTANCE4;
        if (!strcmp(name, "GL_CLIP_DISTANCE5")) return GL_CLIP_DISTANCE5;
        if (!strcmp(name, "GL_COLOR3_BIT_PGI")) return GL_COLOR3_BIT_PGI;
        if (!strcmp(name, "GL_COLOR4_BIT_PGI")) return GL_COLOR4_BIT_PGI;
        if (!strcmp(name, "GL_COLOR_LOGIC_OP")) return GL_COLOR_LOGIC_OP;
        if (!strcmp(name, "GL_COLOR_MATERIAL")) return GL_COLOR_MATERIAL;
        if (!strcmp(name, "GL_COMPILE_STATUS")) return GL_COMPILE_STATUS;
        if (!strcmp(name, "GL_COMPRESSED_RED")) return GL_COMPRESSED_RED;
        if (!strcmp(name, "GL_COMPRESSED_RGB")) return GL_COMPRESSED_RGB;
        if (!strcmp(name, "GL_CONSTANT_ALPHA")) return GL_CONSTANT_ALPHA;
        if (!strcmp(name, "GL_CONSTANT_COLOR")) return GL_CONSTANT_COLOR;
        if (!strcmp(name, "GL_CONVOLUTION_1D")) return GL_CONVOLUTION_1D;
        if (!strcmp(name, "GL_CONVOLUTION_2D")) return GL_CONVOLUTION_2D;
        if (!strcmp(name, "GL_CULL_FACE_MODE")) return GL_CULL_FACE_MODE;
        if (!strcmp(name, "GL_CURRENT_NORMAL")) return GL_CURRENT_NORMAL;
        if (!strcmp(name, "GL_DEPTH_CLAMP_NV")) return GL_DEPTH_CLAMP_NV;
        if (!strcmp(name, "GL_DOT_PRODUCT_NV")) return GL_DOT_PRODUCT_NV;
        if (!strcmp(name, "GL_EIGHTH_BIT_ATI")) return GL_EIGHTH_BIT_ATI;
        if (!strcmp(name, "GL_EXT_422_pixels")) return GL_EXT_422_pixels;
        if (!strcmp(name, "GL_EXT_index_func")) return GL_EXT_index_func;
        if (!strcmp(name, "GL_EXT_subtexture")) return GL_EXT_subtexture;
        if (!strcmp(name, "GL_EYE_POINT_SGIS")) return GL_EYE_POINT_SGIS;
        if (!strcmp(name, "GL_FIXED_ONLY_ARB")) return GL_FIXED_ONLY_ARB;
        if (!strcmp(name, "GL_FLOAT_MAT2_ARB")) return GL_FLOAT_MAT2_ARB;
        if (!strcmp(name, "GL_FLOAT_MAT3_ARB")) return GL_FLOAT_MAT3_ARB;
        if (!strcmp(name, "GL_FLOAT_MAT4_ARB")) return GL_FLOAT_MAT4_ARB;
        if (!strcmp(name, "GL_FLOAT_RGB16_NV")) return GL_FLOAT_RGB16_NV;
        if (!strcmp(name, "GL_FLOAT_RGB32_NV")) return GL_FLOAT_RGB32_NV;
        if (!strcmp(name, "GL_FLOAT_VEC2_ARB")) return GL_FLOAT_VEC2_ARB;
        if (!strcmp(name, "GL_FLOAT_VEC3_ARB")) return GL_FLOAT_VEC3_ARB;
        if (!strcmp(name, "GL_FLOAT_VEC4_ARB")) return GL_FLOAT_VEC4_ARB;
        if (!strcmp(name, "GL_FOG_COORDINATE")) return GL_FOG_COORDINATE;
        if (!strcmp(name, "GL_FRAGMENT_DEPTH")) return GL_FRAGMENT_DEPTH;
        if (!strcmp(name, "GL_FRONT_AND_BACK")) return GL_FRONT_AND_BACK;
        if (!strcmp(name, "GL_FULL_RANGE_EXT")) return GL_FULL_RANGE_EXT;
        if (!strcmp(name, "GL_HALF_FLOAT_ARB")) return GL_HALF_FLOAT_ARB;
        if (!strcmp(name, "GL_HISTOGRAM_SINK")) return GL_HISTOGRAM_SINK;
        if (!strcmp(name, "GL_INDEX_LOGIC_OP")) return GL_INDEX_LOGIC_OP;
        if (!strcmp(name, "GL_INTENSITY4_EXT")) return GL_INTENSITY4_EXT;
        if (!strcmp(name, "GL_INTENSITY8_EXT")) return GL_INTENSITY8_EXT;
        if (!strcmp(name, "GL_INTERLACE_SGIX")) return GL_INTERLACE_SGIX;
        if (!strcmp(name, "GL_INT_SAMPLER_1D")) return GL_INT_SAMPLER_1D;
        if (!strcmp(name, "GL_INT_SAMPLER_2D")) return GL_INT_SAMPLER_2D;
        if (!strcmp(name, "GL_INT_SAMPLER_3D")) return GL_INT_SAMPLER_3D;
        if (!strcmp(name, "GL_LUMINANCE4_EXT")) return GL_LUMINANCE4_EXT;
        if (!strcmp(name, "GL_LUMINANCE8_EXT")) return GL_LUMINANCE8_EXT;
        if (!strcmp(name, "GL_MAX_ALPHA_SGIS")) return GL_MAX_ALPHA_SGIS;
        if (!strcmp(name, "GL_MAX_EVAL_ORDER")) return GL_MAX_EVAL_ORDER;
        if (!strcmp(name, "GL_MAX_GREEN_SGIS")) return GL_MAX_GREEN_SGIS;
        if (!strcmp(name, "GL_MIN_ALPHA_SGIS")) return GL_MIN_ALPHA_SGIS;
        if (!strcmp(name, "GL_MIN_GREEN_SGIS")) return GL_MIN_GREEN_SGIS;
        if (!strcmp(name, "GL_MODELVIEW0_ARB")) return GL_MODELVIEW0_ARB;
        if (!strcmp(name, "GL_MODELVIEW0_EXT")) return GL_MODELVIEW0_EXT;
        if (!strcmp(name, "GL_MODELVIEW1_ARB")) return GL_MODELVIEW1_ARB;
        if (!strcmp(name, "GL_MODELVIEW1_EXT")) return GL_MODELVIEW1_EXT;
        if (!strcmp(name, "GL_MODELVIEW2_ARB")) return GL_MODELVIEW2_ARB;
        if (!strcmp(name, "GL_MODELVIEW3_ARB")) return GL_MODELVIEW3_ARB;
        if (!strcmp(name, "GL_MODELVIEW4_ARB")) return GL_MODELVIEW4_ARB;
        if (!strcmp(name, "GL_MODELVIEW5_ARB")) return GL_MODELVIEW5_ARB;
        if (!strcmp(name, "GL_MODELVIEW6_ARB")) return GL_MODELVIEW6_ARB;
        if (!strcmp(name, "GL_MODELVIEW7_ARB")) return GL_MODELVIEW7_ARB;
        if (!strcmp(name, "GL_MODELVIEW8_ARB")) return GL_MODELVIEW8_ARB;
        if (!strcmp(name, "GL_MODELVIEW9_ARB")) return GL_MODELVIEW9_ARB;
        if (!strcmp(name, "GL_MVP_MATRIX_EXT")) return GL_MVP_MATRIX_EXT;
        if (!strcmp(name, "GL_NEGATE_BIT_ATI")) return GL_NEGATE_BIT_ATI;
        if (!strcmp(name, "GL_NEGATIVE_W_EXT")) return GL_NEGATIVE_W_EXT;
        if (!strcmp(name, "GL_NEGATIVE_X_EXT")) return GL_NEGATIVE_X_EXT;
        if (!strcmp(name, "GL_NEGATIVE_Y_EXT")) return GL_NEGATIVE_Y_EXT;
        if (!strcmp(name, "GL_NEGATIVE_Z_EXT")) return GL_NEGATIVE_Z_EXT;
        if (!strcmp(name, "GL_NORMAL_BIT_PGI")) return GL_NORMAL_BIT_PGI;
        if (!strcmp(name, "GL_NORMAL_MAP_ARB")) return GL_NORMAL_MAP_ARB;
        if (!strcmp(name, "GL_NORMAL_MAP_EXT")) return GL_NORMAL_MAP_EXT;
        if (!strcmp(name, "GL_NUM_EXTENSIONS")) return GL_NUM_EXTENSIONS;
        if (!strcmp(name, "GL_NUM_PASSES_ATI")) return GL_NUM_PASSES_ATI;
        if (!strcmp(name, "GL_NV_depth_clamp")) return GL_NV_depth_clamp;
        if (!strcmp(name, "GL_OPERAND0_ALPHA")) return GL_OPERAND0_ALPHA;
        if (!strcmp(name, "GL_OPERAND1_ALPHA")) return GL_OPERAND1_ALPHA;
        if (!strcmp(name, "GL_OPERAND2_ALPHA")) return GL_OPERAND2_ALPHA;
        if (!strcmp(name, "GL_OUTPUT_FOG_EXT")) return GL_OUTPUT_FOG_EXT;
        if (!strcmp(name, "GL_PACK_ALIGNMENT")) return GL_PACK_ALIGNMENT;
        if (!strcmp(name, "GL_PACK_LSB_FIRST")) return GL_PACK_LSB_FIRST;
        if (!strcmp(name, "GL_PACK_SKIP_ROWS")) return GL_PACK_SKIP_ROWS;
        if (!strcmp(name, "GL_PERCENTAGE_AMD")) return GL_PERCENTAGE_AMD;
        if (!strcmp(name, "GL_PGI_misc_hints")) return GL_PGI_misc_hints;
        if (!strcmp(name, "GL_PHONG_HINT_WIN")) return GL_PHONG_HINT_WIN;
        if (!strcmp(name, "GL_PIXEL_COUNT_NV")) return GL_PIXEL_COUNT_NV;
        if (!strcmp(name, "GL_PIXEL_MODE_BIT")) return GL_PIXEL_MODE_BIT;
        if (!strcmp(name, "GL_POINT_SIZE_MAX")) return GL_POINT_SIZE_MAX;
        if (!strcmp(name, "GL_POINT_SIZE_MIN")) return GL_POINT_SIZE_MIN;
        if (!strcmp(name, "GL_POLYGON_SMOOTH")) return GL_POLYGON_SMOOTH;
        if (!strcmp(name, "GL_R11F_G11F_B10F")) return GL_R11F_G11F_B10F;
        if (!strcmp(name, "GL_READ_WRITE_ARB")) return GL_READ_WRITE_ARB;
        if (!strcmp(name, "GL_REFLECTION_MAP")) return GL_REFLECTION_MAP;
        if (!strcmp(name, "GL_RESCALE_NORMAL")) return GL_RESCALE_NORMAL;
        if (!strcmp(name, "GL_RGBA_DXT5_S3TC")) return GL_RGBA_DXT5_S3TC;
        if (!strcmp(name, "GL_SAMPLE_BUFFERS")) return GL_SAMPLE_BUFFERS;
        if (!strcmp(name, "GL_SAMPLE_MASK_NV")) return GL_SAMPLE_MASK_NV;
        if (!strcmp(name, "GL_SAMPLER_1D_ARB")) return GL_SAMPLER_1D_ARB;
        if (!strcmp(name, "GL_SAMPLER_2D_ARB")) return GL_SAMPLER_2D_ARB;
        if (!strcmp(name, "GL_SAMPLER_3D_ARB")) return GL_SAMPLER_3D_ARB;
        if (!strcmp(name, "GL_SAMPLES_PASSED")) return GL_SAMPLES_PASSED;
        if (!strcmp(name, "GL_SGIS_texture4D")) return GL_SGIS_texture4D;
        if (!strcmp(name, "GL_SGIX_framezoom")) return GL_SGIX_framezoom;
        if (!strcmp(name, "GL_SGIX_interlace")) return GL_SGIX_interlace;
        if (!strcmp(name, "GL_SIGNED_HILO_NV")) return GL_SIGNED_HILO_NV;
        if (!strcmp(name, "GL_SIGNED_HILO_NV")) return GL_SIGNED_HILO_NV;
        if (!strcmp(name, "GL_SIGNED_RGB8_NV")) return GL_SIGNED_RGB8_NV;
        if (!strcmp(name, "GL_SIGNED_RGB8_NV")) return GL_SIGNED_RGB8_NV;
        if (!strcmp(name, "GL_SIGNED_RGBA_NV")) return GL_SIGNED_RGBA_NV;
        if (!strcmp(name, "GL_SIGNED_RGBA_NV")) return GL_SIGNED_RGBA_NV;
        if (!strcmp(name, "GL_SLUMINANCE_EXT")) return GL_SLUMINANCE_EXT;
        if (!strcmp(name, "GL_SOURCE3_RGB_NV")) return GL_SOURCE3_RGB_NV;
        if (!strcmp(name, "GL_SPOT_DIRECTION")) return GL_SPOT_DIRECTION;
        if (!strcmp(name, "GL_SRGB_ALPHA_EXT")) return GL_SRGB_ALPHA_EXT;
        if (!strcmp(name, "GL_STACK_OVERFLOW")) return GL_STACK_OVERFLOW;
        if (!strcmp(name, "GL_STENCIL_BUFFER")) return GL_STENCIL_BUFFER;
        if (!strcmp(name, "GL_STENCIL_INDEX1")) return GL_STENCIL_INDEX1;
        if (!strcmp(name, "GL_STENCIL_INDEX4")) return GL_STENCIL_INDEX4;
        if (!strcmp(name, "GL_STENCIL_INDEX8")) return GL_STENCIL_INDEX8;
        if (!strcmp(name, "GL_SUN_mesh_array")) return GL_SUN_mesh_array;
        if (!strcmp(name, "GL_TEXTURE_3D_EXT")) return GL_TEXTURE_3D_EXT;
        if (!strcmp(name, "GL_TEXTURE_3D_EXT")) return GL_TEXTURE_3D_EXT;
        if (!strcmp(name, "GL_TEXTURE_BORDER")) return GL_TEXTURE_BORDER;
        if (!strcmp(name, "GL_TEXTURE_HEIGHT")) return GL_TEXTURE_HEIGHT;
        if (!strcmp(name, "GL_TEXTURE_MATRIX")) return GL_TEXTURE_MATRIX;
        if (!strcmp(name, "GL_TEXTURE_WRAP_R")) return GL_TEXTURE_WRAP_R;
        if (!strcmp(name, "GL_TEXTURE_WRAP_S")) return GL_TEXTURE_WRAP_S;
        if (!strcmp(name, "GL_TEXTURE_WRAP_T")) return GL_TEXTURE_WRAP_T;
        if (!strcmp(name, "GL_TRIANGLE_STRIP")) return GL_TRIANGLE_STRIP;
        if (!strcmp(name, "GL_UNSIGNED_SHORT")) return GL_UNSIGNED_SHORT;
        if (!strcmp(name, "GL_WRITE_ONLY_ARB")) return GL_WRITE_ONLY_ARB;
        return _GL_UNDEFINED;
    case 18:
        if (!strcmp(name, "GL_422_AVERAGE_EXT")) return GL_422_AVERAGE_EXT;
        if (!strcmp(name, "GL_ACCUM_BLUE_BITS")) return GL_ACCUM_BLUE_BITS;
        if (!strcmp(name, "GL_ACTIVE_UNIFORMS")) return GL_ACTIVE_UNIFORMS;
        if (!strcmp(name, "GL_ALL_ATTRIB_BITS")) return GL_ALL_ATTRIB_BITS;
        if (!strcmp(name, "GL_ALPHA_TEST_FUNC")) return GL_ALPHA_TEST_FUNC;
        if (!strcmp(name, "GL_APPLE_ycbcr_422")) return GL_APPLE_ycbcr_422;
        if (!strcmp(name, "GL_ARB_multisample")) return GL_ARB_multisample;
        if (!strcmp(name, "GL_ATTENUATION_EXT")) return GL_ATTENUATION_EXT;
        if (!strcmp(name, "GL_BGR_INTEGER_EXT")) return GL_BGR_INTEGER_EXT;
        if (!strcmp(name, "GL_BLEND_COLOR_EXT")) return GL_BLEND_COLOR_EXT;
        if (!strcmp(name, "GL_BLEND_DST_ALPHA")) return GL_BLEND_DST_ALPHA;
        if (!strcmp(name, "GL_BLEND_SRC_ALPHA")) return GL_BLEND_SRC_ALPHA;
        if (!strcmp(name, "GL_BUFFER_SIZE_ARB")) return GL_BUFFER_SIZE_ARB;
        if (!strcmp(name, "GL_BUMP_ENVMAP_ATI")) return GL_BUMP_ENVMAP_ATI;
        if (!strcmp(name, "GL_BUMP_TARGET_ATI")) return GL_BUMP_TARGET_ATI;
        if (!strcmp(name, "GL_CLAMP_TO_BORDER")) return GL_CLAMP_TO_BORDER;
        if (!strcmp(name, "GL_COLOR_ARRAY_EXT")) return GL_COLOR_ARRAY_EXT;
        if (!strcmp(name, "GL_COLOR_TABLE_SGI")) return GL_COLOR_TABLE_SGI;
        if (!strcmp(name, "GL_COLOR_WRITEMASK")) return GL_COLOR_WRITEMASK;
        if (!strcmp(name, "GL_COMBINE_RGB_ARB")) return GL_COMBINE_RGB_ARB;
        if (!strcmp(name, "GL_COMBINE_RGB_EXT")) return GL_COMBINE_RGB_EXT;
        if (!strcmp(name, "GL_COMPRESSED_RGBA")) return GL_COMPRESSED_RGBA;
        if (!strcmp(name, "GL_COMPRESSED_SRGB")) return GL_COMPRESSED_SRGB;
        if (!strcmp(name, "GL_CONSTANT_BORDER")) return GL_CONSTANT_BORDER;
        if (!strcmp(name, "GL_CULL_VERTEX_IBM")) return GL_CULL_VERTEX_IBM;
        if (!strcmp(name, "GL_CURRENT_PROGRAM")) return GL_CURRENT_PROGRAM;
        if (!strcmp(name, "GL_CURRENT_TIME_NV")) return GL_CURRENT_TIME_NV;
        if (!strcmp(name, "GL_DEPTH_COMPONENT")) return GL_DEPTH_COMPONENT;
        if (!strcmp(name, "GL_DEPTH_WRITEMASK")) return GL_DEPTH_WRITEMASK;
        if (!strcmp(name, "GL_DSDT_MAG_VIB_NV")) return GL_DSDT_MAG_VIB_NV;
        if (!strcmp(name, "GL_DSDT_MAG_VIB_NV")) return GL_DSDT_MAG_VIB_NV;
        if (!strcmp(name, "GL_EDGE_FLAG_ARRAY")) return GL_EDGE_FLAG_ARRAY;
        if (!strcmp(name, "GL_EMBOSS_LIGHT_NV")) return GL_EMBOSS_LIGHT_NV;
        if (!strcmp(name, "GL_ENV_REPLACE_EXT")) return GL_ENV_REPLACE_EXT;
        if (!strcmp(name, "GL_EXT_blend_color")) return GL_EXT_blend_color;
        if (!strcmp(name, "GL_EXT_convolution")) return GL_EXT_convolution;
        if (!strcmp(name, "GL_EXT_cull_vertex")) return GL_EXT_cull_vertex;
        if (!strcmp(name, "GL_EXT_gpu_shader4")) return GL_EXT_gpu_shader4;
        if (!strcmp(name, "GL_EXT_multisample")) return GL_EXT_multisample;
        if (!strcmp(name, "GL_EXT_texture_env")) return GL_EXT_texture_env;
        if (!strcmp(name, "GL_EXT_timer_query")) return GL_EXT_timer_query;
        if (!strcmp(name, "GL_FENCE_STATUS_NV")) return GL_FENCE_STATUS_NV;
        if (!strcmp(name, "GL_FLOAT_RGBA16_NV")) return GL_FLOAT_RGBA16_NV;
        if (!strcmp(name, "GL_FLOAT_RGBA32_NV")) return GL_FLOAT_RGBA32_NV;
        if (!strcmp(name, "GL_FOG_COORD_ARRAY")) return GL_FOG_COORD_ARRAY;
        if (!strcmp(name, "GL_FOG_OFFSET_SGIX")) return GL_FOG_OFFSET_SGIX;
        if (!strcmp(name, "GL_FRAGMENT_SHADER")) return GL_FRAGMENT_SHADER;
        if (!strcmp(name, "GL_FRAMEBUFFER_EXT")) return GL_FRAMEBUFFER_EXT;
        if (!strcmp(name, "GL_GENERATE_MIPMAP")) return GL_GENERATE_MIPMAP;
        if (!strcmp(name, "GL_HISTOGRAM_WIDTH")) return GL_HISTOGRAM_WIDTH;
        if (!strcmp(name, "GL_IBM_cull_vertex")) return GL_IBM_cull_vertex;
        if (!strcmp(name, "GL_IBM_static_data")) return GL_IBM_static_data;
        if (!strcmp(name, "GL_INDEX_ARRAY_EXT")) return GL_INDEX_ARRAY_EXT;
        if (!strcmp(name, "GL_INDEX_WRITEMASK")) return GL_INDEX_WRITEMASK;
        if (!strcmp(name, "GL_INFO_LOG_LENGTH")) return GL_INFO_LOG_LENGTH;
        if (!strcmp(name, "GL_INTENSITY12_EXT")) return GL_INTENSITY12_EXT;
        if (!strcmp(name, "GL_INTENSITY16_EXT")) return GL_INTENSITY16_EXT;
        if (!strcmp(name, "GL_INTENSITY8I_EXT")) return GL_INTENSITY8I_EXT;
        if (!strcmp(name, "GL_INTERPOLATE_ARB")) return GL_INTERPOLATE_ARB;
        if (!strcmp(name, "GL_INTERPOLATE_EXT")) return GL_INTERPOLATE_EXT;
        if (!strcmp(name, "GL_KTX_BACK_REGION")) return GL_KTX_BACK_REGION;
        if (!strcmp(name, "GL_LUMINANCE12_EXT")) return GL_LUMINANCE12_EXT;
        if (!strcmp(name, "GL_LUMINANCE16_EXT")) return GL_LUMINANCE16_EXT;
        if (!strcmp(name, "GL_LUMINANCE8I_EXT")) return GL_LUMINANCE8I_EXT;
        if (!strcmp(name, "GL_LUMINANCE_ALPHA")) return GL_LUMINANCE_ALPHA;
        if (!strcmp(name, "GL_MAX_CLIP_PLANES")) return GL_MAX_CLIP_PLANES;
        if (!strcmp(name, "GL_MAX_SAMPLES_EXT")) return GL_MAX_SAMPLES_EXT;
        if (!strcmp(name, "GL_MESA_window_pos")) return GL_MESA_window_pos;
        if (!strcmp(name, "GL_MINMAX_SINK_EXT")) return GL_MINMAX_SINK_EXT;
        if (!strcmp(name, "GL_MIRRORED_REPEAT")) return GL_MIRRORED_REPEAT;
        if (!strcmp(name, "GL_MODELVIEW10_ARB")) return GL_MODELVIEW10_ARB;
        if (!strcmp(name, "GL_MODELVIEW11_ARB")) return GL_MODELVIEW11_ARB;
        if (!strcmp(name, "GL_MODELVIEW12_ARB")) return GL_MODELVIEW12_ARB;
        if (!strcmp(name, "GL_MODELVIEW13_ARB")) return GL_MODELVIEW13_ARB;
        if (!strcmp(name, "GL_MODELVIEW14_ARB")) return GL_MODELVIEW14_ARB;
        if (!strcmp(name, "GL_MODELVIEW15_ARB")) return GL_MODELVIEW15_ARB;
        if (!strcmp(name, "GL_MODELVIEW16_ARB")) return GL_MODELVIEW16_ARB;
        if (!strcmp(name, "GL_MODELVIEW17_ARB")) return GL_MODELVIEW17_ARB;
        if (!strcmp(name, "GL_MODELVIEW18_ARB")) return GL_MODELVIEW18_ARB;
        if (!strcmp(name, "GL_MODELVIEW19_ARB")) return GL_MODELVIEW19_ARB;
        if (!strcmp(name, "GL_MODELVIEW20_ARB")) return GL_MODELVIEW20_ARB;
        if (!strcmp(name, "GL_MODELVIEW21_ARB")) return GL_MODELVIEW21_ARB;
        if (!strcmp(name, "GL_MODELVIEW22_ARB")) return GL_MODELVIEW22_ARB;
        if (!strcmp(name, "GL_MODELVIEW23_ARB")) return GL_MODELVIEW23_ARB;
        if (!strcmp(name, "GL_MODELVIEW24_ARB")) return GL_MODELVIEW24_ARB;
        if (!strcmp(name, "GL_MODELVIEW25_ARB")) return GL_MODELVIEW25_ARB;
        if (!strcmp(name, "GL_MODELVIEW26_ARB")) return GL_MODELVIEW26_ARB;
        if (!strcmp(name, "GL_MODELVIEW27_ARB")) return GL_MODELVIEW27_ARB;
        if (!strcmp(name, "GL_MODELVIEW28_ARB")) return GL_MODELVIEW28_ARB;
        if (!strcmp(name, "GL_MODELVIEW29_ARB")) return GL_MODELVIEW29_ARB;
        if (!strcmp(name, "GL_MODELVIEW30_ARB")) return GL_MODELVIEW30_ARB;
        if (!strcmp(name, "GL_MODELVIEW31_ARB")) return GL_MODELVIEW31_ARB;
        if (!strcmp(name, "GL_MULTISAMPLE_ARB")) return GL_MULTISAMPLE_ARB;
        if (!strcmp(name, "GL_MULTISAMPLE_BIT")) return GL_MULTISAMPLE_BIT;
        if (!strcmp(name, "GL_MULTISAMPLE_EXT")) return GL_MULTISAMPLE_EXT;
        if (!strcmp(name, "GL_NV_blend_square")) return GL_NV_blend_square;
        if (!strcmp(name, "GL_NV_float_buffer")) return GL_NV_float_buffer;
        if (!strcmp(name, "GL_NV_fog_distance")) return GL_NV_fog_distance;
        if (!strcmp(name, "GL_NV_gpu_program4")) return GL_NV_gpu_program4;
        if (!strcmp(name, "GL_NV_point_sprite")) return GL_NV_point_sprite;
        if (!strcmp(name, "GL_OBJECT_TYPE_ARB")) return GL_OBJECT_TYPE_ARB;
        if (!strcmp(name, "GL_OES_read_format")) return GL_OES_read_format;
        if (!strcmp(name, "GL_OPERAND3_RGB_NV")) return GL_OPERAND3_RGB_NV;
        if (!strcmp(name, "GL_PACK_ROW_LENGTH")) return GL_PACK_ROW_LENGTH;
        if (!strcmp(name, "GL_PACK_SWAP_BYTES")) return GL_PACK_SWAP_BYTES;
        if (!strcmp(name, "GL_PASS_THROUGH_NV")) return GL_PASS_THROUGH_NV;
        if (!strcmp(name, "GL_POINT_SPRITE_NV")) return GL_POINT_SPRITE_NV;
        if (!strcmp(name, "GL_POLYGON_STIPPLE")) return GL_POLYGON_STIPPLE;
        if (!strcmp(name, "GL_PRESENT_TIME_NV")) return GL_PRESENT_TIME_NV;
        if (!strcmp(name, "GL_PRIMITIVE_ID_NV")) return GL_PRIMITIVE_ID_NV;
        if (!strcmp(name, "GL_PROXY_HISTOGRAM")) return GL_PROXY_HISTOGRAM;
        if (!strcmp(name, "GL_QUARTER_BIT_ATI")) return GL_QUARTER_BIT_ATI;
        if (!strcmp(name, "GL_QUERY_RESULT_NV")) return GL_QUERY_RESULT_NV;
        if (!strcmp(name, "GL_RED_INTEGER_EXT")) return GL_RED_INTEGER_EXT;
        if (!strcmp(name, "GL_RGBA4_DXT5_S3TC")) return GL_RGBA4_DXT5_S3TC;
        if (!strcmp(name, "GL_RGB_FLOAT16_ATI")) return GL_RGB_FLOAT16_ATI;
        if (!strcmp(name, "GL_RGB_FLOAT32_ATI")) return GL_RGB_FLOAT32_ATI;
        if (!strcmp(name, "GL_RGB_INTEGER_EXT")) return GL_RGB_INTEGER_EXT;
        if (!strcmp(name, "GL_RGB_SIGNED_SGIX")) return GL_RGB_SIGNED_SGIX;
        if (!strcmp(name, "GL_SAMPLE_COUNT_NV")) return GL_SAMPLE_COUNT_NV;
        if (!strcmp(name, "GL_SAMPLE_COVERAGE")) return GL_SAMPLE_COVERAGE;
        if (!strcmp(name, "GL_SAMPLE_MASK_EXT")) return GL_SAMPLE_MASK_EXT;
        if (!strcmp(name, "GL_SCALE_BY_TWO_NV")) return GL_SCALE_BY_TWO_NV;
        if (!strcmp(name, "GL_SGI_color_table")) return GL_SGI_color_table;
        if (!strcmp(name, "GL_SGIX_fog_offset")) return GL_SGIX_fog_offset;
        if (!strcmp(name, "GL_SIGNED_ALPHA_NV")) return GL_SIGNED_ALPHA_NV;
        if (!strcmp(name, "GL_SIGNED_ALPHA_NV")) return GL_SIGNED_ALPHA_NV;
        if (!strcmp(name, "GL_SIGNED_HILO8_NV")) return GL_SIGNED_HILO8_NV;
        if (!strcmp(name, "GL_SIGNED_RGBA8_NV")) return GL_SIGNED_RGBA8_NV;
        if (!strcmp(name, "GL_SIGNED_RGBA8_NV")) return GL_SIGNED_RGBA8_NV;
        if (!strcmp(name, "GL_SLICE_ACCUM_SUN")) return GL_SLICE_ACCUM_SUN;
        if (!strcmp(name, "GL_SLUMINANCE8_EXT")) return GL_SLUMINANCE8_EXT;
        if (!strcmp(name, "GL_SOURCE0_RGB_ARB")) return GL_SOURCE0_RGB_ARB;
        if (!strcmp(name, "GL_SOURCE0_RGB_EXT")) return GL_SOURCE0_RGB_EXT;
        if (!strcmp(name, "GL_SOURCE1_RGB_ARB")) return GL_SOURCE1_RGB_ARB;
        if (!strcmp(name, "GL_SOURCE1_RGB_EXT")) return GL_SOURCE1_RGB_EXT;
        if (!strcmp(name, "GL_SOURCE2_RGB_ARB")) return GL_SOURCE2_RGB_ARB;
        if (!strcmp(name, "GL_SOURCE2_RGB_EXT")) return GL_SOURCE2_RGB_EXT;
        if (!strcmp(name, "GL_STACK_UNDERFLOW")) return GL_STACK_UNDERFLOW;
        if (!strcmp(name, "GL_STATIC_COPY_ARB")) return GL_STATIC_COPY_ARB;
        if (!strcmp(name, "GL_STATIC_DRAW_ARB")) return GL_STATIC_DRAW_ARB;
        if (!strcmp(name, "GL_STATIC_READ_ARB")) return GL_STATIC_READ_ARB;
        if (!strcmp(name, "GL_STENCIL_INDEX16")) return GL_STENCIL_INDEX16;
        if (!strcmp(name, "GL_STREAM_COPY_ARB")) return GL_STREAM_COPY_ARB;
        if (!strcmp(name, "GL_STREAM_DRAW_ARB")) return GL_STREAM_DRAW_ARB;
        if (!strcmp(name, "GL_STREAM_READ_ARB")) return GL_STREAM_READ_ARB;
        if (!strcmp(name, "GL_SUN_slice_accum")) return GL_SUN_slice_accum;
        if (!strcmp(name, "GL_SWIZZLE_STQ_ATI")) return GL_SWIZZLE_STQ_ATI;
        if (!strcmp(name, "GL_SWIZZLE_STR_ATI")) return GL_SWIZZLE_STR_ATI;
        if (!strcmp(name, "GL_T2F_C4F_N3F_V3F")) return GL_T2F_C4F_N3F_V3F;
        if (!strcmp(name, "GL_T4F_C4F_N3F_V4F")) return GL_T4F_C4F_N3F_V4F;
        if (!strcmp(name, "GL_TABLE_TOO_LARGE")) return GL_TABLE_TOO_LARGE;
        if (!strcmp(name, "GL_TEXTURE_MAX_LOD")) return GL_TEXTURE_MAX_LOD;
        if (!strcmp(name, "GL_TEXTURE_MIN_LOD")) return GL_TEXTURE_MIN_LOD;
        if (!strcmp(name, "GL_TRACK_MATRIX_NV")) return GL_TRACK_MATRIX_NV;
        if (!strcmp(name, "GL_VALIDATE_STATUS")) return GL_VALIDATE_STATUS;
        if (!strcmp(name, "GL_VERTEX4_BIT_PGI")) return GL_VERTEX4_BIT_PGI;
        if (!strcmp(name, "GL_WRAP_BORDER_SUN")) return GL_WRAP_BORDER_SUN;
        if (!strcmp(name, "GL_YCBCR_422_APPLE")) return GL_YCBCR_422_APPLE;
        return _GL_UNDEFINED;
    case 19:
        if (!strcmp(name, "GL_3D_COLOR_TEXTURE")) return GL_3D_COLOR_TEXTURE;
        if (!strcmp(name, "GL_3DFX_multisample")) return GL_3DFX_multisample;
        if (!strcmp(name, "GL_4D_COLOR_TEXTURE")) return GL_4D_COLOR_TEXTURE;
        if (!strcmp(name, "GL_ACCUM_ALPHA_BITS")) return GL_ACCUM_ALPHA_BITS;
        if (!strcmp(name, "GL_ACCUM_BUFFER_BIT")) return GL_ACCUM_BUFFER_BIT;
        if (!strcmp(name, "GL_ACCUM_GREEN_BITS")) return GL_ACCUM_GREEN_BITS;
        if (!strcmp(name, "GL_ALL_COMPLETED_NV")) return GL_ALL_COMPLETED_NV;
        if (!strcmp(name, "GL_ARB_draw_buffers")) return GL_ARB_draw_buffers;
        if (!strcmp(name, "GL_ARB_multitexture")) return GL_ARB_multitexture;
        if (!strcmp(name, "GL_ARB_point_sprite")) return GL_ARB_point_sprite;
        if (!strcmp(name, "GL_ARB_vertex_blend")) return GL_ARB_vertex_blend;
        if (!strcmp(name, "GL_ARRAY_BUFFER_ARB")) return GL_ARRAY_BUFFER_ARB;
        if (!strcmp(name, "GL_ATI_draw_buffers")) return GL_ATI_draw_buffers;
        if (!strcmp(name, "GL_ATI_pn_triangles")) return GL_ATI_pn_triangles;
        if (!strcmp(name, "GL_ATTACHED_SHADERS")) return GL_ATTACHED_SHADERS;
        if (!strcmp(name, "GL_BGRA_INTEGER_EXT")) return GL_BGRA_INTEGER_EXT;
        if (!strcmp(name, "GL_BLUE_INTEGER_EXT")) return GL_BLUE_INTEGER_EXT;
        if (!strcmp(name, "GL_BUFFER_USAGE_ARB")) return GL_BUFFER_USAGE_ARB;
        if (!strcmp(name, "GL_CLAMP_READ_COLOR")) return GL_CLAMP_READ_COLOR;
        if (!strcmp(name, "GL_CLIP_DISTANCE_NV")) return GL_CLIP_DISTANCE_NV;
        if (!strcmp(name, "GL_COLOR_ARRAY_SIZE")) return GL_COLOR_ARRAY_SIZE;
        if (!strcmp(name, "GL_COLOR_ARRAY_TYPE")) return GL_COLOR_ARRAY_TYPE;
        if (!strcmp(name, "GL_COLOR_BUFFER_BIT")) return GL_COLOR_BUFFER_BIT;
        if (!strcmp(name, "GL_COLOR_INDEX1_EXT")) return GL_COLOR_INDEX1_EXT;
        if (!strcmp(name, "GL_COLOR_INDEX1_EXT")) return GL_COLOR_INDEX1_EXT;
        if (!strcmp(name, "GL_COLOR_INDEX2_EXT")) return GL_COLOR_INDEX2_EXT;
        if (!strcmp(name, "GL_COLOR_INDEX2_EXT")) return GL_COLOR_INDEX2_EXT;
        if (!strcmp(name, "GL_COLOR_INDEX4_EXT")) return GL_COLOR_INDEX4_EXT;
        if (!strcmp(name, "GL_COLOR_INDEX4_EXT")) return GL_COLOR_INDEX4_EXT;
        if (!strcmp(name, "GL_COLOR_INDEX8_EXT")) return GL_COLOR_INDEX8_EXT;
        if (!strcmp(name, "GL_COLOR_INDEX8_EXT")) return GL_COLOR_INDEX8_EXT;
        if (!strcmp(name, "GL_COLOR_MATRIX_SGI")) return GL_COLOR_MATRIX_SGI;
        if (!strcmp(name, "GL_COLOR_TABLE_BIAS")) return GL_COLOR_TABLE_BIAS;
        if (!strcmp(name, "GL_COMBINER_BIAS_NV")) return GL_COMBINER_BIAS_NV;
        if (!strcmp(name, "GL_COMPRESSED_ALPHA")) return GL_COMPRESSED_ALPHA;
        if (!strcmp(name, "GL_COORD_REPLACE_NV")) return GL_COORD_REPLACE_NV;
        if (!strcmp(name, "GL_COPY_PIXEL_TOKEN")) return GL_COPY_PIXEL_TOKEN;
        if (!strcmp(name, "GL_COUNTER_TYPE_AMD")) return GL_COUNTER_TYPE_AMD;
        if (!strcmp(name, "GL_CULL_FRAGMENT_NV")) return GL_CULL_FRAGMENT_NV;
        if (!strcmp(name, "GL_DEPTH24_STENCIL8")) return GL_DEPTH24_STENCIL8;
        if (!strcmp(name, "GL_DEPTH_ATTACHMENT")) return GL_DEPTH_ATTACHMENT;
        if (!strcmp(name, "GL_DEPTH_BOUNDS_EXT")) return GL_DEPTH_BOUNDS_EXT;
        if (!strcmp(name, "GL_DEPTH_BUFFER_BIT")) return GL_DEPTH_BUFFER_BIT;
        if (!strcmp(name, "GL_DEPTH_STENCIL_NV")) return GL_DEPTH_STENCIL_NV;
        if (!strcmp(name, "GL_DRAW_BUFFER0_ARB")) return GL_DRAW_BUFFER0_ARB;
        if (!strcmp(name, "GL_DRAW_BUFFER0_ATI")) return GL_DRAW_BUFFER0_ATI;
        if (!strcmp(name, "GL_DRAW_BUFFER1_ARB")) return GL_DRAW_BUFFER1_ARB;
        if (!strcmp(name, "GL_DRAW_BUFFER1_ATI")) return GL_DRAW_BUFFER1_ATI;
        if (!strcmp(name, "GL_DRAW_BUFFER2_ARB")) return GL_DRAW_BUFFER2_ARB;
        if (!strcmp(name, "GL_DRAW_BUFFER2_ATI")) return GL_DRAW_BUFFER2_ATI;
        if (!strcmp(name, "GL_DRAW_BUFFER3_ARB")) return GL_DRAW_BUFFER3_ARB;
        if (!strcmp(name, "GL_DRAW_BUFFER3_ATI")) return GL_DRAW_BUFFER3_ATI;
        if (!strcmp(name, "GL_DRAW_BUFFER4_ARB")) return GL_DRAW_BUFFER4_ARB;
        if (!strcmp(name, "GL_DRAW_BUFFER4_ATI")) return GL_DRAW_BUFFER4_ATI;
        if (!strcmp(name, "GL_DRAW_BUFFER5_ARB")) return GL_DRAW_BUFFER5_ARB;
        if (!strcmp(name, "GL_DRAW_BUFFER5_ATI")) return GL_DRAW_BUFFER5_ATI;
        if (!strcmp(name, "GL_DRAW_BUFFER6_ARB")) return GL_DRAW_BUFFER6_ARB;
        if (!strcmp(name, "GL_DRAW_BUFFER6_ATI")) return GL_DRAW_BUFFER6_ATI;
        if (!strcmp(name, "GL_DRAW_BUFFER7_ARB")) return GL_DRAW_BUFFER7_ARB;
        if (!strcmp(name, "GL_DRAW_BUFFER7_ATI")) return GL_DRAW_BUFFER7_ATI;
        if (!strcmp(name, "GL_DRAW_BUFFER8_ARB")) return GL_DRAW_BUFFER8_ARB;
        if (!strcmp(name, "GL_DRAW_BUFFER8_ATI")) return GL_DRAW_BUFFER8_ATI;
        if (!strcmp(name, "GL_DRAW_BUFFER9_ARB")) return GL_DRAW_BUFFER9_ARB;
        if (!strcmp(name, "GL_DRAW_BUFFER9_ATI")) return GL_DRAW_BUFFER9_ATI;
        if (!strcmp(name, "GL_DRAW_FRAMEBUFFER")) return GL_DRAW_FRAMEBUFFER;
        if (!strcmp(name, "GL_DRAW_PIXEL_TOKEN")) return GL_DRAW_PIXEL_TOKEN;
        if (!strcmp(name, "GL_DYNAMIC_COPY_ARB")) return GL_DYNAMIC_COPY_ARB;
        if (!strcmp(name, "GL_DYNAMIC_DRAW_ARB")) return GL_DYNAMIC_DRAW_ARB;
        if (!strcmp(name, "GL_DYNAMIC_READ_ARB")) return GL_DYNAMIC_READ_ARB;
        if (!strcmp(name, "GL_EDGEFLAG_BIT_PGI")) return GL_EDGEFLAG_BIT_PGI;
        if (!strcmp(name, "GL_ENV_MODULATE_EXT")) return GL_ENV_MODULATE_EXT;
        if (!strcmp(name, "GL_ENV_SUBTRACT_EXT")) return GL_ENV_SUBTRACT_EXT;
        if (!strcmp(name, "GL_EXPAND_NEGATE_NV")) return GL_EXPAND_NEGATE_NV;
        if (!strcmp(name, "GL_EXPAND_NORMAL_NV")) return GL_EXPAND_NORMAL_NV;
        if (!strcmp(name, "GL_EXT_blend_minmax")) return GL_EXT_blend_minmax;
        if (!strcmp(name, "GL_EXT_copy_texture")) return GL_EXT_copy_texture;
        if (!strcmp(name, "GL_EXT_packed_float")) return GL_EXT_packed_float;
        if (!strcmp(name, "GL_EXT_scene_marker")) return GL_EXT_scene_marker;
        if (!strcmp(name, "GL_EXT_shadow_funcs")) return GL_EXT_shadow_funcs;
        if (!strcmp(name, "GL_EXT_stencil_wrap")) return GL_EXT_stencil_wrap;
        if (!strcmp(name, "GL_EXT_texture_sRGB")) return GL_EXT_texture_sRGB;
        if (!strcmp(name, "GL_EXT_vertex_array")) return GL_EXT_vertex_array;
        if (!strcmp(name, "GL_FRAMEBUFFER_SRGB")) return GL_FRAMEBUFFER_SRGB;
        if (!strcmp(name, "GL_GLOBAL_ALPHA_SUN")) return GL_GLOBAL_ALPHA_SUN;
        if (!strcmp(name, "GL_HISTOGRAM_FORMAT")) return GL_HISTOGRAM_FORMAT;
        if (!strcmp(name, "GL_INDEX_ARRAY_TYPE")) return GL_INDEX_ARRAY_TYPE;
        if (!strcmp(name, "GL_INGR_color_clamp")) return GL_INGR_color_clamp;
        if (!strcmp(name, "GL_INTENSITY16F_ARB")) return GL_INTENSITY16F_ARB;
        if (!strcmp(name, "GL_INTENSITY16I_EXT")) return GL_INTENSITY16I_EXT;
        if (!strcmp(name, "GL_INTENSITY32F_ARB")) return GL_INTENSITY32F_ARB;
        if (!strcmp(name, "GL_INTENSITY32I_EXT")) return GL_INTENSITY32I_EXT;
        if (!strcmp(name, "GL_INTENSITY8UI_EXT")) return GL_INTENSITY8UI_EXT;
        if (!strcmp(name, "GL_INT_SAMPLER_CUBE")) return GL_INT_SAMPLER_CUBE;
        if (!strcmp(name, "GL_KTX_FRONT_REGION")) return GL_KTX_FRONT_REGION;
        if (!strcmp(name, "GL_LINE_RESET_TOKEN")) return GL_LINE_RESET_TOKEN;
        if (!strcmp(name, "GL_LINE_SMOOTH_HINT")) return GL_LINE_SMOOTH_HINT;
        if (!strcmp(name, "GL_LINE_WIDTH_RANGE")) return GL_LINE_WIDTH_RANGE;
        if (!strcmp(name, "GL_LUMINANCE16F_ARB")) return GL_LUMINANCE16F_ARB;
        if (!strcmp(name, "GL_LUMINANCE16I_EXT")) return GL_LUMINANCE16I_EXT;
        if (!strcmp(name, "GL_LUMINANCE32F_ARB")) return GL_LUMINANCE32F_ARB;
        if (!strcmp(name, "GL_LUMINANCE32I_EXT")) return GL_LUMINANCE32I_EXT;
        if (!strcmp(name, "GL_LUMINANCE8UI_EXT")) return GL_LUMINANCE8UI_EXT;
        if (!strcmp(name, "GL_MAP1_GRID_DOMAIN")) return GL_MAP1_GRID_DOMAIN;
        if (!strcmp(name, "GL_MAP1_TANGENT_EXT")) return GL_MAP1_TANGENT_EXT;
        if (!strcmp(name, "GL_MAP2_GRID_DOMAIN")) return GL_MAP2_GRID_DOMAIN;
        if (!strcmp(name, "GL_MAP2_TANGENT_EXT")) return GL_MAP2_TANGENT_EXT;
        if (!strcmp(name, "GL_MAX_DRAW_BUFFERS")) return GL_MAX_DRAW_BUFFERS;
        if (!strcmp(name, "GL_MAX_LIST_NESTING")) return GL_MAX_LIST_NESTING;
        if (!strcmp(name, "GL_MAX_SHININESS_NV")) return GL_MAX_SHININESS_NV;
        if (!strcmp(name, "GL_MAX_TEXTURE_SIZE")) return GL_MAX_TEXTURE_SIZE;
        if (!strcmp(name, "GL_MESA_pack_invert")) return GL_MESA_pack_invert;
        if (!strcmp(name, "GL_MIRROR_CLAMP_ATI")) return GL_MIRROR_CLAMP_ATI;
        if (!strcmp(name, "GL_MIRROR_CLAMP_EXT")) return GL_MIRROR_CLAMP_EXT;
        if (!strcmp(name, "GL_MODELVIEW_MATRIX")) return GL_MODELVIEW_MATRIX;
        if (!strcmp(name, "GL_MULTISAMPLE_3DFX")) return GL_MULTISAMPLE_3DFX;
        if (!strcmp(name, "GL_MULTISAMPLE_SGIS")) return GL_MULTISAMPLE_SGIS;
        if (!strcmp(name, "GL_NAME_STACK_DEPTH")) return GL_NAME_STACK_DEPTH;
        if (!strcmp(name, "GL_NEGATIVE_ONE_EXT")) return GL_NEGATIVE_ONE_EXT;
        if (!strcmp(name, "GL_NORMAL_ARRAY_EXT")) return GL_NORMAL_ARRAY_EXT;
        if (!strcmp(name, "GL_NV_present_video")) return GL_NV_present_video;
        if (!strcmp(name, "GL_NV_texgen_emboss")) return GL_NV_texgen_emboss;
        if (!strcmp(name, "GL_OBJECT_LINE_SGIS")) return GL_OBJECT_LINE_SGIS;
        if (!strcmp(name, "GL_OPERAND0_RGB_ARB")) return GL_OPERAND0_RGB_ARB;
        if (!strcmp(name, "GL_OPERAND0_RGB_EXT")) return GL_OPERAND0_RGB_EXT;
        if (!strcmp(name, "GL_OPERAND1_RGB_ARB")) return GL_OPERAND1_RGB_ARB;
        if (!strcmp(name, "GL_OPERAND1_RGB_EXT")) return GL_OPERAND1_RGB_EXT;
        if (!strcmp(name, "GL_OPERAND2_RGB_ARB")) return GL_OPERAND2_RGB_ARB;
        if (!strcmp(name, "GL_OPERAND2_RGB_EXT")) return GL_OPERAND2_RGB_EXT;
        if (!strcmp(name, "GL_PACK_INVERT_MESA")) return GL_PACK_INVERT_MESA;
        if (!strcmp(name, "GL_PACK_SKIP_IMAGES")) return GL_PACK_SKIP_IMAGES;
        if (!strcmp(name, "GL_PACK_SKIP_PIXELS")) return GL_PACK_SKIP_PIXELS;
        if (!strcmp(name, "GL_PGI_vertex_hints")) return GL_PGI_vertex_hints;
        if (!strcmp(name, "GL_PIXEL_MAP_A_TO_A")) return GL_PIXEL_MAP_A_TO_A;
        if (!strcmp(name, "GL_PIXEL_MAP_B_TO_B")) return GL_PIXEL_MAP_B_TO_B;
        if (!strcmp(name, "GL_PIXEL_MAP_G_TO_G")) return GL_PIXEL_MAP_G_TO_G;
        if (!strcmp(name, "GL_PIXEL_MAP_I_TO_A")) return GL_PIXEL_MAP_I_TO_A;
        if (!strcmp(name, "GL_PIXEL_MAP_I_TO_B")) return GL_PIXEL_MAP_I_TO_B;
        if (!strcmp(name, "GL_PIXEL_MAP_I_TO_G")) return GL_PIXEL_MAP_I_TO_G;
        if (!strcmp(name, "GL_PIXEL_MAP_I_TO_I")) return GL_PIXEL_MAP_I_TO_I;
        if (!strcmp(name, "GL_PIXEL_MAP_I_TO_R")) return GL_PIXEL_MAP_I_TO_R;
        if (!strcmp(name, "GL_PIXEL_MAP_R_TO_R")) return GL_PIXEL_MAP_R_TO_R;
        if (!strcmp(name, "GL_PIXEL_MAP_S_TO_S")) return GL_PIXEL_MAP_S_TO_S;
        if (!strcmp(name, "GL_PN_TRIANGLES_ATI")) return GL_PN_TRIANGLES_ATI;
        if (!strcmp(name, "GL_POINT_SIZE_RANGE")) return GL_POINT_SIZE_RANGE;
        if (!strcmp(name, "GL_POINT_SPRITE_ARB")) return GL_POINT_SPRITE_ARB;
        if (!strcmp(name, "GL_PRIMARY_COLOR_NV")) return GL_PRIMARY_COLOR_NV;
        if (!strcmp(name, "GL_PROXY_TEXTURE_1D")) return GL_PROXY_TEXTURE_1D;
        if (!strcmp(name, "GL_PROXY_TEXTURE_1D")) return GL_PROXY_TEXTURE_1D;
        if (!strcmp(name, "GL_PROXY_TEXTURE_2D")) return GL_PROXY_TEXTURE_2D;
        if (!strcmp(name, "GL_PROXY_TEXTURE_2D")) return GL_PROXY_TEXTURE_2D;
        if (!strcmp(name, "GL_PROXY_TEXTURE_3D")) return GL_PROXY_TEXTURE_3D;
        if (!strcmp(name, "GL_QUERY_NO_WAIT_NV")) return GL_QUERY_NO_WAIT_NV;
        if (!strcmp(name, "GL_QUERY_RESULT_ARB")) return GL_QUERY_RESULT_ARB;
        if (!strcmp(name, "GL_R1UI_C3F_V3F_SUN")) return GL_R1UI_C3F_V3F_SUN;
        if (!strcmp(name, "GL_R1UI_N3F_V3F_SUN")) return GL_R1UI_N3F_V3F_SUN;
        if (!strcmp(name, "GL_R1UI_T2F_V3F_SUN")) return GL_R1UI_T2F_V3F_SUN;
        if (!strcmp(name, "GL_READ_FRAMEBUFFER")) return GL_READ_FRAMEBUFFER;
        if (!strcmp(name, "GL_RENDERBUFFER_EXT")) return GL_RENDERBUFFER_EXT;
        if (!strcmp(name, "GL_REPLICATE_BORDER")) return GL_REPLICATE_BORDER;
        if (!strcmp(name, "GL_RGBA_FLOAT16_ATI")) return GL_RGBA_FLOAT16_ATI;
        if (!strcmp(name, "GL_RGBA_FLOAT32_ATI")) return GL_RGBA_FLOAT32_ATI;
        if (!strcmp(name, "GL_RGBA_INTEGER_EXT")) return GL_RGBA_INTEGER_EXT;
        if (!strcmp(name, "GL_RGBA_SIGNED_SGIX")) return GL_RGBA_SIGNED_SGIX;
        if (!strcmp(name, "GL_SAMPLE_MASK_SGIS")) return GL_SAMPLE_MASK_SGIS;
        if (!strcmp(name, "GL_SAMPLER_1D_ARRAY")) return GL_SAMPLER_1D_ARRAY;
        if (!strcmp(name, "GL_SAMPLER_2D_ARRAY")) return GL_SAMPLER_2D_ARRAY;
        if (!strcmp(name, "GL_SAMPLER_CUBE_ARB")) return GL_SAMPLER_CUBE_ARB;
        if (!strcmp(name, "GL_SATURATE_BIT_ATI")) return GL_SATURATE_BIT_ATI;
        if (!strcmp(name, "GL_SCALE_BY_FOUR_NV")) return GL_SCALE_BY_FOUR_NV;
        if (!strcmp(name, "GL_SEPARABLE_2D_EXT")) return GL_SEPARABLE_2D_EXT;
        if (!strcmp(name, "GL_SEPARATE_ATTRIBS")) return GL_SEPARATE_ATTRIBS;
        if (!strcmp(name, "GL_SGI_color_matrix")) return GL_SGI_color_matrix;
        if (!strcmp(name, "GL_SGIS_color_range")) return GL_SGIS_color_range;
        if (!strcmp(name, "GL_SGIS_multisample")) return GL_SGIS_multisample;
        if (!strcmp(name, "GL_SGIS_texture_lod")) return GL_SGIS_texture_lod;
        if (!strcmp(name, "GL_SGIX_async_pixel")) return GL_SGIX_async_pixel;
        if (!strcmp(name, "GL_SGIX_fog_texture")) return GL_SGIX_fog_texture;
        if (!strcmp(name, "GL_SIGNED_ALPHA8_NV")) return GL_SIGNED_ALPHA8_NV;
        if (!strcmp(name, "GL_SIGNED_ALPHA8_NV")) return GL_SIGNED_ALPHA8_NV;
        if (!strcmp(name, "GL_SIGNED_HILO16_NV")) return GL_SIGNED_HILO16_NV;
        if (!strcmp(name, "GL_SIGNED_HILO16_NV")) return GL_SIGNED_HILO16_NV;
        if (!strcmp(name, "GL_SIGNED_NEGATE_NV")) return GL_SIGNED_NEGATE_NV;
        if (!strcmp(name, "GL_SINGLE_COLOR_EXT")) return GL_SINGLE_COLOR_EXT;
        if (!strcmp(name, "GL_SLUMINANCE_ALPHA")) return GL_SLUMINANCE_ALPHA;
        if (!strcmp(name, "GL_SOURCE3_ALPHA_NV")) return GL_SOURCE3_ALPHA_NV;
        if (!strcmp(name, "GL_SRGB8_ALPHA8_EXT")) return GL_SRGB8_ALPHA8_EXT;
        if (!strcmp(name, "GL_STENCIL_BACK_REF")) return GL_STENCIL_BACK_REF;
        if (!strcmp(name, "GL_SUN_global_alpha")) return GL_SUN_global_alpha;
        if (!strcmp(name, "GL_SWIZZLE_STRQ_ATI")) return GL_SWIZZLE_STRQ_ATI;
        if (!strcmp(name, "GL_TEXTURE_1D_ARRAY")) return GL_TEXTURE_1D_ARRAY;
        if (!strcmp(name, "GL_TEXTURE_2D_ARRAY")) return GL_TEXTURE_2D_ARRAY;
        if (!strcmp(name, "GL_TEXTURE_COORD_NV")) return GL_TEXTURE_COORD_NV;
        if (!strcmp(name, "GL_TEXTURE_CUBE_MAP")) return GL_TEXTURE_CUBE_MAP;
        if (!strcmp(name, "GL_TEXTURE_ENV0_EXT")) return GL_TEXTURE_ENV0_EXT;
        if (!strcmp(name, "GL_TEXTURE_ENV_MODE")) return GL_TEXTURE_ENV_MODE;
        if (!strcmp(name, "GL_TEXTURE_FOG_SGIX")) return GL_TEXTURE_FOG_SGIX;
        if (!strcmp(name, "GL_TEXTURE_GEN_MODE")) return GL_TEXTURE_GEN_MODE;
        if (!strcmp(name, "GL_TEXTURE_LOD_BIAS")) return GL_TEXTURE_LOD_BIAS;
        if (!strcmp(name, "GL_TEXTURE_PRIORITY")) return GL_TEXTURE_PRIORITY;
        if (!strcmp(name, "GL_TEXTURE_RED_SIZE")) return GL_TEXTURE_RED_SIZE;
        if (!strcmp(name, "GL_TEXTURE_RED_TYPE")) return GL_TEXTURE_RED_TYPE;
        if (!strcmp(name, "GL_TEXTURE_RESIDENT")) return GL_TEXTURE_RESIDENT;
        if (!strcmp(name, "GL_TIME_ELAPSED_EXT")) return GL_TIME_ELAPSED_EXT;
        if (!strcmp(name, "GL_UNPACK_ALIGNMENT")) return GL_UNPACK_ALIGNMENT;
        if (!strcmp(name, "GL_UNPACK_LSB_FIRST")) return GL_UNPACK_LSB_FIRST;
        if (!strcmp(name, "GL_UNPACK_SKIP_ROWS")) return GL_UNPACK_SKIP_ROWS;
        if (!strcmp(name, "GL_VERTEX23_BIT_PGI")) return GL_VERTEX23_BIT_PGI;
        if (!strcmp(name, "GL_VERTEX_ARRAY_EXT")) return GL_VERTEX_ARRAY_EXT;
        if (!strcmp(name, "GL_VERTEX_BLEND_ARB")) return GL_VERTEX_BLEND_ARB;
        if (!strcmp(name, "GL_VIBRANCE_BIAS_NV")) return GL_VIBRANCE_BIAS_NV;
        if (!strcmp(name, "GL_WEIGHT_ARRAY_ARB")) return GL_WEIGHT_ARRAY_ARB;
        if (!strcmp(name, "GL_WIN_specular_fog")) return GL_WIN_specular_fog;
        return _GL_UNDEFINED;
    case 20:
        if (!strcmp(name, "GL_ACCUM_CLEAR_VALUE")) return GL_ACCUM_CLEAR_VALUE;
        if (!strcmp(name, "GL_ACTIVE_ATTRIBUTES")) return GL_ACTIVE_ATTRIBUTES;
        if (!strcmp(name, "GL_ALPHA_FLOAT16_ATI")) return GL_ALPHA_FLOAT16_ATI;
        if (!strcmp(name, "GL_ALPHA_FLOAT32_ATI")) return GL_ALPHA_FLOAT32_ATI;
        if (!strcmp(name, "GL_ALPHA_INTEGER_EXT")) return GL_ALPHA_INTEGER_EXT;
        if (!strcmp(name, "GL_ALPHA_SIGNED_SGIX")) return GL_ALPHA_SIGNED_SGIX;
        if (!strcmp(name, "GL_ARB_depth_texture")) return GL_ARB_depth_texture;
        if (!strcmp(name, "GL_ARB_texture_float")) return GL_ARB_texture_float;
        if (!strcmp(name, "GL_ARB_vertex_shader")) return GL_ARB_vertex_shader;
        if (!strcmp(name, "GL_ASYNC_MARKER_SGIX")) return GL_ASYNC_MARKER_SGIX;
        if (!strcmp(name, "GL_ATI_element_array")) return GL_ATI_element_array;
        if (!strcmp(name, "GL_ATI_texture_float")) return GL_ATI_texture_float;
        if (!strcmp(name, "GL_BLEND_DST_RGB_EXT")) return GL_BLEND_DST_RGB_EXT;
        if (!strcmp(name, "GL_BLEND_SRC_RGB_EXT")) return GL_BLEND_SRC_RGB_EXT;
        if (!strcmp(name, "GL_BUFFER_ACCESS_ARB")) return GL_BUFFER_ACCESS_ARB;
        if (!strcmp(name, "GL_BUFFER_MAPPED_ARB")) return GL_BUFFER_MAPPED_ARB;
        if (!strcmp(name, "GL_CLAMP_TO_EDGE_EXT")) return GL_CLAMP_TO_EDGE_EXT;
        if (!strcmp(name, "GL_CLIP_FAR_HINT_PGI")) return GL_CLIP_FAR_HINT_PGI;
        if (!strcmp(name, "GL_COLOR_ATTACHMENT0")) return GL_COLOR_ATTACHMENT0;
        if (!strcmp(name, "GL_COLOR_ATTACHMENT1")) return GL_COLOR_ATTACHMENT1;
        if (!strcmp(name, "GL_COLOR_ATTACHMENT2")) return GL_COLOR_ATTACHMENT2;
        if (!strcmp(name, "GL_COLOR_ATTACHMENT3")) return GL_COLOR_ATTACHMENT3;
        if (!strcmp(name, "GL_COLOR_ATTACHMENT4")) return GL_COLOR_ATTACHMENT4;
        if (!strcmp(name, "GL_COLOR_ATTACHMENT5")) return GL_COLOR_ATTACHMENT5;
        if (!strcmp(name, "GL_COLOR_ATTACHMENT6")) return GL_COLOR_ATTACHMENT6;
        if (!strcmp(name, "GL_COLOR_ATTACHMENT7")) return GL_COLOR_ATTACHMENT7;
        if (!strcmp(name, "GL_COLOR_ATTACHMENT8")) return GL_COLOR_ATTACHMENT8;
        if (!strcmp(name, "GL_COLOR_ATTACHMENT9")) return GL_COLOR_ATTACHMENT9;
        if (!strcmp(name, "GL_COLOR_CLEAR_VALUE")) return GL_COLOR_CLEAR_VALUE;
        if (!strcmp(name, "GL_COLOR_FLOAT_APPLE")) return GL_COLOR_FLOAT_APPLE;
        if (!strcmp(name, "GL_COLOR_INDEX12_EXT")) return GL_COLOR_INDEX12_EXT;
        if (!strcmp(name, "GL_COLOR_INDEX12_EXT")) return GL_COLOR_INDEX12_EXT;
        if (!strcmp(name, "GL_COLOR_INDEX16_EXT")) return GL_COLOR_INDEX16_EXT;
        if (!strcmp(name, "GL_COLOR_INDEX16_EXT")) return GL_COLOR_INDEX16_EXT;
        if (!strcmp(name, "GL_COLOR_TABLE_SCALE")) return GL_COLOR_TABLE_SCALE;
        if (!strcmp(name, "GL_COLOR_TABLE_WIDTH")) return GL_COLOR_TABLE_WIDTH;
        if (!strcmp(name, "GL_COMBINE_ALPHA_ARB")) return GL_COMBINE_ALPHA_ARB;
        if (!strcmp(name, "GL_COMBINE_ALPHA_EXT")) return GL_COMBINE_ALPHA_EXT;
        if (!strcmp(name, "GL_COMBINER_INPUT_NV")) return GL_COMBINER_INPUT_NV;
        if (!strcmp(name, "GL_COMBINER_SCALE_NV")) return GL_COMBINER_SCALE_NV;
        if (!strcmp(name, "GL_CONVOLUTION_WIDTH")) return GL_CONVOLUTION_WIDTH;
        if (!strcmp(name, "GL_COORD_REPLACE_ARB")) return GL_COORD_REPLACE_ARB;
        if (!strcmp(name, "GL_COUNTER_RANGE_AMD")) return GL_COUNTER_RANGE_AMD;
        if (!strcmp(name, "GL_CURRENT_ATTRIB_NV")) return GL_CURRENT_ATTRIB_NV;
        if (!strcmp(name, "GL_CURRENT_FOG_COORD")) return GL_CURRENT_FOG_COORD;
        if (!strcmp(name, "GL_CURRENT_MATRIX_NV")) return GL_CURRENT_MATRIX_NV;
        if (!strcmp(name, "GL_CURRENT_QUERY_ARB")) return GL_CURRENT_QUERY_ARB;
        if (!strcmp(name, "GL_DEPTH32F_STENCIL8")) return GL_DEPTH32F_STENCIL8;
        if (!strcmp(name, "GL_DEPTH_CLEAR_VALUE")) return GL_DEPTH_CLEAR_VALUE;
        if (!strcmp(name, "GL_DEPTH_COMPONENT16")) return GL_DEPTH_COMPONENT16;
        if (!strcmp(name, "GL_DEPTH_COMPONENT24")) return GL_DEPTH_COMPONENT24;
        if (!strcmp(name, "GL_DEPTH_COMPONENT32")) return GL_DEPTH_COMPONENT32;
        if (!strcmp(name, "GL_DEPTH_STENCIL_EXT")) return GL_DEPTH_STENCIL_EXT;
        if (!strcmp(name, "GL_DRAW_BUFFER10_ARB")) return GL_DRAW_BUFFER10_ARB;
        if (!strcmp(name, "GL_DRAW_BUFFER10_ATI")) return GL_DRAW_BUFFER10_ATI;
        if (!strcmp(name, "GL_DRAW_BUFFER11_ARB")) return GL_DRAW_BUFFER11_ARB;
        if (!strcmp(name, "GL_DRAW_BUFFER11_ATI")) return GL_DRAW_BUFFER11_ATI;
        if (!strcmp(name, "GL_DRAW_BUFFER12_ARB")) return GL_DRAW_BUFFER12_ARB;
        if (!strcmp(name, "GL_DRAW_BUFFER12_ATI")) return GL_DRAW_BUFFER12_ATI;
        if (!strcmp(name, "GL_DRAW_BUFFER13_ARB")) return GL_DRAW_BUFFER13_ARB;
        if (!strcmp(name, "GL_DRAW_BUFFER13_ATI")) return GL_DRAW_BUFFER13_ATI;
        if (!strcmp(name, "GL_DRAW_BUFFER14_ARB")) return GL_DRAW_BUFFER14_ARB;
        if (!strcmp(name, "GL_DRAW_BUFFER14_ATI")) return GL_DRAW_BUFFER14_ATI;
        if (!strcmp(name, "GL_DRAW_BUFFER15_ARB")) return GL_DRAW_BUFFER15_ARB;
        if (!strcmp(name, "GL_DRAW_BUFFER15_ATI")) return GL_DRAW_BUFFER15_ATI;
        if (!strcmp(name, "GL_DRAW_PIXELS_APPLE")) return GL_DRAW_PIXELS_APPLE;
        if (!strcmp(name, "GL_ELEMENT_ARRAY_ATI")) return GL_ELEMENT_ARRAY_ATI;
        if (!strcmp(name, "GL_EXT_draw_buffers2")) return GL_EXT_draw_buffers2;
        if (!strcmp(name, "GL_EXT_index_texture")) return GL_EXT_index_texture;
        if (!strcmp(name, "GL_EXT_light_texture")) return GL_EXT_light_texture;
        if (!strcmp(name, "GL_EXT_packed_pixels")) return GL_EXT_packed_pixels;
        if (!strcmp(name, "GL_EXT_texture_array")) return GL_EXT_texture_array;
        if (!strcmp(name, "GL_EXT_vertex_shader")) return GL_EXT_vertex_shader;
        if (!strcmp(name, "GL_FRAGMENT_FOG_SGIX")) return GL_FRAGMENT_FOG_SGIX;
        if (!strcmp(name, "GL_FUNC_SUBTRACT_EXT")) return GL_FUNC_SUBTRACT_EXT;
        if (!strcmp(name, "GL_GENERIC_ATTRIB_NV")) return GL_GENERIC_ATTRIB_NV;
        if (!strcmp(name, "GL_GREEN_INTEGER_EXT")) return GL_GREEN_INTEGER_EXT;
        if (!strcmp(name, "GL_HP_occlusion_test")) return GL_HP_occlusion_test;
        if (!strcmp(name, "GL_INDEX_CLEAR_VALUE")) return GL_INDEX_CLEAR_VALUE;
        if (!strcmp(name, "GL_INTENSITY16UI_EXT")) return GL_INTENSITY16UI_EXT;
        if (!strcmp(name, "GL_INTENSITY32UI_EXT")) return GL_INTENSITY32UI_EXT;
        if (!strcmp(name, "GL_INVALID_OPERATION")) return GL_INVALID_OPERATION;
        if (!strcmp(name, "GL_KTX_buffer_region")) return GL_KTX_buffer_region;
        if (!strcmp(name, "GL_LUMINANCE16UI_EXT")) return GL_LUMINANCE16UI_EXT;
        if (!strcmp(name, "GL_LUMINANCE32UI_EXT")) return GL_LUMINANCE32UI_EXT;
        if (!strcmp(name, "GL_LUMINANCE4_ALPHA4")) return GL_LUMINANCE4_ALPHA4;
        if (!strcmp(name, "GL_LUMINANCE6_ALPHA2")) return GL_LUMINANCE6_ALPHA2;
        if (!strcmp(name, "GL_LUMINANCE8_ALPHA8")) return GL_LUMINANCE8_ALPHA8;
        if (!strcmp(name, "GL_MAGNITUDE_BIAS_NV")) return GL_MAGNITUDE_BIAS_NV;
        if (!strcmp(name, "GL_MAP1_BINORMAL_EXT")) return GL_MAP1_BINORMAL_EXT;
        if (!strcmp(name, "GL_MAP2_BINORMAL_EXT")) return GL_MAP2_BINORMAL_EXT;
        if (!strcmp(name, "GL_MAX_TEXTURE_UNITS")) return GL_MAX_TEXTURE_UNITS;
        if (!strcmp(name, "GL_MAX_VIEWPORT_DIMS")) return GL_MAX_VIEWPORT_DIMS;
        if (!strcmp(name, "GL_MINMAX_FORMAT_EXT")) return GL_MINMAX_FORMAT_EXT;
        if (!strcmp(name, "GL_MODULATE_ADD_ATIX")) return GL_MODULATE_ADD_ATIX;
        if (!strcmp(name, "GL_NORMAL_ARRAY_TYPE")) return GL_NORMAL_ARRAY_TYPE;
        if (!strcmp(name, "GL_NV_texture_shader")) return GL_NV_texture_shader;
        if (!strcmp(name, "GL_NV_vertex_program")) return GL_NV_vertex_program;
        if (!strcmp(name, "GL_OBJECT_POINT_SGIS")) return GL_OBJECT_POINT_SGIS;
        if (!strcmp(name, "GL_OCCLUSION_TEST_HP")) return GL_OCCLUSION_TEST_HP;
        if (!strcmp(name, "GL_OPERAND3_ALPHA_NV")) return GL_OPERAND3_ALPHA_NV;
        if (!strcmp(name, "GL_OP_EXP_BASE_2_EXT")) return GL_OP_EXP_BASE_2_EXT;
        if (!strcmp(name, "GL_OP_LOG_BASE_2_EXT")) return GL_OP_LOG_BASE_2_EXT;
        if (!strcmp(name, "GL_OP_RECIP_SQRT_EXT")) return GL_OP_RECIP_SQRT_EXT;
        if (!strcmp(name, "GL_OUTPUT_COLOR0_EXT")) return GL_OUTPUT_COLOR0_EXT;
        if (!strcmp(name, "GL_OUTPUT_COLOR1_EXT")) return GL_OUTPUT_COLOR1_EXT;
        if (!strcmp(name, "GL_OUTPUT_VERTEX_EXT")) return GL_OUTPUT_VERTEX_EXT;
        if (!strcmp(name, "GL_PACK_IMAGE_HEIGHT")) return GL_PACK_IMAGE_HEIGHT;
        if (!strcmp(name, "GL_PACK_RESAMPLE_OML")) return GL_PACK_RESAMPLE_OML;
        if (!strcmp(name, "GL_PALETTE4_RGB8_OES")) return GL_PALETTE4_RGB8_OES;
        if (!strcmp(name, "GL_PALETTE8_RGB8_OES")) return GL_PALETTE8_RGB8_OES;
        if (!strcmp(name, "GL_PIXEL_PACK_BUFFER")) return GL_PIXEL_PACK_BUFFER;
        if (!strcmp(name, "GL_POINT_SMOOTH_HINT")) return GL_POINT_SMOOTH_HINT;
        if (!strcmp(name, "GL_PRIMARY_COLOR_ARB")) return GL_PRIMARY_COLOR_ARB;
        if (!strcmp(name, "GL_PRIMARY_COLOR_EXT")) return GL_PRIMARY_COLOR_EXT;
        if (!strcmp(name, "GL_PROGRAM_LENGTH_NV")) return GL_PROGRAM_LENGTH_NV;
        if (!strcmp(name, "GL_PROGRAM_STRING_NV")) return GL_PROGRAM_STRING_NV;
        if (!strcmp(name, "GL_PROGRAM_TARGET_NV")) return GL_PROGRAM_TARGET_NV;
        if (!strcmp(name, "GL_PROJECTION_MATRIX")) return GL_PROJECTION_MATRIX;
        if (!strcmp(name, "GL_PROXY_COLOR_TABLE")) return GL_PROXY_COLOR_TABLE;
        if (!strcmp(name, "GL_R1UI_C4UB_V3F_SUN")) return GL_R1UI_C4UB_V3F_SUN;
        if (!strcmp(name, "GL_REFLECTION_MAP_NV")) return GL_REFLECTION_MAP_NV;
        if (!strcmp(name, "GL_RGB16_SIGNED_SGIX")) return GL_RGB16_SIGNED_SGIX;
        if (!strcmp(name, "GL_RGB_FLOAT16_APPLE")) return GL_RGB_FLOAT16_APPLE;
        if (!strcmp(name, "GL_RGB_FLOAT32_APPLE")) return GL_RGB_FLOAT32_APPLE;
        if (!strcmp(name, "GL_SAMPLER_1D_SHADOW")) return GL_SAMPLER_1D_SHADOW;
        if (!strcmp(name, "GL_SAMPLER_2D_SHADOW")) return GL_SAMPLER_2D_SHADOW;
        if (!strcmp(name, "GL_SGIS_fog_function")) return GL_SGIS_fog_function;
        if (!strcmp(name, "GL_SGIX_flush_raster")) return GL_SGIX_flush_raster;
        if (!strcmp(name, "GL_SHADER_OBJECT_ARB")) return GL_SHADER_OBJECT_ARB;
        if (!strcmp(name, "GL_SOURCE0_ALPHA_ARB")) return GL_SOURCE0_ALPHA_ARB;
        if (!strcmp(name, "GL_SOURCE0_ALPHA_EXT")) return GL_SOURCE0_ALPHA_EXT;
        if (!strcmp(name, "GL_SOURCE1_ALPHA_ARB")) return GL_SOURCE1_ALPHA_ARB;
        if (!strcmp(name, "GL_SOURCE1_ALPHA_EXT")) return GL_SOURCE1_ALPHA_EXT;
        if (!strcmp(name, "GL_SOURCE2_ALPHA_ARB")) return GL_SOURCE2_ALPHA_ARB;
        if (!strcmp(name, "GL_SOURCE2_ALPHA_EXT")) return GL_SOURCE2_ALPHA_EXT;
        if (!strcmp(name, "GL_STENCIL_BACK_FAIL")) return GL_STENCIL_BACK_FAIL;
        if (!strcmp(name, "GL_STENCIL_BACK_FUNC")) return GL_STENCIL_BACK_FUNC;
        if (!strcmp(name, "GL_STENCIL_WRITEMASK")) return GL_STENCIL_WRITEMASK;
        if (!strcmp(name, "GL_SUN_triangle_list")) return GL_SUN_triangle_list;
        if (!strcmp(name, "GL_TANGENT_ARRAY_EXT")) return GL_TANGENT_ARRAY_EXT;
        if (!strcmp(name, "GL_TEXCOORD1_BIT_PGI")) return GL_TEXCOORD1_BIT_PGI;
        if (!strcmp(name, "GL_TEXCOORD2_BIT_PGI")) return GL_TEXCOORD2_BIT_PGI;
        if (!strcmp(name, "GL_TEXCOORD3_BIT_PGI")) return GL_TEXCOORD3_BIT_PGI;
        if (!strcmp(name, "GL_TEXCOORD4_BIT_PGI")) return GL_TEXCOORD4_BIT_PGI;
        if (!strcmp(name, "GL_TEXTURE_BLUE_SIZE")) return GL_TEXTURE_BLUE_SIZE;
        if (!strcmp(name, "GL_TEXTURE_BLUE_TYPE")) return GL_TEXTURE_BLUE_TYPE;
        if (!strcmp(name, "GL_TEXTURE_DEPTH_EXT")) return GL_TEXTURE_DEPTH_EXT;
        if (!strcmp(name, "GL_TEXTURE_ENV_COLOR")) return GL_TEXTURE_ENV_COLOR;
        if (!strcmp(name, "GL_TEXTURE_LIGHT_EXT")) return GL_TEXTURE_LIGHT_EXT;
        if (!strcmp(name, "GL_TEXTURE_MAX_LEVEL")) return GL_TEXTURE_MAX_LEVEL;
        if (!strcmp(name, "GL_TEXTURE_SHADER_NV")) return GL_TEXTURE_SHADER_NV;
        if (!strcmp(name, "GL_TRIANGLE_LIST_SUN")) return GL_TRIANGLE_LIST_SUN;
        if (!strcmp(name, "GL_TRIANGLE_MESH_SUN")) return GL_TRIANGLE_MESH_SUN;
        if (!strcmp(name, "GL_UNPACK_ROW_LENGTH")) return GL_UNPACK_ROW_LENGTH;
        if (!strcmp(name, "GL_UNPACK_SWAP_BYTES")) return GL_UNPACK_SWAP_BYTES;
        if (!strcmp(name, "GL_UNSIGNED_INT_24_8")) return GL_UNSIGNED_INT_24_8;
        if (!strcmp(name, "GL_UNSIGNED_INT_VEC2")) return GL_UNSIGNED_INT_VEC2;
        if (!strcmp(name, "GL_UNSIGNED_INT_VEC3")) return GL_UNSIGNED_INT_VEC3;
        if (!strcmp(name, "GL_UNSIGNED_INT_VEC4")) return GL_UNSIGNED_INT_VEC4;
        if (!strcmp(name, "GL_VARIANT_ARRAY_EXT")) return GL_VARIANT_ARRAY_EXT;
        if (!strcmp(name, "GL_VARIANT_VALUE_EXT")) return GL_VARIANT_VALUE_EXT;
        if (!strcmp(name, "GL_VERTEX_ARRAY_SIZE")) return GL_VERTEX_ARRAY_SIZE;
        if (!strcmp(name, "GL_VERTEX_ARRAY_TYPE")) return GL_VERTEX_ARRAY_TYPE;
        if (!strcmp(name, "GL_VERTEX_PROGRAM_NV")) return GL_VERTEX_PROGRAM_NV;
        if (!strcmp(name, "GL_VERTEX_SHADER_ARB")) return GL_VERTEX_SHADER_ARB;
        if (!strcmp(name, "GL_VERTEX_SHADER_EXT")) return GL_VERTEX_SHADER_EXT;
        if (!strcmp(name, "GL_VERTEX_SOURCE_ATI")) return GL_VERTEX_SOURCE_ATI;
        if (!strcmp(name, "GL_VIBRANCE_SCALE_NV")) return GL_VIBRANCE_SCALE_NV;
        if (!strcmp(name, "GL_WIN_phong_shading")) return GL_WIN_phong_shading;
        return _GL_UNDEFINED;
    case 21:
        if (!strcmp(name, "GL_ACTIVE_TEXTURE_ARB")) return GL_ACTIVE_TEXTURE_ARB;
        if (!strcmp(name, "GL_ACTIVE_VARYINGS_NV")) return GL_ACTIVE_VARYINGS_NV;
        if (!strcmp(name, "GL_APPLE_float_pixels")) return GL_APPLE_float_pixels;
        if (!strcmp(name, "GL_APPLE_pixel_buffer")) return GL_APPLE_pixel_buffer;
        if (!strcmp(name, "GL_ARB_draw_instanced")) return GL_ARB_draw_instanced;
        if (!strcmp(name, "GL_ARB_matrix_palette")) return GL_ARB_matrix_palette;
        if (!strcmp(name, "GL_ARB_shader_objects")) return GL_ARB_shader_objects;
        if (!strcmp(name, "GL_ARB_shadow_ambient")) return GL_ARB_shadow_ambient;
        if (!strcmp(name, "GL_ARB_vertex_program")) return GL_ARB_vertex_program;
        if (!strcmp(name, "GL_ATI_envmap_bumpmap")) return GL_ATI_envmap_bumpmap;
        if (!strcmp(name, "GL_ATI_vertex_streams")) return GL_ATI_vertex_streams;
        if (!strcmp(name, "GL_ATIX_point_sprites")) return GL_ATIX_point_sprites;
        if (!strcmp(name, "GL_ATTRIB_STACK_DEPTH")) return GL_ATTRIB_STACK_DEPTH;
        if (!strcmp(name, "GL_BINORMAL_ARRAY_EXT")) return GL_BINORMAL_ARRAY_EXT;
        if (!strcmp(name, "GL_BLEND_EQUATION_EXT")) return GL_BLEND_EQUATION_EXT;
        if (!strcmp(name, "GL_BLEND_EQUATION_RGB")) return GL_BLEND_EQUATION_RGB;
        if (!strcmp(name, "GL_BUFFER_MAP_POINTER")) return GL_BUFFER_MAP_POINTER;
        if (!strcmp(name, "GL_BUMP_TEX_UNITS_ATI")) return GL_BUMP_TEX_UNITS_ATI;
        if (!strcmp(name, "GL_CLAMP_TO_EDGE_SGIS")) return GL_CLAMP_TO_EDGE_SGIS;
        if (!strcmp(name, "GL_CLAMP_VERTEX_COLOR")) return GL_CLAMP_VERTEX_COLOR;
        if (!strcmp(name, "GL_CLIP_NEAR_HINT_PGI")) return GL_CLIP_NEAR_HINT_PGI;
        if (!strcmp(name, "GL_COLOR_ARRAY_STRIDE")) return GL_COLOR_ARRAY_STRIDE;
        if (!strcmp(name, "GL_COLOR_ATTACHMENT10")) return GL_COLOR_ATTACHMENT10;
        if (!strcmp(name, "GL_COLOR_ATTACHMENT11")) return GL_COLOR_ATTACHMENT11;
        if (!strcmp(name, "GL_COLOR_ATTACHMENT12")) return GL_COLOR_ATTACHMENT12;
        if (!strcmp(name, "GL_COLOR_ATTACHMENT13")) return GL_COLOR_ATTACHMENT13;
        if (!strcmp(name, "GL_COLOR_ATTACHMENT14")) return GL_COLOR_ATTACHMENT14;
        if (!strcmp(name, "GL_COLOR_ATTACHMENT15")) return GL_COLOR_ATTACHMENT15;
        if (!strcmp(name, "GL_COLOR_SUM_CLAMP_NV")) return GL_COLOR_SUM_CLAMP_NV;
        if (!strcmp(name, "GL_COLOR_TABLE_FORMAT")) return GL_COLOR_TABLE_FORMAT;
        if (!strcmp(name, "GL_COMPRESSED_RGB_ARB")) return GL_COMPRESSED_RGB_ARB;
        if (!strcmp(name, "GL_CONSTANT_ALPHA_EXT")) return GL_CONSTANT_ALPHA_EXT;
        if (!strcmp(name, "GL_CONSTANT_COLOR0_NV")) return GL_CONSTANT_COLOR0_NV;
        if (!strcmp(name, "GL_CONSTANT_COLOR1_NV")) return GL_CONSTANT_COLOR1_NV;
        if (!strcmp(name, "GL_CONSTANT_COLOR_EXT")) return GL_CONSTANT_COLOR_EXT;
        if (!strcmp(name, "GL_CONVOLUTION_1D_EXT")) return GL_CONVOLUTION_1D_EXT;
        if (!strcmp(name, "GL_CONVOLUTION_2D_EXT")) return GL_CONVOLUTION_2D_EXT;
        if (!strcmp(name, "GL_CONVOLUTION_FORMAT")) return GL_CONVOLUTION_FORMAT;
        if (!strcmp(name, "GL_CONVOLUTION_HEIGHT")) return GL_CONVOLUTION_HEIGHT;
        if (!strcmp(name, "GL_CURRENT_MATRIX_ARB")) return GL_CURRENT_MATRIX_ARB;
        if (!strcmp(name, "GL_CURRENT_VERTEX_EXT")) return GL_CURRENT_VERTEX_EXT;
        if (!strcmp(name, "GL_CURRENT_WEIGHT_ARB")) return GL_CURRENT_WEIGHT_ARB;
        if (!strcmp(name, "GL_DEPTH_COMPONENT32F")) return GL_DEPTH_COMPONENT32F;
        if (!strcmp(name, "GL_DEPTH_TEXTURE_MODE")) return GL_DEPTH_TEXTURE_MODE;
        if (!strcmp(name, "GL_EMBOSS_CONSTANT_NV")) return GL_EMBOSS_CONSTANT_NV;
        if (!strcmp(name, "GL_EXT_blend_logic_op")) return GL_EXT_blend_logic_op;
        if (!strcmp(name, "GL_EXT_blend_subtract")) return GL_EXT_blend_subtract;
        if (!strcmp(name, "GL_EXT_color_subtable")) return GL_EXT_color_subtable;
        if (!strcmp(name, "GL_EXT_draw_instanced")) return GL_EXT_draw_instanced;
        if (!strcmp(name, "GL_EXT_index_material")) return GL_EXT_index_material;
        if (!strcmp(name, "GL_EXT_misc_attribute")) return GL_EXT_misc_attribute;
        if (!strcmp(name, "GL_EXT_polygon_offset")) return GL_EXT_polygon_offset;
        if (!strcmp(name, "GL_EXT_rescale_normal")) return GL_EXT_rescale_normal;
        if (!strcmp(name, "GL_EXT_texture_object")) return GL_EXT_texture_object;
        if (!strcmp(name, "GL_FENCE_CONDITION_NV")) return GL_FENCE_CONDITION_NV;
        if (!strcmp(name, "GL_FLOAT_RGBA_MODE_NV")) return GL_FLOAT_RGBA_MODE_NV;
        if (!strcmp(name, "GL_FOG_COORDINATE_EXT")) return GL_FOG_COORDINATE_EXT;
        if (!strcmp(name, "GL_FRAGMENT_COLOR_EXT")) return GL_FRAGMENT_COLOR_EXT;
        if (!strcmp(name, "GL_FRAGMENT_DEPTH_EXT")) return GL_FRAGMENT_DEPTH_EXT;
        if (!strcmp(name, "GL_FRAGMENT_DEPTH_EXT")) return GL_FRAGMENT_DEPTH_EXT;
        if (!strcmp(name, "GL_HISTOGRAM_RED_SIZE")) return GL_HISTOGRAM_RED_SIZE;
        if (!strcmp(name, "GL_HISTOGRAM_SINK_EXT")) return GL_HISTOGRAM_SINK_EXT;
        if (!strcmp(name, "GL_HP_image_transform")) return GL_HP_image_transform;
        if (!strcmp(name, "GL_IBM_rasterpos_clip")) return GL_IBM_rasterpos_clip;
        if (!strcmp(name, "GL_INDEX_ARRAY_STRIDE")) return GL_INDEX_ARRAY_STRIDE;
        if (!strcmp(name, "GL_INTERLACE_READ_OML")) return GL_INTERLACE_READ_OML;
        if (!strcmp(name, "GL_INT_SAMPLER_1D_EXT")) return GL_INT_SAMPLER_1D_EXT;
        if (!strcmp(name, "GL_INT_SAMPLER_2D_EXT")) return GL_INT_SAMPLER_2D_EXT;
        if (!strcmp(name, "GL_INT_SAMPLER_3D_EXT")) return GL_INT_SAMPLER_3D_EXT;
        if (!strcmp(name, "GL_KTX_STENCIL_REGION")) return GL_KTX_STENCIL_REGION;
        if (!strcmp(name, "GL_LIGHT_ENV_MODE_EXT")) return GL_LIGHT_ENV_MODE_EXT;
        if (!strcmp(name, "GL_LINEAR_ATTENUATION")) return GL_LINEAR_ATTENUATION;
        if (!strcmp(name, "GL_LOCAL_CONSTANT_EXT")) return GL_LOCAL_CONSTANT_EXT;
        if (!strcmp(name, "GL_LUMINANCE12_ALPHA4")) return GL_LUMINANCE12_ALPHA4;
        if (!strcmp(name, "GL_MAGNITUDE_SCALE_NV")) return GL_MAGNITUDE_SCALE_NV;
        if (!strcmp(name, "GL_MAP1_GRID_SEGMENTS")) return GL_MAP1_GRID_SEGMENTS;
        if (!strcmp(name, "GL_MAP2_GRID_SEGMENTS")) return GL_MAP2_GRID_SEGMENTS;
        if (!strcmp(name, "GL_MATRIX_PALETTE_ARB")) return GL_MATRIX_PALETTE_ARB;
        if (!strcmp(name, "GL_MAX_CLIP_DISTANCES")) return GL_MAX_CLIP_DISTANCES;
        if (!strcmp(name, "GL_MAX_INTENSITY_SGIS")) return GL_MAX_INTENSITY_SGIS;
        if (!strcmp(name, "GL_MAX_LUMINANCE_SGIS")) return GL_MAX_LUMINANCE_SGIS;
        if (!strcmp(name, "GL_MAX_TEXTURE_COORDS")) return GL_MAX_TEXTURE_COORDS;
        if (!strcmp(name, "GL_MAX_VARYING_FLOATS")) return GL_MAX_VARYING_FLOATS;
        if (!strcmp(name, "GL_MAX_VERTEX_ATTRIBS")) return GL_MAX_VERTEX_ATTRIBS;
        if (!strcmp(name, "GL_MESA_ycbcr_texture")) return GL_MESA_ycbcr_texture;
        if (!strcmp(name, "GL_MIN_INTENSITY_SGIS")) return GL_MIN_INTENSITY_SGIS;
        if (!strcmp(name, "GL_MIN_LUMINANCE_SGIS")) return GL_MIN_LUMINANCE_SGIS;
        if (!strcmp(name, "GL_NV_occlusion_query")) return GL_NV_occlusion_query;
        if (!strcmp(name, "GL_NV_texture_shader2")) return GL_NV_texture_shader2;
        if (!strcmp(name, "GL_NV_texture_shader3")) return GL_NV_texture_shader3;
        if (!strcmp(name, "GL_NV_vertex_program2")) return GL_NV_vertex_program2;
        if (!strcmp(name, "GL_NV_vertex_program3")) return GL_NV_vertex_program3;
        if (!strcmp(name, "GL_NV_vertex_program4")) return GL_NV_vertex_program4;
        if (!strcmp(name, "GL_OBJECT_SUBTYPE_ARB")) return GL_OBJECT_SUBTYPE_ARB;
        if (!strcmp(name, "GL_OPERAND0_ALPHA_ARB")) return GL_OPERAND0_ALPHA_ARB;
        if (!strcmp(name, "GL_OPERAND0_ALPHA_EXT")) return GL_OPERAND0_ALPHA_EXT;
        if (!strcmp(name, "GL_OPERAND1_ALPHA_ARB")) return GL_OPERAND1_ALPHA_ARB;
        if (!strcmp(name, "GL_OPERAND1_ALPHA_EXT")) return GL_OPERAND1_ALPHA_EXT;
        if (!strcmp(name, "GL_OPERAND2_ALPHA_ARB")) return GL_OPERAND2_ALPHA_ARB;
        if (!strcmp(name, "GL_OPERAND2_ALPHA_EXT")) return GL_OPERAND2_ALPHA_EXT;
        if (!strcmp(name, "GL_PACK_CMYK_HINT_EXT")) return GL_PACK_CMYK_HINT_EXT;
        if (!strcmp(name, "GL_PACK_RESAMPLE_SGIX")) return GL_PACK_RESAMPLE_SGIX;
        if (!strcmp(name, "GL_PALETTE4_RGBA4_OES")) return GL_PALETTE4_RGBA4_OES;
        if (!strcmp(name, "GL_PALETTE4_RGBA8_OES")) return GL_PALETTE4_RGBA8_OES;
        if (!strcmp(name, "GL_PALETTE8_RGBA4_OES")) return GL_PALETTE8_RGBA4_OES;
        if (!strcmp(name, "GL_PALETTE8_RGBA8_OES")) return GL_PALETTE8_RGBA8_OES;
        if (!strcmp(name, "GL_PASS_THROUGH_TOKEN")) return GL_PASS_THROUGH_TOKEN;
        if (!strcmp(name, "GL_PERFMON_RESULT_AMD")) return GL_PERFMON_RESULT_AMD;
        if (!strcmp(name, "GL_POINT_SIZE_MAX_ARB")) return GL_POINT_SIZE_MAX_ARB;
        if (!strcmp(name, "GL_POINT_SIZE_MAX_EXT")) return GL_POINT_SIZE_MAX_EXT;
        if (!strcmp(name, "GL_POINT_SIZE_MIN_ARB")) return GL_POINT_SIZE_MIN_ARB;
        if (!strcmp(name, "GL_POINT_SIZE_MIN_EXT")) return GL_POINT_SIZE_MIN_EXT;
        if (!strcmp(name, "GL_POLYGON_OFFSET_EXT")) return GL_POLYGON_OFFSET_EXT;
        if (!strcmp(name, "GL_PROGRAM_FORMAT_ARB")) return GL_PROGRAM_FORMAT_ARB;
        if (!strcmp(name, "GL_PROGRAM_LENGTH_ARB")) return GL_PROGRAM_LENGTH_ARB;
        if (!strcmp(name, "GL_PROGRAM_MATRIX_EXT")) return GL_PROGRAM_MATRIX_EXT;
        if (!strcmp(name, "GL_PROGRAM_OBJECT_ARB")) return GL_PROGRAM_OBJECT_ARB;
        if (!strcmp(name, "GL_PROGRAM_STRING_ARB")) return GL_PROGRAM_STRING_ARB;
        if (!strcmp(name, "GL_QUERY_COUNTER_BITS")) return GL_QUERY_COUNTER_BITS;
        if (!strcmp(name, "GL_R11F_G11F_B10F_EXT")) return GL_R11F_G11F_B10F_EXT;
        if (!strcmp(name, "GL_RASTERIZER_DISCARD")) return GL_RASTERIZER_DISCARD;
        if (!strcmp(name, "GL_RED_MAX_CLAMP_INGR")) return GL_RED_MAX_CLAMP_INGR;
        if (!strcmp(name, "GL_RED_MIN_CLAMP_INGR")) return GL_RED_MIN_CLAMP_INGR;
        if (!strcmp(name, "GL_REFLECTION_MAP_ARB")) return GL_REFLECTION_MAP_ARB;
        if (!strcmp(name, "GL_REFLECTION_MAP_EXT")) return GL_REFLECTION_MAP_EXT;
        if (!strcmp(name, "GL_RENDERBUFFER_WIDTH")) return GL_RENDERBUFFER_WIDTH;
        if (!strcmp(name, "GL_REPLACE_MIDDLE_SUN")) return GL_REPLACE_MIDDLE_SUN;
        if (!strcmp(name, "GL_REPLACE_OLDEST_SUN")) return GL_REPLACE_OLDEST_SUN;
        if (!strcmp(name, "GL_RESCALE_NORMAL_EXT")) return GL_RESCALE_NORMAL_EXT;
        if (!strcmp(name, "GL_RGBA16_SIGNED_SGIX")) return GL_RGBA16_SIGNED_SGIX;
        if (!strcmp(name, "GL_RGBA_FLOAT16_APPLE")) return GL_RGBA_FLOAT16_APPLE;
        if (!strcmp(name, "GL_RGBA_FLOAT32_APPLE")) return GL_RGBA_FLOAT32_APPLE;
        if (!strcmp(name, "GL_SAMPLE_BUFFERS_ARB")) return GL_SAMPLE_BUFFERS_ARB;
        if (!strcmp(name, "GL_SAMPLE_BUFFERS_EXT")) return GL_SAMPLE_BUFFERS_EXT;
        if (!strcmp(name, "GL_SAMPLE_PATTERN_EXT")) return GL_SAMPLE_PATTERN_EXT;
        if (!strcmp(name, "GL_SAMPLE_POSITION_NV")) return GL_SAMPLE_POSITION_NV;
        if (!strcmp(name, "GL_SAMPLER_BUFFER_EXT")) return GL_SAMPLER_BUFFER_EXT;
        if (!strcmp(name, "GL_SAMPLES_PASSED_ARB")) return GL_SAMPLES_PASSED_ARB;
        if (!strcmp(name, "GL_SECONDARY_COLOR_NV")) return GL_SECONDARY_COLOR_NV;
        if (!strcmp(name, "GL_SGIS_pixel_texture")) return GL_SGIS_pixel_texture;
        if (!strcmp(name, "GL_SGIX_depth_texture")) return GL_SGIX_depth_texture;
        if (!strcmp(name, "GL_SGIX_list_priority")) return GL_SGIX_list_priority;
        if (!strcmp(name, "GL_SGIX_pixel_texture")) return GL_SGIX_pixel_texture;
        if (!strcmp(name, "GL_SGIX_texture_range")) return GL_SGIX_texture_range;
        if (!strcmp(name, "GL_SIGNED_IDENTITY_NV")) return GL_SIGNED_IDENTITY_NV;
        if (!strcmp(name, "GL_SLUMINANCE8_ALPHA8")) return GL_SLUMINANCE8_ALPHA8;
        if (!strcmp(name, "GL_SRC_ALPHA_SATURATE")) return GL_SRC_ALPHA_SATURATE;
        if (!strcmp(name, "GL_STENCIL_ATTACHMENT")) return GL_STENCIL_ATTACHMENT;
        if (!strcmp(name, "GL_STENCIL_BUFFER_BIT")) return GL_STENCIL_BUFFER_BIT;
        if (!strcmp(name, "GL_STENCIL_INDEX1_EXT")) return GL_STENCIL_INDEX1_EXT;
        if (!strcmp(name, "GL_STENCIL_INDEX4_EXT")) return GL_STENCIL_INDEX4_EXT;
        if (!strcmp(name, "GL_STENCIL_INDEX8_EXT")) return GL_STENCIL_INDEX8_EXT;
        if (!strcmp(name, "GL_STENCIL_VALUE_MASK")) return GL_STENCIL_VALUE_MASK;
        if (!strcmp(name, "GL_SUNX_constant_data")) return GL_SUNX_constant_data;
        if (!strcmp(name, "GL_SWIZZLE_STQ_DQ_ATI")) return GL_SWIZZLE_STQ_DQ_ATI;
        if (!strcmp(name, "GL_SWIZZLE_STR_DR_ATI")) return GL_SWIZZLE_STR_DR_ATI;
        if (!strcmp(name, "GL_TEXTURE_ALPHA_SIZE")) return GL_TEXTURE_ALPHA_SIZE;
        if (!strcmp(name, "GL_TEXTURE_ALPHA_TYPE")) return GL_TEXTURE_ALPHA_TYPE;
        if (!strcmp(name, "GL_TEXTURE_BASE_LEVEL")) return GL_TEXTURE_BASE_LEVEL;
        if (!strcmp(name, "GL_TEXTURE_BINDING_1D")) return GL_TEXTURE_BINDING_1D;
        if (!strcmp(name, "GL_TEXTURE_BINDING_2D")) return GL_TEXTURE_BINDING_2D;
        if (!strcmp(name, "GL_TEXTURE_BINDING_3D")) return GL_TEXTURE_BINDING_3D;
        if (!strcmp(name, "GL_TEXTURE_BUFFER_ARB")) return GL_TEXTURE_BUFFER_ARB;
        if (!strcmp(name, "GL_TEXTURE_BUFFER_EXT")) return GL_TEXTURE_BUFFER_EXT;
        if (!strcmp(name, "GL_TEXTURE_COMPONENTS")) return GL_TEXTURE_COMPONENTS;
        if (!strcmp(name, "GL_TEXTURE_COMPRESSED")) return GL_TEXTURE_COMPRESSED;
        if (!strcmp(name, "GL_TEXTURE_DEPTH_SIZE")) return GL_TEXTURE_DEPTH_SIZE;
        if (!strcmp(name, "GL_TEXTURE_DEPTH_TYPE")) return GL_TEXTURE_DEPTH_TYPE;
        if (!strcmp(name, "GL_TEXTURE_DS_SIZE_NV")) return GL_TEXTURE_DS_SIZE_NV;
        if (!strcmp(name, "GL_TEXTURE_DT_SIZE_NV")) return GL_TEXTURE_DT_SIZE_NV;
        if (!strcmp(name, "GL_TEXTURE_GREEN_SIZE")) return GL_TEXTURE_GREEN_SIZE;
        if (!strcmp(name, "GL_TEXTURE_GREEN_TYPE")) return GL_TEXTURE_GREEN_TYPE;
        if (!strcmp(name, "GL_TEXTURE_HI_SIZE_NV")) return GL_TEXTURE_HI_SIZE_NV;
        if (!strcmp(name, "GL_TEXTURE_LO_SIZE_NV")) return GL_TEXTURE_LO_SIZE_NV;
        if (!strcmp(name, "GL_TEXTURE_MAG_FILTER")) return GL_TEXTURE_MAG_FILTER;
        if (!strcmp(name, "GL_TEXTURE_MIN_FILTER")) return GL_TEXTURE_MIN_FILTER;
        if (!strcmp(name, "GL_TEXTURE_NORMAL_EXT")) return GL_TEXTURE_NORMAL_EXT;
        if (!strcmp(name, "GL_TEXTURE_WRAP_R_EXT")) return GL_TEXTURE_WRAP_R_EXT;
        if (!strcmp(name, "GL_UNIFORM_BUFFER_EXT")) return GL_UNIFORM_BUFFER_EXT;
        if (!strcmp(name, "GL_UNPACK_SKIP_IMAGES")) return GL_UNPACK_SKIP_IMAGES;
        if (!strcmp(name, "GL_UNPACK_SKIP_PIXELS")) return GL_UNPACK_SKIP_PIXELS;
        if (!strcmp(name, "GL_UNSIGNED_INT64_AMD")) return GL_UNSIGNED_INT64_AMD;
        if (!strcmp(name, "GL_UNSIGNED_INVERT_NV")) return GL_UNSIGNED_INVERT_NV;
        if (!strcmp(name, "GL_VERTEX_PROGRAM_ARB")) return GL_VERTEX_PROGRAM_ARB;
        if (!strcmp(name, "GL_VERTEX_STREAM0_ATI")) return GL_VERTEX_STREAM0_ATI;
        if (!strcmp(name, "GL_VERTEX_STREAM1_ATI")) return GL_VERTEX_STREAM1_ATI;
        if (!strcmp(name, "GL_VERTEX_STREAM2_ATI")) return GL_VERTEX_STREAM2_ATI;
        if (!strcmp(name, "GL_VERTEX_STREAM3_ATI")) return GL_VERTEX_STREAM3_ATI;
        if (!strcmp(name, "GL_VERTEX_STREAM4_ATI")) return GL_VERTEX_STREAM4_ATI;
        if (!strcmp(name, "GL_VERTEX_STREAM5_ATI")) return GL_VERTEX_STREAM5_ATI;
        if (!strcmp(name, "GL_VERTEX_STREAM6_ATI")) return GL_VERTEX_STREAM6_ATI;
        if (!strcmp(name, "GL_VERTEX_STREAM7_ATI")) return GL_VERTEX_STREAM7_ATI;
        if (!strcmp(name, "GL_WIDE_LINE_HINT_PGI")) return GL_WIDE_LINE_HINT_PGI;
        return _GL_UNDEFINED;
    case 22:
        if (!strcmp(name, "GL_422_REV_AVERAGE_EXT")) return GL_422_REV_AVERAGE_EXT;
        if (!strcmp(name, "GL_ALL_STATIC_DATA_IBM")) return GL_ALL_STATIC_DATA_IBM;
        if (!strcmp(name, "GL_ALPHA16_SIGNED_SGIX")) return GL_ALPHA16_SIGNED_SGIX;
        if (!strcmp(name, "GL_ALPHA_FLOAT16_APPLE")) return GL_ALPHA_FLOAT16_APPLE;
        if (!strcmp(name, "GL_ALPHA_FLOAT32_APPLE")) return GL_ALPHA_FLOAT32_APPLE;
        if (!strcmp(name, "GL_AMBIENT_AND_DIFFUSE")) return GL_AMBIENT_AND_DIFFUSE;
        if (!strcmp(name, "GL_APPLE_element_array")) return GL_APPLE_element_array;
        if (!strcmp(name, "GL_APPLE_texture_range")) return GL_APPLE_texture_range;
        if (!strcmp(name, "GL_ARB_fragment_shader")) return GL_ARB_fragment_shader;
        if (!strcmp(name, "GL_ARB_occlusion_query")) return GL_ARB_occlusion_query;
        if (!strcmp(name, "GL_ARB_texture_env_add")) return GL_ARB_texture_env_add;
        if (!strcmp(name, "GL_ATI_fragment_shader")) return GL_ATI_fragment_shader;
        if (!strcmp(name, "GL_BLEND_DST_ALPHA_EXT")) return GL_BLEND_DST_ALPHA_EXT;
        if (!strcmp(name, "GL_BLEND_SRC_ALPHA_EXT")) return GL_BLEND_SRC_ALPHA_EXT;
        if (!strcmp(name, "GL_BLUE_MAX_CLAMP_INGR")) return GL_BLUE_MAX_CLAMP_INGR;
        if (!strcmp(name, "GL_BLUE_MIN_CLAMP_INGR")) return GL_BLUE_MIN_CLAMP_INGR;
        if (!strcmp(name, "GL_BUMP_ROT_MATRIX_ATI")) return GL_BUMP_ROT_MATRIX_ATI;
        if (!strcmp(name, "GL_CLAMP_TO_BORDER_ARB")) return GL_CLAMP_TO_BORDER_ARB;
        if (!strcmp(name, "GL_COLOR_ARRAY_POINTER")) return GL_COLOR_ARRAY_POINTER;
        if (!strcmp(name, "GL_COLOR_MATERIAL_FACE")) return GL_COLOR_MATERIAL_FACE;
        if (!strcmp(name, "GL_COMBINER_MAPPING_NV")) return GL_COMBINER_MAPPING_NV;
        if (!strcmp(name, "GL_COMBINER_MUX_SUM_NV")) return GL_COMBINER_MUX_SUM_NV;
        if (!strcmp(name, "GL_COMPILE_AND_EXECUTE")) return GL_COMPILE_AND_EXECUTE;
        if (!strcmp(name, "GL_COMPRESSED_RGBA_ARB")) return GL_COMPRESSED_RGBA_ARB;
        if (!strcmp(name, "GL_COMPRESSED_RG_RGTC2")) return GL_COMPRESSED_RG_RGTC2;
        if (!strcmp(name, "GL_COMPRESSED_SRGB_EXT")) return GL_COMPRESSED_SRGB_EXT;
        if (!strcmp(name, "GL_CURRENT_TANGENT_EXT")) return GL_CURRENT_TANGENT_EXT;
        if (!strcmp(name, "GL_EDGE_FLAG_ARRAY_EXT")) return GL_EDGE_FLAG_ARRAY_EXT;
        if (!strcmp(name, "GL_ELEMENT_ARRAY_APPLE")) return GL_ELEMENT_ARRAY_APPLE;
        if (!strcmp(name, "GL_EXTENDED_RANGE_SGIS")) return GL_EXTENDED_RANGE_SGIS;
        if (!strcmp(name, "GL_EXT_pixel_transform")) return GL_EXT_pixel_transform;
        if (!strcmp(name, "GL_EXT_secondary_color")) return GL_EXT_secondary_color;
        if (!strcmp(name, "GL_EXT_texture_env_add")) return GL_EXT_texture_env_add;
        if (!strcmp(name, "GL_EXT_texture_integer")) return GL_EXT_texture_integer;
        if (!strcmp(name, "GL_EXT_texture_swizzle")) return GL_EXT_texture_swizzle;
        if (!strcmp(name, "GL_FRAGMENT_LIGHT0_EXT")) return GL_FRAGMENT_LIGHT0_EXT;
        if (!strcmp(name, "GL_FRAGMENT_LIGHT7_EXT")) return GL_FRAGMENT_LIGHT7_EXT;
        if (!strcmp(name, "GL_FRAGMENT_NORMAL_EXT")) return GL_FRAGMENT_NORMAL_EXT;
        if (!strcmp(name, "GL_FRAGMENT_PROGRAM_NV")) return GL_FRAGMENT_PROGRAM_NV;
        if (!strcmp(name, "GL_FRAGMENT_SHADER_ARB")) return GL_FRAGMENT_SHADER_ARB;
        if (!strcmp(name, "GL_FRAGMENT_SHADER_ATI")) return GL_FRAGMENT_SHADER_ATI;
        if (!strcmp(name, "GL_FRAMEBUFFER_BINDING")) return GL_FRAMEBUFFER_BINDING;
        if (!strcmp(name, "GL_FRAMEBUFFER_DEFAULT")) return GL_FRAMEBUFFER_DEFAULT;
        if (!strcmp(name, "GL_GEOMETRY_PROGRAM_NV")) return GL_GEOMETRY_PROGRAM_NV;
        if (!strcmp(name, "GL_GEOMETRY_SHADER_ARB")) return GL_GEOMETRY_SHADER_ARB;
        if (!strcmp(name, "GL_GEOMETRY_SHADER_EXT")) return GL_GEOMETRY_SHADER_EXT;
        if (!strcmp(name, "GL_HALF_BIAS_NEGATE_NV")) return GL_HALF_BIAS_NEGATE_NV;
        if (!strcmp(name, "GL_HALF_BIAS_NORMAL_NV")) return GL_HALF_BIAS_NORMAL_NV;
        if (!strcmp(name, "GL_HISTOGRAM_BLUE_SIZE")) return GL_HISTOGRAM_BLUE_SIZE;
        if (!strcmp(name, "GL_HISTOGRAM_WIDTH_EXT")) return GL_HISTOGRAM_WIDTH_EXT;
        if (!strcmp(name, "GL_HP_texture_lighting")) return GL_HP_texture_lighting;
        if (!strcmp(name, "GL_INDEX_ARRAY_POINTER")) return GL_INDEX_ARRAY_POINTER;
        if (!strcmp(name, "GL_INGR_interlace_read")) return GL_INGR_interlace_read;
        if (!strcmp(name, "GL_INTERLACE_READ_INGR")) return GL_INTERLACE_READ_INGR;
        if (!strcmp(name, "GL_INTERLEAVED_ATTRIBS")) return GL_INTERLEAVED_ATTRIBS;
        if (!strcmp(name, "GL_INVARIANT_VALUE_EXT")) return GL_INVARIANT_VALUE_EXT;
        if (!strcmp(name, "GL_LIGHT_MODEL_AMBIENT")) return GL_LIGHT_MODEL_AMBIENT;
        if (!strcmp(name, "GL_LINES_ADJACENCY_ARB")) return GL_LINES_ADJACENCY_ARB;
        if (!strcmp(name, "GL_LINES_ADJACENCY_EXT")) return GL_LINES_ADJACENCY_EXT;
        if (!strcmp(name, "GL_LINE_STIPPLE_REPEAT")) return GL_LINE_STIPPLE_REPEAT;
        if (!strcmp(name, "GL_LUMINANCE12_ALPHA12")) return GL_LUMINANCE12_ALPHA12;
        if (!strcmp(name, "GL_LUMINANCE16_ALPHA16")) return GL_LUMINANCE16_ALPHA16;
        if (!strcmp(name, "GL_MAP_TESSELLATION_NV")) return GL_MAP_TESSELLATION_NV;
        if (!strcmp(name, "GL_MAT_AMBIENT_BIT_PGI")) return GL_MAT_AMBIENT_BIT_PGI;
        if (!strcmp(name, "GL_MAT_DIFFUSE_BIT_PGI")) return GL_MAT_DIFFUSE_BIT_PGI;
        if (!strcmp(name, "GL_MAX_3D_TEXTURE_SIZE")) return GL_MAX_3D_TEXTURE_SIZE;
        if (!strcmp(name, "GL_MAX_PIXEL_MAP_TABLE")) return GL_MAX_PIXEL_MAP_TABLE;
        if (!strcmp(name, "GL_MAX_VERTEX_HINT_PGI")) return GL_MAX_VERTEX_HINT_PGI;
        if (!strcmp(name, "GL_MESA_resize_buffers")) return GL_MESA_resize_buffers;
        if (!strcmp(name, "GL_MESAX_texture_stack")) return GL_MESAX_texture_stack;
        if (!strcmp(name, "GL_MIRRORED_REPEAT_ARB")) return GL_MIRRORED_REPEAT_ARB;
        if (!strcmp(name, "GL_MIRRORED_REPEAT_IBM")) return GL_MIRRORED_REPEAT_IBM;
        if (!strcmp(name, "GL_MULTISAMPLE_BIT_ARB")) return GL_MULTISAMPLE_BIT_ARB;
        if (!strcmp(name, "GL_MULTISAMPLE_BIT_EXT")) return GL_MULTISAMPLE_BIT_EXT;
        if (!strcmp(name, "GL_MULTISAMPLE_BIT_EXT")) return GL_MULTISAMPLE_BIT_EXT;
        if (!strcmp(name, "GL_NORMAL_ARRAY_STRIDE")) return GL_NORMAL_ARRAY_STRIDE;
        if (!strcmp(name, "GL_NUM_FILL_STREAMS_NV")) return GL_NUM_FILL_STREAMS_NV;
        if (!strcmp(name, "GL_NV_fragment_program")) return GL_NV_fragment_program;
        if (!strcmp(name, "GL_NV_geometry_shader4")) return GL_NV_geometry_shader4;
        if (!strcmp(name, "GL_NV_pixel_data_range")) return GL_NV_pixel_data_range;
        if (!strcmp(name, "GL_ONE_MINUS_DST_ALPHA")) return GL_ONE_MINUS_DST_ALPHA;
        if (!strcmp(name, "GL_ONE_MINUS_DST_COLOR")) return GL_ONE_MINUS_DST_COLOR;
        if (!strcmp(name, "GL_ONE_MINUS_SRC_ALPHA")) return GL_ONE_MINUS_SRC_ALPHA;
        if (!strcmp(name, "GL_ONE_MINUS_SRC_COLOR")) return GL_ONE_MINUS_SRC_COLOR;
        if (!strcmp(name, "GL_PIXEL_UNPACK_BUFFER")) return GL_PIXEL_UNPACK_BUFFER;
        if (!strcmp(name, "GL_POLYGON_OFFSET_FILL")) return GL_POLYGON_OFFSET_FILL;
        if (!strcmp(name, "GL_POLYGON_OFFSET_LINE")) return GL_POLYGON_OFFSET_LINE;
        if (!strcmp(name, "GL_POLYGON_SMOOTH_HINT")) return GL_POLYGON_SMOOTH_HINT;
        if (!strcmp(name, "GL_POLYGON_STIPPLE_BIT")) return GL_POLYGON_STIPPLE_BIT;
        if (!strcmp(name, "GL_PRESENT_DURATION_NV")) return GL_PRESENT_DURATION_NV;
        if (!strcmp(name, "GL_PROGRAM_ATTRIBS_ARB")) return GL_PROGRAM_ATTRIBS_ARB;
        if (!strcmp(name, "GL_PROGRAM_BINDING_ARB")) return GL_PROGRAM_BINDING_ARB;
        if (!strcmp(name, "GL_PROGRAM_RESIDENT_NV")) return GL_PROGRAM_RESIDENT_NV;
        if (!strcmp(name, "GL_PROXY_HISTOGRAM_EXT")) return GL_PROXY_HISTOGRAM_EXT;
        if (!strcmp(name, "GL_RENDERBUFFER_HEIGHT")) return GL_RENDERBUFFER_HEIGHT;
        if (!strcmp(name, "GL_RGBA_FLOAT_MODE_ARB")) return GL_RGBA_FLOAT_MODE_ARB;
        if (!strcmp(name, "GL_SAMPLE_ALPHA_TO_ONE")) return GL_SAMPLE_ALPHA_TO_ONE;
        if (!strcmp(name, "GL_SAMPLE_BUFFERS_3DFX")) return GL_SAMPLE_BUFFERS_3DFX;
        if (!strcmp(name, "GL_SAMPLE_BUFFERS_SGIS")) return GL_SAMPLE_BUFFERS_SGIS;
        if (!strcmp(name, "GL_SAMPLE_COVERAGE_ARB")) return GL_SAMPLE_COVERAGE_ARB;
        if (!strcmp(name, "GL_SAMPLE_PATTERN_SGIS")) return GL_SAMPLE_PATTERN_SGIS;
        if (!strcmp(name, "GL_SAMPLER_2D_RECT_ARB")) return GL_SAMPLER_2D_RECT_ARB;
        if (!strcmp(name, "GL_SAMPLER_2D_RECT_ARB")) return GL_SAMPLER_2D_RECT_ARB;
        if (!strcmp(name, "GL_SAMPLER_CUBE_SHADOW")) return GL_SAMPLER_CUBE_SHADOW;
        if (!strcmp(name, "GL_SEPARATE_ATTRIBS_NV")) return GL_SEPARATE_ATTRIBS_NV;
        if (!strcmp(name, "GL_SGIS_detail_texture")) return GL_SGIS_detail_texture;
        if (!strcmp(name, "GL_SGIS_texture_select")) return GL_SGIS_texture_select;
        if (!strcmp(name, "GL_SGIX_ir_instrument1")) return GL_SGIX_ir_instrument1;
        if (!strcmp(name, "GL_SGIX_shadow_ambient")) return GL_SGIX_shadow_ambient;
        if (!strcmp(name, "GL_SGIX_vertex_preclip")) return GL_SGIX_vertex_preclip;
        if (!strcmp(name, "GL_SHADER_OPERATION_NV")) return GL_SHADER_OPERATION_NV;
        if (!strcmp(name, "GL_SHADOW_AMBIENT_SGIX")) return GL_SHADOW_AMBIENT_SGIX;
        if (!strcmp(name, "GL_SIGNED_INTENSITY_NV")) return GL_SIGNED_INTENSITY_NV;
        if (!strcmp(name, "GL_SIGNED_INTENSITY_NV")) return GL_SIGNED_INTENSITY_NV;
        if (!strcmp(name, "GL_SIGNED_LUMINANCE_NV")) return GL_SIGNED_LUMINANCE_NV;
        if (!strcmp(name, "GL_SIGNED_LUMINANCE_NV")) return GL_SIGNED_LUMINANCE_NV;
        if (!strcmp(name, "GL_STENCIL_CLEAR_VALUE")) return GL_STENCIL_CLEAR_VALUE;
        if (!strcmp(name, "GL_STENCIL_INDEX16_EXT")) return GL_STENCIL_INDEX16_EXT;
        if (!strcmp(name, "GL_SWIZZLE_STRQ_DQ_ATI")) return GL_SWIZZLE_STRQ_DQ_ATI;
        if (!strcmp(name, "GL_TEXTURE_COORD_ARRAY")) return GL_TEXTURE_COORD_ARRAY;
        if (!strcmp(name, "GL_TEXTURE_MAG_SIZE_NV")) return GL_TEXTURE_MAG_SIZE_NV;
        if (!strcmp(name, "GL_TEXTURE_SHARED_SIZE")) return GL_TEXTURE_SHARED_SIZE;
        if (!strcmp(name, "GL_TEXTURE_STACK_DEPTH")) return GL_TEXTURE_STACK_DEPTH;
        if (!strcmp(name, "GL_UNPACK_IMAGE_HEIGHT")) return GL_UNPACK_IMAGE_HEIGHT;
        if (!strcmp(name, "GL_UNPACK_RESAMPLE_OML")) return GL_UNPACK_RESAMPLE_OML;
        if (!strcmp(name, "GL_UNSIGNED_BYTE_3_3_2")) return GL_UNSIGNED_BYTE_3_3_2;
        if (!strcmp(name, "GL_UNSIGNED_NORMALIZED")) return GL_UNSIGNED_NORMALIZED;
        if (!strcmp(name, "GL_UNSIGNED_NORMALIZED")) return GL_UNSIGNED_NORMALIZED;
        if (!strcmp(name, "GL_VBO_FREE_MEMORY_ATI")) return GL_VBO_FREE_MEMORY_ATI;
        if (!strcmp(name, "GL_VERTEX_ARRAY_STRIDE")) return GL_VERTEX_ARRAY_STRIDE;
        if (!strcmp(name, "GL_VERTEX_PRECLIP_SGIX")) return GL_VERTEX_PRECLIP_SGIX;
        if (!strcmp(name, "GL_VERTEX_PRECLIP_SGIX")) return GL_VERTEX_PRECLIP_SGIX;
        return _GL_UNDEFINED;
    case 23:
        if (!strcmp(name, "GL_ALPHA_MAX_CLAMP_INGR")) return GL_ALPHA_MAX_CLAMP_INGR;
        if (!strcmp(name, "GL_ALPHA_MIN_CLAMP_INGR")) return GL_ALPHA_MIN_CLAMP_INGR;
        if (!strcmp(name, "GL_ALWAYS_FAST_HINT_PGI")) return GL_ALWAYS_FAST_HINT_PGI;
        if (!strcmp(name, "GL_ALWAYS_SOFT_HINT_PGI")) return GL_ALWAYS_SOFT_HINT_PGI;
        if (!strcmp(name, "GL_APPLE_client_storage")) return GL_APPLE_client_storage;
        if (!strcmp(name, "GL_APPLE_transform_hint")) return GL_APPLE_transform_hint;
        if (!strcmp(name, "GL_ARB_fragment_program")) return GL_ARB_fragment_program;
        if (!strcmp(name, "GL_ARB_framebuffer_sRGB")) return GL_ARB_framebuffer_sRGB;
        if (!strcmp(name, "GL_ARB_geometry_shader4")) return GL_ARB_geometry_shader4;
        if (!strcmp(name, "GL_ARB_half_float_pixel")) return GL_ARB_half_float_pixel;
        if (!strcmp(name, "GL_ARB_instanced_arrays")) return GL_ARB_instanced_arrays;
        if (!strcmp(name, "GL_ARB_map_buffer_range")) return GL_ARB_map_buffer_range;
        if (!strcmp(name, "GL_ARB_point_parameters")) return GL_ARB_point_parameters;
        if (!strcmp(name, "GL_ARB_texture_cube_map")) return GL_ARB_texture_cube_map;
        if (!strcmp(name, "GL_ARB_texture_env_dot3")) return GL_ARB_texture_env_dot3;
        if (!strcmp(name, "GL_ARB_transpose_matrix")) return GL_ARB_transpose_matrix;
        if (!strcmp(name, "GL_ARRAY_BUFFER_BINDING")) return GL_ARRAY_BUFFER_BINDING;
        if (!strcmp(name, "GL_ASYNC_HISTOGRAM_SGIX")) return GL_ASYNC_HISTOGRAM_SGIX;
        if (!strcmp(name, "GL_ASYNC_TEX_IMAGE_SGIX")) return GL_ASYNC_TEX_IMAGE_SGIX;
        if (!strcmp(name, "GL_ATI_separate_stencil")) return GL_ATI_separate_stencil;
        if (!strcmp(name, "GL_ATTRIB_ARRAY_SIZE_NV")) return GL_ATTRIB_ARRAY_SIZE_NV;
        if (!strcmp(name, "GL_ATTRIB_ARRAY_TYPE_NV")) return GL_ATTRIB_ARRAY_TYPE_NV;
        if (!strcmp(name, "GL_BLEND_EQUATION_ALPHA")) return GL_BLEND_EQUATION_ALPHA;
        if (!strcmp(name, "GL_CG_VERTEX_SHADER_EXT")) return GL_CG_VERTEX_SHADER_EXT;
        if (!strcmp(name, "GL_CLAMP_FRAGMENT_COLOR")) return GL_CLAMP_FRAGMENT_COLOR;
        if (!strcmp(name, "GL_CLAMP_READ_COLOR_ARB")) return GL_CLAMP_READ_COLOR_ARB;
        if (!strcmp(name, "GL_CLAMP_TO_BORDER_SGIS")) return GL_CLAMP_TO_BORDER_SGIS;
        if (!strcmp(name, "GL_COLOR_ARRAY_LIST_IBM")) return GL_COLOR_ARRAY_LIST_IBM;
        if (!strcmp(name, "GL_COLOR_ARRAY_SIZE_EXT")) return GL_COLOR_ARRAY_SIZE_EXT;
        if (!strcmp(name, "GL_COLOR_ARRAY_TYPE_EXT")) return GL_COLOR_ARRAY_TYPE_EXT;
        if (!strcmp(name, "GL_COLOR_TABLE_BIAS_SGI")) return GL_COLOR_TABLE_BIAS_SGI;
        if (!strcmp(name, "GL_COLOR_TABLE_RED_SIZE")) return GL_COLOR_TABLE_RED_SIZE;
        if (!strcmp(name, "GL_COMPARE_R_TO_TEXTURE")) return GL_COMPARE_R_TO_TEXTURE;
        if (!strcmp(name, "GL_COMPRESSED_ALPHA_ARB")) return GL_COMPRESSED_ALPHA_ARB;
        if (!strcmp(name, "GL_COMPRESSED_INTENSITY")) return GL_COMPRESSED_INTENSITY;
        if (!strcmp(name, "GL_COMPRESSED_LUMINANCE")) return GL_COMPRESSED_LUMINANCE;
        if (!strcmp(name, "GL_COMPRESSED_RED_RGTC1")) return GL_COMPRESSED_RED_RGTC1;
        if (!strcmp(name, "GL_CONSTANT_ATTENUATION")) return GL_CONSTANT_ATTENUATION;
        if (!strcmp(name, "GL_CURRENT_BINORMAL_EXT")) return GL_CURRENT_BINORMAL_EXT;
        if (!strcmp(name, "GL_CURRENT_RASTER_COLOR")) return GL_CURRENT_RASTER_COLOR;
        if (!strcmp(name, "GL_CURRENT_RASTER_INDEX")) return GL_CURRENT_RASTER_INDEX;
        if (!strcmp(name, "GL_DEPTH24_STENCIL8_EXT")) return GL_DEPTH24_STENCIL8_EXT;
        if (!strcmp(name, "GL_DEPTH32F_STENCIL8_NV")) return GL_DEPTH32F_STENCIL8_NV;
        if (!strcmp(name, "GL_DEPTH_ATTACHMENT_EXT")) return GL_DEPTH_ATTACHMENT_EXT;
        if (!strcmp(name, "GL_DRAW_FRAMEBUFFER_EXT")) return GL_DRAW_FRAMEBUFFER_EXT;
        if (!strcmp(name, "GL_ELEMENT_ARRAY_BUFFER")) return GL_ELEMENT_ARRAY_BUFFER;
        if (!strcmp(name, "GL_EXT_bindable_uniform")) return GL_EXT_bindable_uniform;
        if (!strcmp(name, "GL_EXT_clip_volume_hint")) return GL_EXT_clip_volume_hint;
        if (!strcmp(name, "GL_EXT_coordinate_frame")) return GL_EXT_coordinate_frame;
        if (!strcmp(name, "GL_EXT_framebuffer_blit")) return GL_EXT_framebuffer_blit;
        if (!strcmp(name, "GL_EXT_framebuffer_sRGB")) return GL_EXT_framebuffer_sRGB;
        if (!strcmp(name, "GL_EXT_geometry_shader4")) return GL_EXT_geometry_shader4;
        if (!strcmp(name, "GL_EXT_paletted_texture")) return GL_EXT_paletted_texture;
        if (!strcmp(name, "GL_EXT_point_parameters")) return GL_EXT_point_parameters;
        if (!strcmp(name, "GL_EXT_stencil_two_side")) return GL_EXT_stencil_two_side;
        if (!strcmp(name, "GL_EXT_texture_cube_map")) return GL_EXT_texture_cube_map;
        if (!strcmp(name, "GL_EXT_texture_env_dot3")) return GL_EXT_texture_env_dot3;
        if (!strcmp(name, "GL_EXT_texture_lod_bias")) return GL_EXT_texture_lod_bias;
        if (!strcmp(name, "GL_EXT_vertex_weighting")) return GL_EXT_vertex_weighting;
        if (!strcmp(name, "GL_FEEDBACK_BUFFER_SIZE")) return GL_FEEDBACK_BUFFER_SIZE;
        if (!strcmp(name, "GL_FEEDBACK_BUFFER_TYPE")) return GL_FEEDBACK_BUFFER_TYPE;
        if (!strcmp(name, "GL_FOG_COORD_ARRAY_TYPE")) return GL_FOG_COORD_ARRAY_TYPE;
        if (!strcmp(name, "GL_FOG_COORDINATE_ARRAY")) return GL_FOG_COORDINATE_ARRAY;
        if (!strcmp(name, "GL_FOG_DISTANCE_MODE_NV")) return GL_FOG_DISTANCE_MODE_NV;
        if (!strcmp(name, "GL_FORCE_BLUE_TO_ONE_NV")) return GL_FORCE_BLUE_TO_ONE_NV;
        if (!strcmp(name, "GL_FRAGMENT_PROGRAM_ARB")) return GL_FRAGMENT_PROGRAM_ARB;
        if (!strcmp(name, "GL_FRAMEBUFFER_COMPLETE")) return GL_FRAMEBUFFER_COMPLETE;
        if (!strcmp(name, "GL_FRAMEBUFFER_SRGB_EXT")) return GL_FRAMEBUFFER_SRGB_EXT;
        if (!strcmp(name, "GL_GENERATE_MIPMAP_HINT")) return GL_GENERATE_MIPMAP_HINT;
        if (!strcmp(name, "GL_GENERATE_MIPMAP_SGIS")) return GL_GENERATE_MIPMAP_SGIS;
        if (!strcmp(name, "GL_GREEN_MAX_CLAMP_INGR")) return GL_GREEN_MAX_CLAMP_INGR;
        if (!strcmp(name, "GL_GREEN_MIN_CLAMP_INGR")) return GL_GREEN_MIN_CLAMP_INGR;
        if (!strcmp(name, "GL_HISTOGRAM_ALPHA_SIZE")) return GL_HISTOGRAM_ALPHA_SIZE;
        if (!strcmp(name, "GL_HISTOGRAM_FORMAT_EXT")) return GL_HISTOGRAM_FORMAT_EXT;
        if (!strcmp(name, "GL_HISTOGRAM_GREEN_SIZE")) return GL_HISTOGRAM_GREEN_SIZE;
        if (!strcmp(name, "GL_INDEX_ARRAY_LIST_IBM")) return GL_INDEX_ARRAY_LIST_IBM;
        if (!strcmp(name, "GL_INDEX_ARRAY_TYPE_EXT")) return GL_INDEX_ARRAY_TYPE_EXT;
        if (!strcmp(name, "GL_INT_SAMPLER_1D_ARRAY")) return GL_INT_SAMPLER_1D_ARRAY;
        if (!strcmp(name, "GL_INT_SAMPLER_2D_ARRAY")) return GL_INT_SAMPLER_2D_ARRAY;
        if (!strcmp(name, "GL_INT_SAMPLER_CUBE_EXT")) return GL_INT_SAMPLER_CUBE_EXT;
        if (!strcmp(name, "GL_INVERSE_TRANSPOSE_NV")) return GL_INVERSE_TRANSPOSE_NV;
        if (!strcmp(name, "GL_LIGHT_MODEL_TWO_SIDE")) return GL_LIGHT_MODEL_TWO_SIDE;
        if (!strcmp(name, "GL_LINEAR_MIPMAP_LINEAR")) return GL_LINEAR_MIPMAP_LINEAR;
        if (!strcmp(name, "GL_LINE_STIPPLE_PATTERN")) return GL_LINE_STIPPLE_PATTERN;
        if (!strcmp(name, "GL_MAP1_TEXTURE_COORD_1")) return GL_MAP1_TEXTURE_COORD_1;
        if (!strcmp(name, "GL_MAP1_TEXTURE_COORD_2")) return GL_MAP1_TEXTURE_COORD_2;
        if (!strcmp(name, "GL_MAP1_TEXTURE_COORD_3")) return GL_MAP1_TEXTURE_COORD_3;
        if (!strcmp(name, "GL_MAP1_TEXTURE_COORD_4")) return GL_MAP1_TEXTURE_COORD_4;
        if (!strcmp(name, "GL_MAP2_TEXTURE_COORD_1")) return GL_MAP2_TEXTURE_COORD_1;
        if (!strcmp(name, "GL_MAP2_TEXTURE_COORD_2")) return GL_MAP2_TEXTURE_COORD_2;
        if (!strcmp(name, "GL_MAP2_TEXTURE_COORD_3")) return GL_MAP2_TEXTURE_COORD_3;
        if (!strcmp(name, "GL_MAP2_TEXTURE_COORD_4")) return GL_MAP2_TEXTURE_COORD_4;
        if (!strcmp(name, "GL_MAT_EMISSION_BIT_PGI")) return GL_MAT_EMISSION_BIT_PGI;
        if (!strcmp(name, "GL_MAT_SPECULAR_BIT_PGI")) return GL_MAT_SPECULAR_BIT_PGI;
        if (!strcmp(name, "GL_MAX_DRAW_BUFFERS_ARB")) return GL_MAX_DRAW_BUFFERS_ARB;
        if (!strcmp(name, "GL_MAX_DRAW_BUFFERS_ATI")) return GL_MAX_DRAW_BUFFERS_ATI;
        if (!strcmp(name, "GL_MAX_ELEMENTS_INDICES")) return GL_MAX_ELEMENTS_INDICES;
        if (!strcmp(name, "GL_MAX_ELEMENTS_INDICES")) return GL_MAX_ELEMENTS_INDICES;
        if (!strcmp(name, "GL_MAX_NAME_STACK_DEPTH")) return GL_MAX_NAME_STACK_DEPTH;
        if (!strcmp(name, "GL_MAX_SPOT_EXPONENT_NV")) return GL_MAX_SPOT_EXPONENT_NV;
        if (!strcmp(name, "GL_MAX_TEXTURE_LOD_BIAS")) return GL_MAX_TEXTURE_LOD_BIAS;
        if (!strcmp(name, "GL_MAX_VERTEX_UNITS_ARB")) return GL_MAX_VERTEX_UNITS_ARB;
        if (!strcmp(name, "GL_MULTISAMPLE_BIT_3DFX")) return GL_MULTISAMPLE_BIT_3DFX;
        if (!strcmp(name, "GL_NORMAL_ARRAY_POINTER")) return GL_NORMAL_ARRAY_POINTER;
        if (!strcmp(name, "GL_NORMALIZED_RANGE_EXT")) return GL_NORMALIZED_RANGE_EXT;
        if (!strcmp(name, "GL_NV_fragment_program2")) return GL_NV_fragment_program2;
        if (!strcmp(name, "GL_NV_fragment_program4")) return GL_NV_fragment_program4;
        if (!strcmp(name, "GL_NV_geometry_program4")) return GL_NV_geometry_program4;
        if (!strcmp(name, "GL_NV_primitive_restart")) return GL_NV_primitive_restart;
        if (!strcmp(name, "GL_NV_texgen_reflection")) return GL_NV_texgen_reflection;
        if (!strcmp(name, "GL_NV_texture_rectangle")) return GL_NV_texture_rectangle;
        if (!strcmp(name, "GL_NV_vertex_program1_1")) return GL_NV_vertex_program1_1;
        if (!strcmp(name, "GL_OES_byte_coordinates")) return GL_OES_byte_coordinates;
        if (!strcmp(name, "GL_OES_single_precision")) return GL_OES_single_precision;
        if (!strcmp(name, "GL_OFFSET_TEXTURE_2D_NV")) return GL_OFFSET_TEXTURE_2D_NV;
        if (!strcmp(name, "GL_OP_CROSS_PRODUCT_EXT")) return GL_OP_CROSS_PRODUCT_EXT;
        if (!strcmp(name, "GL_PACK_SKIP_IMAGES_EXT")) return GL_PACK_SKIP_IMAGES_EXT;
        if (!strcmp(name, "GL_PALETTE4_RGB5_A1_OES")) return GL_PALETTE4_RGB5_A1_OES;
        if (!strcmp(name, "GL_PALETTE8_RGB5_A1_OES")) return GL_PALETTE8_RGB5_A1_OES;
        if (!strcmp(name, "GL_PIXEL_MAG_FILTER_EXT")) return GL_PIXEL_MAG_FILTER_EXT;
        if (!strcmp(name, "GL_PIXEL_MIN_FILTER_EXT")) return GL_PIXEL_MIN_FILTER_EXT;
        if (!strcmp(name, "GL_POLYGON_OFFSET_POINT")) return GL_POLYGON_OFFSET_POINT;
        if (!strcmp(name, "GL_POLYGON_OFFSET_UNITS")) return GL_POLYGON_OFFSET_UNITS;
        if (!strcmp(name, "GL_PRIMITIVE_RESTART_NV")) return GL_PRIMITIVE_RESTART_NV;
        if (!strcmp(name, "GL_PRIMITIVES_GENERATED")) return GL_PRIMITIVES_GENERATED;
        if (!strcmp(name, "GL_PROGRAM_PARAMETER_NV")) return GL_PROGRAM_PARAMETER_NV;
        if (!strcmp(name, "GL_PROXY_TEXTURE_1D_EXT")) return GL_PROXY_TEXTURE_1D_EXT;
        if (!strcmp(name, "GL_PROXY_TEXTURE_2D_EXT")) return GL_PROXY_TEXTURE_2D_EXT;
        if (!strcmp(name, "GL_PROXY_TEXTURE_3D_EXT")) return GL_PROXY_TEXTURE_3D_EXT;
        if (!strcmp(name, "GL_PROXY_TEXTURE_3D_EXT")) return GL_PROXY_TEXTURE_3D_EXT;
        if (!strcmp(name, "GL_QUERY_BY_REGION_WAIT")) return GL_QUERY_BY_REGION_WAIT;
        if (!strcmp(name, "GL_R1UI_C4F_N3F_V3F_SUN")) return GL_R1UI_C4F_N3F_V3F_SUN;
        if (!strcmp(name, "GL_R1UI_T2F_N3F_V3F_SUN")) return GL_R1UI_T2F_N3F_V3F_SUN;
        if (!strcmp(name, "GL_READ_FRAMEBUFFER_EXT")) return GL_READ_FRAMEBUFFER_EXT;
        if (!strcmp(name, "GL_RENDERBUFFER_BINDING")) return GL_RENDERBUFFER_BINDING;
        if (!strcmp(name, "GL_RENDERBUFFER_SAMPLES")) return GL_RENDERBUFFER_SAMPLES;
        if (!strcmp(name, "GL_REPLACEMENT_CODE_SUN")) return GL_REPLACEMENT_CODE_SUN;
        if (!strcmp(name, "GL_RESAMPLE_AVERAGE_OML")) return GL_RESAMPLE_AVERAGE_OML;
        if (!strcmp(name, "GL_SAMPLE_COUNT_BITS_NV")) return GL_SAMPLE_COUNT_BITS_NV;
        if (!strcmp(name, "GL_SAMPLE_MASK_VALUE_NV")) return GL_SAMPLE_MASK_VALUE_NV;
        if (!strcmp(name, "GL_SAMPLER_1D_ARRAY_EXT")) return GL_SAMPLER_1D_ARRAY_EXT;
        if (!strcmp(name, "GL_SAMPLER_2D_ARRAY_EXT")) return GL_SAMPLER_2D_ARRAY_EXT;
        if (!strcmp(name, "GL_SCALE_BY_ONE_HALF_NV")) return GL_SCALE_BY_ONE_HALF_NV;
        if (!strcmp(name, "GL_SECONDARY_COLOR_ATIX")) return GL_SECONDARY_COLOR_ATIX;
        if (!strcmp(name, "GL_SEPARATE_ATTRIBS_EXT")) return GL_SEPARATE_ATTRIBS_EXT;
        if (!strcmp(name, "GL_SGIS_generate_mipmap")) return GL_SGIS_generate_mipmap;
        if (!strcmp(name, "GL_SGIS_sharpen_texture")) return GL_SGIS_sharpen_texture;
        if (!strcmp(name, "GL_SGIS_texture_filter4")) return GL_SGIS_texture_filter4;
        if (!strcmp(name, "GL_SGIX_async_histogram")) return GL_SGIX_async_histogram;
        if (!strcmp(name, "GL_SGIX_reference_plane")) return GL_SGIX_reference_plane;
        if (!strcmp(name, "GL_SGIX_texture_add_env")) return GL_SGIX_texture_add_env;
        if (!strcmp(name, "GL_SHADER_CONSISTENT_NV")) return GL_SHADER_CONSISTENT_NV;
        if (!strcmp(name, "GL_SHADER_SOURCE_LENGTH")) return GL_SHADER_SOURCE_LENGTH;
        if (!strcmp(name, "GL_SIGNED_INTENSITY8_NV")) return GL_SIGNED_INTENSITY8_NV;
        if (!strcmp(name, "GL_SIGNED_INTENSITY8_NV")) return GL_SIGNED_INTENSITY8_NV;
        if (!strcmp(name, "GL_SIGNED_LUMINANCE8_NV")) return GL_SIGNED_LUMINANCE8_NV;
        if (!strcmp(name, "GL_SIGNED_LUMINANCE8_NV")) return GL_SIGNED_LUMINANCE8_NV;
        if (!strcmp(name, "GL_SLUMINANCE_ALPHA_EXT")) return GL_SLUMINANCE_ALPHA_EXT;
        if (!strcmp(name, "GL_STENCIL_TAG_BITS_EXT")) return GL_STENCIL_TAG_BITS_EXT;
        if (!strcmp(name, "GL_STORAGE_CACHED_APPLE")) return GL_STORAGE_CACHED_APPLE;
        if (!strcmp(name, "GL_STORAGE_CACHED_APPLE")) return GL_STORAGE_CACHED_APPLE;
        if (!strcmp(name, "GL_STORAGE_SHARED_APPLE")) return GL_STORAGE_SHARED_APPLE;
        if (!strcmp(name, "GL_STORAGE_SHARED_APPLE")) return GL_STORAGE_SHARED_APPLE;
        if (!strcmp(name, "GL_TEXTURE_1D_ARRAY_EXT")) return GL_TEXTURE_1D_ARRAY_EXT;
        if (!strcmp(name, "GL_TEXTURE_2D_ARRAY_EXT")) return GL_TEXTURE_2D_ARRAY_EXT;
        if (!strcmp(name, "GL_TEXTURE_BORDER_COLOR")) return GL_TEXTURE_BORDER_COLOR;
        if (!strcmp(name, "GL_TEXTURE_COMPARE_FUNC")) return GL_TEXTURE_COMPARE_FUNC;
        if (!strcmp(name, "GL_TEXTURE_COMPARE_MODE")) return GL_TEXTURE_COMPARE_MODE;
        if (!strcmp(name, "GL_TEXTURE_COMPARE_SGIX")) return GL_TEXTURE_COMPARE_SGIX;
        if (!strcmp(name, "GL_TEXTURE_CUBE_MAP_ARB")) return GL_TEXTURE_CUBE_MAP_ARB;
        if (!strcmp(name, "GL_TEXTURE_CUBE_MAP_ARB")) return GL_TEXTURE_CUBE_MAP_ARB;
        if (!strcmp(name, "GL_TEXTURE_CUBE_MAP_EXT")) return GL_TEXTURE_CUBE_MAP_EXT;
        if (!strcmp(name, "GL_TEXTURE_LOD_BIAS_EXT")) return GL_TEXTURE_LOD_BIAS_EXT;
        if (!strcmp(name, "GL_TEXTURE_MAX_LOD_SGIS")) return GL_TEXTURE_MAX_LOD_SGIS;
        if (!strcmp(name, "GL_TEXTURE_MIN_LOD_SGIS")) return GL_TEXTURE_MIN_LOD_SGIS;
        if (!strcmp(name, "GL_TEXTURE_PRIORITY_EXT")) return GL_TEXTURE_PRIORITY_EXT;
        if (!strcmp(name, "GL_TEXTURE_RECTANGLE_NV")) return GL_TEXTURE_RECTANGLE_NV;
        if (!strcmp(name, "GL_TEXTURE_RED_SIZE_EXT")) return GL_TEXTURE_RED_SIZE_EXT;
        if (!strcmp(name, "GL_TEXTURE_RED_TYPE_ARB")) return GL_TEXTURE_RED_TYPE_ARB;
        if (!strcmp(name, "GL_TEXTURE_RESIDENT_EXT")) return GL_TEXTURE_RESIDENT_EXT;
        if (!strcmp(name, "GL_TEXTURE_STENCIL_SIZE")) return GL_TEXTURE_STENCIL_SIZE;
        if (!strcmp(name, "GL_TRANSFORM_HINT_APPLE")) return GL_TRANSFORM_HINT_APPLE;
        if (!strcmp(name, "GL_UNPACK_CMYK_HINT_EXT")) return GL_UNPACK_CMYK_HINT_EXT;
        if (!strcmp(name, "GL_UNPACK_RESAMPLE_SGIX")) return GL_UNPACK_RESAMPLE_SGIX;
        if (!strcmp(name, "GL_UNSIGNED_IDENTITY_NV")) return GL_UNSIGNED_IDENTITY_NV;
        if (!strcmp(name, "GL_UNSIGNED_INT_24_8_NV")) return GL_UNSIGNED_INT_24_8_NV;
        if (!strcmp(name, "GL_UNSIGNED_INT_8_8_8_8")) return GL_UNSIGNED_INT_8_8_8_8;
        if (!strcmp(name, "GL_UNSIGNED_SHORT_5_6_5")) return GL_UNSIGNED_SHORT_5_6_5;
        if (!strcmp(name, "GL_VARIANT_DATATYPE_EXT")) return GL_VARIANT_DATATYPE_EXT;
        if (!strcmp(name, "GL_VERTEX_ARRAY_BINDING")) return GL_VERTEX_ARRAY_BINDING;
        if (!strcmp(name, "GL_VERTEX_ARRAY_POINTER")) return GL_VERTEX_ARRAY_POINTER;
        if (!strcmp(name, "GL_VERTEX_DATA_HINT_PGI")) return GL_VERTEX_DATA_HINT_PGI;
        if (!strcmp(name, "GL_VERTEX_WEIGHTING_EXT")) return GL_VERTEX_WEIGHTING_EXT;
        if (!strcmp(name, "GL_WEIGHT_SUM_UNITY_ARB")) return GL_WEIGHT_SUM_UNITY_ARB;
        return _GL_UNDEFINED;
    case 24:
        if (!strcmp(name, "GL_APPLE_specular_vector")) return GL_APPLE_specular_vector;
        if (!strcmp(name, "GL_ARB_half_float_vertex")) return GL_ARB_half_float_vertex;
        if (!strcmp(name, "GL_ARB_texture_rectangle")) return GL_ARB_texture_rectangle;
        if (!strcmp(name, "GL_ATI_map_object_buffer")) return GL_ATI_map_object_buffer;
        if (!strcmp(name, "GL_BACK_NORMALS_HINT_PGI")) return GL_BACK_NORMALS_HINT_PGI;
        if (!strcmp(name, "GL_BACK_PRIMARY_COLOR_NV")) return GL_BACK_PRIMARY_COLOR_NV;
        if (!strcmp(name, "GL_CLIENT_ACTIVE_TEXTURE")) return GL_CLIENT_ACTIVE_TEXTURE;
        if (!strcmp(name, "GL_COLOR_ARRAY_COUNT_EXT")) return GL_COLOR_ARRAY_COUNT_EXT;
        if (!strcmp(name, "GL_COLOR_ATTACHMENT0_EXT")) return GL_COLOR_ATTACHMENT0_EXT;
        if (!strcmp(name, "GL_COLOR_ATTACHMENT1_EXT")) return GL_COLOR_ATTACHMENT1_EXT;
        if (!strcmp(name, "GL_COLOR_ATTACHMENT2_EXT")) return GL_COLOR_ATTACHMENT2_EXT;
        if (!strcmp(name, "GL_COLOR_ATTACHMENT3_EXT")) return GL_COLOR_ATTACHMENT3_EXT;
        if (!strcmp(name, "GL_COLOR_ATTACHMENT4_EXT")) return GL_COLOR_ATTACHMENT4_EXT;
        if (!strcmp(name, "GL_COLOR_ATTACHMENT5_EXT")) return GL_COLOR_ATTACHMENT5_EXT;
        if (!strcmp(name, "GL_COLOR_ATTACHMENT6_EXT")) return GL_COLOR_ATTACHMENT6_EXT;
        if (!strcmp(name, "GL_COLOR_ATTACHMENT7_EXT")) return GL_COLOR_ATTACHMENT7_EXT;
        if (!strcmp(name, "GL_COLOR_ATTACHMENT8_EXT")) return GL_COLOR_ATTACHMENT8_EXT;
        if (!strcmp(name, "GL_COLOR_ATTACHMENT9_EXT")) return GL_COLOR_ATTACHMENT9_EXT;
        if (!strcmp(name, "GL_COLOR_TABLE_BLUE_SIZE")) return GL_COLOR_TABLE_BLUE_SIZE;
        if (!strcmp(name, "GL_COLOR_TABLE_SCALE_SGI")) return GL_COLOR_TABLE_SCALE_SGI;
        if (!strcmp(name, "GL_COLOR_TABLE_WIDTH_EXT")) return GL_COLOR_TABLE_WIDTH_EXT;
        if (!strcmp(name, "GL_COLOR_TABLE_WIDTH_SGI")) return GL_COLOR_TABLE_WIDTH_SGI;
        if (!strcmp(name, "GL_COMBINER_AB_OUTPUT_NV")) return GL_COMBINER_AB_OUTPUT_NV;
        if (!strcmp(name, "GL_COMBINER_CD_OUTPUT_NV")) return GL_COMBINER_CD_OUTPUT_NV;
        if (!strcmp(name, "GL_COMPRESSED_SLUMINANCE")) return GL_COMPRESSED_SLUMINANCE;
        if (!strcmp(name, "GL_COMPRESSED_SRGB_ALPHA")) return GL_COMPRESSED_SRGB_ALPHA;
        if (!strcmp(name, "GL_CONVOLUTION_HINT_SGIX")) return GL_CONVOLUTION_HINT_SGIX;
        if (!strcmp(name, "GL_CONVOLUTION_WIDTH_EXT")) return GL_CONVOLUTION_WIDTH_EXT;
        if (!strcmp(name, "GL_CURRENT_VERTEX_ATTRIB")) return GL_CURRENT_VERTEX_ATTRIB;
        if (!strcmp(name, "GL_DEPTH_BOUNDS_TEST_EXT")) return GL_DEPTH_BOUNDS_TEST_EXT;
        if (!strcmp(name, "GL_DEPTH_COMPONENT16_ARB")) return GL_DEPTH_COMPONENT16_ARB;
        if (!strcmp(name, "GL_DEPTH_COMPONENT24_ARB")) return GL_DEPTH_COMPONENT24_ARB;
        if (!strcmp(name, "GL_DEPTH_COMPONENT32_ARB")) return GL_DEPTH_COMPONENT32_ARB;
        if (!strcmp(name, "GL_DEPTH_COMPONENT32F_NV")) return GL_DEPTH_COMPONENT32F_NV;
        if (!strcmp(name, "GL_DSDT_MAG_INTENSITY_NV")) return GL_DSDT_MAG_INTENSITY_NV;
        if (!strcmp(name, "GL_DSDT_MAG_INTENSITY_NV")) return GL_DSDT_MAG_INTENSITY_NV;
        if (!strcmp(name, "GL_ENV_REVERSE_BLEND_EXT")) return GL_ENV_REVERSE_BLEND_EXT;
        if (!strcmp(name, "GL_EVAL_TRIANGULAR_2D_NV")) return GL_EVAL_TRIANGULAR_2D_NV;
        if (!strcmp(name, "GL_EXT_depth_bounds_test")) return GL_EXT_depth_bounds_test;
        if (!strcmp(name, "GL_EXT_fragment_lighting")) return GL_EXT_fragment_lighting;
        if (!strcmp(name, "GL_EXT_multi_draw_arrays")) return GL_EXT_multi_draw_arrays;
        if (!strcmp(name, "GL_EXT_stencil_clear_tag")) return GL_EXT_stencil_clear_tag;
        if (!strcmp(name, "GL_EXT_texture_rectangle")) return GL_EXT_texture_rectangle;
        if (!strcmp(name, "GL_EXT_vertex_array_bgra")) return GL_EXT_vertex_array_bgra;
        if (!strcmp(name, "GL_EYE_PLANE_ABSOLUTE_NV")) return GL_EYE_PLANE_ABSOLUTE_NV;
        if (!strcmp(name, "GL_FOG_COORDINATE_SOURCE")) return GL_FOG_COORDINATE_SOURCE;
        if (!strcmp(name, "GL_FOG_OFFSET_VALUE_SGIX")) return GL_FOG_OFFSET_VALUE_SGIX;
        if (!strcmp(name, "GL_FRAGMENT_LIGHTING_EXT")) return GL_FRAGMENT_LIGHTING_EXT;
        if (!strcmp(name, "GL_FRAGMENT_MATERIAL_EXT")) return GL_FRAGMENT_MATERIAL_EXT;
        if (!strcmp(name, "GL_FRAMEBUFFER_UNDEFINED")) return GL_FRAMEBUFFER_UNDEFINED;
        if (!strcmp(name, "GL_FULL_STIPPLE_HINT_PGI")) return GL_FULL_STIPPLE_HINT_PGI;
        if (!strcmp(name, "GL_FUNC_REVERSE_SUBTRACT")) return GL_FUNC_REVERSE_SUBTRACT;
        if (!strcmp(name, "GL_GREMEDY_string_marker")) return GL_GREMEDY_string_marker;
        if (!strcmp(name, "GL_INDEX_ARRAY_COUNT_EXT")) return GL_INDEX_ARRAY_COUNT_EXT;
        if (!strcmp(name, "GL_INTEL_parallel_arrays")) return GL_INTEL_parallel_arrays;
        if (!strcmp(name, "GL_INTEL_texture_scissor")) return GL_INTEL_texture_scissor;
        if (!strcmp(name, "GL_INTENSITY_FLOAT16_ATI")) return GL_INTENSITY_FLOAT16_ATI;
        if (!strcmp(name, "GL_INTENSITY_FLOAT32_ATI")) return GL_INTENSITY_FLOAT32_ATI;
        if (!strcmp(name, "GL_INTENSITY_SIGNED_SGIX")) return GL_INTENSITY_SIGNED_SGIX;
        if (!strcmp(name, "GL_LINEAR_MIPMAP_NEAREST")) return GL_LINEAR_MIPMAP_NEAREST;
        if (!strcmp(name, "GL_LUMINANCE4_ALPHA4_EXT")) return GL_LUMINANCE4_ALPHA4_EXT;
        if (!strcmp(name, "GL_LUMINANCE6_ALPHA2_EXT")) return GL_LUMINANCE6_ALPHA2_EXT;
        if (!strcmp(name, "GL_LUMINANCE8_ALPHA8_EXT")) return GL_LUMINANCE8_ALPHA8_EXT;
        if (!strcmp(name, "GL_LUMINANCE_ALPHA8I_EXT")) return GL_LUMINANCE_ALPHA8I_EXT;
        if (!strcmp(name, "GL_LUMINANCE_FLOAT16_ATI")) return GL_LUMINANCE_FLOAT16_ATI;
        if (!strcmp(name, "GL_LUMINANCE_FLOAT32_ATI")) return GL_LUMINANCE_FLOAT32_ATI;
        if (!strcmp(name, "GL_LUMINANCE_INTEGER_EXT")) return GL_LUMINANCE_INTEGER_EXT;
        if (!strcmp(name, "GL_LUMINANCE_SIGNED_SGIX")) return GL_LUMINANCE_SIGNED_SGIX;
        if (!strcmp(name, "GL_MAP_ATTRIB_U_ORDER_NV")) return GL_MAP_ATTRIB_U_ORDER_NV;
        if (!strcmp(name, "GL_MAP_ATTRIB_V_ORDER_NV")) return GL_MAP_ATTRIB_V_ORDER_NV;
        if (!strcmp(name, "GL_MAT_SHININESS_BIT_PGI")) return GL_MAT_SHININESS_BIT_PGI;
        if (!strcmp(name, "GL_MAX_ACTIVE_LIGHTS_EXT")) return GL_MAX_ACTIVE_LIGHTS_EXT;
        if (!strcmp(name, "GL_MAX_COLOR_ATTACHMENTS")) return GL_MAX_COLOR_ATTACHMENTS;
        if (!strcmp(name, "GL_MAX_CONVOLUTION_WIDTH")) return GL_MAX_CONVOLUTION_WIDTH;
        if (!strcmp(name, "GL_MAX_ELEMENTS_VERTICES")) return GL_MAX_ELEMENTS_VERTICES;
        if (!strcmp(name, "GL_MAX_ELEMENTS_VERTICES")) return GL_MAX_ELEMENTS_VERTICES;
        if (!strcmp(name, "GL_MAX_RENDERBUFFER_SIZE")) return GL_MAX_RENDERBUFFER_SIZE;
        if (!strcmp(name, "GL_MAX_TEXTURE_COORDS_NV")) return GL_MAX_TEXTURE_COORDS_NV;
        if (!strcmp(name, "GL_MAX_TEXTURE_UNITS_ARB")) return GL_MAX_TEXTURE_UNITS_ARB;
        if (!strcmp(name, "GL_MAX_TRACK_MATRICES_NV")) return GL_MAX_TRACK_MATRICES_NV;
        if (!strcmp(name, "GL_MODELVIEW0_MATRIX_EXT")) return GL_MODELVIEW0_MATRIX_EXT;
        if (!strcmp(name, "GL_MODELVIEW1_MATRIX_EXT")) return GL_MODELVIEW1_MATRIX_EXT;
        if (!strcmp(name, "GL_MODELVIEW_STACK_DEPTH")) return GL_MODELVIEW_STACK_DEPTH;
        if (!strcmp(name, "GL_NEAREST_MIPMAP_LINEAR")) return GL_NEAREST_MIPMAP_LINEAR;
        if (!strcmp(name, "GL_NORMAL_ARRAY_LIST_IBM")) return GL_NORMAL_ARRAY_LIST_IBM;
        if (!strcmp(name, "GL_NORMAL_ARRAY_TYPE_EXT")) return GL_NORMAL_ARRAY_TYPE_EXT;
        if (!strcmp(name, "GL_NV_conditional_render")) return GL_NV_conditional_render;
        if (!strcmp(name, "GL_NV_depth_buffer_float")) return GL_NV_depth_buffer_float;
        if (!strcmp(name, "GL_NV_light_max_exponent")) return GL_NV_light_max_exponent;
        if (!strcmp(name, "GL_NV_register_combiners")) return GL_NV_register_combiners;
        if (!strcmp(name, "GL_NV_transform_feedback")) return GL_NV_transform_feedback;
        if (!strcmp(name, "GL_NV_vertex_array_range")) return GL_NV_vertex_array_range;
        if (!strcmp(name, "GL_PACK_IMAGE_HEIGHT_EXT")) return GL_PACK_IMAGE_HEIGHT_EXT;
        if (!strcmp(name, "GL_PALETTE4_R5_G6_B5_OES")) return GL_PALETTE4_R5_G6_B5_OES;
        if (!strcmp(name, "GL_PALETTE8_R5_G6_B5_OES")) return GL_PALETTE8_R5_G6_B5_OES;
        if (!strcmp(name, "GL_PARALLEL_ARRAYS_INTEL")) return GL_PARALLEL_ARRAYS_INTEL;
        if (!strcmp(name, "GL_PIXEL_COUNTER_BITS_NV")) return GL_PIXEL_COUNTER_BITS_NV;
        if (!strcmp(name, "GL_PIXEL_MAP_A_TO_A_SIZE")) return GL_PIXEL_MAP_A_TO_A_SIZE;
        if (!strcmp(name, "GL_PIXEL_MAP_B_TO_B_SIZE")) return GL_PIXEL_MAP_B_TO_B_SIZE;
        if (!strcmp(name, "GL_PIXEL_MAP_G_TO_G_SIZE")) return GL_PIXEL_MAP_G_TO_G_SIZE;
        if (!strcmp(name, "GL_PIXEL_MAP_I_TO_A_SIZE")) return GL_PIXEL_MAP_I_TO_A_SIZE;
        if (!strcmp(name, "GL_PIXEL_MAP_I_TO_B_SIZE")) return GL_PIXEL_MAP_I_TO_B_SIZE;
        if (!strcmp(name, "GL_PIXEL_MAP_I_TO_G_SIZE")) return GL_PIXEL_MAP_I_TO_G_SIZE;
        if (!strcmp(name, "GL_PIXEL_MAP_I_TO_I_SIZE")) return GL_PIXEL_MAP_I_TO_I_SIZE;
        if (!strcmp(name, "GL_PIXEL_MAP_I_TO_R_SIZE")) return GL_PIXEL_MAP_I_TO_R_SIZE;
        if (!strcmp(name, "GL_PIXEL_MAP_R_TO_R_SIZE")) return GL_PIXEL_MAP_R_TO_R_SIZE;
        if (!strcmp(name, "GL_PIXEL_MAP_S_TO_S_SIZE")) return GL_PIXEL_MAP_S_TO_S_SIZE;
        if (!strcmp(name, "GL_PIXEL_PACK_BUFFER_ARB")) return GL_PIXEL_PACK_BUFFER_ARB;
        if (!strcmp(name, "GL_PIXEL_PACK_BUFFER_EXT")) return GL_PIXEL_PACK_BUFFER_EXT;
        if (!strcmp(name, "GL_POLYGON_OFFSET_FACTOR")) return GL_POLYGON_OFFSET_FACTOR;
        if (!strcmp(name, "GL_PROXY_COLOR_TABLE_SGI")) return GL_PROXY_COLOR_TABLE_SGI;
        if (!strcmp(name, "GL_QUADRATIC_ATTENUATION")) return GL_QUADRATIC_ATTENUATION;
        if (!strcmp(name, "GL_RASTERIZER_DISCARD_NV")) return GL_RASTERIZER_DISCARD_NV;
        if (!strcmp(name, "GL_REGISTER_COMBINERS_NV")) return GL_REGISTER_COMBINERS_NV;
        if (!strcmp(name, "GL_RENDERBUFFER_RED_SIZE")) return GL_RENDERBUFFER_RED_SIZE;
        if (!strcmp(name, "GL_RESAMPLE_DECIMATE_OML")) return GL_RESAMPLE_DECIMATE_OML;
        if (!strcmp(name, "GL_RGBA_INTEGER_MODE_EXT")) return GL_RGBA_INTEGER_MODE_EXT;
        if (!strcmp(name, "GL_SAMPLE_COVERAGE_VALUE")) return GL_SAMPLE_COVERAGE_VALUE;
        if (!strcmp(name, "GL_SAMPLE_MASK_VALUE_EXT")) return GL_SAMPLE_MASK_VALUE_EXT;
        if (!strcmp(name, "GL_SAMPLER_1D_SHADOW_ARB")) return GL_SAMPLER_1D_SHADOW_ARB;
        if (!strcmp(name, "GL_SAMPLER_2D_SHADOW_ARB")) return GL_SAMPLER_2D_SHADOW_ARB;
        if (!strcmp(name, "GL_SECONDARY_COLOR_ARRAY")) return GL_SECONDARY_COLOR_ARRAY;
        if (!strcmp(name, "GL_SELECTION_BUFFER_SIZE")) return GL_SELECTION_BUFFER_SIZE;
        if (!strcmp(name, "GL_SGIX_texture_lod_bias")) return GL_SGIX_texture_lod_bias;
        if (!strcmp(name, "GL_STENCIL_BACK_FAIL_ATI")) return GL_STENCIL_BACK_FAIL_ATI;
        if (!strcmp(name, "GL_STENCIL_BACK_FUNC_ATI")) return GL_STENCIL_BACK_FUNC_ATI;
        if (!strcmp(name, "GL_STORAGE_PRIVATE_APPLE")) return GL_STORAGE_PRIVATE_APPLE;
        if (!strcmp(name, "GL_SUN_read_video_pixels")) return GL_SUN_read_video_pixels;
        if (!strcmp(name, "GL_TEXTURE_BLUE_SIZE_EXT")) return GL_TEXTURE_BLUE_SIZE_EXT;
        if (!strcmp(name, "GL_TEXTURE_BLUE_TYPE_ARB")) return GL_TEXTURE_BLUE_TYPE_ARB;
        if (!strcmp(name, "GL_TEXTURE_ENV_SHIFT_EXT")) return GL_TEXTURE_ENV_SHIFT_EXT;
        if (!strcmp(name, "GL_TEXTURE_GEQUAL_R_SGIX")) return GL_TEXTURE_GEQUAL_R_SGIX;
        if (!strcmp(name, "GL_TEXTURE_LEQUAL_R_SGIX")) return GL_TEXTURE_LEQUAL_R_SGIX;
        if (!strcmp(name, "GL_TEXTURE_RECTANGLE_ARB")) return GL_TEXTURE_RECTANGLE_ARB;
        if (!strcmp(name, "GL_TEXTURE_RECTANGLE_EXT")) return GL_TEXTURE_RECTANGLE_EXT;
        if (!strcmp(name, "GL_TEXTURE_SWIZZLE_A_EXT")) return GL_TEXTURE_SWIZZLE_A_EXT;
        if (!strcmp(name, "GL_TEXTURE_SWIZZLE_B_EXT")) return GL_TEXTURE_SWIZZLE_B_EXT;
        if (!strcmp(name, "GL_TEXTURE_SWIZZLE_G_EXT")) return GL_TEXTURE_SWIZZLE_G_EXT;
        if (!strcmp(name, "GL_TEXTURE_SWIZZLE_R_EXT")) return GL_TEXTURE_SWIZZLE_R_EXT;
        if (!strcmp(name, "GL_TRANSFORM_FEEDBACK_NV")) return GL_TRANSFORM_FEEDBACK_NV;
        if (!strcmp(name, "GL_UNSIGNED_INT_24_8_EXT")) return GL_UNSIGNED_INT_24_8_EXT;
        if (!strcmp(name, "GL_UNSIGNED_INT_VEC2_EXT")) return GL_UNSIGNED_INT_VEC2_EXT;
        if (!strcmp(name, "GL_UNSIGNED_INT_VEC3_EXT")) return GL_UNSIGNED_INT_VEC3_EXT;
        if (!strcmp(name, "GL_UNSIGNED_INT_VEC4_EXT")) return GL_UNSIGNED_INT_VEC4_EXT;
        if (!strcmp(name, "GL_VERTEX_ARRAY_LIST_IBM")) return GL_VERTEX_ARRAY_LIST_IBM;
        if (!strcmp(name, "GL_VERTEX_ARRAY_RANGE_NV")) return GL_VERTEX_ARRAY_RANGE_NV;
        if (!strcmp(name, "GL_VERTEX_ARRAY_SIZE_EXT")) return GL_VERTEX_ARRAY_SIZE_EXT;
        if (!strcmp(name, "GL_VERTEX_ARRAY_TYPE_EXT")) return GL_VERTEX_ARRAY_TYPE_EXT;
        if (!strcmp(name, "GL_WEIGHT_ARRAY_SIZE_ARB")) return GL_WEIGHT_ARRAY_SIZE_ARB;
        if (!strcmp(name, "GL_WEIGHT_ARRAY_TYPE_ARB")) return GL_WEIGHT_ARRAY_TYPE_ARB;
        return _GL_UNDEFINED;
    case 25:
        if (!strcmp(name, "GL_ARB_color_buffer_float")) return GL_ARB_color_buffer_float;
        if (!strcmp(name, "GL_ARB_depth_buffer_float")) return GL_ARB_depth_buffer_float;
        if (!strcmp(name, "GL_ARB_framebuffer_object")) return GL_ARB_framebuffer_object;
        if (!strcmp(name, "GL_ASYNC_DRAW_PIXELS_SGIX")) return GL_ASYNC_DRAW_PIXELS_SGIX;
        if (!strcmp(name, "GL_ASYNC_READ_PIXELS_SGIX")) return GL_ASYNC_READ_PIXELS_SGIX;
        if (!strcmp(name, "GL_ATI_shader_texture_lod")) return GL_ATI_shader_texture_lod;
        if (!strcmp(name, "GL_ATIX_texture_env_route")) return GL_ATIX_texture_env_route;
        if (!strcmp(name, "GL_ATTRIB_ARRAY_STRIDE_NV")) return GL_ATTRIB_ARRAY_STRIDE_NV;
        if (!strcmp(name, "GL_BLEND_EQUATION_RGB_EXT")) return GL_BLEND_EQUATION_RGB_EXT;
        if (!strcmp(name, "GL_BUFFER_MAP_POINTER_ARB")) return GL_BUFFER_MAP_POINTER_ARB;
        if (!strcmp(name, "GL_BUMP_NUM_TEX_UNITS_ATI")) return GL_BUMP_NUM_TEX_UNITS_ATI;
        if (!strcmp(name, "GL_CG_FRAGMENT_SHADER_EXT")) return GL_CG_FRAGMENT_SHADER_EXT;
        if (!strcmp(name, "GL_CLAMP_VERTEX_COLOR_ARB")) return GL_CLAMP_VERTEX_COLOR_ARB;
        if (!strcmp(name, "GL_CLIENT_ALL_ATTRIB_BITS")) return GL_CLIENT_ALL_ATTRIB_BITS;
        if (!strcmp(name, "GL_CLIENT_PIXEL_STORE_BIT")) return GL_CLIENT_PIXEL_STORE_BIT;
        if (!strcmp(name, "GL_COLOR_ARRAY_STRIDE_EXT")) return GL_COLOR_ARRAY_STRIDE_EXT;
        if (!strcmp(name, "GL_COLOR_ATTACHMENT10_EXT")) return GL_COLOR_ATTACHMENT10_EXT;
        if (!strcmp(name, "GL_COLOR_ATTACHMENT11_EXT")) return GL_COLOR_ATTACHMENT11_EXT;
        if (!strcmp(name, "GL_COLOR_ATTACHMENT12_EXT")) return GL_COLOR_ATTACHMENT12_EXT;
        if (!strcmp(name, "GL_COLOR_ATTACHMENT13_EXT")) return GL_COLOR_ATTACHMENT13_EXT;
        if (!strcmp(name, "GL_COLOR_ATTACHMENT14_EXT")) return GL_COLOR_ATTACHMENT14_EXT;
        if (!strcmp(name, "GL_COLOR_ATTACHMENT15_EXT")) return GL_COLOR_ATTACHMENT15_EXT;
        if (!strcmp(name, "GL_COLOR_TABLE_ALPHA_SIZE")) return GL_COLOR_TABLE_ALPHA_SIZE;
        if (!strcmp(name, "GL_COLOR_TABLE_FORMAT_EXT")) return GL_COLOR_TABLE_FORMAT_EXT;
        if (!strcmp(name, "GL_COLOR_TABLE_FORMAT_SGI")) return GL_COLOR_TABLE_FORMAT_SGI;
        if (!strcmp(name, "GL_COLOR_TABLE_GREEN_SIZE")) return GL_COLOR_TABLE_GREEN_SIZE;
        if (!strcmp(name, "GL_COMBINER_SUM_OUTPUT_NV")) return GL_COMBINER_SUM_OUTPUT_NV;
        if (!strcmp(name, "GL_COMPARE_REF_TO_TEXTURE")) return GL_COMPARE_REF_TO_TEXTURE;
        if (!strcmp(name, "GL_CONVOLUTION_FORMAT_EXT")) return GL_CONVOLUTION_FORMAT_EXT;
        if (!strcmp(name, "GL_CONVOLUTION_HEIGHT_EXT")) return GL_CONVOLUTION_HEIGHT_EXT;
        if (!strcmp(name, "GL_CURRENT_FOG_COORDINATE")) return GL_CURRENT_FOG_COORDINATE;
        if (!strcmp(name, "GL_CURRENT_TEXTURE_COORDS")) return GL_CURRENT_TEXTURE_COORDS;
        if (!strcmp(name, "GL_DEPTH_COMPONENT16_SGIX")) return GL_DEPTH_COMPONENT16_SGIX;
        if (!strcmp(name, "GL_DEPTH_COMPONENT24_SGIX")) return GL_DEPTH_COMPONENT24_SGIX;
        if (!strcmp(name, "GL_DEPTH_COMPONENT32_SGIX")) return GL_DEPTH_COMPONENT32_SGIX;
        if (!strcmp(name, "GL_DEPTH_TEXTURE_MODE_ARB")) return GL_DEPTH_TEXTURE_MODE_ARB;
        if (!strcmp(name, "GL_EDGE_FLAG_ARRAY_STRIDE")) return GL_EDGE_FLAG_ARRAY_STRIDE;
        if (!strcmp(name, "GL_ELEMENT_ARRAY_TYPE_ATI")) return GL_ELEMENT_ARRAY_TYPE_ATI;
        if (!strcmp(name, "GL_EVAL_VERTEX_ATTRIB0_NV")) return GL_EVAL_VERTEX_ATTRIB0_NV;
        if (!strcmp(name, "GL_EVAL_VERTEX_ATTRIB1_NV")) return GL_EVAL_VERTEX_ATTRIB1_NV;
        if (!strcmp(name, "GL_EVAL_VERTEX_ATTRIB2_NV")) return GL_EVAL_VERTEX_ATTRIB2_NV;
        if (!strcmp(name, "GL_EVAL_VERTEX_ATTRIB3_NV")) return GL_EVAL_VERTEX_ATTRIB3_NV;
        if (!strcmp(name, "GL_EVAL_VERTEX_ATTRIB4_NV")) return GL_EVAL_VERTEX_ATTRIB4_NV;
        if (!strcmp(name, "GL_EVAL_VERTEX_ATTRIB5_NV")) return GL_EVAL_VERTEX_ATTRIB5_NV;
        if (!strcmp(name, "GL_EVAL_VERTEX_ATTRIB6_NV")) return GL_EVAL_VERTEX_ATTRIB6_NV;
        if (!strcmp(name, "GL_EVAL_VERTEX_ATTRIB7_NV")) return GL_EVAL_VERTEX_ATTRIB7_NV;
        if (!strcmp(name, "GL_EVAL_VERTEX_ATTRIB8_NV")) return GL_EVAL_VERTEX_ATTRIB8_NV;
        if (!strcmp(name, "GL_EVAL_VERTEX_ATTRIB9_NV")) return GL_EVAL_VERTEX_ATTRIB9_NV;
        if (!strcmp(name, "GL_EXT_framebuffer_object")) return GL_EXT_framebuffer_object;
        if (!strcmp(name, "GL_EXT_texture_edge_clamp")) return GL_EXT_texture_edge_clamp;
        if (!strcmp(name, "GL_EXT_transform_feedback")) return GL_EXT_transform_feedback;
        if (!strcmp(name, "GL_FOG_COORD_ARRAY_STRIDE")) return GL_FOG_COORD_ARRAY_STRIDE;
        if (!strcmp(name, "GL_FOG_PATCHY_FACTOR_SGIX")) return GL_FOG_PATCHY_FACTOR_SGIX;
        if (!strcmp(name, "GL_HISTOGRAM_RED_SIZE_EXT")) return GL_HISTOGRAM_RED_SIZE_EXT;
        if (!strcmp(name, "GL_IBM_vertex_array_lists")) return GL_IBM_vertex_array_lists;
        if (!strcmp(name, "GL_INDEX_ARRAY_STRIDE_EXT")) return GL_INDEX_ARRAY_STRIDE_EXT;
        if (!strcmp(name, "GL_INTERLEAVED_ATTRIBS_NV")) return GL_INTERLEAVED_ATTRIBS_NV;
        if (!strcmp(name, "GL_INT_SAMPLER_BUFFER_EXT")) return GL_INT_SAMPLER_BUFFER_EXT;
        if (!strcmp(name, "GL_INVARIANT_DATATYPE_EXT")) return GL_INVARIANT_DATATYPE_EXT;
        if (!strcmp(name, "GL_INVERTED_SCREEN_W_REND")) return GL_INVERTED_SCREEN_W_REND;
        if (!strcmp(name, "GL_LINE_WIDTH_GRANULARITY")) return GL_LINE_WIDTH_GRANULARITY;
        if (!strcmp(name, "GL_LUMINANCE12_ALPHA4_EXT")) return GL_LUMINANCE12_ALPHA4_EXT;
        if (!strcmp(name, "GL_LUMINANCE_ALPHA16F_ARB")) return GL_LUMINANCE_ALPHA16F_ARB;
        if (!strcmp(name, "GL_LUMINANCE_ALPHA16I_EXT")) return GL_LUMINANCE_ALPHA16I_EXT;
        if (!strcmp(name, "GL_LUMINANCE_ALPHA32F_ARB")) return GL_LUMINANCE_ALPHA32F_ARB;
        if (!strcmp(name, "GL_LUMINANCE_ALPHA32I_EXT")) return GL_LUMINANCE_ALPHA32I_EXT;
        if (!strcmp(name, "GL_LUMINANCE_ALPHA8UI_EXT")) return GL_LUMINANCE_ALPHA8UI_EXT;
        if (!strcmp(name, "GL_MAP_FLUSH_EXPLICIT_BIT")) return GL_MAP_FLUSH_EXPLICIT_BIT;
        if (!strcmp(name, "GL_MAP_UNSYNCHRONIZED_BIT")) return GL_MAP_UNSYNCHRONIZED_BIT;
        if (!strcmp(name, "GL_MATERIAL_SIDE_HINT_PGI")) return GL_MATERIAL_SIDE_HINT_PGI;
        if (!strcmp(name, "GL_MATRIX_INDEX_ARRAY_ARB")) return GL_MATRIX_INDEX_ARRAY_ARB;
        if (!strcmp(name, "GL_MAX_ATTRIB_STACK_DEPTH")) return GL_MAX_ATTRIB_STACK_DEPTH;
        if (!strcmp(name, "GL_MAX_CONVOLUTION_HEIGHT")) return GL_MAX_CONVOLUTION_HEIGHT;
        if (!strcmp(name, "GL_MAX_TEXTURE_COORDS_ARB")) return GL_MAX_TEXTURE_COORDS_ARB;
        if (!strcmp(name, "GL_MAX_VARYING_COMPONENTS")) return GL_MAX_VARYING_COMPONENTS;
        if (!strcmp(name, "GL_MAX_VARYING_FLOATS_ARB")) return GL_MAX_VARYING_FLOATS_ARB;
        if (!strcmp(name, "GL_MAX_VERTEX_ATTRIBS_ARB")) return GL_MAX_VERTEX_ATTRIBS_ARB;
        if (!strcmp(name, "GL_MAX_VERTEX_STREAMS_ATI")) return GL_MAX_VERTEX_STREAMS_ATI;
        if (!strcmp(name, "GL_MODULATE_SUBTRACT_ATIX")) return GL_MODULATE_SUBTRACT_ATIX;
        if (!strcmp(name, "GL_NEAREST_MIPMAP_NEAREST")) return GL_NEAREST_MIPMAP_NEAREST;
        if (!strcmp(name, "GL_NORMAL_ARRAY_COUNT_EXT")) return GL_NORMAL_ARRAY_COUNT_EXT;
        if (!strcmp(name, "GL_NV_copy_depth_to_color")) return GL_NV_copy_depth_to_color;
        if (!strcmp(name, "GL_NV_register_combiners2")) return GL_NV_register_combiners2;
        if (!strcmp(name, "GL_NV_transform_feedback2")) return GL_NV_transform_feedback2;
        if (!strcmp(name, "GL_NV_vertex_array_range2")) return GL_NV_vertex_array_range2;
        if (!strcmp(name, "GL_OBJECT_BUFFER_SIZE_ATI")) return GL_OBJECT_BUFFER_SIZE_ATI;
        if (!strcmp(name, "GL_OBJECT_LINK_STATUS_ARB")) return GL_OBJECT_LINK_STATUS_ARB;
        if (!strcmp(name, "GL_OFFSET_TEXTURE_BIAS_NV")) return GL_OFFSET_TEXTURE_BIAS_NV;
        if (!strcmp(name, "GL_OP_MULTIPLY_MATRIX_EXT")) return GL_OP_MULTIPLY_MATRIX_EXT;
        if (!strcmp(name, "GL_OUTPUT_POINT_SIZE_ATIX")) return GL_OUTPUT_POINT_SIZE_ATIX;
        if (!strcmp(name, "GL_PER_STAGE_CONSTANTS_NV")) return GL_PER_STAGE_CONSTANTS_NV;
        if (!strcmp(name, "GL_PIXEL_CUBIC_WEIGHT_EXT")) return GL_PIXEL_CUBIC_WEIGHT_EXT;
        if (!strcmp(name, "GL_PIXEL_TRANSFORM_2D_EXT")) return GL_PIXEL_TRANSFORM_2D_EXT;
        if (!strcmp(name, "GL_POINT_SIZE_GRANULARITY")) return GL_POINT_SIZE_GRANULARITY;
        if (!strcmp(name, "GL_POINT_SPRITE_R_MODE_NV")) return GL_POINT_SPRITE_R_MODE_NV;
        if (!strcmp(name, "GL_PROGRAM_PARAMETERS_ARB")) return GL_PROGRAM_PARAMETERS_ARB;
        if (!strcmp(name, "GL_PROGRAM_POINT_SIZE_ARB")) return GL_PROGRAM_POINT_SIZE_ARB;
        if (!strcmp(name, "GL_PROGRAM_POINT_SIZE_EXT")) return GL_PROGRAM_POINT_SIZE_EXT;
        if (!strcmp(name, "GL_PROJECTION_STACK_DEPTH")) return GL_PROJECTION_STACK_DEPTH;
        if (!strcmp(name, "GL_PROXY_TEXTURE_1D_ARRAY")) return GL_PROXY_TEXTURE_1D_ARRAY;
        if (!strcmp(name, "GL_PROXY_TEXTURE_2D_ARRAY")) return GL_PROXY_TEXTURE_2D_ARRAY;
        if (!strcmp(name, "GL_PROXY_TEXTURE_CUBE_MAP")) return GL_PROXY_TEXTURE_CUBE_MAP;
        if (!strcmp(name, "GL_QUERY_COUNTER_BITS_ARB")) return GL_QUERY_COUNTER_BITS_ARB;
        if (!strcmp(name, "GL_QUERY_RESULT_AVAILABLE")) return GL_QUERY_RESULT_AVAILABLE;
        if (!strcmp(name, "GL_RASTERIZER_DISCARD_EXT")) return GL_RASTERIZER_DISCARD_EXT;
        if (!strcmp(name, "GL_RENDERBUFFER_BLUE_SIZE")) return GL_RENDERBUFFER_BLUE_SIZE;
        if (!strcmp(name, "GL_RENDERBUFFER_WIDTH_EXT")) return GL_RENDERBUFFER_WIDTH_EXT;
        if (!strcmp(name, "GL_RESAMPLE_DECIMATE_SGIX")) return GL_RESAMPLE_DECIMATE_SGIX;
        if (!strcmp(name, "GL_RESAMPLE_REPLICATE_OML")) return GL_RESAMPLE_REPLICATE_OML;
        if (!strcmp(name, "GL_RESAMPLE_ZERO_FILL_OML")) return GL_RESAMPLE_ZERO_FILL_OML;
        if (!strcmp(name, "GL_SAMPLE_COVERAGE_INVERT")) return GL_SAMPLE_COVERAGE_INVERT;
        if (!strcmp(name, "GL_SAMPLE_MASK_INVERT_EXT")) return GL_SAMPLE_MASK_INVERT_EXT;
        if (!strcmp(name, "GL_SAMPLE_MASK_VALUE_SGIS")) return GL_SAMPLE_MASK_VALUE_SGIS;
        if (!strcmp(name, "GL_SGIS_point_line_texgen")) return GL_SGIS_point_line_texgen;
        if (!strcmp(name, "GL_SGIX_tag_sample_buffer")) return GL_SGIX_tag_sample_buffer;
        if (!strcmp(name, "GL_SHADOW_ATTENUATION_EXT")) return GL_SHADOW_ATTENUATION_EXT;
        if (!strcmp(name, "GL_SLUMINANCE8_ALPHA8_EXT")) return GL_SLUMINANCE8_ALPHA8_EXT;
        if (!strcmp(name, "GL_STENCIL_ATTACHMENT_EXT")) return GL_STENCIL_ATTACHMENT_EXT;
        if (!strcmp(name, "GL_STENCIL_BACK_WRITEMASK")) return GL_STENCIL_BACK_WRITEMASK;
        if (!strcmp(name, "GL_TANGENT_ARRAY_TYPE_EXT")) return GL_TANGENT_ARRAY_TYPE_EXT;
        if (!strcmp(name, "GL_TEXTURE_1D_BINDING_EXT")) return GL_TEXTURE_1D_BINDING_EXT;
        if (!strcmp(name, "GL_TEXTURE_1D_STACK_MESAX")) return GL_TEXTURE_1D_STACK_MESAX;
        if (!strcmp(name, "GL_TEXTURE_2D_BINDING_EXT")) return GL_TEXTURE_2D_BINDING_EXT;
        if (!strcmp(name, "GL_TEXTURE_2D_STACK_MESAX")) return GL_TEXTURE_2D_STACK_MESAX;
        if (!strcmp(name, "GL_TEXTURE_3D_BINDING_EXT")) return GL_TEXTURE_3D_BINDING_EXT;
        if (!strcmp(name, "GL_TEXTURE_ALPHA_SIZE_EXT")) return GL_TEXTURE_ALPHA_SIZE_EXT;
        if (!strcmp(name, "GL_TEXTURE_ALPHA_TYPE_ARB")) return GL_TEXTURE_ALPHA_TYPE_ARB;
        if (!strcmp(name, "GL_TEXTURE_COMPRESSED_ARB")) return GL_TEXTURE_COMPRESSED_ARB;
        if (!strcmp(name, "GL_TEXTURE_DEPTH_SIZE_ARB")) return GL_TEXTURE_DEPTH_SIZE_ARB;
        if (!strcmp(name, "GL_TEXTURE_DEPTH_TYPE_ARB")) return GL_TEXTURE_DEPTH_TYPE_ARB;
        if (!strcmp(name, "GL_TEXTURE_FILTER_CONTROL")) return GL_TEXTURE_FILTER_CONTROL;
        if (!strcmp(name, "GL_TEXTURE_GREEN_SIZE_EXT")) return GL_TEXTURE_GREEN_SIZE_EXT;
        if (!strcmp(name, "GL_TEXTURE_GREEN_TYPE_ARB")) return GL_TEXTURE_GREEN_TYPE_ARB;
        if (!strcmp(name, "GL_TEXTURE_INDEX_SIZE_EXT")) return GL_TEXTURE_INDEX_SIZE_EXT;
        if (!strcmp(name, "GL_TEXTURE_INTENSITY_SIZE")) return GL_TEXTURE_INTENSITY_SIZE;
        if (!strcmp(name, "GL_TEXTURE_INTENSITY_TYPE")) return GL_TEXTURE_INTENSITY_TYPE;
        if (!strcmp(name, "GL_TEXTURE_LUMINANCE_SIZE")) return GL_TEXTURE_LUMINANCE_SIZE;
        if (!strcmp(name, "GL_TEXTURE_LUMINANCE_TYPE")) return GL_TEXTURE_LUMINANCE_TYPE;
        if (!strcmp(name, "GL_TEXTURE_MAX_LEVEL_SGIS")) return GL_TEXTURE_MAX_LEVEL_SGIS;
        if (!strcmp(name, "GL_TRANSPOSE_COLOR_MATRIX")) return GL_TRANSPOSE_COLOR_MATRIX;
        if (!strcmp(name, "GL_UNPACK_SKIP_IMAGES_EXT")) return GL_UNPACK_SKIP_IMAGES_EXT;
        if (!strcmp(name, "GL_UNSIGNED_SHORT_4_4_4_4")) return GL_UNSIGNED_SHORT_4_4_4_4;
        if (!strcmp(name, "GL_UNSIGNED_SHORT_5_5_5_1")) return GL_UNSIGNED_SHORT_5_5_5_1;
        if (!strcmp(name, "GL_VARIANT_ARRAY_TYPE_EXT")) return GL_VARIANT_ARRAY_TYPE_EXT;
        if (!strcmp(name, "GL_VERTEX_ARRAY_COUNT_EXT")) return GL_VERTEX_ARRAY_COUNT_EXT;
        return _GL_UNDEFINED;
    case 26:
        if (!strcmp(name, "GL_ACTIVE_STENCIL_FACE_EXT")) return GL_ACTIVE_STENCIL_FACE_EXT;
        if (!strcmp(name, "GL_ACTIVE_VERTEX_UNITS_ARB")) return GL_ACTIVE_VERTEX_UNITS_ARB;
        if (!strcmp(name, "GL_ALLOW_DRAW_FRG_HINT_PGI")) return GL_ALLOW_DRAW_FRG_HINT_PGI;
        if (!strcmp(name, "GL_ALLOW_DRAW_MEM_HINT_PGI")) return GL_ALLOW_DRAW_MEM_HINT_PGI;
        if (!strcmp(name, "GL_ALLOW_DRAW_OBJ_HINT_PGI")) return GL_ALLOW_DRAW_OBJ_HINT_PGI;
        if (!strcmp(name, "GL_ALLOW_DRAW_WIN_HINT_PGI")) return GL_ALLOW_DRAW_WIN_HINT_PGI;
        if (!strcmp(name, "GL_AMD_performance_monitor")) return GL_AMD_performance_monitor;
        if (!strcmp(name, "GL_ARB_pixel_buffer_object")) return GL_ARB_pixel_buffer_object;
        if (!strcmp(name, "GL_ARB_texture_compression")) return GL_ARB_texture_compression;
        if (!strcmp(name, "GL_ARB_texture_env_combine")) return GL_ARB_texture_env_combine;
        if (!strcmp(name, "GL_ARB_vertex_array_object")) return GL_ARB_vertex_array_object;
        if (!strcmp(name, "GL_ARRAY_OBJECT_BUFFER_ATI")) return GL_ARRAY_OBJECT_BUFFER_ATI;
        if (!strcmp(name, "GL_ARRAY_OBJECT_OFFSET_ATI")) return GL_ARRAY_OBJECT_OFFSET_ATI;
        if (!strcmp(name, "GL_ATI_texture_mirror_once")) return GL_ATI_texture_mirror_once;
        if (!strcmp(name, "GL_ATI_vertex_array_object")) return GL_ATI_vertex_array_object;
        if (!strcmp(name, "GL_ATTRIB_ARRAY_POINTER_NV")) return GL_ATTRIB_ARRAY_POINTER_NV;
        if (!strcmp(name, "GL_BACK_SECONDARY_COLOR_NV")) return GL_BACK_SECONDARY_COLOR_NV;
        if (!strcmp(name, "GL_BINORMAL_ARRAY_TYPE_EXT")) return GL_BINORMAL_ARRAY_TYPE_EXT;
        if (!strcmp(name, "GL_CLIENT_VERTEX_ARRAY_BIT")) return GL_CLIENT_VERTEX_ARRAY_BIT;
        if (!strcmp(name, "GL_COLOR_ALPHA_PAIRING_ATI")) return GL_COLOR_ALPHA_PAIRING_ATI;
        if (!strcmp(name, "GL_COLOR_ARRAY_POINTER_EXT")) return GL_COLOR_ARRAY_POINTER_EXT;
        if (!strcmp(name, "GL_CONVOLUTION_BORDER_MODE")) return GL_CONVOLUTION_BORDER_MODE;
        if (!strcmp(name, "GL_CONVOLUTION_FILTER_BIAS")) return GL_CONVOLUTION_FILTER_BIAS;
        if (!strcmp(name, "GL_CURRENT_RASTER_DISTANCE")) return GL_CURRENT_RASTER_DISTANCE;
        if (!strcmp(name, "GL_CURRENT_RASTER_POSITION")) return GL_CURRENT_RASTER_POSITION;
        if (!strcmp(name, "GL_CURRENT_SECONDARY_COLOR")) return GL_CURRENT_SECONDARY_COLOR;
        if (!strcmp(name, "GL_EDGE_FLAG_ARRAY_POINTER")) return GL_EDGE_FLAG_ARRAY_POINTER;
        if (!strcmp(name, "GL_EVAL_VERTEX_ATTRIB10_NV")) return GL_EVAL_VERTEX_ATTRIB10_NV;
        if (!strcmp(name, "GL_EVAL_VERTEX_ATTRIB11_NV")) return GL_EVAL_VERTEX_ATTRIB11_NV;
        if (!strcmp(name, "GL_EVAL_VERTEX_ATTRIB12_NV")) return GL_EVAL_VERTEX_ATTRIB12_NV;
        if (!strcmp(name, "GL_EVAL_VERTEX_ATTRIB13_NV")) return GL_EVAL_VERTEX_ATTRIB13_NV;
        if (!strcmp(name, "GL_EVAL_VERTEX_ATTRIB14_NV")) return GL_EVAL_VERTEX_ATTRIB14_NV;
        if (!strcmp(name, "GL_EVAL_VERTEX_ATTRIB15_NV")) return GL_EVAL_VERTEX_ATTRIB15_NV;
        if (!strcmp(name, "GL_EXT_blend_func_separate")) return GL_EXT_blend_func_separate;
        if (!strcmp(name, "GL_EXT_direct_state_access")) return GL_EXT_direct_state_access;
        if (!strcmp(name, "GL_EXT_draw_range_elements")) return GL_EXT_draw_range_elements;
        if (!strcmp(name, "GL_EXT_index_array_formats")) return GL_EXT_index_array_formats;
        if (!strcmp(name, "GL_EXT_pixel_buffer_object")) return GL_EXT_pixel_buffer_object;
        if (!strcmp(name, "GL_EXT_texture_env_combine")) return GL_EXT_texture_env_combine;
        if (!strcmp(name, "GL_FEEDBACK_BUFFER_POINTER")) return GL_FEEDBACK_BUFFER_POINTER;
        if (!strcmp(name, "GL_FOG_COORD_ARRAY_POINTER")) return GL_FOG_COORD_ARRAY_POINTER;
        if (!strcmp(name, "GL_FRAMEBUFFER_BINDING_EXT")) return GL_FRAMEBUFFER_BINDING_EXT;
        if (!strcmp(name, "GL_FRAMEBUFFER_UNSUPPORTED")) return GL_FRAMEBUFFER_UNSUPPORTED;
        if (!strcmp(name, "GL_GEOMETRY_INPUT_TYPE_ARB")) return GL_GEOMETRY_INPUT_TYPE_ARB;
        if (!strcmp(name, "GL_GEOMETRY_INPUT_TYPE_EXT")) return GL_GEOMETRY_INPUT_TYPE_EXT;
        if (!strcmp(name, "GL_GLOBAL_ALPHA_FACTOR_SUN")) return GL_GLOBAL_ALPHA_FACTOR_SUN;
        if (!strcmp(name, "GL_HISTOGRAM_BLUE_SIZE_EXT")) return GL_HISTOGRAM_BLUE_SIZE_EXT;
        if (!strcmp(name, "GL_INDEX_ARRAY_POINTER_EXT")) return GL_INDEX_ARRAY_POINTER_EXT;
        if (!strcmp(name, "GL_INTENSITY16_SIGNED_SGIX")) return GL_INTENSITY16_SIGNED_SGIX;
        if (!strcmp(name, "GL_INTENSITY_FLOAT16_APPLE")) return GL_INTENSITY_FLOAT16_APPLE;
        if (!strcmp(name, "GL_INTENSITY_FLOAT32_APPLE")) return GL_INTENSITY_FLOAT32_APPLE;
        if (!strcmp(name, "GL_INTERLEAVED_ATTRIBS_EXT")) return GL_INTERLEAVED_ATTRIBS_EXT;
        if (!strcmp(name, "GL_INT_SAMPLER_2D_RECT_EXT")) return GL_INT_SAMPLER_2D_RECT_EXT;
        if (!strcmp(name, "GL_LUMINANCE12_ALPHA12_EXT")) return GL_LUMINANCE12_ALPHA12_EXT;
        if (!strcmp(name, "GL_LUMINANCE16_ALPHA16_EXT")) return GL_LUMINANCE16_ALPHA16_EXT;
        if (!strcmp(name, "GL_LUMINANCE16_SIGNED_SGIX")) return GL_LUMINANCE16_SIGNED_SGIX;
        if (!strcmp(name, "GL_LUMINANCE_ALPHA16UI_EXT")) return GL_LUMINANCE_ALPHA16UI_EXT;
        if (!strcmp(name, "GL_LUMINANCE_ALPHA32UI_EXT")) return GL_LUMINANCE_ALPHA32UI_EXT;
        if (!strcmp(name, "GL_LUMINANCE_FLOAT16_APPLE")) return GL_LUMINANCE_FLOAT16_APPLE;
        if (!strcmp(name, "GL_LUMINANCE_FLOAT32_APPLE")) return GL_LUMINANCE_FLOAT32_APPLE;
        if (!strcmp(name, "GL_MAX_3D_TEXTURE_SIZE_EXT")) return GL_MAX_3D_TEXTURE_SIZE_EXT;
        if (!strcmp(name, "GL_MAX_FRAGMENT_LIGHTS_EXT")) return GL_MAX_FRAGMENT_LIGHTS_EXT;
        if (!strcmp(name, "GL_MAX_MAP_TESSELLATION_NV")) return GL_MAX_MAP_TESSELLATION_NV;
        if (!strcmp(name, "GL_MAX_PROGRAM_ATTRIBS_ARB")) return GL_MAX_PROGRAM_ATTRIBS_ARB;
        if (!strcmp(name, "GL_MAX_PROGRAM_IF_DEPTH_NV")) return GL_MAX_PROGRAM_IF_DEPTH_NV;
        if (!strcmp(name, "GL_MAX_TEXTURE_IMAGE_UNITS")) return GL_MAX_TEXTURE_IMAGE_UNITS;
        if (!strcmp(name, "GL_MAX_TEXTURE_STACK_DEPTH")) return GL_MAX_TEXTURE_STACK_DEPTH;
        if (!strcmp(name, "GL_MODELVIEW_PROJECTION_NV")) return GL_MODELVIEW_PROJECTION_NV;
        if (!strcmp(name, "GL_NORMAL_ARRAY_STRIDE_EXT")) return GL_NORMAL_ARRAY_STRIDE_EXT;
        if (!strcmp(name, "GL_NV_explicit_multisample")) return GL_NV_explicit_multisample;
        if (!strcmp(name, "GL_NV_packed_depth_stencil")) return GL_NV_packed_depth_stencil;
        if (!strcmp(name, "GL_NV_texture_env_combine4")) return GL_NV_texture_env_combine4;
        if (!strcmp(name, "GL_OBJECT_BUFFER_USAGE_ATI")) return GL_OBJECT_BUFFER_USAGE_ATI;
        if (!strcmp(name, "GL_OFFSET_TEXTURE_SCALE_NV")) return GL_OFFSET_TEXTURE_SCALE_NV;
        if (!strcmp(name, "GL_PERFMON_RESULT_SIZE_AMD")) return GL_PERFMON_RESULT_SIZE_AMD;
        if (!strcmp(name, "GL_PIXEL_UNPACK_BUFFER_ARB")) return GL_PIXEL_UNPACK_BUFFER_ARB;
        if (!strcmp(name, "GL_PIXEL_UNPACK_BUFFER_EXT")) return GL_PIXEL_UNPACK_BUFFER_EXT;
        if (!strcmp(name, "GL_POLYGON_OFFSET_BIAS_EXT")) return GL_POLYGON_OFFSET_BIAS_EXT;
        if (!strcmp(name, "GL_PRIMITIVES_GENERATED_NV")) return GL_PRIMITIVES_GENERATED_NV;
        if (!strcmp(name, "GL_PROGRAM_ERROR_STRING_NV")) return GL_PROGRAM_ERROR_STRING_NV;
        if (!strcmp(name, "GL_PROGRAM_TEMPORARIES_ARB")) return GL_PROGRAM_TEMPORARIES_ARB;
        if (!strcmp(name, "GL_QUERY_BY_REGION_NO_WAIT")) return GL_QUERY_BY_REGION_NO_WAIT;
        if (!strcmp(name, "GL_QUERY_BY_REGION_WAIT_NV")) return GL_QUERY_BY_REGION_WAIT_NV;
        if (!strcmp(name, "GL_RECLAIM_MEMORY_HINT_PGI")) return GL_RECLAIM_MEMORY_HINT_PGI;
        if (!strcmp(name, "GL_RENDERBUFFER_ALPHA_SIZE")) return GL_RENDERBUFFER_ALPHA_SIZE;
        if (!strcmp(name, "GL_RENDERBUFFER_DEPTH_SIZE")) return GL_RENDERBUFFER_DEPTH_SIZE;
        if (!strcmp(name, "GL_RENDERBUFFER_GREEN_SIZE")) return GL_RENDERBUFFER_GREEN_SIZE;
        if (!strcmp(name, "GL_RENDERBUFFER_HEIGHT_EXT")) return GL_RENDERBUFFER_HEIGHT_EXT;
        if (!strcmp(name, "GL_REND_screen_coordinates")) return GL_REND_screen_coordinates;
        if (!strcmp(name, "GL_RESAMPLE_REPLICATE_SGIX")) return GL_RESAMPLE_REPLICATE_SGIX;
        if (!strcmp(name, "GL_RESAMPLE_ZERO_FILL_SGIX")) return GL_RESAMPLE_ZERO_FILL_SGIX;
        if (!strcmp(name, "GL_RGB_EXTENDED_RANGE_SGIX")) return GL_RGB_EXTENDED_RANGE_SGIX;
        if (!strcmp(name, "GL_SAMPLE_ALPHA_TO_ONE_ARB")) return GL_SAMPLE_ALPHA_TO_ONE_ARB;
        if (!strcmp(name, "GL_SAMPLE_ALPHA_TO_ONE_EXT")) return GL_SAMPLE_ALPHA_TO_ONE_EXT;
        if (!strcmp(name, "GL_SAMPLE_MASK_INVERT_SGIS")) return GL_SAMPLE_MASK_INVERT_SGIS;
        if (!strcmp(name, "GL_SAMPLER_1D_ARRAY_SHADOW")) return GL_SAMPLER_1D_ARRAY_SHADOW;
        if (!strcmp(name, "GL_SAMPLER_2D_ARRAY_SHADOW")) return GL_SAMPLER_2D_ARRAY_SHADOW;
        if (!strcmp(name, "GL_SAMPLER_CUBE_SHADOW_EXT")) return GL_SAMPLER_CUBE_SHADOW_EXT;
        if (!strcmp(name, "GL_SAMPLER_RENDERBUFFER_NV")) return GL_SAMPLER_RENDERBUFFER_NV;
        if (!strcmp(name, "GL_SCREEN_COORDINATES_REND")) return GL_SCREEN_COORDINATES_REND;
        if (!strcmp(name, "GL_SEPARATE_SPECULAR_COLOR")) return GL_SEPARATE_SPECULAR_COLOR;
        if (!strcmp(name, "GL_SGIS_texture_edge_clamp")) return GL_SGIS_texture_edge_clamp;
        if (!strcmp(name, "GL_SGI_texture_color_table")) return GL_SGI_texture_color_table;
        if (!strcmp(name, "GL_SGIX_blend_alpha_minmax")) return GL_SGIX_blend_alpha_minmax;
        if (!strcmp(name, "GL_SGIX_pixel_texture_bits")) return GL_SGIX_pixel_texture_bits;
        if (!strcmp(name, "GL_SGIX_texture_scale_bias")) return GL_SGIX_texture_scale_bias;
        if (!strcmp(name, "GL_SMOOTH_LINE_WIDTH_RANGE")) return GL_SMOOTH_LINE_WIDTH_RANGE;
        if (!strcmp(name, "GL_SMOOTH_POINT_SIZE_RANGE")) return GL_SMOOTH_POINT_SIZE_RANGE;
        if (!strcmp(name, "GL_STATIC_VERTEX_ARRAY_IBM")) return GL_STATIC_VERTEX_ARRAY_IBM;
        if (!strcmp(name, "GL_STENCIL_BACK_VALUE_MASK")) return GL_STENCIL_BACK_VALUE_MASK;
        if (!strcmp(name, "GL_STENCIL_PASS_DEPTH_FAIL")) return GL_STENCIL_PASS_DEPTH_FAIL;
        if (!strcmp(name, "GL_STENCIL_PASS_DEPTH_PASS")) return GL_STENCIL_PASS_DEPTH_PASS;
        if (!strcmp(name, "GL_STRICT_SCISSOR_HINT_PGI")) return GL_STRICT_SCISSOR_HINT_PGI;
        if (!strcmp(name, "GL_TEXTURE_BASE_LEVEL_SGIS")) return GL_TEXTURE_BASE_LEVEL_SGIS;
        if (!strcmp(name, "GL_TEXTURE_COLOR_TABLE_SGI")) return GL_TEXTURE_COLOR_TABLE_SGI;
        if (!strcmp(name, "GL_TEXTURE_COORD_ARRAY_EXT")) return GL_TEXTURE_COORD_ARRAY_EXT;
        if (!strcmp(name, "GL_TEXTURE_FREE_MEMORY_ATI")) return GL_TEXTURE_FREE_MEMORY_ATI;
        if (!strcmp(name, "GL_TEXTURE_INTERNAL_FORMAT")) return GL_TEXTURE_INTERNAL_FORMAT;
        if (!strcmp(name, "GL_TEXTURE_OUTPUT_RGB_ATIX")) return GL_TEXTURE_OUTPUT_RGB_ATIX;
        if (!strcmp(name, "GL_TEXTURE_POINT_MODE_ATIX")) return GL_TEXTURE_POINT_MODE_ATIX;
        if (!strcmp(name, "GL_TEXTURE_RENDERBUFFER_NV")) return GL_TEXTURE_RENDERBUFFER_NV;
        if (!strcmp(name, "GL_TEXTURE_SHARED_SIZE_EXT")) return GL_TEXTURE_SHARED_SIZE_EXT;
        if (!strcmp(name, "GL_TRIANGLES_ADJACENCY_ARB")) return GL_TRIANGLES_ADJACENCY_ARB;
        if (!strcmp(name, "GL_TRIANGLES_ADJACENCY_EXT")) return GL_TRIANGLES_ADJACENCY_EXT;
        if (!strcmp(name, "GL_UNPACK_IMAGE_HEIGHT_EXT")) return GL_UNPACK_IMAGE_HEIGHT_EXT;
        if (!strcmp(name, "GL_UNSIGNED_BYTE_2_3_3_REV")) return GL_UNSIGNED_BYTE_2_3_3_REV;
        if (!strcmp(name, "GL_UNSIGNED_BYTE_3_3_2_EXT")) return GL_UNSIGNED_BYTE_3_3_2_EXT;
        if (!strcmp(name, "GL_UNSIGNED_INT_10_10_10_2")) return GL_UNSIGNED_INT_10_10_10_2;
        if (!strcmp(name, "GL_UNSIGNED_INT_SAMPLER_1D")) return GL_UNSIGNED_INT_SAMPLER_1D;
        if (!strcmp(name, "GL_UNSIGNED_INT_SAMPLER_2D")) return GL_UNSIGNED_INT_SAMPLER_2D;
        if (!strcmp(name, "GL_UNSIGNED_INT_SAMPLER_3D")) return GL_UNSIGNED_INT_SAMPLER_3D;
        if (!strcmp(name, "GL_UNSIGNED_NORMALIZED_ARB")) return GL_UNSIGNED_NORMALIZED_ARB;
        if (!strcmp(name, "GL_UNSIGNED_SHORT_8_8_MESA")) return GL_UNSIGNED_SHORT_8_8_MESA;
        if (!strcmp(name, "GL_VERTEX_ARRAY_STRIDE_EXT")) return GL_VERTEX_ARRAY_STRIDE_EXT;
        if (!strcmp(name, "GL_VERTEX_ATTRIB_ARRAY0_NV")) return GL_VERTEX_ATTRIB_ARRAY0_NV;
        if (!strcmp(name, "GL_VERTEX_ATTRIB_ARRAY1_NV")) return GL_VERTEX_ATTRIB_ARRAY1_NV;
        if (!strcmp(name, "GL_VERTEX_ATTRIB_ARRAY2_NV")) return GL_VERTEX_ATTRIB_ARRAY2_NV;
        if (!strcmp(name, "GL_VERTEX_ATTRIB_ARRAY3_NV")) return GL_VERTEX_ATTRIB_ARRAY3_NV;
        if (!strcmp(name, "GL_VERTEX_ATTRIB_ARRAY4_NV")) return GL_VERTEX_ATTRIB_ARRAY4_NV;
        if (!strcmp(name, "GL_VERTEX_ATTRIB_ARRAY5_NV")) return GL_VERTEX_ATTRIB_ARRAY5_NV;
        if (!strcmp(name, "GL_VERTEX_ATTRIB_ARRAY6_NV")) return GL_VERTEX_ATTRIB_ARRAY6_NV;
        if (!strcmp(name, "GL_VERTEX_ATTRIB_ARRAY7_NV")) return GL_VERTEX_ATTRIB_ARRAY7_NV;
        if (!strcmp(name, "GL_VERTEX_ATTRIB_ARRAY8_NV")) return GL_VERTEX_ATTRIB_ARRAY8_NV;
        if (!strcmp(name, "GL_VERTEX_ATTRIB_ARRAY9_NV")) return GL_VERTEX_ATTRIB_ARRAY9_NV;
        if (!strcmp(name, "GL_VERTEX_PROGRAM_TWO_SIDE")) return GL_VERTEX_PROGRAM_TWO_SIDE;
        if (!strcmp(name, "GL_VERTEX_STATE_PROGRAM_NV")) return GL_VERTEX_STATE_PROGRAM_NV;
        if (!strcmp(name, "GL_VERTEX_WEIGHT_ARRAY_EXT")) return GL_VERTEX_WEIGHT_ARRAY_EXT;
        if (!strcmp(name, "GL_WEIGHT_ARRAY_STRIDE_ARB")) return GL_WEIGHT_ARRAY_STRIDE_ARB;
        return _GL_UNDEFINED;
    case 27:
        if (!strcmp(name, "GL_ALIASED_LINE_WIDTH_RANGE")) return GL_ALIASED_LINE_WIDTH_RANGE;
        if (!strcmp(name, "GL_ALIASED_POINT_SIZE_RANGE")) return GL_ALIASED_POINT_SIZE_RANGE;
        if (!strcmp(name, "GL_APPLE_flush_buffer_range")) return GL_APPLE_flush_buffer_range;
        if (!strcmp(name, "GL_APPLE_vertex_array_range")) return GL_APPLE_vertex_array_range;
        if (!strcmp(name, "GL_ARB_shading_language_100")) return GL_ARB_shading_language_100;
        if (!strcmp(name, "GL_ARB_texture_border_clamp")) return GL_ARB_texture_border_clamp;
        if (!strcmp(name, "GL_ARB_texture_env_crossbar")) return GL_ARB_texture_env_crossbar;
        if (!strcmp(name, "GL_ARB_vertex_buffer_object")) return GL_ARB_vertex_buffer_object;
        if (!strcmp(name, "GL_ARRAY_BUFFER_BINDING_ARB")) return GL_ARRAY_BUFFER_BINDING_ARB;
        if (!strcmp(name, "GL_ATI_text_fragment_shader")) return GL_ATI_text_fragment_shader;
        if (!strcmp(name, "GL_BLEND_EQUATION_ALPHA_EXT")) return GL_BLEND_EQUATION_ALPHA_EXT;
        if (!strcmp(name, "GL_BUMP_ROT_MATRIX_SIZE_ATI")) return GL_BUMP_ROT_MATRIX_SIZE_ATI;
        if (!strcmp(name, "GL_CLAMP_FRAGMENT_COLOR_ARB")) return GL_CLAMP_FRAGMENT_COLOR_ARB;
        if (!strcmp(name, "GL_COLOR_MATERIAL_PARAMETER")) return GL_COLOR_MATERIAL_PARAMETER;
        if (!strcmp(name, "GL_COLOR_MATRIX_STACK_DEPTH")) return GL_COLOR_MATRIX_STACK_DEPTH;
        if (!strcmp(name, "GL_COLOR_TABLE_RED_SIZE_EXT")) return GL_COLOR_TABLE_RED_SIZE_EXT;
        if (!strcmp(name, "GL_COLOR_TABLE_RED_SIZE_SGI")) return GL_COLOR_TABLE_RED_SIZE_SGI;
        if (!strcmp(name, "GL_COMPARE_R_TO_TEXTURE_ARB")) return GL_COMPARE_R_TO_TEXTURE_ARB;
        if (!strcmp(name, "GL_COMPRESSED_INTENSITY_ARB")) return GL_COMPRESSED_INTENSITY_ARB;
        if (!strcmp(name, "GL_COMPRESSED_LUMINANCE_ARB")) return GL_COMPRESSED_LUMINANCE_ARB;
        if (!strcmp(name, "GL_COMPRESSED_RED_RGTC1_EXT")) return GL_COMPRESSED_RED_RGTC1_EXT;
        if (!strcmp(name, "GL_COMPRESSED_RGB_FXT1_3DFX")) return GL_COMPRESSED_RGB_FXT1_3DFX;
        if (!strcmp(name, "GL_CONSERVE_MEMORY_HINT_PGI")) return GL_CONSERVE_MEMORY_HINT_PGI;
        if (!strcmp(name, "GL_CONVOLUTION_BORDER_COLOR")) return GL_CONVOLUTION_BORDER_COLOR;
        if (!strcmp(name, "GL_CONVOLUTION_FILTER_SCALE")) return GL_CONVOLUTION_FILTER_SCALE;
        if (!strcmp(name, "GL_CURRENT_MATRIX_INDEX_ARB")) return GL_CURRENT_MATRIX_INDEX_ARB;
        if (!strcmp(name, "GL_DEPTH_STENCIL_ATTACHMENT")) return GL_DEPTH_STENCIL_ATTACHMENT;
        if (!strcmp(name, "GL_DEPTH_STENCIL_TO_BGRA_NV")) return GL_DEPTH_STENCIL_TO_BGRA_NV;
        if (!strcmp(name, "GL_DEPTH_STENCIL_TO_RGBA_NV")) return GL_DEPTH_STENCIL_TO_RGBA_NV;
        if (!strcmp(name, "GL_DISTANCE_ATTENUATION_EXT")) return GL_DISTANCE_ATTENUATION_EXT;
        if (!strcmp(name, "GL_DRAW_FRAMEBUFFER_BINDING")) return GL_DRAW_FRAMEBUFFER_BINDING;
        if (!strcmp(name, "GL_DSDT8_MAG8_INTENSITY8_NV")) return GL_DSDT8_MAG8_INTENSITY8_NV;
        if (!strcmp(name, "GL_DSDT8_MAG8_INTENSITY8_NV")) return GL_DSDT8_MAG8_INTENSITY8_NV;
        if (!strcmp(name, "GL_EDGE_FLAG_ARRAY_LIST_IBM")) return GL_EDGE_FLAG_ARRAY_LIST_IBM;
        if (!strcmp(name, "GL_ELEMENT_ARRAY_BUFFER_ARB")) return GL_ELEMENT_ARRAY_BUFFER_ARB;
        if (!strcmp(name, "GL_ELEMENT_ARRAY_TYPE_APPLE")) return GL_ELEMENT_ARRAY_TYPE_APPLE;
        if (!strcmp(name, "GL_ENV_REVERSE_SUBTRACT_EXT")) return GL_ENV_REVERSE_SUBTRACT_EXT;
        if (!strcmp(name, "GL_EXT_packed_depth_stencil")) return GL_EXT_packed_depth_stencil;
        if (!strcmp(name, "GL_EXT_texture_mirror_clamp")) return GL_EXT_texture_mirror_clamp;
        if (!strcmp(name, "GL_FOG_COORDINATE_ARRAY_EXT")) return GL_FOG_COORDINATE_ARRAY_EXT;
        if (!strcmp(name, "GL_FOG_SPECULAR_TEXTURE_WIN")) return GL_FOG_SPECULAR_TEXTURE_WIN;
        if (!strcmp(name, "GL_FRAMEBUFFER_COMPLETE_EXT")) return GL_FRAMEBUFFER_COMPLETE_EXT;
        if (!strcmp(name, "GL_GEOMETRY_OUTPUT_TYPE_ARB")) return GL_GEOMETRY_OUTPUT_TYPE_ARB;
        if (!strcmp(name, "GL_GEOMETRY_OUTPUT_TYPE_EXT")) return GL_GEOMETRY_OUTPUT_TYPE_EXT;
        if (!strcmp(name, "GL_GREMEDY_frame_terminator")) return GL_GREMEDY_frame_terminator;
        if (!strcmp(name, "GL_HISTOGRAM_ALPHA_SIZE_EXT")) return GL_HISTOGRAM_ALPHA_SIZE_EXT;
        if (!strcmp(name, "GL_HISTOGRAM_GREEN_SIZE_EXT")) return GL_HISTOGRAM_GREEN_SIZE_EXT;
        if (!strcmp(name, "GL_HISTOGRAM_LUMINANCE_SIZE")) return GL_HISTOGRAM_LUMINANCE_SIZE;
        if (!strcmp(name, "GL_INT_SAMPLER_1D_ARRAY_EXT")) return GL_INT_SAMPLER_1D_ARRAY_EXT;
        if (!strcmp(name, "GL_INT_SAMPLER_2D_ARRAY_EXT")) return GL_INT_SAMPLER_2D_ARRAY_EXT;
        if (!strcmp(name, "GL_LIGHT_MODEL_LOCAL_VIEWER")) return GL_LIGHT_MODEL_LOCAL_VIEWER;
        if (!strcmp(name, "GL_LINE_STRIP_ADJACENCY_ARB")) return GL_LINE_STRIP_ADJACENCY_ARB;
        if (!strcmp(name, "GL_LINE_STRIP_ADJACENCY_EXT")) return GL_LINE_STRIP_ADJACENCY_EXT;
        if (!strcmp(name, "GL_LOCAL_CONSTANT_VALUE_EXT")) return GL_LOCAL_CONSTANT_VALUE_EXT;
        if (!strcmp(name, "GL_MAP1_VERTEX_ATTRIB0_4_NV")) return GL_MAP1_VERTEX_ATTRIB0_4_NV;
        if (!strcmp(name, "GL_MAP1_VERTEX_ATTRIB1_4_NV")) return GL_MAP1_VERTEX_ATTRIB1_4_NV;
        if (!strcmp(name, "GL_MAP1_VERTEX_ATTRIB2_4_NV")) return GL_MAP1_VERTEX_ATTRIB2_4_NV;
        if (!strcmp(name, "GL_MAP1_VERTEX_ATTRIB3_4_NV")) return GL_MAP1_VERTEX_ATTRIB3_4_NV;
        if (!strcmp(name, "GL_MAP1_VERTEX_ATTRIB4_4_NV")) return GL_MAP1_VERTEX_ATTRIB4_4_NV;
        if (!strcmp(name, "GL_MAP1_VERTEX_ATTRIB5_4_NV")) return GL_MAP1_VERTEX_ATTRIB5_4_NV;
        if (!strcmp(name, "GL_MAP1_VERTEX_ATTRIB6_4_NV")) return GL_MAP1_VERTEX_ATTRIB6_4_NV;
        if (!strcmp(name, "GL_MAP1_VERTEX_ATTRIB7_4_NV")) return GL_MAP1_VERTEX_ATTRIB7_4_NV;
        if (!strcmp(name, "GL_MAP1_VERTEX_ATTRIB8_4_NV")) return GL_MAP1_VERTEX_ATTRIB8_4_NV;
        if (!strcmp(name, "GL_MAP1_VERTEX_ATTRIB9_4_NV")) return GL_MAP1_VERTEX_ATTRIB9_4_NV;
        if (!strcmp(name, "GL_MAP2_VERTEX_ATTRIB0_4_NV")) return GL_MAP2_VERTEX_ATTRIB0_4_NV;
        if (!strcmp(name, "GL_MAP2_VERTEX_ATTRIB1_4_NV")) return GL_MAP2_VERTEX_ATTRIB1_4_NV;
        if (!strcmp(name, "GL_MAP2_VERTEX_ATTRIB2_4_NV")) return GL_MAP2_VERTEX_ATTRIB2_4_NV;
        if (!strcmp(name, "GL_MAP2_VERTEX_ATTRIB3_4_NV")) return GL_MAP2_VERTEX_ATTRIB3_4_NV;
        if (!strcmp(name, "GL_MAP2_VERTEX_ATTRIB4_4_NV")) return GL_MAP2_VERTEX_ATTRIB4_4_NV;
        if (!strcmp(name, "GL_MAP2_VERTEX_ATTRIB5_4_NV")) return GL_MAP2_VERTEX_ATTRIB5_4_NV;
        if (!strcmp(name, "GL_MAP2_VERTEX_ATTRIB6_4_NV")) return GL_MAP2_VERTEX_ATTRIB6_4_NV;
        if (!strcmp(name, "GL_MAP2_VERTEX_ATTRIB7_4_NV")) return GL_MAP2_VERTEX_ATTRIB7_4_NV;
        if (!strcmp(name, "GL_MAP2_VERTEX_ATTRIB8_4_NV")) return GL_MAP2_VERTEX_ATTRIB8_4_NV;
        if (!strcmp(name, "GL_MAP2_VERTEX_ATTRIB9_4_NV")) return GL_MAP2_VERTEX_ATTRIB9_4_NV;
        if (!strcmp(name, "GL_MAP_INVALIDATE_RANGE_BIT")) return GL_MAP_INVALIDATE_RANGE_BIT;
        if (!strcmp(name, "GL_MAX_ARRAY_TEXTURE_LAYERS")) return GL_MAX_ARRAY_TEXTURE_LAYERS;
        if (!strcmp(name, "GL_MAX_ASYNC_HISTOGRAM_SGIX")) return GL_MAX_ASYNC_HISTOGRAM_SGIX;
        if (!strcmp(name, "GL_MAX_ASYNC_TEX_IMAGE_SGIX")) return GL_MAX_ASYNC_TEX_IMAGE_SGIX;
        if (!strcmp(name, "GL_MAX_GENERAL_COMBINERS_NV")) return GL_MAX_GENERAL_COMBINERS_NV;
        if (!strcmp(name, "GL_MAX_PALETTE_MATRICES_ARB")) return GL_MAX_PALETTE_MATRICES_ARB;
        if (!strcmp(name, "GL_MAX_PROGRAM_MATRICES_ARB")) return GL_MAX_PROGRAM_MATRICES_ARB;
        if (!strcmp(name, "GL_MAX_PROGRAM_TEXEL_OFFSET")) return GL_MAX_PROGRAM_TEXEL_OFFSET;
        if (!strcmp(name, "GL_MAX_SAMPLE_MASK_WORDS_NV")) return GL_MAX_SAMPLE_MASK_WORDS_NV;
        if (!strcmp(name, "GL_MAX_TEXTURE_LOD_BIAS_EXT")) return GL_MAX_TEXTURE_LOD_BIAS_EXT;
        if (!strcmp(name, "GL_MIN_PROGRAM_TEXEL_OFFSET")) return GL_MIN_PROGRAM_TEXEL_OFFSET;
        if (!strcmp(name, "GL_MIRROR_CLAMP_TO_EDGE_ATI")) return GL_MIRROR_CLAMP_TO_EDGE_ATI;
        if (!strcmp(name, "GL_MIRROR_CLAMP_TO_EDGE_EXT")) return GL_MIRROR_CLAMP_TO_EDGE_EXT;
        if (!strcmp(name, "GL_MODULATE_SIGNED_ADD_ATIX")) return GL_MODULATE_SIGNED_ADD_ATIX;
        if (!strcmp(name, "GL_NORMAL_ARRAY_POINTER_EXT")) return GL_NORMAL_ARRAY_POINTER_EXT;
        if (!strcmp(name, "GL_NUM_GENERAL_COMBINERS_NV")) return GL_NUM_GENERAL_COMBINERS_NV;
        if (!strcmp(name, "GL_NV_depth_range_unclamped")) return GL_NV_depth_range_unclamped;
        if (!strcmp(name, "GL_NV_texture_expand_normal")) return GL_NV_texture_expand_normal;
        if (!strcmp(name, "GL_OBJECT_DELETE_STATUS_ARB")) return GL_OBJECT_DELETE_STATUS_ARB;
        if (!strcmp(name, "GL_OCCLUSION_TEST_RESULT_HP")) return GL_OCCLUSION_TEST_RESULT_HP;
        if (!strcmp(name, "GL_OFFSET_TEXTURE_MATRIX_NV")) return GL_OFFSET_TEXTURE_MATRIX_NV;
        if (!strcmp(name, "GL_ONE_MINUS_CONSTANT_ALPHA")) return GL_ONE_MINUS_CONSTANT_ALPHA;
        if (!strcmp(name, "GL_ONE_MINUS_CONSTANT_COLOR")) return GL_ONE_MINUS_CONSTANT_COLOR;
        if (!strcmp(name, "GL_PIXEL_COUNT_AVAILABLE_NV")) return GL_PIXEL_COUNT_AVAILABLE_NV;
        if (!strcmp(name, "GL_PRIMITIVES_GENERATED_EXT")) return GL_PRIMITIVES_GENERATED_EXT;
        if (!strcmp(name, "GL_PROGRAM_ERROR_STRING_ARB")) return GL_PROGRAM_ERROR_STRING_ARB;
        if (!strcmp(name, "GL_PROGRAM_FORMAT_ASCII_ARB")) return GL_PROGRAM_FORMAT_ASCII_ARB;
        if (!strcmp(name, "GL_PROGRAM_INSTRUCTIONS_ARB")) return GL_PROGRAM_INSTRUCTIONS_ARB;
        if (!strcmp(name, "GL_R1UI_T2F_C4F_N3F_V3F_SUN")) return GL_R1UI_T2F_C4F_N3F_V3F_SUN;
        if (!strcmp(name, "GL_READ_FRAMEBUFFER_BINDING")) return GL_READ_FRAMEBUFFER_BINDING;
        if (!strcmp(name, "GL_READ_PIXEL_DATA_RANGE_NV")) return GL_READ_PIXEL_DATA_RANGE_NV;
        if (!strcmp(name, "GL_RENDERBUFFER_BINDING_EXT")) return GL_RENDERBUFFER_BINDING_EXT;
        if (!strcmp(name, "GL_RENDERBUFFER_SAMPLES_EXT")) return GL_RENDERBUFFER_SAMPLES_EXT;
        if (!strcmp(name, "GL_RGBA_EXTENDED_RANGE_SGIX")) return GL_RGBA_EXTENDED_RANGE_SGIX;
        if (!strcmp(name, "GL_SAMPLE_ALPHA_TO_COVERAGE")) return GL_SAMPLE_ALPHA_TO_COVERAGE;
        if (!strcmp(name, "GL_SAMPLE_ALPHA_TO_MASK_EXT")) return GL_SAMPLE_ALPHA_TO_MASK_EXT;
        if (!strcmp(name, "GL_SAMPLE_ALPHA_TO_ONE_SGIS")) return GL_SAMPLE_ALPHA_TO_ONE_SGIS;
        if (!strcmp(name, "GL_SELECTION_BUFFER_POINTER")) return GL_SELECTION_BUFFER_POINTER;
        if (!strcmp(name, "GL_SGIX_vertex_preclip_hint")) return GL_SGIX_vertex_preclip_hint;
        if (!strcmp(name, "GL_SHADING_LANGUAGE_VERSION")) return GL_SHADING_LANGUAGE_VERSION;
        if (!strcmp(name, "GL_STRICT_LIGHTING_HINT_PGI")) return GL_STRICT_LIGHTING_HINT_PGI;
        if (!strcmp(name, "GL_TANGENT_ARRAY_STRIDE_EXT")) return GL_TANGENT_ARRAY_STRIDE_EXT;
        if (!strcmp(name, "GL_TEXT_FRAGMENT_SHADER_ATI")) return GL_TEXT_FRAGMENT_SHADER_ATI;
        if (!strcmp(name, "GL_TEXTURE_BINDING_1D_ARRAY")) return GL_TEXTURE_BINDING_1D_ARRAY;
        if (!strcmp(name, "GL_TEXTURE_BINDING_2D_ARRAY")) return GL_TEXTURE_BINDING_2D_ARRAY;
        if (!strcmp(name, "GL_TEXTURE_BINDING_CUBE_MAP")) return GL_TEXTURE_BINDING_CUBE_MAP;
        if (!strcmp(name, "GL_TEXTURE_BORDER_VALUES_NV")) return GL_TEXTURE_BORDER_VALUES_NV;
        if (!strcmp(name, "GL_TEXTURE_COMPARE_FUNC_ARB")) return GL_TEXTURE_COMPARE_FUNC_ARB;
        if (!strcmp(name, "GL_TEXTURE_COMPARE_MODE_ARB")) return GL_TEXTURE_COMPARE_MODE_ARB;
        if (!strcmp(name, "GL_TEXTURE_COMPRESSION_HINT")) return GL_TEXTURE_COMPRESSION_HINT;
        if (!strcmp(name, "GL_TEXTURE_COORD_ARRAY_SIZE")) return GL_TEXTURE_COORD_ARRAY_SIZE;
        if (!strcmp(name, "GL_TEXTURE_COORD_ARRAY_TYPE")) return GL_TEXTURE_COORD_ARRAY_TYPE;
        if (!strcmp(name, "GL_TEXTURE_MAX_CLAMP_R_SGIX")) return GL_TEXTURE_MAX_CLAMP_R_SGIX;
        if (!strcmp(name, "GL_TEXTURE_MAX_CLAMP_S_SGIX")) return GL_TEXTURE_MAX_CLAMP_S_SGIX;
        if (!strcmp(name, "GL_TEXTURE_MAX_CLAMP_T_SGIX")) return GL_TEXTURE_MAX_CLAMP_T_SGIX;
        if (!strcmp(name, "GL_TEXTURE_STENCIL_SIZE_EXT")) return GL_TEXTURE_STENCIL_SIZE_EXT;
        if (!strcmp(name, "GL_TEXTURE_SWIZZLE_RGBA_EXT")) return GL_TEXTURE_SWIZZLE_RGBA_EXT;
        if (!strcmp(name, "GL_TRANSPOSE_TEXTURE_MATRIX")) return GL_TRANSPOSE_TEXTURE_MATRIX;
        if (!strcmp(name, "GL_UNSIGNED_INT_5_9_9_9_REV")) return GL_UNSIGNED_INT_5_9_9_9_REV;
        if (!strcmp(name, "GL_UNSIGNED_INT_8_8_8_8_EXT")) return GL_UNSIGNED_INT_8_8_8_8_EXT;
        if (!strcmp(name, "GL_UNSIGNED_INT_8_8_8_8_REV")) return GL_UNSIGNED_INT_8_8_8_8_REV;
        if (!strcmp(name, "GL_UNSIGNED_SHORT_5_6_5_REV")) return GL_UNSIGNED_SHORT_5_6_5_REV;
        if (!strcmp(name, "GL_UNSIGNED_SHORT_8_8_APPLE")) return GL_UNSIGNED_SHORT_8_8_APPLE;
        if (!strcmp(name, "GL_VARIANT_ARRAY_STRIDE_EXT")) return GL_VARIANT_ARRAY_STRIDE_EXT;
        if (!strcmp(name, "GL_VERTEX_ARRAY_POINTER_EXT")) return GL_VERTEX_ARRAY_POINTER_EXT;
        if (!strcmp(name, "GL_VERTEX_ARRAY_RANGE_APPLE")) return GL_VERTEX_ARRAY_RANGE_APPLE;
        if (!strcmp(name, "GL_VERTEX_ATTRIB_ARRAY10_NV")) return GL_VERTEX_ATTRIB_ARRAY10_NV;
        if (!strcmp(name, "GL_VERTEX_ATTRIB_ARRAY11_NV")) return GL_VERTEX_ATTRIB_ARRAY11_NV;
        if (!strcmp(name, "GL_VERTEX_ATTRIB_ARRAY12_NV")) return GL_VERTEX_ATTRIB_ARRAY12_NV;
        if (!strcmp(name, "GL_VERTEX_ATTRIB_ARRAY13_NV")) return GL_VERTEX_ATTRIB_ARRAY13_NV;
        if (!strcmp(name, "GL_VERTEX_ATTRIB_ARRAY14_NV")) return GL_VERTEX_ATTRIB_ARRAY14_NV;
        if (!strcmp(name, "GL_VERTEX_ATTRIB_ARRAY15_NV")) return GL_VERTEX_ATTRIB_ARRAY15_NV;
        if (!strcmp(name, "GL_VERTEX_ATTRIB_ARRAY_SIZE")) return GL_VERTEX_ATTRIB_ARRAY_SIZE;
        if (!strcmp(name, "GL_VERTEX_ATTRIB_ARRAY_TYPE")) return GL_VERTEX_ATTRIB_ARRAY_TYPE;
        if (!strcmp(name, "GL_VERTEX_PRECLIP_HINT_SGIX")) return GL_VERTEX_PRECLIP_HINT_SGIX;
        if (!strcmp(name, "GL_VERTEX_PRECLIP_HINT_SGIX")) return GL_VERTEX_PRECLIP_HINT_SGIX;
        if (!strcmp(name, "GL_VERTEX_SHADER_LOCALS_EXT")) return GL_VERTEX_SHADER_LOCALS_EXT;
        if (!strcmp(name, "GL_WEIGHT_ARRAY_POINTER_ARB")) return GL_WEIGHT_ARRAY_POINTER_ARB;
        return _GL_UNDEFINED;
    case 28:
        if (!strcmp(name, "GL_ACTIVE_UNIFORM_MAX_LENGTH")) return GL_ACTIVE_UNIFORM_MAX_LENGTH;
        if (!strcmp(name, "GL_ALPHA_EXTENDED_RANGE_SGIX")) return GL_ALPHA_EXTENDED_RANGE_SGIX;
        if (!strcmp(name, "GL_APPLE_vertex_array_object")) return GL_APPLE_vertex_array_object;
        if (!strcmp(name, "GL_ARB_texture_buffer_object")) return GL_ARB_texture_buffer_object;
        if (!strcmp(name, "GL_ATIX_texture_env_combine3")) return GL_ATIX_texture_env_combine3;
        if (!strcmp(name, "GL_BINORMAL_ARRAY_STRIDE_EXT")) return GL_BINORMAL_ARRAY_STRIDE_EXT;
        if (!strcmp(name, "GL_CLIENT_ACTIVE_TEXTURE_ARB")) return GL_CLIENT_ACTIVE_TEXTURE_ARB;
        if (!strcmp(name, "GL_CLIENT_ATTRIB_STACK_DEPTH")) return GL_CLIENT_ATTRIB_STACK_DEPTH;
        if (!strcmp(name, "GL_COLOR_TABLE_BLUE_SIZE_EXT")) return GL_COLOR_TABLE_BLUE_SIZE_EXT;
        if (!strcmp(name, "GL_COLOR_TABLE_BLUE_SIZE_SGI")) return GL_COLOR_TABLE_BLUE_SIZE_SGI;
        if (!strcmp(name, "GL_COMPRESSED_RGBA_FXT1_3DFX")) return GL_COMPRESSED_RGBA_FXT1_3DFX;
        if (!strcmp(name, "GL_COMPRESSED_SLUMINANCE_EXT")) return GL_COMPRESSED_SLUMINANCE_EXT;
        if (!strcmp(name, "GL_COMPRESSED_SRGB_ALPHA_EXT")) return GL_COMPRESSED_SRGB_ALPHA_EXT;
        if (!strcmp(name, "GL_CURRENT_RASTER_NORMAL_EXT")) return GL_CURRENT_RASTER_NORMAL_EXT;
        if (!strcmp(name, "GL_CURRENT_VERTEX_ATTRIB_ARB")) return GL_CURRENT_VERTEX_ATTRIB_ARB;
        if (!strcmp(name, "GL_CURRENT_VERTEX_WEIGHT_EXT")) return GL_CURRENT_VERTEX_WEIGHT_EXT;
        if (!strcmp(name, "GL_DOT_PRODUCT_TEXTURE_1D_NV")) return GL_DOT_PRODUCT_TEXTURE_1D_NV;
        if (!strcmp(name, "GL_DOT_PRODUCT_TEXTURE_2D_NV")) return GL_DOT_PRODUCT_TEXTURE_2D_NV;
        if (!strcmp(name, "GL_DOT_PRODUCT_TEXTURE_3D_NV")) return GL_DOT_PRODUCT_TEXTURE_3D_NV;
        if (!strcmp(name, "GL_EDGE_FLAG_ARRAY_COUNT_EXT")) return GL_EDGE_FLAG_ARRAY_COUNT_EXT;
        if (!strcmp(name, "GL_ELEMENT_ARRAY_POINTER_ATI")) return GL_ELEMENT_ARRAY_POINTER_ATI;
        if (!strcmp(name, "GL_EXT_compiled_vertex_array")) return GL_EXT_compiled_vertex_array;
        if (!strcmp(name, "GL_EXT_texture_buffer_object")) return GL_EXT_texture_buffer_object;
        if (!strcmp(name, "GL_EYE_DISTANCE_TO_LINE_SGIS")) return GL_EYE_DISTANCE_TO_LINE_SGIS;
        if (!strcmp(name, "GL_FOG_COORDINATE_ARRAY_TYPE")) return GL_FOG_COORDINATE_ARRAY_TYPE;
        if (!strcmp(name, "GL_FOG_COORDINATE_SOURCE_EXT")) return GL_FOG_COORDINATE_SOURCE_EXT;
        if (!strcmp(name, "GL_FUNC_REVERSE_SUBTRACT_EXT")) return GL_FUNC_REVERSE_SUBTRACT_EXT;
        if (!strcmp(name, "GL_GENERATE_MIPMAP_HINT_SGIS")) return GL_GENERATE_MIPMAP_HINT_SGIS;
        if (!strcmp(name, "GL_GEOMETRY_VERTICES_OUT_ARB")) return GL_GEOMETRY_VERTICES_OUT_ARB;
        if (!strcmp(name, "GL_GEOMETRY_VERTICES_OUT_EXT")) return GL_GEOMETRY_VERTICES_OUT_EXT;
        if (!strcmp(name, "GL_IBM_multimode_draw_arrays")) return GL_IBM_multimode_draw_arrays;
        if (!strcmp(name, "GL_LIGHT_MODEL_COLOR_CONTROL")) return GL_LIGHT_MODEL_COLOR_CONTROL;
        if (!strcmp(name, "GL_MAP1_VERTEX_ATTRIB10_4_NV")) return GL_MAP1_VERTEX_ATTRIB10_4_NV;
        if (!strcmp(name, "GL_MAP1_VERTEX_ATTRIB11_4_NV")) return GL_MAP1_VERTEX_ATTRIB11_4_NV;
        if (!strcmp(name, "GL_MAP1_VERTEX_ATTRIB12_4_NV")) return GL_MAP1_VERTEX_ATTRIB12_4_NV;
        if (!strcmp(name, "GL_MAP1_VERTEX_ATTRIB13_4_NV")) return GL_MAP1_VERTEX_ATTRIB13_4_NV;
        if (!strcmp(name, "GL_MAP1_VERTEX_ATTRIB14_4_NV")) return GL_MAP1_VERTEX_ATTRIB14_4_NV;
        if (!strcmp(name, "GL_MAP1_VERTEX_ATTRIB15_4_NV")) return GL_MAP1_VERTEX_ATTRIB15_4_NV;
        if (!strcmp(name, "GL_MAP2_VERTEX_ATTRIB10_4_NV")) return GL_MAP2_VERTEX_ATTRIB10_4_NV;
        if (!strcmp(name, "GL_MAP2_VERTEX_ATTRIB11_4_NV")) return GL_MAP2_VERTEX_ATTRIB11_4_NV;
        if (!strcmp(name, "GL_MAP2_VERTEX_ATTRIB12_4_NV")) return GL_MAP2_VERTEX_ATTRIB12_4_NV;
        if (!strcmp(name, "GL_MAP2_VERTEX_ATTRIB13_4_NV")) return GL_MAP2_VERTEX_ATTRIB13_4_NV;
        if (!strcmp(name, "GL_MAP2_VERTEX_ATTRIB14_4_NV")) return GL_MAP2_VERTEX_ATTRIB14_4_NV;
        if (!strcmp(name, "GL_MAP2_VERTEX_ATTRIB15_4_NV")) return GL_MAP2_VERTEX_ATTRIB15_4_NV;
        if (!strcmp(name, "GL_MAP_INVALIDATE_BUFFER_BIT")) return GL_MAP_INVALIDATE_BUFFER_BIT;
        if (!strcmp(name, "GL_MAT_COLOR_INDEXES_BIT_PGI")) return GL_MAT_COLOR_INDEXES_BIT_PGI;
        if (!strcmp(name, "GL_MAX_COLOR_ATTACHMENTS_EXT")) return GL_MAX_COLOR_ATTACHMENTS_EXT;
        if (!strcmp(name, "GL_MAX_CONVOLUTION_WIDTH_EXT")) return GL_MAX_CONVOLUTION_WIDTH_EXT;
        if (!strcmp(name, "GL_MAX_CUBE_MAP_TEXTURE_SIZE")) return GL_MAX_CUBE_MAP_TEXTURE_SIZE;
        if (!strcmp(name, "GL_MAX_MODELVIEW_STACK_DEPTH")) return GL_MAX_MODELVIEW_STACK_DEPTH;
        if (!strcmp(name, "GL_MAX_PROGRAM_CALL_DEPTH_NV")) return GL_MAX_PROGRAM_CALL_DEPTH_NV;
        if (!strcmp(name, "GL_MAX_PROGRAM_CALL_DEPTH_NV")) return GL_MAX_PROGRAM_CALL_DEPTH_NV;
        if (!strcmp(name, "GL_MAX_PROGRAM_LOOP_COUNT_NV")) return GL_MAX_PROGRAM_LOOP_COUNT_NV;
        if (!strcmp(name, "GL_MAX_PROGRAM_LOOP_DEPTH_NV")) return GL_MAX_PROGRAM_LOOP_DEPTH_NV;
        if (!strcmp(name, "GL_MAX_RENDERBUFFER_SIZE_EXT")) return GL_MAX_RENDERBUFFER_SIZE_EXT;
        if (!strcmp(name, "GL_NV_vertex_program2_option")) return GL_NV_vertex_program2_option;
        if (!strcmp(name, "GL_OBJECT_COMPILE_STATUS_ARB")) return GL_OBJECT_COMPILE_STATUS_ARB;
        if (!strcmp(name, "GL_OFFSET_HILO_TEXTURE_2D_NV")) return GL_OFFSET_HILO_TEXTURE_2D_NV;
        if (!strcmp(name, "GL_OFFSET_TEXTURE_2D_BIAS_NV")) return GL_OFFSET_TEXTURE_2D_BIAS_NV;
        if (!strcmp(name, "GL_OUTPUT_TEXTURE_COORD0_EXT")) return GL_OUTPUT_TEXTURE_COORD0_EXT;
        if (!strcmp(name, "GL_OUTPUT_TEXTURE_COORD1_EXT")) return GL_OUTPUT_TEXTURE_COORD1_EXT;
        if (!strcmp(name, "GL_OUTPUT_TEXTURE_COORD2_EXT")) return GL_OUTPUT_TEXTURE_COORD2_EXT;
        if (!strcmp(name, "GL_OUTPUT_TEXTURE_COORD3_EXT")) return GL_OUTPUT_TEXTURE_COORD3_EXT;
        if (!strcmp(name, "GL_OUTPUT_TEXTURE_COORD4_EXT")) return GL_OUTPUT_TEXTURE_COORD4_EXT;
        if (!strcmp(name, "GL_OUTPUT_TEXTURE_COORD5_EXT")) return GL_OUTPUT_TEXTURE_COORD5_EXT;
        if (!strcmp(name, "GL_OUTPUT_TEXTURE_COORD6_EXT")) return GL_OUTPUT_TEXTURE_COORD6_EXT;
        if (!strcmp(name, "GL_OUTPUT_TEXTURE_COORD7_EXT")) return GL_OUTPUT_TEXTURE_COORD7_EXT;
        if (!strcmp(name, "GL_OUTPUT_TEXTURE_COORD8_EXT")) return GL_OUTPUT_TEXTURE_COORD8_EXT;
        if (!strcmp(name, "GL_OUTPUT_TEXTURE_COORD9_EXT")) return GL_OUTPUT_TEXTURE_COORD9_EXT;
        if (!strcmp(name, "GL_PIXEL_PACK_BUFFER_BINDING")) return GL_PIXEL_PACK_BUFFER_BINDING;
        if (!strcmp(name, "GL_POINT_FADE_THRESHOLD_SIZE")) return GL_POINT_FADE_THRESHOLD_SIZE;
        if (!strcmp(name, "GL_POINT_SPRITE_COORD_ORIGIN")) return GL_POINT_SPRITE_COORD_ORIGIN;
        if (!strcmp(name, "GL_POLYGON_OFFSET_FACTOR_EXT")) return GL_POLYGON_OFFSET_FACTOR_EXT;
        if (!strcmp(name, "GL_POST_CONVOLUTION_RED_BIAS")) return GL_POST_CONVOLUTION_RED_BIAS;
        if (!strcmp(name, "GL_PREVIOUS_TEXTURE_INPUT_NV")) return GL_PREVIOUS_TEXTURE_INPUT_NV;
        if (!strcmp(name, "GL_PROGRAM_ERROR_POSITION_NV")) return GL_PROGRAM_ERROR_POSITION_NV;
        if (!strcmp(name, "GL_QUERY_RESULT_AVAILABLE_NV")) return GL_QUERY_RESULT_AVAILABLE_NV;
        if (!strcmp(name, "GL_RENDERBUFFER_RED_SIZE_EXT")) return GL_RENDERBUFFER_RED_SIZE_EXT;
        if (!strcmp(name, "GL_RENDERBUFFER_STENCIL_SIZE")) return GL_RENDERBUFFER_STENCIL_SIZE;
        if (!strcmp(name, "GL_RGB16_EXTENDED_RANGE_SGIX")) return GL_RGB16_EXTENDED_RANGE_SGIX;
        if (!strcmp(name, "GL_SAMPLE_ALPHA_TO_MASK_SGIS")) return GL_SAMPLE_ALPHA_TO_MASK_SGIS;
        if (!strcmp(name, "GL_SAMPLE_COVERAGE_VALUE_ARB")) return GL_SAMPLE_COVERAGE_VALUE_ARB;
        if (!strcmp(name, "GL_SECONDARY_COLOR_ARRAY_EXT")) return GL_SECONDARY_COLOR_ARRAY_EXT;
        if (!strcmp(name, "GL_SGIS_texture_border_clamp")) return GL_SGIS_texture_border_clamp;
        if (!strcmp(name, "GL_SGIX_convolution_accuracy")) return GL_SGIX_convolution_accuracy;
        if (!strcmp(name, "GL_SGIX_texture_multi_buffer")) return GL_SGIX_texture_multi_buffer;
        if (!strcmp(name, "GL_SIGNED_LUMINANCE_ALPHA_NV")) return GL_SIGNED_LUMINANCE_ALPHA_NV;
        if (!strcmp(name, "GL_SIGNED_LUMINANCE_ALPHA_NV")) return GL_SIGNED_LUMINANCE_ALPHA_NV;
        if (!strcmp(name, "GL_STENCIL_TEST_TWO_SIDE_EXT")) return GL_STENCIL_TEST_TWO_SIDE_EXT;
        if (!strcmp(name, "GL_STRICT_DEPTHFUNC_HINT_PGI")) return GL_STRICT_DEPTHFUNC_HINT_PGI;
        if (!strcmp(name, "GL_TANGENT_ARRAY_POINTER_EXT")) return GL_TANGENT_ARRAY_POINTER_EXT;
        if (!strcmp(name, "GL_TEXTURE_BUFFER_FORMAT_ARB")) return GL_TEXTURE_BUFFER_FORMAT_ARB;
        if (!strcmp(name, "GL_TEXTURE_BUFFER_FORMAT_EXT")) return GL_TEXTURE_BUFFER_FORMAT_EXT;
        if (!strcmp(name, "GL_TEXTURE_MATERIAL_FACE_EXT")) return GL_TEXTURE_MATERIAL_FACE_EXT;
        if (!strcmp(name, "GL_TEXTURE_OUTPUT_ALPHA_ATIX")) return GL_TEXTURE_OUTPUT_ALPHA_ATIX;
        if (!strcmp(name, "GL_TEXTURE_POINT_SPRITE_ATIX")) return GL_TEXTURE_POINT_SPRITE_ATIX;
        if (!strcmp(name, "GL_TRACK_MATRIX_TRANSFORM_NV")) return GL_TRACK_MATRIX_TRANSFORM_NV;
        if (!strcmp(name, "GL_TRANSFORM_FEEDBACK_BUFFER")) return GL_TRANSFORM_FEEDBACK_BUFFER;
        if (!strcmp(name, "GL_UNPACK_CONSTANT_DATA_SUNX")) return GL_UNPACK_CONSTANT_DATA_SUNX;
        if (!strcmp(name, "GL_UNSIGNED_INT_S8_S8_8_8_NV")) return GL_UNSIGNED_INT_S8_S8_8_8_NV;
        if (!strcmp(name, "GL_UNSIGNED_INT_S8_S8_8_8_NV")) return GL_UNSIGNED_INT_S8_S8_8_8_NV;
        if (!strcmp(name, "GL_UNSIGNED_INT_SAMPLER_CUBE")) return GL_UNSIGNED_INT_SAMPLER_CUBE;
        if (!strcmp(name, "GL_VARIANT_ARRAY_POINTER_EXT")) return GL_VARIANT_ARRAY_POINTER_EXT;
        if (!strcmp(name, "GL_VERTEX_PROGRAM_BINDING_NV")) return GL_VERTEX_PROGRAM_BINDING_NV;
        if (!strcmp(name, "GL_VERTEX_PROGRAM_POINT_SIZE")) return GL_VERTEX_PROGRAM_POINT_SIZE;
        if (!strcmp(name, "GL_VERTEX_SHADER_BINDING_EXT")) return GL_VERTEX_SHADER_BINDING_EXT;
        if (!strcmp(name, "GL_WRITE_PIXEL_DATA_RANGE_NV")) return GL_WRITE_PIXEL_DATA_RANGE_NV;
        return _GL_UNDEFINED;
    case 29:
        if (!strcmp(name, "GL_BINORMAL_ARRAY_POINTER_EXT")) return GL_BINORMAL_ARRAY_POINTER_EXT;
        if (!strcmp(name, "GL_COLOR_ARRAY_BUFFER_BINDING")) return GL_COLOR_ARRAY_BUFFER_BINDING;
        if (!strcmp(name, "GL_COLOR_TABLE_ALPHA_SIZE_EXT")) return GL_COLOR_TABLE_ALPHA_SIZE_EXT;
        if (!strcmp(name, "GL_COLOR_TABLE_ALPHA_SIZE_SGI")) return GL_COLOR_TABLE_ALPHA_SIZE_SGI;
        if (!strcmp(name, "GL_COLOR_TABLE_GREEN_SIZE_EXT")) return GL_COLOR_TABLE_GREEN_SIZE_EXT;
        if (!strcmp(name, "GL_COLOR_TABLE_GREEN_SIZE_SGI")) return GL_COLOR_TABLE_GREEN_SIZE_SGI;
        if (!strcmp(name, "GL_COLOR_TABLE_INTENSITY_SIZE")) return GL_COLOR_TABLE_INTENSITY_SIZE;
        if (!strcmp(name, "GL_COLOR_TABLE_LUMINANCE_SIZE")) return GL_COLOR_TABLE_LUMINANCE_SIZE;
        if (!strcmp(name, "GL_COMBINER_AB_DOT_PRODUCT_NV")) return GL_COMBINER_AB_DOT_PRODUCT_NV;
        if (!strcmp(name, "GL_COMBINER_CD_DOT_PRODUCT_NV")) return GL_COMBINER_CD_DOT_PRODUCT_NV;
        if (!strcmp(name, "GL_COMPRESSED_LUMINANCE_ALPHA")) return GL_COMPRESSED_LUMINANCE_ALPHA;
        if (!strcmp(name, "GL_COMPRESSED_SIGNED_RG_RGTC2")) return GL_COMPRESSED_SIGNED_RG_RGTC2;
        if (!strcmp(name, "GL_COMPRESSED_TEXTURE_FORMATS")) return GL_COMPRESSED_TEXTURE_FORMATS;
        if (!strcmp(name, "GL_CURRENT_FOG_COORDINATE_EXT")) return GL_CURRENT_FOG_COORDINATE_EXT;
        if (!strcmp(name, "GL_CURRENT_PALETTE_MATRIX_ARB")) return GL_CURRENT_PALETTE_MATRIX_ARB;
        if (!strcmp(name, "GL_DEPENDENT_AR_TEXTURE_2D_NV")) return GL_DEPENDENT_AR_TEXTURE_2D_NV;
        if (!strcmp(name, "GL_DEPENDENT_GB_TEXTURE_2D_NV")) return GL_DEPENDENT_GB_TEXTURE_2D_NV;
        if (!strcmp(name, "GL_DEPTH_BUFFER_FLOAT_MODE_NV")) return GL_DEPTH_BUFFER_FLOAT_MODE_NV;
        if (!strcmp(name, "GL_EDGE_FLAG_ARRAY_STRIDE_EXT")) return GL_EDGE_FLAG_ARRAY_STRIDE_EXT;
        if (!strcmp(name, "GL_EXT_gpu_program_parameters")) return GL_EXT_gpu_program_parameters;
        if (!strcmp(name, "GL_EXT_shared_texture_palette")) return GL_EXT_shared_texture_palette;
        if (!strcmp(name, "GL_EXT_texture_perturb_normal")) return GL_EXT_texture_perturb_normal;
        if (!strcmp(name, "GL_EYE_DISTANCE_TO_POINT_SGIS")) return GL_EYE_DISTANCE_TO_POINT_SGIS;
        if (!strcmp(name, "GL_FLOAT_CLEAR_COLOR_VALUE_NV")) return GL_FLOAT_CLEAR_COLOR_VALUE_NV;
        if (!strcmp(name, "GL_FORMAT_SUBSAMPLE_24_24_OML")) return GL_FORMAT_SUBSAMPLE_24_24_OML;
        if (!strcmp(name, "GL_INDEX_ARRAY_BUFFER_BINDING")) return GL_INDEX_ARRAY_BUFFER_BINDING;
        if (!strcmp(name, "GL_MAX_ASYNC_DRAW_PIXELS_SGIX")) return GL_MAX_ASYNC_DRAW_PIXELS_SGIX;
        if (!strcmp(name, "GL_MAX_ASYNC_READ_PIXELS_SGIX")) return GL_MAX_ASYNC_READ_PIXELS_SGIX;
        if (!strcmp(name, "GL_MAX_CONVOLUTION_HEIGHT_EXT")) return GL_MAX_CONVOLUTION_HEIGHT_EXT;
        if (!strcmp(name, "GL_MAX_PROGRAM_PARAMETERS_ARB")) return GL_MAX_PROGRAM_PARAMETERS_ARB;
        if (!strcmp(name, "GL_MAX_PROJECTION_STACK_DEPTH")) return GL_MAX_PROJECTION_STACK_DEPTH;
        if (!strcmp(name, "GL_MAX_RATIONAL_EVAL_ORDER_NV")) return GL_MAX_RATIONAL_EVAL_ORDER_NV;
        if (!strcmp(name, "GL_MAX_TEXTURE_IMAGE_UNITS_NV")) return GL_MAX_TEXTURE_IMAGE_UNITS_NV;
        if (!strcmp(name, "GL_MAX_VARYING_COMPONENTS_EXT")) return GL_MAX_VARYING_COMPONENTS_EXT;
        if (!strcmp(name, "GL_MIRROR_CLAMP_TO_BORDER_EXT")) return GL_MIRROR_CLAMP_TO_BORDER_EXT;
        if (!strcmp(name, "GL_MODELVIEW0_STACK_DEPTH_EXT")) return GL_MODELVIEW0_STACK_DEPTH_EXT;
        if (!strcmp(name, "GL_MODELVIEW1_STACK_DEPTH_EXT")) return GL_MODELVIEW1_STACK_DEPTH_EXT;
        if (!strcmp(name, "GL_MULTISAMPLE_FILTER_HINT_NV")) return GL_MULTISAMPLE_FILTER_HINT_NV;
        if (!strcmp(name, "GL_NATIVE_GRAPHICS_HANDLE_PGI")) return GL_NATIVE_GRAPHICS_HANDLE_PGI;
        if (!strcmp(name, "GL_NUM_FRAGMENT_CONSTANTS_ATI")) return GL_NUM_FRAGMENT_CONSTANTS_ATI;
        if (!strcmp(name, "GL_NUM_FRAGMENT_REGISTERS_ATI")) return GL_NUM_FRAGMENT_REGISTERS_ATI;
        if (!strcmp(name, "GL_NUM_INSTRUCTIONS_TOTAL_ATI")) return GL_NUM_INSTRUCTIONS_TOTAL_ATI;
        if (!strcmp(name, "GL_NV_fragment_program_option")) return GL_NV_fragment_program_option;
        if (!strcmp(name, "GL_NV_multisample_filter_hint")) return GL_NV_multisample_filter_hint;
        if (!strcmp(name, "GL_NV_parameter_buffer_object")) return GL_NV_parameter_buffer_object;
        if (!strcmp(name, "GL_NV_texture_compression_vtc")) return GL_NV_texture_compression_vtc;
        if (!strcmp(name, "GL_OBJECT_ACTIVE_UNIFORMS_ARB")) return GL_OBJECT_ACTIVE_UNIFORMS_ARB;
        if (!strcmp(name, "GL_OBJECT_INFO_LOG_LENGTH_ARB")) return GL_OBJECT_INFO_LOG_LENGTH_ARB;
        if (!strcmp(name, "GL_OBJECT_VALIDATE_STATUS_ARB")) return GL_OBJECT_VALIDATE_STATUS_ARB;
        if (!strcmp(name, "GL_OFFSET_TEXTURE_2D_SCALE_NV")) return GL_OFFSET_TEXTURE_2D_SCALE_NV;
        if (!strcmp(name, "GL_OUTPUT_TEXTURE_COORD10_EXT")) return GL_OUTPUT_TEXTURE_COORD10_EXT;
        if (!strcmp(name, "GL_OUTPUT_TEXTURE_COORD11_EXT")) return GL_OUTPUT_TEXTURE_COORD11_EXT;
        if (!strcmp(name, "GL_OUTPUT_TEXTURE_COORD12_EXT")) return GL_OUTPUT_TEXTURE_COORD12_EXT;
        if (!strcmp(name, "GL_OUTPUT_TEXTURE_COORD13_EXT")) return GL_OUTPUT_TEXTURE_COORD13_EXT;
        if (!strcmp(name, "GL_OUTPUT_TEXTURE_COORD14_EXT")) return GL_OUTPUT_TEXTURE_COORD14_EXT;
        if (!strcmp(name, "GL_OUTPUT_TEXTURE_COORD15_EXT")) return GL_OUTPUT_TEXTURE_COORD15_EXT;
        if (!strcmp(name, "GL_OUTPUT_TEXTURE_COORD16_EXT")) return GL_OUTPUT_TEXTURE_COORD16_EXT;
        if (!strcmp(name, "GL_OUTPUT_TEXTURE_COORD17_EXT")) return GL_OUTPUT_TEXTURE_COORD17_EXT;
        if (!strcmp(name, "GL_OUTPUT_TEXTURE_COORD18_EXT")) return GL_OUTPUT_TEXTURE_COORD18_EXT;
        if (!strcmp(name, "GL_OUTPUT_TEXTURE_COORD19_EXT")) return GL_OUTPUT_TEXTURE_COORD19_EXT;
        if (!strcmp(name, "GL_OUTPUT_TEXTURE_COORD20_EXT")) return GL_OUTPUT_TEXTURE_COORD20_EXT;
        if (!strcmp(name, "GL_OUTPUT_TEXTURE_COORD21_EXT")) return GL_OUTPUT_TEXTURE_COORD21_EXT;
        if (!strcmp(name, "GL_OUTPUT_TEXTURE_COORD22_EXT")) return GL_OUTPUT_TEXTURE_COORD22_EXT;
        if (!strcmp(name, "GL_OUTPUT_TEXTURE_COORD23_EXT")) return GL_OUTPUT_TEXTURE_COORD23_EXT;
        if (!strcmp(name, "GL_OUTPUT_TEXTURE_COORD24_EXT")) return GL_OUTPUT_TEXTURE_COORD24_EXT;
        if (!strcmp(name, "GL_OUTPUT_TEXTURE_COORD25_EXT")) return GL_OUTPUT_TEXTURE_COORD25_EXT;
        if (!strcmp(name, "GL_OUTPUT_TEXTURE_COORD26_EXT")) return GL_OUTPUT_TEXTURE_COORD26_EXT;
        if (!strcmp(name, "GL_OUTPUT_TEXTURE_COORD27_EXT")) return GL_OUTPUT_TEXTURE_COORD27_EXT;
        if (!strcmp(name, "GL_OUTPUT_TEXTURE_COORD28_EXT")) return GL_OUTPUT_TEXTURE_COORD28_EXT;
        if (!strcmp(name, "GL_OUTPUT_TEXTURE_COORD29_EXT")) return GL_OUTPUT_TEXTURE_COORD29_EXT;
        if (!strcmp(name, "GL_OUTPUT_TEXTURE_COORD30_EXT")) return GL_OUTPUT_TEXTURE_COORD30_EXT;
        if (!strcmp(name, "GL_OUTPUT_TEXTURE_COORD31_EXT")) return GL_OUTPUT_TEXTURE_COORD31_EXT;
        if (!strcmp(name, "GL_POINT_DISTANCE_ATTENUATION")) return GL_POINT_DISTANCE_ATTENUATION;
        if (!strcmp(name, "GL_POST_COLOR_MATRIX_RED_BIAS")) return GL_POST_COLOR_MATRIX_RED_BIAS;
        if (!strcmp(name, "GL_POST_CONVOLUTION_BLUE_BIAS")) return GL_POST_CONVOLUTION_BLUE_BIAS;
        if (!strcmp(name, "GL_POST_CONVOLUTION_RED_SCALE")) return GL_POST_CONVOLUTION_RED_SCALE;
        if (!strcmp(name, "GL_PRIMITIVE_RESTART_INDEX_NV")) return GL_PRIMITIVE_RESTART_INDEX_NV;
        if (!strcmp(name, "GL_PROGRAM_ERROR_POSITION_ARB")) return GL_PROGRAM_ERROR_POSITION_ARB;
        if (!strcmp(name, "GL_PROGRAM_NATIVE_ATTRIBS_ARB")) return GL_PROGRAM_NATIVE_ATTRIBS_ARB;
        if (!strcmp(name, "GL_PROXY_TEXTURE_1D_ARRAY_EXT")) return GL_PROXY_TEXTURE_1D_ARRAY_EXT;
        if (!strcmp(name, "GL_PROXY_TEXTURE_2D_ARRAY_EXT")) return GL_PROXY_TEXTURE_2D_ARRAY_EXT;
        if (!strcmp(name, "GL_PROXY_TEXTURE_CUBE_MAP_ARB")) return GL_PROXY_TEXTURE_CUBE_MAP_ARB;
        if (!strcmp(name, "GL_PROXY_TEXTURE_CUBE_MAP_ARB")) return GL_PROXY_TEXTURE_CUBE_MAP_ARB;
        if (!strcmp(name, "GL_PROXY_TEXTURE_CUBE_MAP_EXT")) return GL_PROXY_TEXTURE_CUBE_MAP_EXT;
        if (!strcmp(name, "GL_PROXY_TEXTURE_RECTANGLE_NV")) return GL_PROXY_TEXTURE_RECTANGLE_NV;
        if (!strcmp(name, "GL_QUERY_BY_REGION_NO_WAIT_NV")) return GL_QUERY_BY_REGION_NO_WAIT_NV;
        if (!strcmp(name, "GL_QUERY_RESULT_AVAILABLE_ARB")) return GL_QUERY_RESULT_AVAILABLE_ARB;
        if (!strcmp(name, "GL_RENDERBUFFER_BLUE_SIZE_EXT")) return GL_RENDERBUFFER_BLUE_SIZE_EXT;
        if (!strcmp(name, "GL_REPLACEMENT_CODE_ARRAY_SUN")) return GL_REPLACEMENT_CODE_ARRAY_SUN;
        if (!strcmp(name, "GL_RGBA16_EXTENDED_RANGE_SGIX")) return GL_RGBA16_EXTENDED_RANGE_SGIX;
        if (!strcmp(name, "GL_RGBA_SIGNED_COMPONENTS_EXT")) return GL_RGBA_SIGNED_COMPONENTS_EXT;
        if (!strcmp(name, "GL_SAMPLE_COVERAGE_INVERT_ARB")) return GL_SAMPLE_COVERAGE_INVERT_ARB;
        if (!strcmp(name, "GL_SAMPLER_2D_RECT_SHADOW_ARB")) return GL_SAMPLER_2D_RECT_SHADOW_ARB;
        if (!strcmp(name, "GL_SAMPLER_2D_RECT_SHADOW_ARB")) return GL_SAMPLER_2D_RECT_SHADOW_ARB;
        if (!strcmp(name, "GL_SECONDARY_COLOR_ARRAY_SIZE")) return GL_SECONDARY_COLOR_ARRAY_SIZE;
        if (!strcmp(name, "GL_SECONDARY_COLOR_ARRAY_TYPE")) return GL_SECONDARY_COLOR_ARRAY_TYPE;
        if (!strcmp(name, "GL_SECONDARY_INTERPOLATOR_ATI")) return GL_SECONDARY_INTERPOLATOR_ATI;
        if (!strcmp(name, "GL_SHARED_TEXTURE_PALETTE_EXT")) return GL_SHARED_TEXTURE_PALETTE_EXT;
        if (!strcmp(name, "GL_TEXTURE_BINDING_BUFFER_ARB")) return GL_TEXTURE_BINDING_BUFFER_ARB;
        if (!strcmp(name, "GL_TEXTURE_BINDING_BUFFER_EXT")) return GL_TEXTURE_BINDING_BUFFER_EXT;
        if (!strcmp(name, "GL_TEXTURE_CONSTANT_DATA_SUNX")) return GL_TEXTURE_CONSTANT_DATA_SUNX;
        if (!strcmp(name, "GL_TEXTURE_COORD_ARRAY_STRIDE")) return GL_TEXTURE_COORD_ARRAY_STRIDE;
        if (!strcmp(name, "GL_TEXTURE_ENV_MODE_ALPHA_EXT")) return GL_TEXTURE_ENV_MODE_ALPHA_EXT;
        if (!strcmp(name, "GL_TEXTURE_FILTER_CONTROL_EXT")) return GL_TEXTURE_FILTER_CONTROL_EXT;
        if (!strcmp(name, "GL_TEXTURE_INTENSITY_SIZE_EXT")) return GL_TEXTURE_INTENSITY_SIZE_EXT;
        if (!strcmp(name, "GL_TEXTURE_INTENSITY_TYPE_ARB")) return GL_TEXTURE_INTENSITY_TYPE_ARB;
        if (!strcmp(name, "GL_TEXTURE_LUMINANCE_SIZE_EXT")) return GL_TEXTURE_LUMINANCE_SIZE_EXT;
        if (!strcmp(name, "GL_TEXTURE_LUMINANCE_TYPE_ARB")) return GL_TEXTURE_LUMINANCE_TYPE_ARB;
        if (!strcmp(name, "GL_TEXTURE_MAX_ANISOTROPY_EXT")) return GL_TEXTURE_MAX_ANISOTROPY_EXT;
        if (!strcmp(name, "GL_TEXTURE_RANGE_LENGTH_APPLE")) return GL_TEXTURE_RANGE_LENGTH_APPLE;
        if (!strcmp(name, "GL_TEXTURE_STORAGE_HINT_APPLE")) return GL_TEXTURE_STORAGE_HINT_APPLE;
        if (!strcmp(name, "GL_TRANSPOSE_COLOR_MATRIX_ARB")) return GL_TRANSPOSE_COLOR_MATRIX_ARB;
        if (!strcmp(name, "GL_TRANSPOSE_MODELVIEW_MATRIX")) return GL_TRANSPOSE_MODELVIEW_MATRIX;
        if (!strcmp(name, "GL_UNIFORM_BUFFER_BINDING_EXT")) return GL_UNIFORM_BUFFER_BINDING_EXT;
        if (!strcmp(name, "GL_UNSIGNED_SHORT_1_5_5_5_REV")) return GL_UNSIGNED_SHORT_1_5_5_5_REV;
        if (!strcmp(name, "GL_UNSIGNED_SHORT_4_4_4_4_EXT")) return GL_UNSIGNED_SHORT_4_4_4_4_EXT;
        if (!strcmp(name, "GL_UNSIGNED_SHORT_4_4_4_4_REV")) return GL_UNSIGNED_SHORT_4_4_4_4_REV;
        if (!strcmp(name, "GL_UNSIGNED_SHORT_5_5_5_1_EXT")) return GL_UNSIGNED_SHORT_5_5_5_1_EXT;
        if (!strcmp(name, "GL_VERTEX_ARRAY_BINDING_APPLE")) return GL_VERTEX_ARRAY_BINDING_APPLE;
        if (!strcmp(name, "GL_VERTEX_ATTRIB_ARRAY_STRIDE")) return GL_VERTEX_ATTRIB_ARRAY_STRIDE;
        if (!strcmp(name, "GL_VERTEX_CONSISTENT_HINT_PGI")) return GL_VERTEX_CONSISTENT_HINT_PGI;
        if (!strcmp(name, "GL_VERTEX_PROGRAM_TWO_SIDE_NV")) return GL_VERTEX_PROGRAM_TWO_SIDE_NV;
        if (!strcmp(name, "GL_VERTEX_SHADER_VARIANTS_EXT")) return GL_VERTEX_SHADER_VARIANTS_EXT;
        return _GL_UNDEFINED;
    case 30:
        if (!strcmp(name, "GL_ACTIVE_ATTRIBUTE_MAX_LENGTH")) return GL_ACTIVE_ATTRIBUTE_MAX_LENGTH;
        if (!strcmp(name, "GL_ALPHA16_EXTENDED_RANGE_SGIX")) return GL_ALPHA16_EXTENDED_RANGE_SGIX;
        if (!strcmp(name, "GL_ARB_fragment_program_shadow")) return GL_ARB_fragment_program_shadow;
        if (!strcmp(name, "GL_ARB_texture_mirrored_repeat")) return GL_ARB_texture_mirrored_repeat;
        if (!strcmp(name, "GL_ATI_texture_compression_3dc")) return GL_ATI_texture_compression_3dc;
        if (!strcmp(name, "GL_BUFFER_FLUSHING_UNMAP_APPLE")) return GL_BUFFER_FLUSHING_UNMAP_APPLE;
        if (!strcmp(name, "GL_COLOR_ARRAY_LIST_STRIDE_IBM")) return GL_COLOR_ARRAY_LIST_STRIDE_IBM;
        if (!strcmp(name, "GL_COMBINER_COMPONENT_USAGE_NV")) return GL_COMBINER_COMPONENT_USAGE_NV;
        if (!strcmp(name, "GL_COMPRESSED_SIGNED_RED_RGTC1")) return GL_COMPRESSED_SIGNED_RED_RGTC1;
        if (!strcmp(name, "GL_COMPRESSED_SLUMINANCE_ALPHA")) return GL_COMPRESSED_SLUMINANCE_ALPHA;
        if (!strcmp(name, "GL_CONVOLUTION_BORDER_MODE_EXT")) return GL_CONVOLUTION_BORDER_MODE_EXT;
        if (!strcmp(name, "GL_CONVOLUTION_FILTER_BIAS_EXT")) return GL_CONVOLUTION_FILTER_BIAS_EXT;
        if (!strcmp(name, "GL_CURRENT_SECONDARY_COLOR_EXT")) return GL_CURRENT_SECONDARY_COLOR_EXT;
        if (!strcmp(name, "GL_DEPENDENT_RGB_TEXTURE_3D_NV")) return GL_DEPENDENT_RGB_TEXTURE_3D_NV;
        if (!strcmp(name, "GL_DOT_PRODUCT_PASS_THROUGH_NV")) return GL_DOT_PRODUCT_PASS_THROUGH_NV;
        if (!strcmp(name, "GL_EDGE_FLAG_ARRAY_POINTER_EXT")) return GL_EDGE_FLAG_ARRAY_POINTER_EXT;
        if (!strcmp(name, "GL_ELEMENT_ARRAY_POINTER_APPLE")) return GL_ELEMENT_ARRAY_POINTER_APPLE;
        if (!strcmp(name, "GL_EXT_blend_equation_separate")) return GL_EXT_blend_equation_separate;
        if (!strcmp(name, "GL_EXT_framebuffer_multisample")) return GL_EXT_framebuffer_multisample;
        if (!strcmp(name, "GL_EXT_separate_specular_color")) return GL_EXT_separate_specular_color;
        if (!strcmp(name, "GL_EXT_texture_shared_exponent")) return GL_EXT_texture_shared_exponent;
        if (!strcmp(name, "GL_FOG_COORDINATE_ARRAY_STRIDE")) return GL_FOG_COORDINATE_ARRAY_STRIDE;
        if (!strcmp(name, "GL_FRAGMENT_COLOR_MATERIAL_EXT")) return GL_FRAGMENT_COLOR_MATERIAL_EXT;
        if (!strcmp(name, "GL_FRAGMENT_PROGRAM_BINDING_NV")) return GL_FRAGMENT_PROGRAM_BINDING_NV;
        if (!strcmp(name, "GL_FRAMEBUFFER_UNSUPPORTED_EXT")) return GL_FRAMEBUFFER_UNSUPPORTED_EXT;
        if (!strcmp(name, "GL_HP_convolution_border_modes")) return GL_HP_convolution_border_modes;
        if (!strcmp(name, "GL_IBM_texture_mirrored_repeat")) return GL_IBM_texture_mirrored_repeat;
        if (!strcmp(name, "GL_INDEX_ARRAY_LIST_STRIDE_IBM")) return GL_INDEX_ARRAY_LIST_STRIDE_IBM;
        if (!strcmp(name, "GL_INT_SAMPLER_RENDERBUFFER_NV")) return GL_INT_SAMPLER_RENDERBUFFER_NV;
        if (!strcmp(name, "GL_LOCAL_CONSTANT_DATATYPE_EXT")) return GL_LOCAL_CONSTANT_DATATYPE_EXT;
        if (!strcmp(name, "GL_LUMINANCE_ALPHA_FLOAT16_ATI")) return GL_LUMINANCE_ALPHA_FLOAT16_ATI;
        if (!strcmp(name, "GL_LUMINANCE_ALPHA_FLOAT32_ATI")) return GL_LUMINANCE_ALPHA_FLOAT32_ATI;
        if (!strcmp(name, "GL_LUMINANCE_ALPHA_INTEGER_EXT")) return GL_LUMINANCE_ALPHA_INTEGER_EXT;
        if (!strcmp(name, "GL_LUMINANCE_ALPHA_SIGNED_SGIX")) return GL_LUMINANCE_ALPHA_SIGNED_SGIX;
        if (!strcmp(name, "GL_MATRIX_INDEX_ARRAY_SIZE_ARB")) return GL_MATRIX_INDEX_ARRAY_SIZE_ARB;
        if (!strcmp(name, "GL_MATRIX_INDEX_ARRAY_TYPE_ARB")) return GL_MATRIX_INDEX_ARRAY_TYPE_ARB;
        if (!strcmp(name, "GL_MAX_PROGRAM_TEMPORARIES_ARB")) return GL_MAX_PROGRAM_TEMPORARIES_ARB;
        if (!strcmp(name, "GL_MAX_PROGRAM_TEXEL_OFFSET_NV")) return GL_MAX_PROGRAM_TEXEL_OFFSET_NV;
        if (!strcmp(name, "GL_MAX_TEXTURE_BUFFER_SIZE_ARB")) return GL_MAX_TEXTURE_BUFFER_SIZE_ARB;
        if (!strcmp(name, "GL_MAX_TEXTURE_BUFFER_SIZE_EXT")) return GL_MAX_TEXTURE_BUFFER_SIZE_EXT;
        if (!strcmp(name, "GL_MAX_TEXTURE_IMAGE_UNITS_ARB")) return GL_MAX_TEXTURE_IMAGE_UNITS_ARB;
        if (!strcmp(name, "GL_MIN_PROGRAM_TEXEL_OFFSET_NV")) return GL_MIN_PROGRAM_TEXEL_OFFSET_NV;
        if (!strcmp(name, "GL_NORMAL_ARRAY_BUFFER_BINDING")) return GL_NORMAL_ARRAY_BUFFER_BINDING;
        if (!strcmp(name, "GL_NUM_LOOPBACK_COMPONENTS_ATI")) return GL_NUM_LOOPBACK_COMPONENTS_ATI;
        if (!strcmp(name, "GL_OBJECT_ATTACHED_OBJECTS_ARB")) return GL_OBJECT_ATTACHED_OBJECTS_ARB;
        if (!strcmp(name, "GL_OFFSET_TEXTURE_2D_MATRIX_NV")) return GL_OFFSET_TEXTURE_2D_MATRIX_NV;
        if (!strcmp(name, "GL_OFFSET_TEXTURE_RECTANGLE_NV")) return GL_OFFSET_TEXTURE_RECTANGLE_NV;
        if (!strcmp(name, "GL_PERSPECTIVE_CORRECTION_HINT")) return GL_PERSPECTIVE_CORRECTION_HINT;
        if (!strcmp(name, "GL_PIXEL_UNPACK_BUFFER_BINDING")) return GL_PIXEL_UNPACK_BUFFER_BINDING;
        if (!strcmp(name, "GL_PN_TRIANGLES_POINT_MODE_ATI")) return GL_PN_TRIANGLES_POINT_MODE_ATI;
        if (!strcmp(name, "GL_POINT_SPRITE_CULL_CLIP_ATIX")) return GL_POINT_SPRITE_CULL_CLIP_ATIX;
        if (!strcmp(name, "GL_POINT_SPRITE_CULL_MODE_ATIX")) return GL_POINT_SPRITE_CULL_MODE_ATIX;
        if (!strcmp(name, "GL_POST_COLOR_MATRIX_BLUE_BIAS")) return GL_POST_COLOR_MATRIX_BLUE_BIAS;
        if (!strcmp(name, "GL_POST_COLOR_MATRIX_RED_SCALE")) return GL_POST_COLOR_MATRIX_RED_SCALE;
        if (!strcmp(name, "GL_POST_CONVOLUTION_ALPHA_BIAS")) return GL_POST_CONVOLUTION_ALPHA_BIAS;
        if (!strcmp(name, "GL_POST_CONVOLUTION_BLUE_SCALE")) return GL_POST_CONVOLUTION_BLUE_SCALE;
        if (!strcmp(name, "GL_POST_CONVOLUTION_GREEN_BIAS")) return GL_POST_CONVOLUTION_GREEN_BIAS;
        if (!strcmp(name, "GL_PROXY_TEXTURE_RECTANGLE_ARB")) return GL_PROXY_TEXTURE_RECTANGLE_ARB;
        if (!strcmp(name, "GL_PROXY_TEXTURE_RECTANGLE_EXT")) return GL_PROXY_TEXTURE_RECTANGLE_EXT;
        if (!strcmp(name, "GL_RENDERBUFFER_ALPHA_SIZE_EXT")) return GL_RENDERBUFFER_ALPHA_SIZE_EXT;
        if (!strcmp(name, "GL_RENDERBUFFER_DEPTH_SIZE_EXT")) return GL_RENDERBUFFER_DEPTH_SIZE_EXT;
        if (!strcmp(name, "GL_RENDERBUFFER_GREEN_SIZE_EXT")) return GL_RENDERBUFFER_GREEN_SIZE_EXT;
        if (!strcmp(name, "GL_SAMPLER_1D_ARRAY_SHADOW_EXT")) return GL_SAMPLER_1D_ARRAY_SHADOW_EXT;
        if (!strcmp(name, "GL_SAMPLER_2D_ARRAY_SHADOW_EXT")) return GL_SAMPLER_2D_ARRAY_SHADOW_EXT;
        if (!strcmp(name, "GL_SEPARATE_SPECULAR_COLOR_EXT")) return GL_SEPARATE_SPECULAR_COLOR_EXT;
        if (!strcmp(name, "GL_SIGNED_LUMINANCE8_ALPHA8_NV")) return GL_SIGNED_LUMINANCE8_ALPHA8_NV;
        if (!strcmp(name, "GL_SIGNED_LUMINANCE8_ALPHA8_NV")) return GL_SIGNED_LUMINANCE8_ALPHA8_NV;
        if (!strcmp(name, "GL_STENCIL_CLEAR_TAG_VALUE_EXT")) return GL_STENCIL_CLEAR_TAG_VALUE_EXT;
        if (!strcmp(name, "GL_TEXTURE_COORD_ARRAY_POINTER")) return GL_TEXTURE_COORD_ARRAY_POINTER;
        if (!strcmp(name, "GL_TEXTURE_CUBE_MAP_NEGATIVE_X")) return GL_TEXTURE_CUBE_MAP_NEGATIVE_X;
        if (!strcmp(name, "GL_TEXTURE_CUBE_MAP_NEGATIVE_Y")) return GL_TEXTURE_CUBE_MAP_NEGATIVE_Y;
        if (!strcmp(name, "GL_TEXTURE_CUBE_MAP_NEGATIVE_Z")) return GL_TEXTURE_CUBE_MAP_NEGATIVE_Z;
        if (!strcmp(name, "GL_TEXTURE_CUBE_MAP_POSITIVE_X")) return GL_TEXTURE_CUBE_MAP_POSITIVE_X;
        if (!strcmp(name, "GL_TEXTURE_CUBE_MAP_POSITIVE_Y")) return GL_TEXTURE_CUBE_MAP_POSITIVE_Y;
        if (!strcmp(name, "GL_TEXTURE_CUBE_MAP_POSITIVE_Z")) return GL_TEXTURE_CUBE_MAP_POSITIVE_Z;
        if (!strcmp(name, "GL_TEXTURE_FLOAT_COMPONENTS_NV")) return GL_TEXTURE_FLOAT_COMPONENTS_NV;
        if (!strcmp(name, "GL_TEXTURE_RANGE_POINTER_APPLE")) return GL_TEXTURE_RANGE_POINTER_APPLE;
        if (!strcmp(name, "GL_TRANSFORM_FEEDBACK_VARYINGS")) return GL_TRANSFORM_FEEDBACK_VARYINGS;
        if (!strcmp(name, "GL_TRANSPOSE_PROJECTION_MATRIX")) return GL_TRANSPOSE_PROJECTION_MATRIX;
        if (!strcmp(name, "GL_UNPACK_CLIENT_STORAGE_APPLE")) return GL_UNPACK_CLIENT_STORAGE_APPLE;
        if (!strcmp(name, "GL_UNSIGNED_INT_10_10_10_2_EXT")) return GL_UNSIGNED_INT_10_10_10_2_EXT;
        if (!strcmp(name, "GL_UNSIGNED_INT_2_10_10_10_REV")) return GL_UNSIGNED_INT_2_10_10_10_REV;
        if (!strcmp(name, "GL_UNSIGNED_INT_SAMPLER_1D_EXT")) return GL_UNSIGNED_INT_SAMPLER_1D_EXT;
        if (!strcmp(name, "GL_UNSIGNED_INT_SAMPLER_2D_EXT")) return GL_UNSIGNED_INT_SAMPLER_2D_EXT;
        if (!strcmp(name, "GL_UNSIGNED_INT_SAMPLER_3D_EXT")) return GL_UNSIGNED_INT_SAMPLER_3D_EXT;
        if (!strcmp(name, "GL_UNSIGNED_SHORT_8_8_REV_MESA")) return GL_UNSIGNED_SHORT_8_8_REV_MESA;
        if (!strcmp(name, "GL_VERTEX_ARRAY_BUFFER_BINDING")) return GL_VERTEX_ARRAY_BUFFER_BINDING;
        if (!strcmp(name, "GL_VERTEX_ARRAY_RANGE_VALID_NV")) return GL_VERTEX_ARRAY_RANGE_VALID_NV;
        if (!strcmp(name, "GL_VERTEX_ATTRIB_ARRAY_ENABLED")) return GL_VERTEX_ATTRIB_ARRAY_ENABLED;
        if (!strcmp(name, "GL_VERTEX_ATTRIB_ARRAY_INTEGER")) return GL_VERTEX_ATTRIB_ARRAY_INTEGER;
        if (!strcmp(name, "GL_VERTEX_ATTRIB_ARRAY_POINTER")) return GL_VERTEX_ATTRIB_ARRAY_POINTER;
        if (!strcmp(name, "GL_VERTEX_PROGRAM_TWO_SIDE_ARB")) return GL_VERTEX_PROGRAM_TWO_SIDE_ARB;
        if (!strcmp(name, "GL_VERTEX_SHADER_OPTIMIZED_EXT")) return GL_VERTEX_SHADER_OPTIMIZED_EXT;
        if (!strcmp(name, "GL_WEIGHT_ARRAY_BUFFER_BINDING")) return GL_WEIGHT_ARRAY_BUFFER_BINDING;
        return _GL_UNDEFINED;
    case 31:
        if (!strcmp(name, "GL_ACTIVE_VARYING_MAX_LENGTH_NV")) return GL_ACTIVE_VARYING_MAX_LENGTH_NV;
        if (!strcmp(name, "GL_ARB_texture_compression_rgtc")) return GL_ARB_texture_compression_rgtc;
        if (!strcmp(name, "GL_ARB_texture_non_power_of_two")) return GL_ARB_texture_non_power_of_two;
        if (!strcmp(name, "GL_ARRAY_ELEMENT_LOCK_COUNT_EXT")) return GL_ARRAY_ELEMENT_LOCK_COUNT_EXT;
        if (!strcmp(name, "GL_ARRAY_ELEMENT_LOCK_FIRST_EXT")) return GL_ARRAY_ELEMENT_LOCK_FIRST_EXT;
        if (!strcmp(name, "GL_BIAS_BY_NEGATIVE_ONE_HALF_NV")) return GL_BIAS_BY_NEGATIVE_ONE_HALF_NV;
        if (!strcmp(name, "GL_COLOR_MATRIX_STACK_DEPTH_SGI")) return GL_COLOR_MATRIX_STACK_DEPTH_SGI;
        if (!strcmp(name, "GL_COMPRESSED_RGB_S3TC_DXT1_EXT")) return GL_COMPRESSED_RGB_S3TC_DXT1_EXT;
        if (!strcmp(name, "GL_COMPRESSED_RGB_S3TC_DXT1_EXT")) return GL_COMPRESSED_RGB_S3TC_DXT1_EXT;
        if (!strcmp(name, "GL_CONVOLUTION_FILTER_SCALE_EXT")) return GL_CONVOLUTION_FILTER_SCALE_EXT;
        if (!strcmp(name, "GL_DEPENDENT_HILO_TEXTURE_2D_NV")) return GL_DEPENDENT_HILO_TEXTURE_2D_NV;
        if (!strcmp(name, "GL_DOT_PRODUCT_DEPTH_REPLACE_NV")) return GL_DOT_PRODUCT_DEPTH_REPLACE_NV;
        if (!strcmp(name, "GL_DRAW_FRAMEBUFFER_BINDING_EXT")) return GL_DRAW_FRAMEBUFFER_BINDING_EXT;
        if (!strcmp(name, "GL_ELEMENT_ARRAY_BUFFER_BINDING")) return GL_ELEMENT_ARRAY_BUFFER_BINDING;
        if (!strcmp(name, "GL_EXT_texture_compression_dxt1")) return GL_EXT_texture_compression_dxt1;
        if (!strcmp(name, "GL_EXT_texture_compression_latc")) return GL_EXT_texture_compression_latc;
        if (!strcmp(name, "GL_EXT_texture_compression_rgtc")) return GL_EXT_texture_compression_rgtc;
        if (!strcmp(name, "GL_EXT_texture_compression_s3tc")) return GL_EXT_texture_compression_s3tc;
        if (!strcmp(name, "GL_FOG_COORDINATE_ARRAY_POINTER")) return GL_FOG_COORDINATE_ARRAY_POINTER;
        if (!strcmp(name, "GL_FORMAT_SUBSAMPLE_244_244_OML")) return GL_FORMAT_SUBSAMPLE_244_244_OML;
        if (!strcmp(name, "GL_FRAMEBUFFER_SRGB_CAPABLE_EXT")) return GL_FRAMEBUFFER_SRGB_CAPABLE_EXT;
        if (!strcmp(name, "GL_HISTOGRAM_LUMINANCE_SIZE_EXT")) return GL_HISTOGRAM_LUMINANCE_SIZE_EXT;
        if (!strcmp(name, "GL_MAX_ARRAY_TEXTURE_LAYERS_EXT")) return GL_MAX_ARRAY_TEXTURE_LAYERS_EXT;
        if (!strcmp(name, "GL_MAX_COLOR_MATRIX_STACK_DEPTH")) return GL_MAX_COLOR_MATRIX_STACK_DEPTH;
        if (!strcmp(name, "GL_MAX_PROGRAM_INSTRUCTIONS_ARB")) return GL_MAX_PROGRAM_INSTRUCTIONS_ARB;
        if (!strcmp(name, "GL_MAX_VERTEX_SHADER_LOCALS_EXT")) return GL_MAX_VERTEX_SHADER_LOCALS_EXT;
        if (!strcmp(name, "GL_NATIVE_GRAPHICS_END_HINT_PGI")) return GL_NATIVE_GRAPHICS_END_HINT_PGI;
        if (!strcmp(name, "GL_NORMAL_ARRAY_LIST_STRIDE_IBM")) return GL_NORMAL_ARRAY_LIST_STRIDE_IBM;
        if (!strcmp(name, "GL_OBJECT_ACTIVE_ATTRIBUTES_ARB")) return GL_OBJECT_ACTIVE_ATTRIBUTES_ARB;
        if (!strcmp(name, "GL_OBJECT_DISTANCE_TO_LINE_SGIS")) return GL_OBJECT_DISTANCE_TO_LINE_SGIS;
        if (!strcmp(name, "GL_ONE_MINUS_CONSTANT_ALPHA_EXT")) return GL_ONE_MINUS_CONSTANT_ALPHA_EXT;
        if (!strcmp(name, "GL_ONE_MINUS_CONSTANT_COLOR_EXT")) return GL_ONE_MINUS_CONSTANT_COLOR_EXT;
        if (!strcmp(name, "GL_PERFMON_RESULT_AVAILABLE_AMD")) return GL_PERFMON_RESULT_AVAILABLE_AMD;
        if (!strcmp(name, "GL_PN_TRIANGLES_NORMAL_MODE_ATI")) return GL_PN_TRIANGLES_NORMAL_MODE_ATI;
        if (!strcmp(name, "GL_POST_COLOR_MATRIX_ALPHA_BIAS")) return GL_POST_COLOR_MATRIX_ALPHA_BIAS;
        if (!strcmp(name, "GL_POST_COLOR_MATRIX_BLUE_SCALE")) return GL_POST_COLOR_MATRIX_BLUE_SCALE;
        if (!strcmp(name, "GL_POST_COLOR_MATRIX_GREEN_BIAS")) return GL_POST_COLOR_MATRIX_GREEN_BIAS;
        if (!strcmp(name, "GL_POST_CONVOLUTION_ALPHA_SCALE")) return GL_POST_CONVOLUTION_ALPHA_SCALE;
        if (!strcmp(name, "GL_POST_CONVOLUTION_COLOR_TABLE")) return GL_POST_CONVOLUTION_COLOR_TABLE;
        if (!strcmp(name, "GL_POST_CONVOLUTION_GREEN_SCALE")) return GL_POST_CONVOLUTION_GREEN_SCALE;
        if (!strcmp(name, "GL_PREFER_DOUBLEBUFFER_HINT_PGI")) return GL_PREFER_DOUBLEBUFFER_HINT_PGI;
        if (!strcmp(name, "GL_PROGRAM_ALU_INSTRUCTIONS_ARB")) return GL_PROGRAM_ALU_INSTRUCTIONS_ARB;
        if (!strcmp(name, "GL_PROGRAM_ATTRIB_COMPONENTS_NV")) return GL_PROGRAM_ATTRIB_COMPONENTS_NV;
        if (!strcmp(name, "GL_PROGRAM_RESULT_COMPONENTS_NV")) return GL_PROGRAM_RESULT_COMPONENTS_NV;
        if (!strcmp(name, "GL_PROGRAM_TEX_INDIRECTIONS_ARB")) return GL_PROGRAM_TEX_INDIRECTIONS_ARB;
        if (!strcmp(name, "GL_PROGRAM_TEX_INSTRUCTIONS_ARB")) return GL_PROGRAM_TEX_INSTRUCTIONS_ARB;
        if (!strcmp(name, "GL_PROXY_TEXTURE_1D_STACK_MESAX")) return GL_PROXY_TEXTURE_1D_STACK_MESAX;
        if (!strcmp(name, "GL_PROXY_TEXTURE_2D_STACK_MESAX")) return GL_PROXY_TEXTURE_2D_STACK_MESAX;
        if (!strcmp(name, "GL_READ_FRAMEBUFFER_BINDING_EXT")) return GL_READ_FRAMEBUFFER_BINDING_EXT;
        if (!strcmp(name, "GL_RENDERBUFFER_FREE_MEMORY_ATI")) return GL_RENDERBUFFER_FREE_MEMORY_ATI;
        if (!strcmp(name, "GL_RENDERBUFFER_INTERNAL_FORMAT")) return GL_RENDERBUFFER_INTERNAL_FORMAT;
        if (!strcmp(name, "GL_SAMPLE_ALPHA_TO_COVERAGE_ARB")) return GL_SAMPLE_ALPHA_TO_COVERAGE_ARB;
        if (!strcmp(name, "GL_SECONDARY_COLOR_ARRAY_STRIDE")) return GL_SECONDARY_COLOR_ARRAY_STRIDE;
        if (!strcmp(name, "GL_SHADING_LANGUAGE_VERSION_ARB")) return GL_SHADING_LANGUAGE_VERSION_ARB;
        if (!strcmp(name, "GL_SIGNED_RGB_UNSIGNED_ALPHA_NV")) return GL_SIGNED_RGB_UNSIGNED_ALPHA_NV;
        if (!strcmp(name, "GL_SIGNED_RGB_UNSIGNED_ALPHA_NV")) return GL_SIGNED_RGB_UNSIGNED_ALPHA_NV;
        if (!strcmp(name, "GL_STENCIL_BACK_PASS_DEPTH_FAIL")) return GL_STENCIL_BACK_PASS_DEPTH_FAIL;
        if (!strcmp(name, "GL_STENCIL_BACK_PASS_DEPTH_PASS")) return GL_STENCIL_BACK_PASS_DEPTH_PASS;
        if (!strcmp(name, "GL_SUN_convolution_border_modes")) return GL_SUN_convolution_border_modes;
        if (!strcmp(name, "GL_TEXTURE_APPLICATION_MODE_EXT")) return GL_TEXTURE_APPLICATION_MODE_EXT;
        if (!strcmp(name, "GL_TEXTURE_BINDING_1D_ARRAY_EXT")) return GL_TEXTURE_BINDING_1D_ARRAY_EXT;
        if (!strcmp(name, "GL_TEXTURE_BINDING_2D_ARRAY_EXT")) return GL_TEXTURE_BINDING_2D_ARRAY_EXT;
        if (!strcmp(name, "GL_TEXTURE_BINDING_CUBE_MAP_ARB")) return GL_TEXTURE_BINDING_CUBE_MAP_ARB;
        if (!strcmp(name, "GL_TEXTURE_BINDING_CUBE_MAP_EXT")) return GL_TEXTURE_BINDING_CUBE_MAP_EXT;
        if (!strcmp(name, "GL_TEXTURE_BINDING_RECTANGLE_NV")) return GL_TEXTURE_BINDING_RECTANGLE_NV;
        if (!strcmp(name, "GL_TEXTURE_COMPRESSION_HINT_ARB")) return GL_TEXTURE_COMPRESSION_HINT_ARB;
        if (!strcmp(name, "GL_TEXTURE_COORD_ARRAY_LIST_IBM")) return GL_TEXTURE_COORD_ARRAY_LIST_IBM;
        if (!strcmp(name, "GL_TEXTURE_COORD_ARRAY_SIZE_EXT")) return GL_TEXTURE_COORD_ARRAY_SIZE_EXT;
        if (!strcmp(name, "GL_TEXTURE_COORD_ARRAY_TYPE_EXT")) return GL_TEXTURE_COORD_ARRAY_TYPE_EXT;
        if (!strcmp(name, "GL_TEXTURE_POINT_ONE_COORD_ATIX")) return GL_TEXTURE_POINT_ONE_COORD_ATIX;
        if (!strcmp(name, "GL_TRANSFORM_FEEDBACK_BUFFER_NV")) return GL_TRANSFORM_FEEDBACK_BUFFER_NV;
        if (!strcmp(name, "GL_TRANSFORM_FEEDBACK_RECORD_NV")) return GL_TRANSFORM_FEEDBACK_RECORD_NV;
        if (!strcmp(name, "GL_TRANSPOSE_CURRENT_MATRIX_ARB")) return GL_TRANSPOSE_CURRENT_MATRIX_ARB;
        if (!strcmp(name, "GL_TRANSPOSE_PROGRAM_MATRIX_EXT")) return GL_TRANSPOSE_PROGRAM_MATRIX_EXT;
        if (!strcmp(name, "GL_TRANSPOSE_TEXTURE_MATRIX_ARB")) return GL_TRANSPOSE_TEXTURE_MATRIX_ARB;
        if (!strcmp(name, "GL_TRIANGLE_STRIP_ADJACENCY_ARB")) return GL_TRIANGLE_STRIP_ADJACENCY_ARB;
        if (!strcmp(name, "GL_TRIANGLE_STRIP_ADJACENCY_EXT")) return GL_TRIANGLE_STRIP_ADJACENCY_EXT;
        if (!strcmp(name, "GL_UNSIGNED_INT_10F_11F_11F_REV")) return GL_UNSIGNED_INT_10F_11F_11F_REV;
        if (!strcmp(name, "GL_UNSIGNED_INT_5_9_9_9_REV_EXT")) return GL_UNSIGNED_INT_5_9_9_9_REV_EXT;
        if (!strcmp(name, "GL_UNSIGNED_SHORT_8_8_REV_APPLE")) return GL_UNSIGNED_SHORT_8_8_REV_APPLE;
        if (!strcmp(name, "GL_VERTEX_ARRAY_LIST_STRIDE_IBM")) return GL_VERTEX_ARRAY_LIST_STRIDE_IBM;
        if (!strcmp(name, "GL_VERTEX_ARRAY_RANGE_LENGTH_NV")) return GL_VERTEX_ARRAY_RANGE_LENGTH_NV;
        if (!strcmp(name, "GL_VERTEX_ATTRIB_ARRAY_SIZE_ARB")) return GL_VERTEX_ATTRIB_ARRAY_SIZE_ARB;
        if (!strcmp(name, "GL_VERTEX_ATTRIB_ARRAY_TYPE_ARB")) return GL_VERTEX_ATTRIB_ARRAY_TYPE_ARB;
        if (!strcmp(name, "GL_VERTEX_PROGRAM_POINT_SIZE_NV")) return GL_VERTEX_PROGRAM_POINT_SIZE_NV;
        if (!strcmp(name, "GL_VERTEX_SHADER_INVARIANTS_EXT")) return GL_VERTEX_SHADER_INVARIANTS_EXT;
        if (!strcmp(name, "GL_VERTEX_WEIGHT_ARRAY_SIZE_EXT")) return GL_VERTEX_WEIGHT_ARRAY_SIZE_EXT;
        if (!strcmp(name, "GL_VERTEX_WEIGHT_ARRAY_TYPE_EXT")) return GL_VERTEX_WEIGHT_ARRAY_TYPE_EXT;
        return _GL_UNDEFINED;
    case 32:
        if (!strcmp(name, "GL_3DFX_texture_compression_FXT1")) return GL_3DFX_texture_compression_FXT1;
        if (!strcmp(name, "GL_CLIP_VOLUME_CLIPPING_HINT_EXT")) return GL_CLIP_VOLUME_CLIPPING_HINT_EXT;
        if (!strcmp(name, "GL_COMPRESSED_RGBA_S3TC_DXT1_EXT")) return GL_COMPRESSED_RGBA_S3TC_DXT1_EXT;
        if (!strcmp(name, "GL_COMPRESSED_RGBA_S3TC_DXT1_EXT")) return GL_COMPRESSED_RGBA_S3TC_DXT1_EXT;
        if (!strcmp(name, "GL_COMPRESSED_RGBA_S3TC_DXT3_EXT")) return GL_COMPRESSED_RGBA_S3TC_DXT3_EXT;
        if (!strcmp(name, "GL_COMPRESSED_RGBA_S3TC_DXT5_EXT")) return GL_COMPRESSED_RGBA_S3TC_DXT5_EXT;
        if (!strcmp(name, "GL_COMPRESSED_SRGB_S3TC_DXT1_EXT")) return GL_COMPRESSED_SRGB_S3TC_DXT1_EXT;
        if (!strcmp(name, "GL_CURRENT_MATRIX_STACK_DEPTH_NV")) return GL_CURRENT_MATRIX_STACK_DEPTH_NV;
        if (!strcmp(name, "GL_CURRENT_OCCLUSION_QUERY_ID_NV")) return GL_CURRENT_OCCLUSION_QUERY_ID_NV;
        if (!strcmp(name, "GL_CURRENT_RASTER_POSITION_VALID")) return GL_CURRENT_RASTER_POSITION_VALID;
        if (!strcmp(name, "GL_CURRENT_RASTER_TEXTURE_COORDS")) return GL_CURRENT_RASTER_TEXTURE_COORDS;
        if (!strcmp(name, "GL_CURRENT_SAMPLE_COUNT_QUERY_NV")) return GL_CURRENT_SAMPLE_COUNT_QUERY_NV;
        if (!strcmp(name, "GL_FOG_COORDINATE_ARRAY_LIST_IBM")) return GL_FOG_COORDINATE_ARRAY_LIST_IBM;
        if (!strcmp(name, "GL_FOG_COORDINATE_ARRAY_TYPE_EXT")) return GL_FOG_COORDINATE_ARRAY_TYPE_EXT;
        if (!strcmp(name, "GL_INTENSITY_EXTENDED_RANGE_SGIX")) return GL_INTENSITY_EXTENDED_RANGE_SGIX;
        if (!strcmp(name, "GL_INVALID_FRAMEBUFFER_OPERATION")) return GL_INVALID_FRAMEBUFFER_OPERATION;
        if (!strcmp(name, "GL_LIGHT_MODEL_COLOR_CONTROL_EXT")) return GL_LIGHT_MODEL_COLOR_CONTROL_EXT;
        if (!strcmp(name, "GL_LUMINANCE_ALPHA_FLOAT16_APPLE")) return GL_LUMINANCE_ALPHA_FLOAT16_APPLE;
        if (!strcmp(name, "GL_LUMINANCE_ALPHA_FLOAT32_APPLE")) return GL_LUMINANCE_ALPHA_FLOAT32_APPLE;
        if (!strcmp(name, "GL_LUMINANCE_EXTENDED_RANGE_SGIX")) return GL_LUMINANCE_EXTENDED_RANGE_SGIX;
        if (!strcmp(name, "GL_MATRIX_INDEX_ARRAY_STRIDE_ARB")) return GL_MATRIX_INDEX_ARRAY_STRIDE_ARB;
        if (!strcmp(name, "GL_MAX_BINDABLE_UNIFORM_SIZE_EXT")) return GL_MAX_BINDABLE_UNIFORM_SIZE_EXT;
        if (!strcmp(name, "GL_MAX_CLIENT_ATTRIB_STACK_DEPTH")) return GL_MAX_CLIENT_ATTRIB_STACK_DEPTH;
        if (!strcmp(name, "GL_MAX_CUBE_MAP_TEXTURE_SIZE_ARB")) return GL_MAX_CUBE_MAP_TEXTURE_SIZE_ARB;
        if (!strcmp(name, "GL_MAX_CUBE_MAP_TEXTURE_SIZE_EXT")) return GL_MAX_CUBE_MAP_TEXTURE_SIZE_EXT;
        if (!strcmp(name, "GL_MAX_RECTANGLE_TEXTURE_SIZE_NV")) return GL_MAX_RECTANGLE_TEXTURE_SIZE_NV;
        if (!strcmp(name, "GL_MAX_VERTEX_UNIFORM_COMPONENTS")) return GL_MAX_VERTEX_UNIFORM_COMPONENTS;
        if (!strcmp(name, "GL_MULTISAMPLE_COVERAGE_MODES_NV")) return GL_MULTISAMPLE_COVERAGE_MODES_NV;
        if (!strcmp(name, "GL_NUM_INSTRUCTIONS_PER_PASS_ATI")) return GL_NUM_INSTRUCTIONS_PER_PASS_ATI;
        if (!strcmp(name, "GL_OBJECT_DISTANCE_TO_POINT_SGIS")) return GL_OBJECT_DISTANCE_TO_POINT_SGIS;
        if (!strcmp(name, "GL_PIXEL_PACK_BUFFER_BINDING_ARB")) return GL_PIXEL_PACK_BUFFER_BINDING_ARB;
        if (!strcmp(name, "GL_PIXEL_PACK_BUFFER_BINDING_EXT")) return GL_PIXEL_PACK_BUFFER_BINDING_EXT;
        if (!strcmp(name, "GL_PIXEL_TRANSFORM_2D_MATRIX_EXT")) return GL_PIXEL_TRANSFORM_2D_MATRIX_EXT;
        if (!strcmp(name, "GL_POINT_FADE_THRESHOLD_SIZE_ARB")) return GL_POINT_FADE_THRESHOLD_SIZE_ARB;
        if (!strcmp(name, "GL_POINT_FADE_THRESHOLD_SIZE_EXT")) return GL_POINT_FADE_THRESHOLD_SIZE_EXT;
        if (!strcmp(name, "GL_POINT_SPRITE_CULL_CENTER_ATIX")) return GL_POINT_SPRITE_CULL_CENTER_ATIX;
        if (!strcmp(name, "GL_POST_COLOR_MATRIX_ALPHA_SCALE")) return GL_POST_COLOR_MATRIX_ALPHA_SCALE;
        if (!strcmp(name, "GL_POST_COLOR_MATRIX_COLOR_TABLE")) return GL_POST_COLOR_MATRIX_COLOR_TABLE;
        if (!strcmp(name, "GL_POST_COLOR_MATRIX_GREEN_SCALE")) return GL_POST_COLOR_MATRIX_GREEN_SCALE;
        if (!strcmp(name, "GL_POST_CONVOLUTION_RED_BIAS_EXT")) return GL_POST_CONVOLUTION_RED_BIAS_EXT;
        if (!strcmp(name, "GL_POST_TEXTURE_FILTER_BIAS_SGIX")) return GL_POST_TEXTURE_FILTER_BIAS_SGIX;
        if (!strcmp(name, "GL_PROGRAM_ADDRESS_REGISTERS_ARB")) return GL_PROGRAM_ADDRESS_REGISTERS_ARB;
        if (!strcmp(name, "GL_PROGRAM_NATIVE_PARAMETERS_ARB")) return GL_PROGRAM_NATIVE_PARAMETERS_ARB;
        if (!strcmp(name, "GL_PROXY_TEXTURE_COLOR_TABLE_SGI")) return GL_PROXY_TEXTURE_COLOR_TABLE_SGI;
        if (!strcmp(name, "GL_RASTER_POSITION_UNCLIPPED_IBM")) return GL_RASTER_POSITION_UNCLIPPED_IBM;
        if (!strcmp(name, "GL_RENDERBUFFER_COLOR_SAMPLES_NV")) return GL_RENDERBUFFER_COLOR_SAMPLES_NV;
        if (!strcmp(name, "GL_RENDERBUFFER_STENCIL_SIZE_EXT")) return GL_RENDERBUFFER_STENCIL_SIZE_EXT;
        if (!strcmp(name, "GL_SECONDARY_COLOR_ARRAY_POINTER")) return GL_SECONDARY_COLOR_ARRAY_POINTER;
        if (!strcmp(name, "GL_SGIX_texture_coordinate_clamp")) return GL_SGIX_texture_coordinate_clamp;
        if (!strcmp(name, "GL_SMOOTH_LINE_WIDTH_GRANULARITY")) return GL_SMOOTH_LINE_WIDTH_GRANULARITY;
        if (!strcmp(name, "GL_SMOOTH_POINT_SIZE_GRANULARITY")) return GL_SMOOTH_POINT_SIZE_GRANULARITY;
        if (!strcmp(name, "GL_TEXTURE_BINDING_RECTANGLE_ARB")) return GL_TEXTURE_BINDING_RECTANGLE_ARB;
        if (!strcmp(name, "GL_TEXTURE_BINDING_RECTANGLE_EXT")) return GL_TEXTURE_BINDING_RECTANGLE_EXT;
        if (!strcmp(name, "GL_TEXTURE_COMPARE_OPERATOR_SGIX")) return GL_TEXTURE_COMPARE_OPERATOR_SGIX;
        if (!strcmp(name, "GL_TEXTURE_COMPRESSED_IMAGE_SIZE")) return GL_TEXTURE_COMPRESSED_IMAGE_SIZE;
        if (!strcmp(name, "GL_TEXTURE_COORD_ARRAY_COUNT_EXT")) return GL_TEXTURE_COORD_ARRAY_COUNT_EXT;
        if (!strcmp(name, "GL_TRANSFORM_FEEDBACK_ATTRIBS_NV")) return GL_TRANSFORM_FEEDBACK_ATTRIBS_NV;
        if (!strcmp(name, "GL_TRANSFORM_FEEDBACK_BINDING_NV")) return GL_TRANSFORM_FEEDBACK_BINDING_NV;
        if (!strcmp(name, "GL_TRANSFORM_FEEDBACK_BUFFER_EXT")) return GL_TRANSFORM_FEEDBACK_BUFFER_EXT;
        if (!strcmp(name, "GL_UNSIGNED_INT_8_8_S8_S8_REV_NV")) return GL_UNSIGNED_INT_8_8_S8_S8_REV_NV;
        if (!strcmp(name, "GL_UNSIGNED_INT_8_8_S8_S8_REV_NV")) return GL_UNSIGNED_INT_8_8_S8_S8_REV_NV;
        if (!strcmp(name, "GL_UNSIGNED_INT_SAMPLER_1D_ARRAY")) return GL_UNSIGNED_INT_SAMPLER_1D_ARRAY;
        if (!strcmp(name, "GL_UNSIGNED_INT_SAMPLER_2D_ARRAY")) return GL_UNSIGNED_INT_SAMPLER_2D_ARRAY;
        if (!strcmp(name, "GL_UNSIGNED_INT_SAMPLER_CUBE_EXT")) return GL_UNSIGNED_INT_SAMPLER_CUBE_EXT;
        if (!strcmp(name, "GL_VERTEX_ARRAY_RANGE_POINTER_NV")) return GL_VERTEX_ARRAY_RANGE_POINTER_NV;
        if (!strcmp(name, "GL_VERTEX_PROGRAM_POINT_SIZE_ARB")) return GL_VERTEX_PROGRAM_POINT_SIZE_ARB;
        return _GL_UNDEFINED;
    case 33:
        if (!strcmp(name, "GL_ATI_vertex_attrib_array_object")) return GL_ATI_vertex_attrib_array_object;
        if (!strcmp(name, "GL_BUFFER_SERIALIZED_MODIFY_APPLE")) return GL_BUFFER_SERIALIZED_MODIFY_APPLE;
        if (!strcmp(name, "GL_COLOR_ARRAY_BUFFER_BINDING_ARB")) return GL_COLOR_ARRAY_BUFFER_BINDING_ARB;
        if (!strcmp(name, "GL_COLOR_TABLE_INTENSITY_SIZE_EXT")) return GL_COLOR_TABLE_INTENSITY_SIZE_EXT;
        if (!strcmp(name, "GL_COLOR_TABLE_INTENSITY_SIZE_SGI")) return GL_COLOR_TABLE_INTENSITY_SIZE_SGI;
        if (!strcmp(name, "GL_COLOR_TABLE_LUMINANCE_SIZE_EXT")) return GL_COLOR_TABLE_LUMINANCE_SIZE_EXT;
        if (!strcmp(name, "GL_COLOR_TABLE_LUMINANCE_SIZE_SGI")) return GL_COLOR_TABLE_LUMINANCE_SIZE_SGI;
        if (!strcmp(name, "GL_COMPRESSED_LUMINANCE_ALPHA_ARB")) return GL_COMPRESSED_LUMINANCE_ALPHA_ARB;
        if (!strcmp(name, "GL_COMPRESSED_LUMINANCE_LATC1_EXT")) return GL_COMPRESSED_LUMINANCE_LATC1_EXT;
        if (!strcmp(name, "GL_COMPRESSED_RED_GREEN_RGTC2_EXT")) return GL_COMPRESSED_RED_GREEN_RGTC2_EXT;
        if (!strcmp(name, "GL_COMPRESSED_TEXTURE_FORMATS_ARB")) return GL_COMPRESSED_TEXTURE_FORMATS_ARB;
        if (!strcmp(name, "GL_CURRENT_MATRIX_STACK_DEPTH_ARB")) return GL_CURRENT_MATRIX_STACK_DEPTH_ARB;
        if (!strcmp(name, "GL_CURRENT_RASTER_SECONDARY_COLOR")) return GL_CURRENT_RASTER_SECONDARY_COLOR;
        if (!strcmp(name, "GL_EDGE_FLAG_ARRAY_BUFFER_BINDING")) return GL_EDGE_FLAG_ARRAY_BUFFER_BINDING;
        if (!strcmp(name, "GL_EXT_texture_filter_anisotropic")) return GL_EXT_texture_filter_anisotropic;
        if (!strcmp(name, "GL_FLOAT_32_UNSIGNED_INT_24_8_REV")) return GL_FLOAT_32_UNSIGNED_INT_24_8_REV;
        if (!strcmp(name, "GL_FOG_COORD_ARRAY_BUFFER_BINDING")) return GL_FOG_COORD_ARRAY_BUFFER_BINDING;
        if (!strcmp(name, "GL_INDEX_ARRAY_BUFFER_BINDING_ARB")) return GL_INDEX_ARRAY_BUFFER_BINDING_ARB;
        if (!strcmp(name, "GL_MATRIX_INDEX_ARRAY_POINTER_ARB")) return GL_MATRIX_INDEX_ARRAY_POINTER_ARB;
        if (!strcmp(name, "GL_MAX_PROGRAM_ENV_PARAMETERS_ARB")) return GL_MAX_PROGRAM_ENV_PARAMETERS_ARB;
        if (!strcmp(name, "GL_MAX_PROGRAM_GENERIC_ATTRIBS_NV")) return GL_MAX_PROGRAM_GENERIC_ATTRIBS_NV;
        if (!strcmp(name, "GL_MAX_PROGRAM_GENERIC_RESULTS_NV")) return GL_MAX_PROGRAM_GENERIC_RESULTS_NV;
        if (!strcmp(name, "GL_MAX_PROGRAM_NATIVE_ATTRIBS_ARB")) return GL_MAX_PROGRAM_NATIVE_ATTRIBS_ARB;
        if (!strcmp(name, "GL_MAX_PROGRAM_OUTPUT_VERTICES_NV")) return GL_MAX_PROGRAM_OUTPUT_VERTICES_NV;
        if (!strcmp(name, "GL_MAX_RECTANGLE_TEXTURE_SIZE_ARB")) return GL_MAX_RECTANGLE_TEXTURE_SIZE_ARB;
        if (!strcmp(name, "GL_MAX_RECTANGLE_TEXTURE_SIZE_EXT")) return GL_MAX_RECTANGLE_TEXTURE_SIZE_EXT;
        if (!strcmp(name, "GL_MAX_TEXTURE_MAX_ANISOTROPY_EXT")) return GL_MAX_TEXTURE_MAX_ANISOTROPY_EXT;
        if (!strcmp(name, "GL_MAX_VERTEX_SHADER_VARIANTS_EXT")) return GL_MAX_VERTEX_SHADER_VARIANTS_EXT;
        if (!strcmp(name, "GL_MAX_VERTEX_TEXTURE_IMAGE_UNITS")) return GL_MAX_VERTEX_TEXTURE_IMAGE_UNITS;
        if (!strcmp(name, "GL_NATIVE_GRAPHICS_BEGIN_HINT_PGI")) return GL_NATIVE_GRAPHICS_BEGIN_HINT_PGI;
        if (!strcmp(name, "GL_NUM_COMPRESSED_TEXTURE_FORMATS")) return GL_NUM_COMPRESSED_TEXTURE_FORMATS;
        if (!strcmp(name, "GL_POINT_DISTANCE_ATTENUATION_ARB")) return GL_POINT_DISTANCE_ATTENUATION_ARB;
        if (!strcmp(name, "GL_POST_COLOR_MATRIX_RED_BIAS_SGI")) return GL_POST_COLOR_MATRIX_RED_BIAS_SGI;
        if (!strcmp(name, "GL_POST_CONVOLUTION_BLUE_BIAS_EXT")) return GL_POST_CONVOLUTION_BLUE_BIAS_EXT;
        if (!strcmp(name, "GL_POST_CONVOLUTION_RED_SCALE_EXT")) return GL_POST_CONVOLUTION_RED_SCALE_EXT;
        if (!strcmp(name, "GL_POST_TEXTURE_FILTER_SCALE_SGIX")) return GL_POST_TEXTURE_FILTER_SCALE_SGIX;
        if (!strcmp(name, "GL_PROGRAM_MATRIX_STACK_DEPTH_EXT")) return GL_PROGRAM_MATRIX_STACK_DEPTH_EXT;
        if (!strcmp(name, "GL_PROGRAM_NATIVE_TEMPORARIES_ARB")) return GL_PROGRAM_NATIVE_TEMPORARIES_ARB;
        if (!strcmp(name, "GL_SECONDARY_COLOR_ARRAY_LIST_IBM")) return GL_SECONDARY_COLOR_ARRAY_LIST_IBM;
        if (!strcmp(name, "GL_SECONDARY_COLOR_ARRAY_SIZE_EXT")) return GL_SECONDARY_COLOR_ARRAY_SIZE_EXT;
        if (!strcmp(name, "GL_SECONDARY_COLOR_ARRAY_TYPE_EXT")) return GL_SECONDARY_COLOR_ARRAY_TYPE_EXT;
        if (!strcmp(name, "GL_SIGNED_RGB8_UNSIGNED_ALPHA8_NV")) return GL_SIGNED_RGB8_UNSIGNED_ALPHA8_NV;
        if (!strcmp(name, "GL_SIGNED_RGB8_UNSIGNED_ALPHA8_NV")) return GL_SIGNED_RGB8_UNSIGNED_ALPHA8_NV;
        if (!strcmp(name, "GL_SPARE0_PLUS_SECONDARY_COLOR_NV")) return GL_SPARE0_PLUS_SECONDARY_COLOR_NV;
        if (!strcmp(name, "GL_TEXTURE_1D_STACK_BINDING_MESAX")) return GL_TEXTURE_1D_STACK_BINDING_MESAX;
        if (!strcmp(name, "GL_TEXTURE_2D_STACK_BINDING_MESAX")) return GL_TEXTURE_2D_STACK_BINDING_MESAX;
        if (!strcmp(name, "GL_TEXTURE_COMPARE_FAIL_VALUE_ARB")) return GL_TEXTURE_COMPARE_FAIL_VALUE_ARB;
        if (!strcmp(name, "GL_TEXTURE_COORD_ARRAY_STRIDE_EXT")) return GL_TEXTURE_COORD_ARRAY_STRIDE_EXT;
        if (!strcmp(name, "GL_TEXTURE_MATERIAL_PARAMETER_EXT")) return GL_TEXTURE_MATERIAL_PARAMETER_EXT;
        if (!strcmp(name, "GL_TEXTURE_MULTI_BUFFER_HINT_SGIX")) return GL_TEXTURE_MULTI_BUFFER_HINT_SGIX;
        if (!strcmp(name, "GL_TEXTURE_UNSIGNED_REMAP_MODE_NV")) return GL_TEXTURE_UNSIGNED_REMAP_MODE_NV;
        if (!strcmp(name, "GL_TRANSFORM_FEEDBACK_BUFFER_MODE")) return GL_TRANSFORM_FEEDBACK_BUFFER_MODE;
        if (!strcmp(name, "GL_TRANSFORM_FEEDBACK_BUFFER_SIZE")) return GL_TRANSFORM_FEEDBACK_BUFFER_SIZE;
        if (!strcmp(name, "GL_TRANSFORM_FEEDBACK_VARYINGS_NV")) return GL_TRANSFORM_FEEDBACK_VARYINGS_NV;
        if (!strcmp(name, "GL_TRANSPOSE_MODELVIEW_MATRIX_ARB")) return GL_TRANSPOSE_MODELVIEW_MATRIX_ARB;
        if (!strcmp(name, "GL_VERTEX_ATTRIB_ARRAY_NORMALIZED")) return GL_VERTEX_ATTRIB_ARRAY_NORMALIZED;
        if (!strcmp(name, "GL_VERTEX_ATTRIB_ARRAY_STRIDE_ARB")) return GL_VERTEX_ATTRIB_ARRAY_STRIDE_ARB;
        if (!strcmp(name, "GL_VERTEX_SHADER_INSTRUCTIONS_EXT")) return GL_VERTEX_SHADER_INSTRUCTIONS_EXT;
        if (!strcmp(name, "GL_VERTEX_WEIGHT_ARRAY_STRIDE_EXT")) return GL_VERTEX_WEIGHT_ARRAY_STRIDE_EXT;
        return _GL_UNDEFINED;
    case 34:
        if (!strcmp(name, "GL_COMPRESSED_SIGNED_RED_RGTC1_EXT")) return GL_COMPRESSED_SIGNED_RED_RGTC1_EXT;
        if (!strcmp(name, "GL_COMPRESSED_SLUMINANCE_ALPHA_EXT")) return GL_COMPRESSED_SLUMINANCE_ALPHA_EXT;
        if (!strcmp(name, "GL_DOT_PRODUCT_DIFFUSE_CUBE_MAP_NV")) return GL_DOT_PRODUCT_DIFFUSE_CUBE_MAP_NV;
        if (!strcmp(name, "GL_DOT_PRODUCT_REFLECT_CUBE_MAP_NV")) return GL_DOT_PRODUCT_REFLECT_CUBE_MAP_NV;
        if (!strcmp(name, "GL_DOT_PRODUCT_TEXTURE_CUBE_MAP_NV")) return GL_DOT_PRODUCT_TEXTURE_CUBE_MAP_NV;
        if (!strcmp(name, "GL_EDGE_FLAG_ARRAY_LIST_STRIDE_IBM")) return GL_EDGE_FLAG_ARRAY_LIST_STRIDE_IBM;
        if (!strcmp(name, "GL_EVAL_FRACTIONAL_TESSELLATION_NV")) return GL_EVAL_FRACTIONAL_TESSELLATION_NV;
        if (!strcmp(name, "GL_EXT_pixel_transform_color_table")) return GL_EXT_pixel_transform_color_table;
        if (!strcmp(name, "GL_FOG_COORDINATE_ARRAY_STRIDE_EXT")) return GL_FOG_COORDINATE_ARRAY_STRIDE_EXT;
        if (!strcmp(name, "GL_FRAGMENT_SHADER_DERIVATIVE_HINT")) return GL_FRAGMENT_SHADER_DERIVATIVE_HINT;
        if (!strcmp(name, "GL_FRAMEBUFFER_ATTACHMENT_RED_SIZE")) return GL_FRAMEBUFFER_ATTACHMENT_RED_SIZE;
        if (!strcmp(name, "GL_INTENSITY16_EXTENDED_RANGE_SGIX")) return GL_INTENSITY16_EXTENDED_RANGE_SGIX;
        if (!strcmp(name, "GL_LUMINANCE16_ALPHA16_SIGNED_SGIX")) return GL_LUMINANCE16_ALPHA16_SIGNED_SGIX;
        if (!strcmp(name, "GL_LUMINANCE16_EXTENDED_RANGE_SGIX")) return GL_LUMINANCE16_EXTENDED_RANGE_SGIX;
        if (!strcmp(name, "GL_MAT_AMBIENT_AND_DIFFUSE_BIT_PGI")) return GL_MAT_AMBIENT_AND_DIFFUSE_BIT_PGI;
        if (!strcmp(name, "GL_MAX_FRAGMENT_UNIFORM_COMPONENTS")) return GL_MAX_FRAGMENT_UNIFORM_COMPONENTS;
        if (!strcmp(name, "GL_MAX_TRACK_MATRIX_STACK_DEPTH_NV")) return GL_MAX_TRACK_MATRIX_STACK_DEPTH_NV;
        if (!strcmp(name, "GL_MIN_PBUFFER_VIEWPORT_DIMS_APPLE")) return GL_MIN_PBUFFER_VIEWPORT_DIMS_APPLE;
        if (!strcmp(name, "GL_NORMAL_ARRAY_BUFFER_BINDING_ARB")) return GL_NORMAL_ARRAY_BUFFER_BINDING_ARB;
        if (!strcmp(name, "GL_OBJECT_SHADER_SOURCE_LENGTH_ARB")) return GL_OBJECT_SHADER_SOURCE_LENGTH_ARB;
        if (!strcmp(name, "GL_OES_compressed_paletted_texture")) return GL_OES_compressed_paletted_texture;
        if (!strcmp(name, "GL_OFFSET_PROJECTIVE_TEXTURE_2D_NV")) return GL_OFFSET_PROJECTIVE_TEXTURE_2D_NV;
        if (!strcmp(name, "GL_PIXEL_UNPACK_BUFFER_BINDING_ARB")) return GL_PIXEL_UNPACK_BUFFER_BINDING_ARB;
        if (!strcmp(name, "GL_PIXEL_UNPACK_BUFFER_BINDING_EXT")) return GL_PIXEL_UNPACK_BUFFER_BINDING_EXT;
        if (!strcmp(name, "GL_POST_COLOR_MATRIX_BLUE_BIAS_SGI")) return GL_POST_COLOR_MATRIX_BLUE_BIAS_SGI;
        if (!strcmp(name, "GL_POST_COLOR_MATRIX_RED_SCALE_SGI")) return GL_POST_COLOR_MATRIX_RED_SCALE_SGI;
        if (!strcmp(name, "GL_POST_CONVOLUTION_ALPHA_BIAS_EXT")) return GL_POST_CONVOLUTION_ALPHA_BIAS_EXT;
        if (!strcmp(name, "GL_POST_CONVOLUTION_BLUE_SCALE_EXT")) return GL_POST_CONVOLUTION_BLUE_SCALE_EXT;
        if (!strcmp(name, "GL_POST_CONVOLUTION_GREEN_BIAS_EXT")) return GL_POST_CONVOLUTION_GREEN_BIAS_EXT;
        if (!strcmp(name, "GL_PROGRAM_NATIVE_INSTRUCTIONS_ARB")) return GL_PROGRAM_NATIVE_INSTRUCTIONS_ARB;
        if (!strcmp(name, "GL_PROGRAM_UNDER_NATIVE_LIMITS_ARB")) return GL_PROGRAM_UNDER_NATIVE_LIMITS_ARB;
        if (!strcmp(name, "GL_READ_PIXEL_DATA_RANGE_LENGTH_NV")) return GL_READ_PIXEL_DATA_RANGE_LENGTH_NV;
        if (!strcmp(name, "GL_REPLACEMENT_CODE_ARRAY_TYPE_SUN")) return GL_REPLACEMENT_CODE_ARRAY_TYPE_SUN;
        if (!strcmp(name, "GL_SGIX_fragment_specular_lighting")) return GL_SGIX_fragment_specular_lighting;
        if (!strcmp(name, "GL_TEXTURE_BINDING_RENDERBUFFER_NV")) return GL_TEXTURE_BINDING_RENDERBUFFER_NV;
        if (!strcmp(name, "GL_TEXTURE_COORD_ARRAY_POINTER_EXT")) return GL_TEXTURE_COORD_ARRAY_POINTER_EXT;
        if (!strcmp(name, "GL_TEXTURE_CUBE_MAP_NEGATIVE_X_ARB")) return GL_TEXTURE_CUBE_MAP_NEGATIVE_X_ARB;
        if (!strcmp(name, "GL_TEXTURE_CUBE_MAP_NEGATIVE_X_EXT")) return GL_TEXTURE_CUBE_MAP_NEGATIVE_X_EXT;
        if (!strcmp(name, "GL_TEXTURE_CUBE_MAP_NEGATIVE_Y_ARB")) return GL_TEXTURE_CUBE_MAP_NEGATIVE_Y_ARB;
        if (!strcmp(name, "GL_TEXTURE_CUBE_MAP_NEGATIVE_Y_EXT")) return GL_TEXTURE_CUBE_MAP_NEGATIVE_Y_EXT;
        if (!strcmp(name, "GL_TEXTURE_CUBE_MAP_NEGATIVE_Z_ARB")) return GL_TEXTURE_CUBE_MAP_NEGATIVE_Z_ARB;
        if (!strcmp(name, "GL_TEXTURE_CUBE_MAP_NEGATIVE_Z_EXT")) return GL_TEXTURE_CUBE_MAP_NEGATIVE_Z_EXT;
        if (!strcmp(name, "GL_TEXTURE_CUBE_MAP_POSITIVE_X_ARB")) return GL_TEXTURE_CUBE_MAP_POSITIVE_X_ARB;
        if (!strcmp(name, "GL_TEXTURE_CUBE_MAP_POSITIVE_X_EXT")) return GL_TEXTURE_CUBE_MAP_POSITIVE_X_EXT;
        if (!strcmp(name, "GL_TEXTURE_CUBE_MAP_POSITIVE_Y_ARB")) return GL_TEXTURE_CUBE_MAP_POSITIVE_Y_ARB;
        if (!strcmp(name, "GL_TEXTURE_CUBE_MAP_POSITIVE_Y_EXT")) return GL_TEXTURE_CUBE_MAP_POSITIVE_Y_EXT;
        if (!strcmp(name, "GL_TEXTURE_CUBE_MAP_POSITIVE_Z_ARB")) return GL_TEXTURE_CUBE_MAP_POSITIVE_Z_ARB;
        if (!strcmp(name, "GL_TEXTURE_CUBE_MAP_POSITIVE_Z_EXT")) return GL_TEXTURE_CUBE_MAP_POSITIVE_Z_EXT;
        if (!strcmp(name, "GL_TRANSFORM_FEEDBACK_BUFFER_START")) return GL_TRANSFORM_FEEDBACK_BUFFER_START;
        if (!strcmp(name, "GL_TRANSFORM_FEEDBACK_VARYINGS_EXT")) return GL_TRANSFORM_FEEDBACK_VARYINGS_EXT;
        if (!strcmp(name, "GL_TRANSPOSE_PROJECTION_MATRIX_ARB")) return GL_TRANSPOSE_PROJECTION_MATRIX_ARB;
        if (!strcmp(name, "GL_UNSIGNED_INT_SAMPLER_BUFFER_EXT")) return GL_UNSIGNED_INT_SAMPLER_BUFFER_EXT;
        if (!strcmp(name, "GL_VERTEX_ARRAY_BUFFER_BINDING_ARB")) return GL_VERTEX_ARRAY_BUFFER_BINDING_ARB;
        if (!strcmp(name, "GL_VERTEX_ARRAY_RANGE_LENGTH_APPLE")) return GL_VERTEX_ARRAY_RANGE_LENGTH_APPLE;
        if (!strcmp(name, "GL_VERTEX_ARRAY_STORAGE_HINT_APPLE")) return GL_VERTEX_ARRAY_STORAGE_HINT_APPLE;
        if (!strcmp(name, "GL_VERTEX_ATTRIB_ARRAY_DIVISOR_ARB")) return GL_VERTEX_ATTRIB_ARRAY_DIVISOR_ARB;
        if (!strcmp(name, "GL_VERTEX_ATTRIB_ARRAY_ENABLED_ARB")) return GL_VERTEX_ATTRIB_ARRAY_ENABLED_ARB;
        if (!strcmp(name, "GL_VERTEX_ATTRIB_ARRAY_INTEGER_EXT")) return GL_VERTEX_ATTRIB_ARRAY_INTEGER_EXT;
        if (!strcmp(name, "GL_VERTEX_ATTRIB_ARRAY_POINTER_ARB")) return GL_VERTEX_ATTRIB_ARRAY_POINTER_ARB;
        if (!strcmp(name, "GL_VERTEX_WEIGHT_ARRAY_POINTER_EXT")) return GL_VERTEX_WEIGHT_ARRAY_POINTER_EXT;
        if (!strcmp(name, "GL_WEIGHT_ARRAY_BUFFER_BINDING_ARB")) return GL_WEIGHT_ARRAY_BUFFER_BINDING_ARB;
        return _GL_UNDEFINED;
    case 35:
        if (!strcmp(name, "GL_COMPARE_REF_DEPTH_TO_TEXTURE_EXT")) return GL_COMPARE_REF_DEPTH_TO_TEXTURE_EXT;
        if (!strcmp(name, "GL_DOT_PRODUCT_TEXTURE_RECTANGLE_NV")) return GL_DOT_PRODUCT_TEXTURE_RECTANGLE_NV;
        if (!strcmp(name, "GL_ELEMENT_ARRAY_BUFFER_BINDING_ARB")) return GL_ELEMENT_ARRAY_BUFFER_BINDING_ARB;
        if (!strcmp(name, "GL_FOG_COORDINATE_ARRAY_POINTER_EXT")) return GL_FOG_COORDINATE_ARRAY_POINTER_EXT;
        if (!strcmp(name, "GL_FRAGMENT_COLOR_MATERIAL_FACE_EXT")) return GL_FRAGMENT_COLOR_MATERIAL_FACE_EXT;
        if (!strcmp(name, "GL_FRAGMENT_LIGHT_MODEL_AMBIENT_EXT")) return GL_FRAGMENT_LIGHT_MODEL_AMBIENT_EXT;
        if (!strcmp(name, "GL_FRAMEBUFFER_ATTACHMENT_BLUE_SIZE")) return GL_FRAMEBUFFER_ATTACHMENT_BLUE_SIZE;
        if (!strcmp(name, "GL_MAX_COLOR_MATRIX_STACK_DEPTH_SGI")) return GL_MAX_COLOR_MATRIX_STACK_DEPTH_SGI;
        if (!strcmp(name, "GL_MAX_COMBINED_TEXTURE_IMAGE_UNITS")) return GL_MAX_COMBINED_TEXTURE_IMAGE_UNITS;
        if (!strcmp(name, "GL_MAX_GEOMETRY_OUTPUT_VERTICES_ARB")) return GL_MAX_GEOMETRY_OUTPUT_VERTICES_ARB;
        if (!strcmp(name, "GL_MAX_GEOMETRY_OUTPUT_VERTICES_EXT")) return GL_MAX_GEOMETRY_OUTPUT_VERTICES_EXT;
        if (!strcmp(name, "GL_MAX_PROGRAM_ALU_INSTRUCTIONS_ARB")) return GL_MAX_PROGRAM_ALU_INSTRUCTIONS_ARB;
        if (!strcmp(name, "GL_MAX_PROGRAM_ATTRIB_COMPONENTS_NV")) return GL_MAX_PROGRAM_ATTRIB_COMPONENTS_NV;
        if (!strcmp(name, "GL_MAX_PROGRAM_EXEC_INSTRUCTIONS_NV")) return GL_MAX_PROGRAM_EXEC_INSTRUCTIONS_NV;
        if (!strcmp(name, "GL_MAX_PROGRAM_EXEC_INSTRUCTIONS_NV")) return GL_MAX_PROGRAM_EXEC_INSTRUCTIONS_NV;
        if (!strcmp(name, "GL_MAX_PROGRAM_LOCAL_PARAMETERS_ARB")) return GL_MAX_PROGRAM_LOCAL_PARAMETERS_ARB;
        if (!strcmp(name, "GL_MAX_PROGRAM_RESULT_COMPONENTS_NV")) return GL_MAX_PROGRAM_RESULT_COMPONENTS_NV;
        if (!strcmp(name, "GL_MAX_PROGRAM_TEX_INDIRECTIONS_ARB")) return GL_MAX_PROGRAM_TEX_INDIRECTIONS_ARB;
        if (!strcmp(name, "GL_MAX_PROGRAM_TEX_INSTRUCTIONS_ARB")) return GL_MAX_PROGRAM_TEX_INSTRUCTIONS_ARB;
        if (!strcmp(name, "GL_MAX_VERTEX_BINDABLE_UNIFORMS_EXT")) return GL_MAX_VERTEX_BINDABLE_UNIFORMS_EXT;
        if (!strcmp(name, "GL_MAX_VERTEX_SHADER_INVARIANTS_EXT")) return GL_MAX_VERTEX_SHADER_INVARIANTS_EXT;
        if (!strcmp(name, "GL_OFFSET_HILO_TEXTURE_RECTANGLE_NV")) return GL_OFFSET_HILO_TEXTURE_RECTANGLE_NV;
        if (!strcmp(name, "GL_POST_COLOR_MATRIX_ALPHA_BIAS_SGI")) return GL_POST_COLOR_MATRIX_ALPHA_BIAS_SGI;
        if (!strcmp(name, "GL_POST_COLOR_MATRIX_BLUE_SCALE_SGI")) return GL_POST_COLOR_MATRIX_BLUE_SCALE_SGI;
        if (!strcmp(name, "GL_POST_COLOR_MATRIX_GREEN_BIAS_SGI")) return GL_POST_COLOR_MATRIX_GREEN_BIAS_SGI;
        if (!strcmp(name, "GL_POST_CONVOLUTION_ALPHA_SCALE_EXT")) return GL_POST_CONVOLUTION_ALPHA_SCALE_EXT;
        if (!strcmp(name, "GL_POST_CONVOLUTION_COLOR_TABLE_SGI")) return GL_POST_CONVOLUTION_COLOR_TABLE_SGI;
        if (!strcmp(name, "GL_POST_CONVOLUTION_GREEN_SCALE_EXT")) return GL_POST_CONVOLUTION_GREEN_SCALE_EXT;
        if (!strcmp(name, "GL_READ_PIXEL_DATA_RANGE_POINTER_NV")) return GL_READ_PIXEL_DATA_RANGE_POINTER_NV;
        if (!strcmp(name, "GL_RENDERBUFFER_COVERAGE_SAMPLES_NV")) return GL_RENDERBUFFER_COVERAGE_SAMPLES_NV;
        if (!strcmp(name, "GL_RENDERBUFFER_INTERNAL_FORMAT_EXT")) return GL_RENDERBUFFER_INTERNAL_FORMAT_EXT;
        if (!strcmp(name, "GL_SECONDARY_COLOR_ARRAY_STRIDE_EXT")) return GL_SECONDARY_COLOR_ARRAY_STRIDE_EXT;
        if (!strcmp(name, "GL_STENCIL_BACK_PASS_DEPTH_FAIL_ATI")) return GL_STENCIL_BACK_PASS_DEPTH_FAIL_ATI;
        if (!strcmp(name, "GL_STENCIL_BACK_PASS_DEPTH_PASS_ATI")) return GL_STENCIL_BACK_PASS_DEPTH_PASS_ATI;
        if (!strcmp(name, "GL_UNSIGNED_INT_10F_11F_11F_REV_EXT")) return GL_UNSIGNED_INT_10F_11F_11F_REV_EXT;
        if (!strcmp(name, "GL_UNSIGNED_INT_SAMPLER_2D_RECT_EXT")) return GL_UNSIGNED_INT_SAMPLER_2D_RECT_EXT;
        if (!strcmp(name, "GL_VERTEX_ARRAY_RANGE_POINTER_APPLE")) return GL_VERTEX_ARRAY_RANGE_POINTER_APPLE;
        if (!strcmp(name, "GL_WRITE_PIXEL_DATA_RANGE_LENGTH_NV")) return GL_WRITE_PIXEL_DATA_RANGE_LENGTH_NV;
        return _GL_UNDEFINED;
    case 36:
        if (!strcmp(name, "GL_DEPENDENT_RGB_TEXTURE_CUBE_MAP_NV")) return GL_DEPENDENT_RGB_TEXTURE_CUBE_MAP_NV;
        if (!strcmp(name, "GL_FLOAT_32_UNSIGNED_INT_24_8_REV_NV")) return GL_FLOAT_32_UNSIGNED_INT_24_8_REV_NV;
        if (!strcmp(name, "GL_FRAGMENT_LIGHT_MODEL_TWO_SIDE_EXT")) return GL_FRAGMENT_LIGHT_MODEL_TWO_SIDE_EXT;
        if (!strcmp(name, "GL_FRAMEBUFFER_ATTACHMENT_ALPHA_SIZE")) return GL_FRAMEBUFFER_ATTACHMENT_ALPHA_SIZE;
        if (!strcmp(name, "GL_FRAMEBUFFER_ATTACHMENT_DEPTH_SIZE")) return GL_FRAMEBUFFER_ATTACHMENT_DEPTH_SIZE;
        if (!strcmp(name, "GL_FRAMEBUFFER_ATTACHMENT_GREEN_SIZE")) return GL_FRAMEBUFFER_ATTACHMENT_GREEN_SIZE;
        if (!strcmp(name, "GL_FRAMEBUFFER_INCOMPLETE_ATTACHMENT")) return GL_FRAMEBUFFER_INCOMPLETE_ATTACHMENT;
        if (!strcmp(name, "GL_INVALID_FRAMEBUFFER_OPERATION_EXT")) return GL_INVALID_FRAMEBUFFER_OPERATION_EXT;
        if (!strcmp(name, "GL_LIGHT_MODEL_SPECULAR_VECTOR_APPLE")) return GL_LIGHT_MODEL_SPECULAR_VECTOR_APPLE;
        if (!strcmp(name, "GL_MAX_MULTISAMPLE_COVERAGE_MODES_NV")) return GL_MAX_MULTISAMPLE_COVERAGE_MODES_NV;
        if (!strcmp(name, "GL_MAX_PROGRAM_ADDRESS_REGISTERS_ARB")) return GL_MAX_PROGRAM_ADDRESS_REGISTERS_ARB;
        if (!strcmp(name, "GL_MAX_PROGRAM_NATIVE_PARAMETERS_ARB")) return GL_MAX_PROGRAM_NATIVE_PARAMETERS_ARB;
        if (!strcmp(name, "GL_MAX_VERTEX_ARRAY_RANGE_ELEMENT_NV")) return GL_MAX_VERTEX_ARRAY_RANGE_ELEMENT_NV;
        if (!strcmp(name, "GL_MAX_VERTEX_UNIFORM_COMPONENTS_ARB")) return GL_MAX_VERTEX_UNIFORM_COMPONENTS_ARB;
        if (!strcmp(name, "GL_MAX_VERTEX_VARYING_COMPONENTS_ARB")) return GL_MAX_VERTEX_VARYING_COMPONENTS_ARB;
        if (!strcmp(name, "GL_MAX_VERTEX_VARYING_COMPONENTS_EXT")) return GL_MAX_VERTEX_VARYING_COMPONENTS_EXT;
        if (!strcmp(name, "GL_OFFSET_TEXTURE_RECTANGLE_SCALE_NV")) return GL_OFFSET_TEXTURE_RECTANGLE_SCALE_NV;
        if (!strcmp(name, "GL_PN_TRIANGLES_POINT_MODE_CUBIC_ATI")) return GL_PN_TRIANGLES_POINT_MODE_CUBIC_ATI;
        if (!strcmp(name, "GL_POST_COLOR_MATRIX_ALPHA_SCALE_SGI")) return GL_POST_COLOR_MATRIX_ALPHA_SCALE_SGI;
        if (!strcmp(name, "GL_POST_COLOR_MATRIX_COLOR_TABLE_SGI")) return GL_POST_COLOR_MATRIX_COLOR_TABLE_SGI;
        if (!strcmp(name, "GL_POST_COLOR_MATRIX_GREEN_SCALE_SGI")) return GL_POST_COLOR_MATRIX_GREEN_SCALE_SGI;
        if (!strcmp(name, "GL_REPLACEMENT_CODE_ARRAY_STRIDE_SUN")) return GL_REPLACEMENT_CODE_ARRAY_STRIDE_SUN;
        if (!strcmp(name, "GL_SECONDARY_COLOR_ARRAY_POINTER_EXT")) return GL_SECONDARY_COLOR_ARRAY_POINTER_EXT;
        if (!strcmp(name, "GL_TEXTURE_COMPRESSED_IMAGE_SIZE_ARB")) return GL_TEXTURE_COMPRESSED_IMAGE_SIZE_ARB;
        if (!strcmp(name, "GL_TRANSFORM_FEEDBACK_BUFFER_BINDING")) return GL_TRANSFORM_FEEDBACK_BUFFER_BINDING;
        if (!strcmp(name, "GL_TRANSFORM_FEEDBACK_BUFFER_MODE_NV")) return GL_TRANSFORM_FEEDBACK_BUFFER_MODE_NV;
        if (!strcmp(name, "GL_TRANSFORM_FEEDBACK_BUFFER_SIZE_NV")) return GL_TRANSFORM_FEEDBACK_BUFFER_SIZE_NV;
        if (!strcmp(name, "GL_UNSIGNED_INT_SAMPLER_1D_ARRAY_EXT")) return GL_UNSIGNED_INT_SAMPLER_1D_ARRAY_EXT;
        if (!strcmp(name, "GL_UNSIGNED_INT_SAMPLER_2D_ARRAY_EXT")) return GL_UNSIGNED_INT_SAMPLER_2D_ARRAY_EXT;
        if (!strcmp(name, "GL_VERTEX_SHADER_LOCAL_CONSTANTS_EXT")) return GL_VERTEX_SHADER_LOCAL_CONSTANTS_EXT;
        if (!strcmp(name, "GL_WRITE_PIXEL_DATA_RANGE_POINTER_NV")) return GL_WRITE_PIXEL_DATA_RANGE_POINTER_NV;
        return _GL_UNDEFINED;
    case 37:
        if (!strcmp(name, "GL_COMPRESSED_LUMINANCE_ALPHA_3DC_ATI")) return GL_COMPRESSED_LUMINANCE_ALPHA_3DC_ATI;
        if (!strcmp(name, "GL_EDGE_FLAG_ARRAY_BUFFER_BINDING_ARB")) return GL_EDGE_FLAG_ARRAY_BUFFER_BINDING_ARB;
        if (!strcmp(name, "GL_FRAMEBUFFER_ATTACHMENT_LAYERED_ARB")) return GL_FRAMEBUFFER_ATTACHMENT_LAYERED_ARB;
        if (!strcmp(name, "GL_FRAMEBUFFER_ATTACHMENT_LAYERED_EXT")) return GL_FRAMEBUFFER_ATTACHMENT_LAYERED_EXT;
        if (!strcmp(name, "GL_FRAMEBUFFER_ATTACHMENT_OBJECT_NAME")) return GL_FRAMEBUFFER_ATTACHMENT_OBJECT_NAME;
        if (!strcmp(name, "GL_FRAMEBUFFER_ATTACHMENT_OBJECT_TYPE")) return GL_FRAMEBUFFER_ATTACHMENT_OBJECT_TYPE;
        if (!strcmp(name, "GL_FRAMEBUFFER_INCOMPLETE_DRAW_BUFFER")) return GL_FRAMEBUFFER_INCOMPLETE_DRAW_BUFFER;
        if (!strcmp(name, "GL_FRAMEBUFFER_INCOMPLETE_FORMATS_EXT")) return GL_FRAMEBUFFER_INCOMPLETE_FORMATS_EXT;
        if (!strcmp(name, "GL_FRAMEBUFFER_INCOMPLETE_MULTISAMPLE")) return GL_FRAMEBUFFER_INCOMPLETE_MULTISAMPLE;
        if (!strcmp(name, "GL_FRAMEBUFFER_INCOMPLETE_READ_BUFFER")) return GL_FRAMEBUFFER_INCOMPLETE_READ_BUFFER;
        if (!strcmp(name, "GL_IMPLEMENTATION_COLOR_READ_TYPE_OES")) return GL_IMPLEMENTATION_COLOR_READ_TYPE_OES;
        if (!strcmp(name, "GL_MAX_FRAGMENT_BINDABLE_UNIFORMS_EXT")) return GL_MAX_FRAGMENT_BINDABLE_UNIFORMS_EXT;
        if (!strcmp(name, "GL_MAX_GEOMETRY_BINDABLE_UNIFORMS_EXT")) return GL_MAX_GEOMETRY_BINDABLE_UNIFORMS_EXT;
        if (!strcmp(name, "GL_MAX_MATRIX_PALETTE_STACK_DEPTH_ARB")) return GL_MAX_MATRIX_PALETTE_STACK_DEPTH_ARB;
        if (!strcmp(name, "GL_MAX_PROGRAM_MATRIX_STACK_DEPTH_ARB")) return GL_MAX_PROGRAM_MATRIX_STACK_DEPTH_ARB;
        if (!strcmp(name, "GL_MAX_PROGRAM_NATIVE_TEMPORARIES_ARB")) return GL_MAX_PROGRAM_NATIVE_TEMPORARIES_ARB;
        if (!strcmp(name, "GL_MAX_VERTEX_SHADER_INSTRUCTIONS_EXT")) return GL_MAX_VERTEX_SHADER_INSTRUCTIONS_EXT;
        if (!strcmp(name, "GL_MAX_VERTEX_TEXTURE_IMAGE_UNITS_ARB")) return GL_MAX_VERTEX_TEXTURE_IMAGE_UNITS_ARB;
        if (!strcmp(name, "GL_NUM_COMPRESSED_TEXTURE_FORMATS_ARB")) return GL_NUM_COMPRESSED_TEXTURE_FORMATS_ARB;
        if (!strcmp(name, "GL_PIXEL_TRANSFORM_2D_STACK_DEPTH_EXT")) return GL_PIXEL_TRANSFORM_2D_STACK_DEPTH_EXT;
        if (!strcmp(name, "GL_PN_TRIANGLES_POINT_MODE_LINEAR_ATI")) return GL_PN_TRIANGLES_POINT_MODE_LINEAR_ATI;
        if (!strcmp(name, "GL_PN_TRIANGLES_TESSELATION_LEVEL_ATI")) return GL_PN_TRIANGLES_TESSELATION_LEVEL_ATI;
        if (!strcmp(name, "GL_PROXY_POST_CONVOLUTION_COLOR_TABLE")) return GL_PROXY_POST_CONVOLUTION_COLOR_TABLE;
        if (!strcmp(name, "GL_REPLACEMENT_CODE_ARRAY_POINTER_SUN")) return GL_REPLACEMENT_CODE_ARRAY_POINTER_SUN;
        if (!strcmp(name, "GL_TEXTURE_COORD_ARRAY_BUFFER_BINDING")) return GL_TEXTURE_COORD_ARRAY_BUFFER_BINDING;
        if (!strcmp(name, "GL_TRANSFORM_FEEDBACK_BUFFER_MODE_EXT")) return GL_TRANSFORM_FEEDBACK_BUFFER_MODE_EXT;
        if (!strcmp(name, "GL_TRANSFORM_FEEDBACK_BUFFER_SIZE_EXT")) return GL_TRANSFORM_FEEDBACK_BUFFER_SIZE_EXT;
        if (!strcmp(name, "GL_TRANSFORM_FEEDBACK_BUFFER_START_NV")) return GL_TRANSFORM_FEEDBACK_BUFFER_START_NV;
        if (!strcmp(name, "GL_VERTEX_ATTRIB_ARRAY_BUFFER_BINDING")) return GL_VERTEX_ATTRIB_ARRAY_BUFFER_BINDING;
        if (!strcmp(name, "GL_VERTEX_ATTRIB_ARRAY_NORMALIZED_ARB")) return GL_VERTEX_ATTRIB_ARRAY_NORMALIZED_ARB;
        if (!strcmp(name, "GL_VERTEX_PROGRAM_PARAMETER_BUFFER_NV")) return GL_VERTEX_PROGRAM_PARAMETER_BUFFER_NV;
        return _GL_UNDEFINED;
    case 38:
        if (!strcmp(name, "GL_COLOR_ARRAY_PARALLEL_POINTERS_INTEL")) return GL_COLOR_ARRAY_PARALLEL_POINTERS_INTEL;
        if (!strcmp(name, "GL_COMPRESSED_SRGB_ALPHA_S3TC_DXT1_EXT")) return GL_COMPRESSED_SRGB_ALPHA_S3TC_DXT1_EXT;
        if (!strcmp(name, "GL_COMPRESSED_SRGB_ALPHA_S3TC_DXT3_EXT")) return GL_COMPRESSED_SRGB_ALPHA_S3TC_DXT3_EXT;
        if (!strcmp(name, "GL_COMPRESSED_SRGB_ALPHA_S3TC_DXT5_EXT")) return GL_COMPRESSED_SRGB_ALPHA_S3TC_DXT5_EXT;
        if (!strcmp(name, "GL_CONTEXT_FLAG_FORWARD_COMPATIBLE_BIT")) return GL_CONTEXT_FLAG_FORWARD_COMPATIBLE_BIT;
        if (!strcmp(name, "GL_DOT_PRODUCT_AFFINE_DEPTH_REPLACE_NV")) return GL_DOT_PRODUCT_AFFINE_DEPTH_REPLACE_NV;
        if (!strcmp(name, "GL_FOG_COORDINATE_ARRAY_BUFFER_BINDING")) return GL_FOG_COORDINATE_ARRAY_BUFFER_BINDING;
        if (!strcmp(name, "GL_FRAGMENT_SHADER_DERIVATIVE_HINT_ARB")) return GL_FRAGMENT_SHADER_DERIVATIVE_HINT_ARB;
        if (!strcmp(name, "GL_FRAMEBUFFER_ATTACHMENT_STENCIL_SIZE")) return GL_FRAMEBUFFER_ATTACHMENT_STENCIL_SIZE;
        if (!strcmp(name, "GL_LUMINANCE_ALPHA_EXTENDED_RANGE_SGIX")) return GL_LUMINANCE_ALPHA_EXTENDED_RANGE_SGIX;
        if (!strcmp(name, "GL_MAX_FRAGMENT_UNIFORM_COMPONENTS_ARB")) return GL_MAX_FRAGMENT_UNIFORM_COMPONENTS_ARB;
        if (!strcmp(name, "GL_MAX_GEOMETRY_UNIFORM_COMPONENTS_ARB")) return GL_MAX_GEOMETRY_UNIFORM_COMPONENTS_ARB;
        if (!strcmp(name, "GL_MAX_GEOMETRY_UNIFORM_COMPONENTS_EXT")) return GL_MAX_GEOMETRY_UNIFORM_COMPONENTS_EXT;
        if (!strcmp(name, "GL_MAX_GEOMETRY_VARYING_COMPONENTS_ARB")) return GL_MAX_GEOMETRY_VARYING_COMPONENTS_ARB;
        if (!strcmp(name, "GL_MAX_GEOMETRY_VARYING_COMPONENTS_EXT")) return GL_MAX_GEOMETRY_VARYING_COMPONENTS_EXT;
        if (!strcmp(name, "GL_MAX_PROGRAM_NATIVE_INSTRUCTIONS_ARB")) return GL_MAX_PROGRAM_NATIVE_INSTRUCTIONS_ARB;
        if (!strcmp(name, "GL_NV_framebuffer_multisample_coverage")) return GL_NV_framebuffer_multisample_coverage;
        if (!strcmp(name, "GL_PN_TRIANGLES_NORMAL_MODE_LINEAR_ATI")) return GL_PN_TRIANGLES_NORMAL_MODE_LINEAR_ATI;
        if (!strcmp(name, "GL_POST_TEXTURE_FILTER_BIAS_RANGE_SGIX")) return GL_POST_TEXTURE_FILTER_BIAS_RANGE_SGIX;
        if (!strcmp(name, "GL_PROGRAM_NATIVE_ALU_INSTRUCTIONS_ARB")) return GL_PROGRAM_NATIVE_ALU_INSTRUCTIONS_ARB;
        if (!strcmp(name, "GL_PROGRAM_NATIVE_TEX_INDIRECTIONS_ARB")) return GL_PROGRAM_NATIVE_TEX_INDIRECTIONS_ARB;
        if (!strcmp(name, "GL_PROGRAM_NATIVE_TEX_INSTRUCTIONS_ARB")) return GL_PROGRAM_NATIVE_TEX_INSTRUCTIONS_ARB;
        if (!strcmp(name, "GL_PROXY_POST_COLOR_MATRIX_COLOR_TABLE")) return GL_PROXY_POST_COLOR_MATRIX_COLOR_TABLE;
        if (!strcmp(name, "GL_TEXTURE_COORD_ARRAY_LIST_STRIDE_IBM")) return GL_TEXTURE_COORD_ARRAY_LIST_STRIDE_IBM;
        if (!strcmp(name, "GL_TRANSFORM_FEEDBACK_BUFFER_ACTIVE_NV")) return GL_TRANSFORM_FEEDBACK_BUFFER_ACTIVE_NV;
        if (!strcmp(name, "GL_TRANSFORM_FEEDBACK_BUFFER_PAUSED_NV")) return GL_TRANSFORM_FEEDBACK_BUFFER_PAUSED_NV;
        if (!strcmp(name, "GL_TRANSFORM_FEEDBACK_BUFFER_START_EXT")) return GL_TRANSFORM_FEEDBACK_BUFFER_START_EXT;
        if (!strcmp(name, "GL_VERTEX_ARRAY_RANGE_WITHOUT_FLUSH_NV")) return GL_VERTEX_ARRAY_RANGE_WITHOUT_FLUSH_NV;
        return _GL_UNDEFINED;
    case 39:
        if (!strcmp(name, "GL_ATIX_vertex_shader_output_point_size")) return GL_ATIX_vertex_shader_output_point_size;
        if (!strcmp(name, "GL_COMPRESSED_LUMINANCE_ALPHA_LATC2_EXT")) return GL_COMPRESSED_LUMINANCE_ALPHA_LATC2_EXT;
        if (!strcmp(name, "GL_FOG_COORDINATE_ARRAY_LIST_STRIDE_IBM")) return GL_FOG_COORDINATE_ARRAY_LIST_STRIDE_IBM;
        if (!strcmp(name, "GL_FRAGMENT_PROGRAM_PARAMETER_BUFFER_NV")) return GL_FRAGMENT_PROGRAM_PARAMETER_BUFFER_NV;
        if (!strcmp(name, "GL_FRAMEBUFFER_ATTACHMENT_TEXTURE_LAYER")) return GL_FRAMEBUFFER_ATTACHMENT_TEXTURE_LAYER;
        if (!strcmp(name, "GL_FRAMEBUFFER_ATTACHMENT_TEXTURE_LAYER")) return GL_FRAMEBUFFER_ATTACHMENT_TEXTURE_LAYER;
        if (!strcmp(name, "GL_FRAMEBUFFER_ATTACHMENT_TEXTURE_LEVEL")) return GL_FRAMEBUFFER_ATTACHMENT_TEXTURE_LEVEL;
        if (!strcmp(name, "GL_GEOMETRY_PROGRAM_PARAMETER_BUFFER_NV")) return GL_GEOMETRY_PROGRAM_PARAMETER_BUFFER_NV;
        if (!strcmp(name, "GL_IMPLEMENTATION_COLOR_READ_FORMAT_OES")) return GL_IMPLEMENTATION_COLOR_READ_FORMAT_OES;
        if (!strcmp(name, "GL_MAX_COMBINED_TEXTURE_IMAGE_UNITS_ARB")) return GL_MAX_COMBINED_TEXTURE_IMAGE_UNITS_ARB;
        if (!strcmp(name, "GL_MAX_GEOMETRY_TEXTURE_IMAGE_UNITS_ARB")) return GL_MAX_GEOMETRY_TEXTURE_IMAGE_UNITS_ARB;
        if (!strcmp(name, "GL_MAX_GEOMETRY_TEXTURE_IMAGE_UNITS_EXT")) return GL_MAX_GEOMETRY_TEXTURE_IMAGE_UNITS_EXT;
        if (!strcmp(name, "GL_MAX_PROGRAM_PARAMETER_BUFFER_SIZE_NV")) return GL_MAX_PROGRAM_PARAMETER_BUFFER_SIZE_NV;
        if (!strcmp(name, "GL_MAX_VERTEX_ARRAY_RANGE_ELEMENT_APPLE")) return GL_MAX_VERTEX_ARRAY_RANGE_ELEMENT_APPLE;
        if (!strcmp(name, "GL_NORMAL_ARRAY_PARALLEL_POINTERS_INTEL")) return GL_NORMAL_ARRAY_PARALLEL_POINTERS_INTEL;
        if (!strcmp(name, "GL_OBJECT_ACTIVE_UNIFORM_MAX_LENGTH_ARB")) return GL_OBJECT_ACTIVE_UNIFORM_MAX_LENGTH_ARB;
        if (!strcmp(name, "GL_OFFSET_HILO_PROJECTIVE_TEXTURE_2D_NV")) return GL_OFFSET_HILO_PROJECTIVE_TEXTURE_2D_NV;
        if (!strcmp(name, "GL_POST_TEXTURE_FILTER_SCALE_RANGE_SGIX")) return GL_POST_TEXTURE_FILTER_SCALE_RANGE_SGIX;
        if (!strcmp(name, "GL_PROGRAM_NATIVE_ADDRESS_REGISTERS_ARB")) return GL_PROGRAM_NATIVE_ADDRESS_REGISTERS_ARB;
        if (!strcmp(name, "GL_RGBA_UNSIGNED_DOT_PRODUCT_MAPPING_NV")) return GL_RGBA_UNSIGNED_DOT_PRODUCT_MAPPING_NV;
        if (!strcmp(name, "GL_SECONDARY_COLOR_ARRAY_BUFFER_BINDING")) return GL_SECONDARY_COLOR_ARRAY_BUFFER_BINDING;
        if (!strcmp(name, "GL_TRANSFORM_FEEDBACK_BUFFER_BINDING_NV")) return GL_TRANSFORM_FEEDBACK_BUFFER_BINDING_NV;
        if (!strcmp(name, "GL_UNSIGNED_INT_SAMPLER_RENDERBUFFER_NV")) return GL_UNSIGNED_INT_SAMPLER_RENDERBUFFER_NV;
        if (!strcmp(name, "GL_VERTEX_ARRAY_PARALLEL_POINTERS_INTEL")) return GL_VERTEX_ARRAY_PARALLEL_POINTERS_INTEL;
        return _GL_UNDEFINED;
    case 40:
        if (!strcmp(name, "GL_COMPRESSED_SIGNED_LUMINANCE_LATC1_EXT")) return GL_COMPRESSED_SIGNED_LUMINANCE_LATC1_EXT;
        if (!strcmp(name, "GL_COMPRESSED_SIGNED_RED_GREEN_RGTC2_EXT")) return GL_COMPRESSED_SIGNED_RED_GREEN_RGTC2_EXT;
        if (!strcmp(name, "GL_FRAGMENT_COLOR_MATERIAL_PARAMETER_EXT")) return GL_FRAGMENT_COLOR_MATERIAL_PARAMETER_EXT;
        if (!strcmp(name, "GL_FRAGMENT_LIGHT_MODEL_LOCAL_VIEWER_EXT")) return GL_FRAGMENT_LIGHT_MODEL_LOCAL_VIEWER_EXT;
        if (!strcmp(name, "GL_FRAMEBUFFER_ATTACHMENT_COLOR_ENCODING")) return GL_FRAMEBUFFER_ATTACHMENT_COLOR_ENCODING;
        if (!strcmp(name, "GL_FRAMEBUFFER_ATTACHMENT_COMPONENT_TYPE")) return GL_FRAMEBUFFER_ATTACHMENT_COMPONENT_TYPE;
        if (!strcmp(name, "GL_FRAMEBUFFER_INCOMPLETE_ATTACHMENT_EXT")) return GL_FRAMEBUFFER_INCOMPLETE_ATTACHMENT_EXT;
        if (!strcmp(name, "GL_FRAMEBUFFER_INCOMPLETE_DIMENSIONS_EXT")) return GL_FRAMEBUFFER_INCOMPLETE_DIMENSIONS_EXT;
        if (!strcmp(name, "GL_MAX_VERTEX_SHADER_LOCAL_CONSTANTS_EXT")) return GL_MAX_VERTEX_SHADER_LOCAL_CONSTANTS_EXT;
        if (!strcmp(name, "GL_NUM_INPUT_INTERPOLATOR_COMPONENTS_ATI")) return GL_NUM_INPUT_INTERPOLATOR_COMPONENTS_ATI;
        if (!strcmp(name, "GL_OFFSET_PROJECTIVE_TEXTURE_2D_SCALE_NV")) return GL_OFFSET_PROJECTIVE_TEXTURE_2D_SCALE_NV;
        if (!strcmp(name, "GL_SECONDARY_COLOR_ARRAY_LIST_STRIDE_IBM")) return GL_SECONDARY_COLOR_ARRAY_LIST_STRIDE_IBM;
        if (!strcmp(name, "GL_TEXTURE_BUFFER_DATA_STORE_BINDING_ARB")) return GL_TEXTURE_BUFFER_DATA_STORE_BINDING_ARB;
        if (!strcmp(name, "GL_TEXTURE_BUFFER_DATA_STORE_BINDING_EXT")) return GL_TEXTURE_BUFFER_DATA_STORE_BINDING_EXT;
        if (!strcmp(name, "GL_TRANSFORM_FEEDBACK_BUFFER_BINDING_EXT")) return GL_TRANSFORM_FEEDBACK_BUFFER_BINDING_EXT;
        if (!strcmp(name, "GL_TRANSFORM_FEEDBACK_PRIMITIVES_WRITTEN")) return GL_TRANSFORM_FEEDBACK_PRIMITIVES_WRITTEN;
        if (!strcmp(name, "GL_TRANSFORM_FEEDBACK_VARYING_MAX_LENGTH")) return GL_TRANSFORM_FEEDBACK_VARYING_MAX_LENGTH;
        return _GL_UNDEFINED;
    case 41:
        if (!strcmp(name, "GL_FRAMEBUFFER_ATTACHMENT_OBJECT_NAME_EXT")) return GL_FRAMEBUFFER_ATTACHMENT_OBJECT_NAME_EXT;
        if (!strcmp(name, "GL_FRAMEBUFFER_ATTACHMENT_OBJECT_TYPE_EXT")) return GL_FRAMEBUFFER_ATTACHMENT_OBJECT_TYPE_EXT;
        if (!strcmp(name, "GL_FRAMEBUFFER_INCOMPLETE_DRAW_BUFFER_EXT")) return GL_FRAMEBUFFER_INCOMPLETE_DRAW_BUFFER_EXT;
        if (!strcmp(name, "GL_FRAMEBUFFER_INCOMPLETE_LAYER_COUNT_ARB")) return GL_FRAMEBUFFER_INCOMPLETE_LAYER_COUNT_ARB;
        if (!strcmp(name, "GL_FRAMEBUFFER_INCOMPLETE_LAYER_COUNT_EXT")) return GL_FRAMEBUFFER_INCOMPLETE_LAYER_COUNT_EXT;
        if (!strcmp(name, "GL_FRAMEBUFFER_INCOMPLETE_MULTISAMPLE_EXT")) return GL_FRAMEBUFFER_INCOMPLETE_MULTISAMPLE_EXT;
        if (!strcmp(name, "GL_FRAMEBUFFER_INCOMPLETE_READ_BUFFER_EXT")) return GL_FRAMEBUFFER_INCOMPLETE_READ_BUFFER_EXT;
        if (!strcmp(name, "GL_MAX_OPTIMIZED_VERTEX_SHADER_LOCALS_EXT")) return GL_MAX_OPTIMIZED_VERTEX_SHADER_LOCALS_EXT;
        if (!strcmp(name, "GL_MAX_PIXEL_TRANSFORM_2D_STACK_DEPTH_EXT")) return GL_MAX_PIXEL_TRANSFORM_2D_STACK_DEPTH_EXT;
        if (!strcmp(name, "GL_MAX_PN_TRIANGLES_TESSELATION_LEVEL_ATI")) return GL_MAX_PN_TRIANGLES_TESSELATION_LEVEL_ATI;
        if (!strcmp(name, "GL_MAX_PROGRAM_TOTAL_OUTPUT_COMPONENTS_NV")) return GL_MAX_PROGRAM_TOTAL_OUTPUT_COMPONENTS_NV;
        if (!strcmp(name, "GL_OBJECT_ACTIVE_ATTRIBUTE_MAX_LENGTH_ARB")) return GL_OBJECT_ACTIVE_ATTRIBUTE_MAX_LENGTH_ARB;
        if (!strcmp(name, "GL_OFFSET_PROJECTIVE_TEXTURE_RECTANGLE_NV")) return GL_OFFSET_PROJECTIVE_TEXTURE_RECTANGLE_NV;
        if (!strcmp(name, "GL_PN_TRIANGLES_NORMAL_MODE_QUADRATIC_ATI")) return GL_PN_TRIANGLES_NORMAL_MODE_QUADRATIC_ATI;
        if (!strcmp(name, "GL_PROXY_POST_CONVOLUTION_COLOR_TABLE_SGI")) return GL_PROXY_POST_CONVOLUTION_COLOR_TABLE_SGI;
        if (!strcmp(name, "GL_TEXTURE_COORD_ARRAY_BUFFER_BINDING_ARB")) return GL_TEXTURE_COORD_ARRAY_BUFFER_BINDING_ARB;
        if (!strcmp(name, "GL_VERTEX_ATTRIB_ARRAY_BUFFER_BINDING_ARB")) return GL_VERTEX_ATTRIB_ARRAY_BUFFER_BINDING_ARB;
        return _GL_UNDEFINED;
    case 42:
        if (!strcmp(name, "GL_FOG_COORDINATE_ARRAY_BUFFER_BINDING_ARB")) return GL_FOG_COORDINATE_ARRAY_BUFFER_BINDING_ARB;
        if (!strcmp(name, "GL_LUMINANCE16_ALPHA16_EXTENDED_RANGE_SGIX")) return GL_LUMINANCE16_ALPHA16_EXTENDED_RANGE_SGIX;
        if (!strcmp(name, "GL_MAX_PROGRAM_NATIVE_ALU_INSTRUCTIONS_ARB")) return GL_MAX_PROGRAM_NATIVE_ALU_INSTRUCTIONS_ARB;
        if (!strcmp(name, "GL_MAX_PROGRAM_NATIVE_TEX_INDIRECTIONS_ARB")) return GL_MAX_PROGRAM_NATIVE_TEX_INDIRECTIONS_ARB;
        if (!strcmp(name, "GL_MAX_PROGRAM_NATIVE_TEX_INSTRUCTIONS_ARB")) return GL_MAX_PROGRAM_NATIVE_TEX_INSTRUCTIONS_ARB;
        if (!strcmp(name, "GL_MAX_TRANSFORM_FEEDBACK_SEPARATE_ATTRIBS")) return GL_MAX_TRANSFORM_FEEDBACK_SEPARATE_ATTRIBS;
        if (!strcmp(name, "GL_PROXY_POST_COLOR_MATRIX_COLOR_TABLE_SGI")) return GL_PROXY_POST_COLOR_MATRIX_COLOR_TABLE_SGI;
        return _GL_UNDEFINED;
    case 43:
        if (!strcmp(name, "GL_FRAMEBUFFER_ATTACHMENT_TEXTURE_LAYER_EXT")) return GL_FRAMEBUFFER_ATTACHMENT_TEXTURE_LAYER_EXT;
        if (!strcmp(name, "GL_FRAMEBUFFER_ATTACHMENT_TEXTURE_LEVEL_EXT")) return GL_FRAMEBUFFER_ATTACHMENT_TEXTURE_LEVEL_EXT;
        if (!strcmp(name, "GL_FRAMEBUFFER_INCOMPLETE_LAYER_TARGETS_ARB")) return GL_FRAMEBUFFER_INCOMPLETE_LAYER_TARGETS_ARB;
        if (!strcmp(name, "GL_FRAMEBUFFER_INCOMPLETE_LAYER_TARGETS_EXT")) return GL_FRAMEBUFFER_INCOMPLETE_LAYER_TARGETS_EXT;
        if (!strcmp(name, "GL_MAX_FRAGMENT_PROGRAM_LOCAL_PARAMETERS_NV")) return GL_MAX_FRAGMENT_PROGRAM_LOCAL_PARAMETERS_NV;
        if (!strcmp(name, "GL_MAX_GEOMETRY_TOTAL_OUTPUT_COMPONENTS_ARB")) return GL_MAX_GEOMETRY_TOTAL_OUTPUT_COMPONENTS_ARB;
        if (!strcmp(name, "GL_MAX_GEOMETRY_TOTAL_OUTPUT_COMPONENTS_EXT")) return GL_MAX_GEOMETRY_TOTAL_OUTPUT_COMPONENTS_EXT;
        if (!strcmp(name, "GL_MAX_OPTIMIZED_VERTEX_SHADER_VARIANTS_EXT")) return GL_MAX_OPTIMIZED_VERTEX_SHADER_VARIANTS_EXT;
        if (!strcmp(name, "GL_MAX_PROGRAM_NATIVE_ADDRESS_REGISTERS_ARB")) return GL_MAX_PROGRAM_NATIVE_ADDRESS_REGISTERS_ARB;
        if (!strcmp(name, "GL_MAX_PROGRAM_PARAMETER_BUFFER_BINDINGS_NV")) return GL_MAX_PROGRAM_PARAMETER_BUFFER_BINDINGS_NV;
        if (!strcmp(name, "GL_SECONDARY_COLOR_ARRAY_BUFFER_BINDING_ARB")) return GL_SECONDARY_COLOR_ARRAY_BUFFER_BINDING_ARB;
        if (!strcmp(name, "GL_TRANSFORM_FEEDBACK_PRIMITIVES_WRITTEN_NV")) return GL_TRANSFORM_FEEDBACK_PRIMITIVES_WRITTEN_NV;
        return _GL_UNDEFINED;
    case 44:
        if (!strcmp(name, "GL_DOT_PRODUCT_CONST_EYE_REFLECT_CUBE_MAP_NV")) return GL_DOT_PRODUCT_CONST_EYE_REFLECT_CUBE_MAP_NV;
        if (!strcmp(name, "GL_FRAMEBUFFER_INCOMPLETE_MISSING_ATTACHMENT")) return GL_FRAMEBUFFER_INCOMPLETE_MISSING_ATTACHMENT;
        if (!strcmp(name, "GL_TRANSFORM_FEEDBACK_PRIMITIVES_WRITTEN_EXT")) return GL_TRANSFORM_FEEDBACK_PRIMITIVES_WRITTEN_EXT;
        if (!strcmp(name, "GL_TRANSFORM_FEEDBACK_VARYING_MAX_LENGTH_EXT")) return GL_TRANSFORM_FEEDBACK_VARYING_MAX_LENGTH_EXT;
        return _GL_UNDEFINED;
    case 45:
        if (!strcmp(name, "GL_MAX_OPTIMIZED_VERTEX_SHADER_INVARIANTS_EXT")) return GL_MAX_OPTIMIZED_VERTEX_SHADER_INVARIANTS_EXT;
        if (!strcmp(name, "GL_MAX_TRANSFORM_FEEDBACK_SEPARATE_ATTRIBS_NV")) return GL_MAX_TRANSFORM_FEEDBACK_SEPARATE_ATTRIBS_NV;
        if (!strcmp(name, "GL_MAX_TRANSFORM_FEEDBACK_SEPARATE_COMPONENTS")) return GL_MAX_TRANSFORM_FEEDBACK_SEPARATE_COMPONENTS;
        if (!strcmp(name, "GL_TEXTURE_RENDERBUFFER_DATA_STORE_BINDING_NV")) return GL_TEXTURE_RENDERBUFFER_DATA_STORE_BINDING_NV;
        return _GL_UNDEFINED;
    case 46:
        if (!strcmp(name, "GL_COMPRESSED_SIGNED_LUMINANCE_ALPHA_LATC2_EXT")) return GL_COMPRESSED_SIGNED_LUMINANCE_ALPHA_LATC2_EXT;
        if (!strcmp(name, "GL_MAX_TRANSFORM_FEEDBACK_SEPARATE_ATTRIBS_EXT")) return GL_MAX_TRANSFORM_FEEDBACK_SEPARATE_ATTRIBS_EXT;
        if (!strcmp(name, "GL_OFFSET_HILO_PROJECTIVE_TEXTURE_RECTANGLE_NV")) return GL_OFFSET_HILO_PROJECTIVE_TEXTURE_RECTANGLE_NV;
        if (!strcmp(name, "GL_TEXTURE_COORD_ARRAY_PARALLEL_POINTERS_INTEL")) return GL_TEXTURE_COORD_ARRAY_PARALLEL_POINTERS_INTEL;
        return _GL_UNDEFINED;
    case 47:
        if (!strcmp(name, "GL_FRAMEBUFFER_ATTACHMENT_TEXTURE_CUBE_MAP_FACE")) return GL_FRAMEBUFFER_ATTACHMENT_TEXTURE_CUBE_MAP_FACE;
        if (!strcmp(name, "GL_MAX_OPTIMIZED_VERTEX_SHADER_INSTRUCTIONS_EXT")) return GL_MAX_OPTIMIZED_VERTEX_SHADER_INSTRUCTIONS_EXT;
        if (!strcmp(name, "GL_OFFSET_PROJECTIVE_TEXTURE_RECTANGLE_SCALE_NV")) return GL_OFFSET_PROJECTIVE_TEXTURE_RECTANGLE_SCALE_NV;
        return _GL_UNDEFINED;
    case 48:
        if (!strcmp(name, "GL_FRAGMENT_LIGHT_MODEL_NORMAL_INTERPOLATION_EXT")) return GL_FRAGMENT_LIGHT_MODEL_NORMAL_INTERPOLATION_EXT;
        if (!strcmp(name, "GL_FRAMEBUFFER_ATTACHMENT_TEXTURE_3D_ZOFFSET_EXT")) return GL_FRAMEBUFFER_ATTACHMENT_TEXTURE_3D_ZOFFSET_EXT;
        if (!strcmp(name, "GL_FRAMEBUFFER_INCOMPLETE_MISSING_ATTACHMENT_EXT")) return GL_FRAMEBUFFER_INCOMPLETE_MISSING_ATTACHMENT_EXT;
        if (!strcmp(name, "GL_MAX_TRANSFORM_FEEDBACK_INTERLEAVED_COMPONENTS")) return GL_MAX_TRANSFORM_FEEDBACK_INTERLEAVED_COMPONENTS;
        if (!strcmp(name, "GL_MAX_TRANSFORM_FEEDBACK_SEPARATE_COMPONENTS_NV")) return GL_MAX_TRANSFORM_FEEDBACK_SEPARATE_COMPONENTS_NV;
        return _GL_UNDEFINED;
    case 49:
        if (!strcmp(name, "GL_MAX_TRANSFORM_FEEDBACK_SEPARATE_COMPONENTS_EXT")) return GL_MAX_TRANSFORM_FEEDBACK_SEPARATE_COMPONENTS_EXT;
        return _GL_UNDEFINED;
    case 50:
        if (!strcmp(name, "GL_MAX_OPTIMIZED_VERTEX_SHADER_LOCAL_CONSTANTS_EXT")) return GL_MAX_OPTIMIZED_VERTEX_SHADER_LOCAL_CONSTANTS_EXT;
        return _GL_UNDEFINED;
    case 51:
        if (!strcmp(name, "GL_FRAMEBUFFER_ATTACHMENT_TEXTURE_CUBE_MAP_FACE_EXT")) return GL_FRAMEBUFFER_ATTACHMENT_TEXTURE_CUBE_MAP_FACE_EXT;
        if (!strcmp(name, "GL_MAX_TRANSFORM_FEEDBACK_INTERLEAVED_COMPONENTS_NV")) return GL_MAX_TRANSFORM_FEEDBACK_INTERLEAVED_COMPONENTS_NV;
        return _GL_UNDEFINED;
    case 52:
        if (!strcmp(name, "GL_MAX_TRANSFORM_FEEDBACK_INTERLEAVED_COMPONENTS_EXT")) return GL_MAX_TRANSFORM_FEEDBACK_INTERLEAVED_COMPONENTS_EXT;
